@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,6 +42,7 @@ import java.util.TimeZone;
 
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.jdbc.JDBCConnection;
 import org.hsqldb.lib.DataOutputStream;
 import org.hsqldb.navigator.RowSetNavigatorClient;
 import org.hsqldb.persist.HsqlProperties;
@@ -63,7 +64,7 @@ import org.hsqldb.types.TimestampData;
  * protocol.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.6
  * @since 1.7.2
  */
 public class ClientConnection implements SessionInterface {
@@ -82,8 +83,8 @@ public class ClientConnection implements SessionInterface {
      * are multiplied by 100 to power p and added up, then negated, to form the
      * integer representation of version string.
      */
-    public static final String NETWORK_COMPATIBILITY_VERSION     = "1.9.0.0";
-    public static final int    NETWORK_COMPATIBILITY_VERSION_INT = -1090000;
+    public static final String NETWORK_COMPATIBILITY_VERSION     = "2.1.0.0";
+    public static final int    NETWORK_COMPATIBILITY_VERSION_INT = -2010000;
 
     //
     static final int             BUFFER_SIZE = 0x1000;
@@ -96,7 +97,7 @@ public class ClientConnection implements SessionInterface {
     protected RowInputBinary     rowIn;
     private Result               resultOut;
     private long                 sessionID;
-    private long                 lobIDSequence;
+    private long                 lobIDSequence = -1;
 
     //
     private boolean  isReadOnlyDefault = false;
@@ -107,6 +108,7 @@ public class ClientConnection implements SessionInterface {
     private Calendar calendar;
 
     //
+    JDBCConnection connection;
     String         host;
     int            port;
     String         path;
@@ -115,6 +117,7 @@ public class ClientConnection implements SessionInterface {
     int            databaseID;
     String         clientPropertiesString;
     HsqlProperties clientProperties;
+    String         databaseUniqueName;
 
     /**
      * Establishes a connection to the server.
@@ -146,6 +149,7 @@ public class ClientConnection implements SessionInterface {
 
         sessionID              = resultIn.getSessionId();
         databaseID             = resultIn.getDatabaseId();
+        databaseUniqueName     = resultIn.getDatabaseName();
         clientPropertiesString = resultIn.getMainString();
     }
 
@@ -481,7 +485,7 @@ public class ClientConnection implements SessionInterface {
     }
 
     protected void write(Result r) throws IOException, HsqlException {
-        r.write(dataOutput, rowOut);
+        r.write(this, dataOutput, rowOut);
     }
 
     protected Result read() throws IOException, HsqlException {
@@ -489,7 +493,7 @@ public class ClientConnection implements SessionInterface {
         Result result = Result.newResult(dataInput, rowIn);
 
         result.readAdditionalResults(this, dataInput, rowIn);
-        rowOut.setBuffer(mainBuffer);
+        rowOut.reset(mainBuffer);
         rowIn.resetRow(mainBuffer.length);
 
         return result;
@@ -503,7 +507,7 @@ public class ClientConnection implements SessionInterface {
     }
 
     public synchronized long getLobId() {
-        return lobIDSequence++;
+        return lobIDSequence--;
     }
 
     public BlobDataID createBlob(long length) {
@@ -559,15 +563,15 @@ public class ClientConnection implements SessionInterface {
     }
 
     public int getStreamBlockSize() {
-        return 512 * 1024;
+        return lobStreamBlockSize;
     }
 
     public HsqlProperties getClientProperties() {
 
         if (clientProperties == null) {
             if (clientPropertiesString.length() > 0) {
-                HsqlProperties.delimitedArgPairsToProps(clientPropertiesString,
-                        "=", ";", null);
+                clientProperties = HsqlProperties.delimitedArgPairsToProps(
+                    clientPropertiesString, "=", ";", null);
             } else {
                 clientProperties = new HsqlProperties();
             }
@@ -576,12 +580,24 @@ public class ClientConnection implements SessionInterface {
         return clientProperties;
     }
 
+    public JDBCConnection getJDBCConnection() {
+        return connection;
+    }
+
+    public void setJDBCConnection(JDBCConnection connection) {
+        this.connection = connection;
+    }
+
+    public String getDatabaseUniqueName() {
+        return databaseUniqueName;
+    }
+
     /**
      * Converts specified encoded integer to a Network Compatibility Version
      * String. The tranmitted integer is negative to distinguish it from
      * 7 bit ASCII characters.
      */
-    static public String toNetCompVersionString(int i) {
+    public static String toNetCompVersionString(int i) {
 
         StringBuffer sb = new StringBuffer();
 
