@@ -1,7 +1,4 @@
-/*
- * For work developed by the HSQL Development Group:
- *
- * Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,42 +26,6 @@
  * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *
- *
- * For work originally developed by the Hypersonic SQL Group:
- *
- * Copyright (c) 1995-2000, The Hypersonic SQL Group.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- * Redistributions of source code must retain the above copyright notice, this
- * list of conditions and the following disclaimer.
- *
- * Redistributions in binary form must reproduce the above copyright notice,
- * this list of conditions and the following disclaimer in the documentation
- * and/or other materials provided with the distribution.
- *
- * Neither the name of the Hypersonic SQL Group nor the names of its
- * contributors may be used to endorse or promote products derived from this
- * software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE HYPERSONIC SQL GROUP,
- * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- * This software consists of voluntary contributions made by many individuals
- * on behalf of the Hypersonic SQL Group.
  */
 
 
@@ -83,20 +44,13 @@ import org.hsqldb.result.Result;
 import org.hsqldb.rights.Grantee;
 import org.hsqldb.types.Type;
 
-// fredt@users 20020225 - patch 1.7.0 by boucherb@users - named constraints
-// fredt@users 20020320 - doc 1.7.0 - update
-// tony_lai@users 20020820 - patch 595156 - violation of Integrity constraint name
-
 /**
  * Implementation of a table constraint with references to the indexes used
  * by the constraint.<p>
  *
- * Partly based on Hypersonic code.
- *
- * @author Thomas Mueller (Hypersonic SQL Group)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
- * @since Hypersonic SQL
+ * @version 2.0.1
+ * @since 1.6.0
  */
 public final class Constraint implements SchemaObject {
 
@@ -116,7 +70,7 @@ public final class Constraint implements SchemaObject {
     OrderedHashSet refColSet;
 
     //
-    final public static Constraint[] emptyArray = new Constraint[]{};
+    public static final Constraint[] emptyArray = new Constraint[]{};
 
     private Constraint() {}
 
@@ -662,7 +616,7 @@ public final class Constraint implements SchemaObject {
             case SchemaObject.ConstraintTypes.FOREIGN_KEY :
                 return ArrayUtil.find(core.refCols, colIndex) != -1
                        && (core.mainCols.length != 1
-                           || core.mainTable == core.refTable);
+                           || core.mainTable != core.refTable);
 
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500, "Constraint");
@@ -689,20 +643,23 @@ public final class Constraint implements SchemaObject {
         }
     }
 
-// fredt@users 20020225 - patch 1.7.0 by fredt - duplicate constraints
-
     /**
      * Compares this with another constraint column set. This is used only for
      * UNIQUE constraints.
      */
     boolean isUniqueWithColumns(int[] cols) {
 
-        if (constType != SchemaObject.ConstraintTypes.UNIQUE
-                || core.mainCols.length != cols.length) {
-            return false;
+        switch (constType) {
+
+            case SchemaObject.ConstraintTypes.PRIMARY_KEY :
+            case SchemaObject.ConstraintTypes.UNIQUE :
+                if (core.mainCols.length == cols.length) {
+                    return ArrayUtil.haveEqualSets(core.mainCols, cols,
+                                                   cols.length);
+                }
         }
 
-        return ArrayUtil.haveEqualSets(core.mainCols, cols, cols.length);
+        return false;
     }
 
     /**
@@ -712,17 +669,22 @@ public final class Constraint implements SchemaObject {
     boolean isEquivalent(Table mainTable, int[] mainCols, Table refTable,
                          int[] refCols) {
 
-        if (constType != SchemaObject.ConstraintTypes.MAIN
-                && constType != SchemaObject.ConstraintTypes.FOREIGN_KEY) {
-            return false;
+        switch (constType) {
+
+            case SchemaObject.ConstraintTypes.MAIN :
+            case SchemaObject.ConstraintTypes.FOREIGN_KEY :
+                if (mainTable != core.mainTable || refTable != core.refTable) {
+                    return false;
+                }
+
+                if (core.mainCols.length == mainCols.length
+                        && core.refCols.length == refCols.length) {
+                    return ArrayUtil.areEqualSets(core.mainCols, mainCols)
+                           && ArrayUtil.areEqualSets(core.refCols, refCols);
+                }
         }
 
-        if (mainTable != core.mainTable || refTable != core.refTable) {
-            return false;
-        }
-
-        return ArrayUtil.areEqualSets(core.mainCols, mainCols)
-               && ArrayUtil.areEqualSets(core.refCols, refCols);
+        return false;
     }
 
     /**
@@ -788,8 +750,7 @@ public final class Constraint implements SchemaObject {
                 return;
 
             case SchemaObject.ConstraintTypes.FOREIGN_KEY :
-                PersistentStore store =
-                    session.sessionData.getRowStore(core.mainTable);
+                PersistentStore store = core.mainTable.getRowStore(session);
 
                 if (ArrayUtil.hasNull(data, core.refCols)) {
                     if (core.matchType == OpTypes.MATCH_SIMPLE) {
@@ -844,7 +805,8 @@ public final class Constraint implements SchemaObject {
         }
     }
 
-    void checkCheckConstraint(Session session, Table table, Object data) {
+    void checkCheckConstraint(Session session, Table table,
+                              ColumnSchema column, Object data) {
 
         session.sessionData.currentValue = data;
 
@@ -853,16 +815,16 @@ public final class Constraint implements SchemaObject {
         session.sessionData.currentValue = null;
 
         if (nomatch) {
-            if (table == null) {
-                throw Error.error(ErrorCode.X_23513, name.name);
-            } else {
-                String[] info = new String[] {
-                    name.name, table.getName().name
-                };
+            String[] info = new String[] {
+                name.statementName,
+                table == null ? ""
+                              : table.getName().statementName,
+                column == null ? ""
+                               : column.getName().statementName,
+            };
 
-                throw Error.error(null, ErrorCode.X_23513,
-                                  ErrorCode.CONSTRAINT, info);
-            }
+            throw Error.error(null, ErrorCode.X_23513,
+                              ErrorCode.COLUMN_CONSTRAINT, info);
         }
     }
 
@@ -871,7 +833,7 @@ public final class Constraint implements SchemaObject {
         switch (this.constType) {
 
             case SchemaObject.ConstraintTypes.CHECK : {
-                String[] info = new String[]{ name.name };
+                String[] info = new String[]{ name.statementName };
 
                 return Error.error(null, ErrorCode.X_23513,
                                    ErrorCode.CONSTRAINT, info);
@@ -888,8 +850,8 @@ public final class Constraint implements SchemaObject {
                 }
 
                 String[] info = new String[] {
-                    getName().statementName,
-                    core.refTable.getName().statementName, sb.toString()
+                    name.statementName, core.refTable.getName().statementName,
+                    sb.toString()
                 };
 
                 return Error.error(null, ErrorCode.X_23503,
@@ -909,8 +871,8 @@ public final class Constraint implements SchemaObject {
 
                 return Error.error(null, ErrorCode.X_23505,
                                    ErrorCode.CONSTRAINT, new String[] {
-                    getName().statementName,
-                    core.mainTable.getName().statementName, sb.toString()
+                    name.statementName, core.mainTable.getName().statementName,
+                    sb.toString()
                 });
             }
             default :
@@ -940,7 +902,7 @@ public final class Constraint implements SchemaObject {
             return core.refIndex.emptyIterator();
         }
 
-        PersistentStore store = session.sessionData.getRowStore(core.refTable);
+        PersistentStore store = core.refTable.getRowStore(session);
 
         return core.refIndex.findFirstRow(session, store, row, core.mainCols);
     }
@@ -987,6 +949,7 @@ public final class Constraint implements SchemaObject {
         Scanner   scanner = new Scanner(ddl);
         ParserDQL parser  = new ParserDQL(session, scanner);
 
+        parser.compileContext.reset(0);
         parser.read();
 
         parser.isCheckOrTriggerCondition = true;
@@ -1019,7 +982,7 @@ public final class Constraint implements SchemaObject {
 
             if (r.getNavigator().getSize() != 0) {
                 String[] info = new String[] {
-                    table.getName().name, ""
+                    name.statementName, table.getName().statementName
                 };
 
                 throw Error.error(null, ErrorCode.X_23513,
