@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,10 @@ import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.dbinfo.DatabaseInformation;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.lib.FrameworkLogger;
+import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HsqlArrayList;
+import org.hsqldb.lib.HsqlTimer;
 import org.hsqldb.persist.HsqlDatabaseProperties;
 import org.hsqldb.persist.HsqlProperties;
 import org.hsqldb.persist.LobManager;
@@ -45,12 +48,12 @@ import org.hsqldb.result.Result;
 import org.hsqldb.rights.GranteeManager;
 import org.hsqldb.rights.User;
 import org.hsqldb.rights.UserManager;
-import org.hsqldb.types.Type;
+import org.hsqldb.store.ValuePool;
+import org.hsqldb.types.Collation;
 
 // incorporates following contributions
 // boucherb@users - javadoc comments
-// Brendan Ryan - data files in Jar
-// Ocke Jansen oj@openoffice.org - file access api
+// Ocke Jansen (oj@openoffice dot org) - file access api
 
 /**
  * Database is the root class for HSQL Database Engine database. <p>
@@ -58,22 +61,23 @@ import org.hsqldb.types.Type;
  * It holds the data structures that form an HSQLDB database instance.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 public class Database {
 
-    int                   databaseID;
-    String                databaseUniqueName;
-    String                databaseType;
-    private final String  canonicalPath;
-    public HsqlProperties urlProperties;
-    private final String  path;
-    DatabaseInformation   dbInfo;
+    int                        databaseID;
+    String                     databaseUniqueName;
+    String                     databaseType;
+    private final String       canonicalPath;
+    public HsqlProperties      urlProperties;
+    private final String       path;
+    public Collation           collation;
+    public DatabaseInformation dbInfo;
 
     /** indicates the state of the database */
-    private int   dbState;
-    public Logger logger;
+    private volatile int dbState;
+    public Logger        logger;
 
     /** true means that all tables are readonly. */
     boolean databaseReadOnly;
@@ -84,12 +88,34 @@ public class Database {
      */
     private boolean filesReadOnly;
 
-    /** true means filesReadOnly but CACHED and TEXT tables are disallowed */
-    private boolean               filesInJar;
-    public boolean                sqlEnforceRefs;
-    public boolean                sqlEnforceSize;
-    public boolean                sqlEnforceNames;
-    private boolean               isReferentialIntegrity;
+    /** true means filesReadOnly */
+    private boolean filesInJar;
+
+    /**
+     * Defaults are used in version upgrades, but overridden by
+     *  databaseProperties or URL properties for new databases.
+     */
+    public boolean                sqlEnforceTypes        = false;
+    public boolean                sqlEnforceRefs         = false;
+    public boolean                sqlEnforceSize         = true;
+    public boolean                sqlEnforceNames        = false;
+    public boolean                sqlRegularNames        = true;
+    public boolean                sqlEnforceTDCD         = true;
+    public boolean                sqlEnforceTDCU         = true;
+    public boolean                sqlTranslateTTI        = true;
+    public boolean                sqlConcatNulls         = true;
+    public boolean                sqlUniqueNulls         = true;
+    public boolean                sqlNullsFirst          = true;
+    public boolean                sqlConvertTruncate     = true;
+    public int                    sqlAvgScale            = 0;
+    public boolean                sqlDoubleNaN           = true;
+    public boolean                sqlLongvarIsLob        = false;
+    public boolean                sqlSyntaxDb2           = false;
+    public boolean                sqlSyntaxMss           = false;
+    public boolean                sqlSyntaxMys           = false;
+    public boolean                sqlSyntaxOra           = false;
+    public boolean                sqlSyntaxPgs           = false;
+    private boolean               isReferentialIntegrity = true;
     public HsqlDatabaseProperties databaseProperties;
     private final boolean         shutdownOnNoConnection;
     int                           resultMaxMemoryRows;
@@ -103,6 +129,7 @@ public class Database {
     public SessionManager     sessionManager;
     public TransactionManager txManager;
     public int defaultIsolationLevel = SessionInterface.TX_READ_COMMITTED;
+    public boolean            txConflictRollback = true;
 
     // schema objects
     public SchemaManager schemaManager;
@@ -112,7 +139,9 @@ public class Database {
 
     //
     public LobManager lobManager;
-    public Collation  collation;
+
+    //
+    public CheckpointRunner checkpointRunner;
 
     //
     public static final int DATABASE_ONLINE       = 1;
@@ -188,15 +217,15 @@ public class Database {
                 new PersistentStoreCollectionDatabase();
             isReferentialIntegrity = true;
             sessionManager         = new SessionManager(this);
-            collation              = collation.getDefaultInstance();
+            collation              = collation.newDatabaseInstance();
             dbInfo = DatabaseInformation.newDatabaseInformation(this);
             txManager              = new TransactionManager2PL(this);
 
             lobManager.createSchema();
             sessionManager.getSysLobSession().setSchema(
                 SqlInvariants.LOBS_SCHEMA);
-
             schemaManager.setSchemaChangeTimestamp();
+            schemaManager.createSystemTables();
 
             // completed metadata
             logger.openPersistence();
@@ -205,7 +234,6 @@ public class Database {
 
             if (isNew) {
                 String username = urlProperties.getProperty("user", "SA");
-
                 String password = urlProperties.getProperty("password", "");
 
                 userManager.createFirstUser(username, password);
@@ -216,6 +244,8 @@ public class Database {
 
             lobManager.open();
             dbInfo.setWithContent(true);
+
+            checkpointRunner = new CheckpointRunner();
         } catch (Throwable e) {
             logger.closePersistence(Database.CLOSEMODE_IMMEDIATELY);
             logger.releaseLock();
@@ -241,15 +271,20 @@ public class Database {
     void clearStructures() {
 
         if (schemaManager != null) {
-            schemaManager.clearStructures();
+            schemaManager.release();
         }
 
-        granteeManager = null;
-        userManager    = null;
-        nameManager    = null;
-        schemaManager  = null;
-        sessionManager = null;
-        dbInfo         = null;
+        if (checkpointRunner != null) {
+            checkpointRunner.stop();
+        }
+
+        granteeManager   = null;
+        userManager      = null;
+        nameManager      = null;
+        schemaManager    = null;
+        sessionManager   = null;
+        dbInfo           = null;
+        checkpointRunner = null;
     }
 
     /**
@@ -309,7 +344,7 @@ public class Database {
     /**
      *  Returns true if database has been shut down, false otherwise
      */
-    synchronized boolean isShutdown() {
+    boolean isShutdown() {
         return dbState == DATABASE_SHUTDOWN;
     }
 
@@ -331,7 +366,7 @@ public class Database {
 
         User user = userManager.getUser(username, password);
         Session session = sessionManager.newSession(this, user,
-            databaseReadOnly, false, zoneString, timeZoneSeconds);
+            databaseReadOnly, true, zoneString, timeZoneSeconds);
 
         return session;
     }
@@ -411,6 +446,13 @@ public class Database {
         sqlEnforceNames = mode;
     }
 
+    public void setRegularNames(boolean mode) {
+
+        sqlRegularNames = mode;
+
+        nameManager.setSqlRegularNames(mode);
+    }
+
     public void setStrictColumnSize(boolean mode) {
         sqlEnforceSize = mode;
     }
@@ -419,8 +461,68 @@ public class Database {
         sqlEnforceRefs = mode;
     }
 
-    public int getDefaultIsolationLevel() {
-        return defaultIsolationLevel;
+    public void setStrictTypes(boolean mode) {
+        sqlEnforceTypes = mode;
+    }
+
+    public void setStrictTDCD(boolean mode) {
+        sqlEnforceTDCD = mode;
+    }
+
+    public void setStrictTDCU(boolean mode) {
+        sqlEnforceTDCU = mode;
+    }
+
+    public void setTranslateTTI(boolean mode) {
+        sqlTranslateTTI = mode;
+    }
+
+    public void setNullsFirst(boolean mode) {
+        sqlNullsFirst = mode;
+    }
+
+    public void setConcatNulls(boolean mode) {
+        sqlConcatNulls = mode;
+    }
+
+    public void setUniqueNulls(boolean mode) {
+        sqlUniqueNulls = mode;
+    }
+
+    public void setConvertTrunc(boolean mode) {
+        sqlConvertTruncate = mode;
+    }
+
+    public void setDoubleNaN(boolean mode) {
+        sqlDoubleNaN = mode;
+    }
+
+    public void setAvgScale(int scale) {
+        sqlAvgScale = scale;
+    }
+
+    public void setLongVarIsLob(boolean mode) {
+        sqlLongvarIsLob = mode;
+    }
+
+    public void setSyntaxDb2(boolean mode) {
+        sqlSyntaxDb2 = mode;
+    }
+
+    public void setSyntaxMss(boolean mode) {
+        sqlSyntaxMss = mode;
+    }
+
+    public void setSyntaxMys(boolean mode) {
+        sqlSyntaxMys = mode;
+    }
+
+    public void setSyntaxOra(boolean mode) {
+        sqlSyntaxOra = mode;
+    }
+
+    public void setSyntaxPgs(boolean mode) {
+        sqlSyntaxPgs = mode;
     }
 
     /**
@@ -441,11 +543,14 @@ public class Database {
 
     void closeIfLast() {
 
-        if (shutdownOnNoConnection && sessionManager.isEmpty()
-                && dbState == this.DATABASE_ONLINE) {
-            try {
-                close(CLOSEMODE_NORMAL);
-            } catch (HsqlException e) {}
+        if (sessionManager.isEmpty() && dbState == this.DATABASE_ONLINE) {
+            if (shutdownOnNoConnection) {
+                try {
+                    close(CLOSEMODE_NORMAL);
+                } catch (HsqlException e) {}
+            } else {
+                logger.synchLog();
+            }
         }
     }
 
@@ -470,9 +575,16 @@ public class Database {
 
         HsqlException he = null;
 
-        setState(DATABASE_CLOSING);
+        // multiple simultaneous close
+        synchronized (this) {
+            if (getState() != DATABASE_ONLINE) {
+                return;
+            }
+
+            setState(DATABASE_CLOSING);
+        }
+
         sessionManager.closeAllSessions();
-        sessionManager.clearAll();
 
         if (filesReadOnly) {
             closemode = CLOSEMODE_IMMEDIATELY;
@@ -484,6 +596,7 @@ public class Database {
          */
         logger.closePersistence(closemode);
         lobManager.close();
+        sessionManager.close();
 
         try {
             if (closemode == CLOSEMODE_COMPACT) {
@@ -497,10 +610,12 @@ public class Database {
             if (t instanceof HsqlException) {
                 he = (HsqlException) t;
             } else {
-                he = Error.error(ErrorCode.GENERAL_ERROR, t.toString());
+                he = Error.error(ErrorCode.GENERAL_ERROR, t);
             }
         }
 
+        lobManager = null;
+
         logger.releaseLock();
         setState(DATABASE_SHUTDOWN);
         clearStructures();
@@ -510,16 +625,19 @@ public class Database {
         // calls
         DatabaseManager.removeDatabase(this);
 
+        // todo - when hsqldb.sql. logging is supported, add another call
+        FrameworkLogger.clearLoggers("hsqldb.db." + getUniqueName());
+
         if (he != null) {
             throw he;
         }
     }
 
-    private synchronized void setState(int state) {
+    private void setState(int state) {
         dbState = state;
     }
 
-    synchronized int getState() {
+    int getState() {
         return dbState;
     }
 
@@ -549,18 +667,36 @@ public class Database {
     public String[] getSettingsSQL() {
 
         HsqlArrayList list = new HsqlArrayList();
+        StringBuffer  sb   = new StringBuffer();
 
         if (!getCatalogName().name.equals(
                 HsqlNameManager.DEFAULT_CATALOG_NAME)) {
             String name = getCatalogName().statementName;
 
-            list.add("ALTER CATALOG PUBLIC RENAME TO " + name);
+            sb.append("ALTER CATALOG PUBLIC RENAME TO ").append(name);
+            list.add(sb.toString());
+            sb.setLength(0);
         }
 
-        if (collation.collator != null) {
-            String name = collation.getName().statementName;
+        if (!collation.isDefaultCollation()) {
+            list.add(collation.getDatabaseCollationSQL());
+        }
 
-            list.add("SET DATABASE COLLATION " + name);
+        HashMappedList lobTables =
+            schemaManager.getTables(SqlInvariants.LOBS_SCHEMA);
+
+        for (int i = 0; i < lobTables.size(); i++) {
+            Table table = (Table) lobTables.get(i);
+
+            if (table.isCached()) {
+                sb.append(Tokens.T_SET).append(' ').append(Tokens.T_TABLE);
+                sb.append(' ');
+                sb.append(table.getName().getSchemaQualifiedStatementName());
+                sb.append(' ').append(Tokens.T_TYPE).append(' ');
+                sb.append(Tokens.T_CACHED);
+                list.add(sb.toString());
+                sb.setLength(0);
+            }
         }
 
         String[] array = new String[list.size()];
@@ -575,10 +711,10 @@ public class Database {
      */
     public Result getScript(boolean indexRoots) {
 
-        Result r = Result.newSingleColumnResult("COMMAND", Type.SQL_VARCHAR);
+        Result r = Result.newSingleColumnResult("COMMAND");
 
         // properties
-        String[] list = logger.getPropertiesSQL();
+        String[] list = logger.getPropertiesSQL(indexRoots);
 
         addRows(r, list);
 
@@ -600,16 +736,6 @@ public class Database {
 
         addRows(r, list);
 
-        // user session start schema names
-        list = getUserManager().getInitialSchemaSQL();
-
-        addRows(r, list);
-
-        // grantee rights
-        list = getGranteeManager().getRightstSQL();
-
-        addRows(r, list);
-
         // index roots
         if (indexRoots) {
             list = schemaManager.getIndexRootsSQL();
@@ -617,11 +743,26 @@ public class Database {
             addRows(r, list);
         }
 
-        // text headers
+        // text headers - readonly - clustered
         list = schemaManager.getTablePropsSQL(!indexRoots);
 
         addRows(r, list);
 
+        // password complexity
+        list = getUserManager().getAuthenticationSQL();
+
+        addRows(r, list);
+
+        // user session start schema names
+        list = getUserManager().getInitialSchemaSQL();
+
+        addRows(r, list);
+
+        // grantee rights
+        list = getGranteeManager().getRightstSQL();
+
+        addRows(r, list);
+
         return r;
     }
 
@@ -651,4 +792,55 @@ public class Database {
     public HsqlProperties getURLProperties() {
         return urlProperties;
     }
+
+    class CheckpointRunner implements Runnable {
+
+        private volatile boolean waiting;
+        private Object           timerTask;
+
+        public void run() {
+
+            try {
+                Session sysSession = sessionManager.newSysSession();
+                Statement checkpoint =
+                    ParserCommand.getAutoCheckpointStatement(Database.this);
+
+                sysSession.executeCompiledStatement(
+                    checkpoint, ValuePool.emptyObjectArray);
+                sysSession.commit(false);
+                sysSession.close();
+
+                waiting = false;
+            } catch (Exception e) {
+
+                // ignore exceptions
+                // may be InterruptedException or IOException
+            }
+        }
+
+        public void start() {
+
+            if (!logger.isLogged()) {
+                return;
+            }
+
+            synchronized (this) {
+                if (waiting) {
+                    return;
+                }
+
+                waiting = true;
+            }
+
+            timerTask = DatabaseManager.getTimer().scheduleAfter(0, this);
+        }
+
+        public void stop() {
+
+            HsqlTimer.cancel(timerTask);
+
+            timerTask = null;
+            waiting   = false;
+        }
+    }
 }
