@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,23 +35,56 @@ import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.HsqlNameManager.SimpleName;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.index.Index;
 import org.hsqldb.lib.ArrayListIdentity;
+import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HsqlList;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.lib.Set;
+import org.hsqldb.navigator.RangeIterator;
+import org.hsqldb.persist.PersistentStore;
+import org.hsqldb.store.ValuePool;
 import org.hsqldb.types.Type;
 
 /**
  * Implementation of column, variable, parameter, etc. access operations.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 public class ExpressionColumn extends Expression {
 
-    public final static ExpressionColumn[] emptyArray =
+    public static final ExpressionColumn[] emptyArray =
         new ExpressionColumn[]{};
+    static final SimpleName rownumName =
+        HsqlNameManager.getSimpleName("ROWNUM", false);
+
+    //
+    public final static HashMappedList diagnosticsList = new HashMappedList();
+    final static String[] diagnosticsVariableTokens    = new String[] {
+        Tokens.T_NUMBER, Tokens.T_MORE, Tokens.T_ROW_COUNT
+    };
+    public final static int            idx_number      = 0;
+    public final static int            idx_more        = 1;
+    public final static int            idx_row_count   = 2;
+
+    static {
+        for (int i = 0; i < diagnosticsVariableTokens.length; i++) {
+            HsqlName name = HsqlNameManager.newSystemObjectName(
+                diagnosticsVariableTokens[i], SchemaObject.VARIABLE);
+            Type type = Type.SQL_INTEGER;
+
+            if (diagnosticsVariableTokens[i] == Tokens.T_MORE) {
+                type = Type.SQL_CHAR;
+            }
+
+            ColumnSchema col = new ColumnSchema(name, type, false, false,
+                                                null);
+
+            diagnosticsList.add(diagnosticsVariableTokens[i], col);
+        }
+    }
 
     //
     ColumnSchema  column;
@@ -68,20 +101,17 @@ public class ExpressionColumn extends Expression {
     boolean isParam;
 
     //
-    boolean strictReference;
 
     /**
-     * Creates a OpCodes.COLUMN expression
+     * Creates a OpTypes.COLUMN expression
      */
-    ExpressionColumn(String schema, String table, String column,
-                     boolean strictReference) {
+    ExpressionColumn(String schema, String table, String column) {
 
         super(OpTypes.COLUMN);
 
-        this.schema          = schema;
-        this.tableName       = table;
-        this.columnName      = column;
-        this.strictReference = strictReference;
+        this.schema     = schema;
+        this.tableName  = table;
+        this.columnName = column;
     }
 
     ExpressionColumn(ColumnSchema column) {
@@ -103,7 +133,7 @@ public class ExpressionColumn extends Expression {
     }
 
     /**
-     * Creates a temporary OpCodes.COLUMN expression
+     * Creates a temporary OpTypes.COLUMN expression
      */
     ExpressionColumn(Expression e, int colIndex, int rangePosition) {
 
@@ -125,9 +155,24 @@ public class ExpressionColumn extends Expression {
 
         if (type == OpTypes.DYNAMIC_PARAM) {
             isParam = true;
+        } else if (type == OpTypes.ROWNUM) {
+            columnName = rownumName.name;
+            dataType   = Type.SQL_INTEGER;
         }
     }
 
+    /**
+     * For diagnostics vars
+     */
+    ExpressionColumn(int type, int columnIndex) {
+
+        super(type);
+
+        this.column      = (ColumnSchema) diagnosticsList.get(columnIndex);
+        this.columnIndex = columnIndex;
+        this.dataType    = column.dataType;
+    }
+
     ExpressionColumn(Expression[] nodes, String name) {
 
         super(OpTypes.COALESCE);
@@ -148,11 +193,11 @@ public class ExpressionColumn extends Expression {
     }
 
     /**
-     * Creates a OpCodes.SEQUENCE expression
+     * Creates a OpTypes.SEQUENCE expression
      */
-    ExpressionColumn(NumberSequence sequence) {
+    ExpressionColumn(NumberSequence sequence, int opType) {
 
-        super(OpTypes.SEQUENCE);
+        super(opType);
 
         this.sequence = sequence;
         dataType      = sequence.getDataType();
@@ -163,8 +208,8 @@ public class ExpressionColumn extends Expression {
         columnIndex   = i;
         column        = range.getColumn(i);
         dataType      = column.getDataType();
-        columnName    = range.getColumnAlias(i);
-        tableName     = range.getTableAlias();
+        columnName    = range.getColumnAlias(i).name;
+        tableName     = range.getTableAlias().name;
         rangeVariable = range;
 
         rangeVariable.addColumn(columnIndex);
@@ -172,18 +217,32 @@ public class ExpressionColumn extends Expression {
 
     void setAttributesAsColumn(RangeVariable range, int i) {
 
-        if (range.variables != null) {
-            columnIndex   = i;
-            column        = range.getColumn(i);
-            dataType      = column.getDataType();
-            rangeVariable = range;
-        } else {
-            columnIndex   = i;
-            column        = range.getColumn(i);
-            dataType      = column.getDataType();
-            rangeVariable = range;
+        columnIndex   = i;
+        column        = range.getColumn(i);
+        dataType      = column.getDataType();
+        rangeVariable = range;
 
-            rangeVariable.addColumn(columnIndex);
+        rangeVariable.addColumn(columnIndex);
+    }
+
+    public byte getNullability() {
+
+        switch (opType) {
+
+            case OpTypes.COLUMN :
+                if (nullability == SchemaObject.Nullability.NULLABLE_UNKNOWN) {
+                    return column.getNullability();
+                }
+
+                return nullability;
+
+            case OpTypes.SEQUENCE :
+            case OpTypes.COALESCE :
+            case OpTypes.ROWNUM :
+                return SchemaObject.Nullability.NO_NULLS;
+
+            default :
+                return SchemaObject.Nullability.NULLABLE_UNKNOWN;
         }
     }
 
@@ -200,12 +259,18 @@ public class ExpressionColumn extends Expression {
             return alias;
         }
 
+        if (rangeVariable != null && rangeVariable.hasColumnAlias()) {
+            return rangeVariable.getColumnAlias(columnIndex);
+        }
+
         if (column != null) {
             return column.getName();
         }
 
         if (opType == OpTypes.COALESCE) {
             return nodes[LEFT].getSimpleName();
+        } else if (opType == OpTypes.ROWNUM) {
+            return rownumName;
         }
 
         return null;
@@ -217,31 +282,17 @@ public class ExpressionColumn extends Expression {
             return alias.name;
         }
 
-        if (opType == OpTypes.COLUMN) {
-            return columnName;
-        }
+        switch (opType) {
 
-        if (opType == OpTypes.COALESCE) {
-            return columnName;
+            case OpTypes.COLUMN :
+            case OpTypes.COALESCE :
+            case OpTypes.ROWNUM :
+                return columnName;
         }
 
         return "";
     }
 
-    public String getBaseColumnName() {
-
-        if (opType == OpTypes.COLUMN && rangeVariable != null) {
-            return rangeVariable.getTable().getColumn(
-                columnIndex).getName().name;
-        }
-
-        return null;
-    }
-
-    public HsqlName getBaseColumnHsqlName() {
-        return column.getName();
-    }
-
     void collectObjectNames(Set set) {
 
         switch (opType) {
@@ -277,14 +328,20 @@ public class ExpressionColumn extends Expression {
 
     String getColumnName() {
 
-        if (opType == OpTypes.COLUMN && column != null) {
-            return column.getName().name;
+        switch (opType) {
+
+            case OpTypes.COLUMN :
+            case OpTypes.PARAMETER :
+            case OpTypes.VARIABLE :
+                if (column != null) {
+                    return column.getName().name;
+                }
         }
 
         return getAlias();
     }
 
-    ColumnSchema getColumn() {
+    public ColumnSchema getColumn() {
         return column;
     }
 
@@ -296,8 +353,9 @@ public class ExpressionColumn extends Expression {
         return rangeVariable;
     }
 
-    public HsqlList resolveColumnReferences(RangeVariable[] rangeVarArray,
-            int rangeCount, HsqlList unresolvedSet, boolean acceptsSequences) {
+    public HsqlList resolveColumnReferences(Session session,
+            RangeGroup rangeGroup, int rangeCount, RangeGroup[] rangeGroups,
+            HsqlList unresolvedSet, boolean acceptsSequences) {
 
         switch (opType) {
 
@@ -307,16 +365,18 @@ public class ExpressionColumn extends Expression {
                 }
                 break;
 
+            case OpTypes.ROWNUM :
             case OpTypes.MULTICOLUMN :
             case OpTypes.DYNAMIC_PARAM :
             case OpTypes.ASTERISK :
             case OpTypes.SIMPLE_COLUMN :
             case OpTypes.COALESCE :
+            case OpTypes.DIAGNOSTICS_VARIABLE :
                 break;
 
+            case OpTypes.COLUMN :
             case OpTypes.PARAMETER :
-            case OpTypes.VARIABLE :
-            case OpTypes.COLUMN : {
+            case OpTypes.VARIABLE : {
                 boolean resolved       = false;
                 boolean tableQualified = tableName != null;
 
@@ -324,6 +384,8 @@ public class ExpressionColumn extends Expression {
                     return unresolvedSet;
                 }
 
+                RangeVariable[] rangeVarArray = rangeGroup.getRangeVariables();
+
                 for (int i = 0; i < rangeCount; i++) {
                     RangeVariable rangeVar = rangeVarArray[i];
 
@@ -332,8 +394,8 @@ public class ExpressionColumn extends Expression {
                     }
 
                     if (resolved) {
-                        if (resolvesDuplicateColumnReference(rangeVar)) {
-                            if (strictReference) {
+                        if (session.database.sqlEnforceRefs) {
+                            if (resolvesDuplicateColumnReference(rangeVar)) {
                                 String message = getColumnName();
 
                                 if (alias != null) {
@@ -350,7 +412,7 @@ public class ExpressionColumn extends Expression {
                             }
                         }
                     } else {
-                        if (resolveColumnReference(rangeVar)) {
+                        if (resolveColumnReference(rangeVar, false)) {
                             if (tableQualified) {
                                 return unresolvedSet;
                             }
@@ -366,6 +428,71 @@ public class ExpressionColumn extends Expression {
                     return unresolvedSet;
                 }
 
+                if (session.database.sqlSyntaxOra) {
+                    if (acceptsSequences && tableName != null) {
+                        if (Tokens.T_CURRVAL.equals(columnName)) {
+                            NumberSequence seq =
+                                session.database.schemaManager.getSequence(
+                                    tableName, session.getSchemaName(schema),
+                                    false);
+
+                            if (seq != null) {
+                                opType     = OpTypes.SEQUENCE_CURRENT;
+                                dataType   = seq.getDataType();
+                                sequence   = seq;
+                                schema     = null;
+                                tableName  = null;
+                                columnName = null;
+                                resolved   = true;
+                            }
+                        } else if (Tokens.T_NEXTVAL.equals(columnName)) {
+                            NumberSequence seq =
+                                session.database.schemaManager.getSequence(
+                                    tableName, session.getSchemaName(schema),
+                                    false);
+
+                            if (seq != null) {
+                                opType     = OpTypes.SEQUENCE;
+                                dataType   = seq.getDataType();
+                                sequence   = seq;
+                                schema     = null;
+                                tableName  = null;
+                                columnName = null;
+                                resolved   = true;
+                            }
+                        }
+                    }
+                }
+
+                if (resolved) {
+                    return unresolvedSet;
+                }
+
+                for (int idx = rangeGroups.length - 1; idx >= 0; idx--) {
+                    rangeVarArray = rangeGroups[idx].getRangeVariables();
+
+                    for (int i = 0; i < rangeVarArray.length; i++) {
+                        RangeVariable rangeVar = rangeVarArray[i];
+
+                        if (rangeVar == null) {
+                            continue;
+                        }
+
+                        if (resolveColumnReference(rangeVar, true)) {
+                            if (opType == OpTypes.COLUMN) {
+                                rangeGroup.setCorrelated();
+
+                                for (int idxx = rangeGroups.length - 1;
+                                        idxx > idx; idxx--) {
+                                    rangeGroups[idxx].setCorrelated();
+                                }
+                            }
+
+                            return unresolvedSet;
+                        }
+                    }
+                }
+
                 if (unresolvedSet == null) {
                     unresolvedSet = new ArrayListIdentity();
                 }
@@ -377,23 +504,30 @@ public class ExpressionColumn extends Expression {
         return unresolvedSet;
     }
 
-    public boolean resolveColumnReference(RangeVariable rangeVar) {
+    private boolean resolveColumnReference(RangeVariable rangeVar,
+                                           boolean outer) {
 
-        if (tableName == null) {
-            Expression e = rangeVar.getColumnExpression(columnName);
+        Expression e = rangeVar.getColumnExpression(columnName);
 
-            if (e != null) {
-                opType   = e.opType;
-                nodes    = e.nodes;
-                dataType = e.dataType;
+        if (e != null) {
+            opType   = e.opType;
+            nodes    = e.nodes;
+            dataType = e.dataType;
 
-                return true;
-            }
+            return true;
+        }
 
-            if (rangeVar.variables != null) {
-                int colIndex = rangeVar.findColumn(columnName);
+        int colIndex = rangeVar.findColumn(schema, tableName, columnName);
 
-                if (colIndex == -1) {
+        if (colIndex == -1) {
+            return false;
+        }
+
+        switch (rangeVar.rangeType) {
+
+            case RangeVariable.PARAMETER_RANGE :
+            case RangeVariable.VARIALBE_RANGE : {
+                if (tableName != null) {
                     return false;
                 }
 
@@ -403,29 +537,35 @@ public class ExpressionColumn extends Expression {
                         == SchemaObject.ParameterModes.PARAM_OUT) {
                     return false;
                 } else {
-                    opType = rangeVar.isVariable ? OpTypes.VARIABLE
-                                                 : OpTypes.PARAMETER;
-
-                    setAttributesAsColumn(rangeVar, colIndex);
-
-                    return true;
+                    opType = rangeVar.rangeType
+                             == RangeVariable.VARIALBE_RANGE ? OpTypes.VARIABLE
+                                                             : OpTypes
+                                                             .PARAMETER;
                 }
+
+                break;
+            }
+            case RangeVariable.TRANSITION_RANGE : {
+                if (tableName == null) {
+                    return false;
+                }
+
+                if (schema != null) {
+                    return false;
+                }
+
+                opType = OpTypes.TRANSITION_VARIABLE;
+
+                break;
+            }
+            default : {
+                break;
             }
         }
 
-        if (!rangeVar.resolvesTableName(this)) {
-            return false;
-        }
+        setAttributesAsColumn(rangeVar, colIndex);
 
-        int colIndex = rangeVar.findColumn(columnName);
-
-        if (colIndex != -1) {
-            setAttributesAsColumn(rangeVar, colIndex);
-
-            return true;
-        }
-
-        return false;
+        return true;
     }
 
     boolean resolvesDuplicateColumnReference(RangeVariable rangeVar) {
@@ -437,34 +577,21 @@ public class ExpressionColumn extends Expression {
                 return false;
             }
 
-            if (rangeVar.variables != null) {
-                int colIndex = rangeVar.findColumn(columnName);
+            switch (rangeVar.rangeType) {
 
-                if (colIndex == -1) {
+                case RangeVariable.PARAMETER_RANGE :
+                case RangeVariable.VARIALBE_RANGE :
+                case RangeVariable.TRANSITION_RANGE :
                     return false;
-                }
 
-                ColumnSchema column = rangeVar.getColumn(colIndex);
+                default :
+                    int colIndex = rangeVar.findColumn(schema, tableName,
+                                                       columnName);
 
-                if (column.getParameterMode()
-                        == SchemaObject.ParameterModes.PARAM_OUT) {
-                    return false;
-                } else {
-                    return true;
-                }
+                    return colIndex != -1;
             }
         }
 
-        if (!rangeVar.resolvesTableName(this)) {
-            return false;
-        }
-
-        int colIndex = rangeVar.findColumn(columnName);
-
-        if (colIndex != -1) {
-            return true;
-        }
-
         return false;
     }
 
@@ -499,18 +626,25 @@ public class ExpressionColumn extends Expression {
             case OpTypes.DEFAULT :
                 return null;
 
+            case OpTypes.DIAGNOSTICS_VARIABLE : {
+                return getDiagnosticsVariable(session);
+            }
             case OpTypes.VARIABLE : {
                 return session.sessionContext.routineVariables[columnIndex];
             }
             case OpTypes.PARAMETER : {
                 return session.sessionContext.routineArguments[columnIndex];
             }
+            case OpTypes.TRANSITION_VARIABLE : {
+                return session.sessionContext
+                    .triggerArguments[rangeVariable.rangePosition][columnIndex];
+            }
             case OpTypes.COLUMN : {
-                Object[] data =
-                    (Object[]) session.sessionContext
-                        .rangeIterators[rangeVariable.rangePosition]
-                        .getCurrent();
-                Object value = data[columnIndex];
+                RangeIterator[] iterators =
+                    session.sessionContext.rangeIterators;
+                Object value =
+                    iterators[rangeVariable.rangePosition].getCurrent(
+                        columnIndex);
 
                 if (dataType != column.dataType) {
                     value = dataType.convertToType(session, value,
@@ -520,11 +654,11 @@ public class ExpressionColumn extends Expression {
                 return value;
             }
             case OpTypes.SIMPLE_COLUMN : {
-                Object[] data =
-                    (Object[]) session.sessionContext
-                        .rangeIterators[rangePosition].getCurrent();
+                Object value =
+                    session.sessionContext.rangeIterators[rangePosition]
+                        .getCurrent(columnIndex);
 
-                return data[columnIndex];
+                return value;
             }
             case OpTypes.COALESCE : {
                 Object value = null;
@@ -545,6 +679,12 @@ public class ExpressionColumn extends Expression {
             case OpTypes.SEQUENCE : {
                 return session.sessionData.getSequenceValue(sequence);
             }
+            case OpTypes.SEQUENCE_CURRENT : {
+                return session.sessionData.getSequenceCurrent(sequence);
+            }
+            case OpTypes.ROWNUM : {
+                return ValuePool.getInt(session.sessionContext.rownum);
+            }
             case OpTypes.ASTERISK :
             case OpTypes.MULTICOLUMN :
             default :
@@ -553,6 +693,10 @@ public class ExpressionColumn extends Expression {
         }
     }
 
+    private Object getDiagnosticsVariable(Session session) {
+        return session.sessionContext.diagnosticsVariables[columnIndex];
+    }
+
     public String getSQL() {
 
         switch (opType) {
@@ -569,14 +713,32 @@ public class ExpressionColumn extends Expression {
             case OpTypes.COALESCE :
                 return alias.getStatementName();
 
+            case OpTypes.DIAGNOSTICS_VARIABLE :
             case OpTypes.VARIABLE :
             case OpTypes.PARAMETER :
+                return column.getName().statementName;
+
+            case OpTypes.ROWNUM : {
+                StringBuffer sb = new StringBuffer(Tokens.T_ROWNUM);
+
+                sb.append('(').append(')');
+            }
             case OpTypes.COLUMN : {
                 if (column == null) {
                     if (alias != null) {
                         return alias.getStatementName();
                     } else {
-                        return columnName;
+                        if (tableName == null) {
+                            return columnName;
+                        }
+
+                        StringBuffer sb = new StringBuffer();
+
+                        sb.append(tableName);
+                        sb.append('.');
+                        sb.append(columnName);
+
+                        return sb.toString();
                     }
                 }
 
@@ -623,8 +785,6 @@ public class ExpressionColumn extends Expression {
 
         StringBuffer sb = new StringBuffer(64);
 
-        sb.append('\n');
-
         for (int i = 0; i < blanks; i++) {
             sb.append(' ');
         }
@@ -682,6 +842,8 @@ public class ExpressionColumn extends Expression {
             // shouldn't get here
         }
 
+        sb.append('\n');
+
         return sb.toString();
     }
 
@@ -724,9 +886,20 @@ public class ExpressionColumn extends Expression {
                     sb.append(c.tableName + '.');
                 }
 
-                throw Error.error(ErrorCode.X_42501,
-                                  sb.toString() + c.getColumnName());
+                sb.append(c.getColumnName());
+
+                throw Error.error(ErrorCode.X_42501, sb.toString());
             } else {
+                OrderedHashSet newSet = new OrderedHashSet();
+
+                e.collectAllExpressions(newSet,
+                                        Expression.columnExpressionSet,
+                                        Expression.emptyExpressionSet);
+
+                // throw with column name
+                checkColumnsResolved(newSet);
+
+                // throw anyway if not found
                 throw Error.error(ErrorCode.X_42501);
             }
         }
@@ -757,23 +930,30 @@ public class ExpressionColumn extends Expression {
     /**
      * collects all range variables in expression tree
      */
-    void collectRangeVariables(RangeVariable[] rangeVariables, Set set) {
+    OrderedHashSet collectRangeVariables(RangeVariable[] rangeVariables,
+                                         OrderedHashSet set) {
 
         for (int i = 0; i < nodes.length; i++) {
             if (nodes[i] != null) {
-                nodes[i].collectRangeVariables(rangeVariables, set);
+                set = nodes[i].collectRangeVariables(rangeVariables, set);
             }
         }
 
         if (rangeVariable != null) {
             for (int i = 0; i < rangeVariables.length; i++) {
                 if (rangeVariables[i] == rangeVariable) {
+                    if (set == null) {
+                        set = new OrderedHashSet();
+                    }
+
                     set.add(rangeVariable);
 
                     break;
                 }
             }
         }
+
+        return set;
     }
 
     Expression replaceAliasInOrderBy(Expression[] columns, int length) {
@@ -840,19 +1020,6 @@ public class ExpressionColumn extends Expression {
         return this;
     }
 
-    int findMatchingRangeVariableIndex(RangeVariable[] rangeVarArray) {
-
-        for (int i = 0; i < rangeVarArray.length; i++) {
-            RangeVariable rangeVar = rangeVarArray[i];
-
-            if (rangeVar.resolvesTableName(this)) {
-                return i;
-            }
-        }
-
-        return -1;
-    }
-
     /**
      * return true if given RangeVariable is used in expression tree
      */
@@ -886,20 +1053,23 @@ public class ExpressionColumn extends Expression {
             return false;
         }
 
-        if (opType != ((Expression) other).opType) {
+        if (opType != other.opType) {
             return false;
         }
 
         switch (opType) {
 
             case OpTypes.SIMPLE_COLUMN :
-                return this.columnIndex == ((Expression) other).columnIndex;
+                return this.columnIndex == other.columnIndex;
 
             case OpTypes.COALESCE :
-                return nodes == ((Expression) other).nodes;
+                return nodes == other.nodes;
 
+            case OpTypes.VARIABLE :
+            case OpTypes.PARAMETER :
             case OpTypes.COLUMN :
-                return column == ((Expression) other).getColumn();
+                return column == other.getColumn()
+                       && rangeVariable == other.getRangeVariable();
 
             default :
                 return false;
@@ -943,4 +1113,69 @@ public class ExpressionColumn extends Expression {
     boolean isDynamicParam() {
         return isParam;
     }
+
+    RangeVariable[] getJoinRangeVariables(RangeVariable[] ranges) {
+
+        if (opType == OpTypes.COLUMN) {
+            return new RangeVariable[]{ rangeVariable };
+        }
+
+        return RangeVariable.emptyArray;
+    }
+
+    /**
+     * For normal tables only. We don't want to create an index on
+     * each column that is checked.
+     */
+    double costFactor(Session session, RangeVariable range, int operation) {
+
+        if (range.rangeTable instanceof TableDerived) {
+            return 1024;
+        }
+
+        PersistentStore store = range.rangeTable.getRowStore(session);
+        int indexType = range.rangeTable.indexTypeForColumn(session,
+            columnIndex);
+        double factor;
+
+        switch (indexType) {
+
+            case Index.INDEX_UNIQUE :
+                if (operation == OpTypes.EQUAL) {
+                    factor = 1;
+                } else {
+                    factor = store.elementCount() / 2;
+                }
+                break;
+
+            case Index.INDEX_NON_UNIQUE :
+                if (operation == OpTypes.EQUAL) {
+                    factor = store.elementCount() / 8;
+
+                    if (factor > 1024) {
+                        factor = 1024;
+                    }
+                } else {
+                    factor = store.elementCount() / 2;
+                }
+                break;
+
+            case Index.INDEX_NONE :
+            default :
+                factor = store.elementCount();
+                break;
+        }
+
+        return factor < Index.minimumSelectivity ? Index.minimumSelectivity
+                                                 : factor;
+    }
+
+    public Expression duplicate() {
+
+        if (opType == OpTypes.PARAMETER) {
+            return this;
+        }
+
+        return super.duplicate();
+    }
 }
