@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,8 +33,12 @@ package org.hsqldb;
 
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.lib.ArrayUtil;
 import org.hsqldb.lib.HsqlList;
 import org.hsqldb.store.ValuePool;
+import org.hsqldb.types.BinaryData;
+import org.hsqldb.types.BinaryType;
+import org.hsqldb.types.CharacterType;
 import org.hsqldb.types.DateTimeType;
 import org.hsqldb.types.IntervalType;
 import org.hsqldb.types.Type;
@@ -43,9 +47,9 @@ import org.hsqldb.types.Types;
 /**
  * Implementation of CAST, CASE, LIMIT and ZONE operations.
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.0.0
+ * @version 2.0.1
  * @since 1.9.0
  */
 public class ExpressionOp extends Expression {
@@ -68,12 +72,18 @@ public class ExpressionOp extends Expression {
 
         switch (opType) {
 
+            case OpTypes.LIKE_ARG :
             case OpTypes.ALTERNATIVE :
             case OpTypes.CASEWHEN :
             case OpTypes.LIMIT :
             case OpTypes.ZONE_MODIFIER :
                 return;
 
+            case OpTypes.PREFIX :
+                dataType = left.dataType;
+
+                return;
+
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500, "ExpressionOp");
         }
@@ -177,6 +187,9 @@ public class ExpressionOp extends Expression {
 
                 return dataType.convertToSQLString(valueData);
 
+            case OpTypes.LIKE_ARG :
+                sb.append(' ').append(Tokens.T_LIKE).append(' ');
+                sb.append(left).append(' ').append(right).append(' ');
             case OpTypes.CAST :
                 sb.append(' ').append(Tokens.T_CAST).append('(');
                 sb.append(left).append(' ').append(Tokens.T_AS).append(' ');
@@ -220,6 +233,8 @@ public class ExpressionOp extends Expression {
                     break;
                 }
 
+                sb.append(Tokens.T_TIME).append(' ').append(Tokens.T_ZONE);
+                sb.append(' ');
                 sb.append(right);
                 break;
 
@@ -243,28 +258,36 @@ public class ExpressionOp extends Expression {
         switch (opType) {
 
             case OpTypes.VALUE :
-                sb.append("VALUE = ").append(valueData);
+                sb.append("VALUE = ").append(
+                    dataType.convertToSQLString(valueData));
                 sb.append(", TYPE = ").append(dataType.getNameString());
 
                 return sb.toString();
 
-            case OpTypes.TABLE :
-                sb.append("VALUELIST ");
+            case OpTypes.LIKE_ARG :
+                sb.append(Tokens.T_LIKE).append(' ').append("ARG ");
+                sb.append(dataType.getTypeDefinition());
+                sb.append(' ');
+                break;
+
+            case OpTypes.VALUELIST :
+                sb.append(Tokens.T_VALUE).append(' ').append("LIST ");
 
                 for (int i = 0; i < nodes.length; i++) {
                     sb.append(nodes[i].describe(session, blanks + 1));
                     sb.append(' ');
                 }
-                break;
+
+                return sb.toString();
 
             case OpTypes.CAST :
-                sb.append("CAST ");
+                sb.append(Tokens.T_CAST).append(' ');
                 sb.append(dataType.getTypeDefinition());
                 sb.append(' ');
                 break;
 
             case OpTypes.CASEWHEN :
-                sb.append("CASEWHEN ");
+                sb.append(Tokens.T_CASEWHEN).append(' ');
                 break;
         }
 
@@ -283,8 +306,9 @@ public class ExpressionOp extends Expression {
         return sb.toString();
     }
 
-    public HsqlList resolveColumnReferences(RangeVariable[] rangeVarArray,
-            int rangeCount, HsqlList unresolvedSet, boolean acceptsSequences) {
+    public HsqlList resolveColumnReferences(Session session,
+            RangeGroup rangeGroup, int rangeCount,
+            RangeGroup[] rangeGroups, HsqlList unresolvedSet, boolean acceptsSequences) {
 
         if (opType == OpTypes.VALUE) {
             return unresolvedSet;
@@ -302,8 +326,8 @@ public class ExpressionOp extends Expression {
                 continue;
             }
 
-            unresolvedSet = nodes[i].resolveColumnReferences(rangeVarArray,
-                    rangeCount, unresolvedSet, acceptsSequences);
+            unresolvedSet = nodes[i].resolveColumnReferences(session,
+                    rangeGroup, rangeCount, rangeGroups, unresolvedSet, acceptsSequences);
         }
 
         return unresolvedSet;
@@ -311,63 +335,66 @@ public class ExpressionOp extends Expression {
 
     public void resolveTypes(Session session, Expression parent) {
 
+        switch (opType) {
+
+            case OpTypes.CASEWHEN :
+                break;
+
+            default :
+                for (int i = 0; i < nodes.length; i++) {
+                    if (nodes[i] != null) {
+                        nodes[i].resolveTypes(session, this);
+                    }
+                }
+        }
+
         switch (opType) {
 
             case OpTypes.VALUE :
                 break;
 
-            case OpTypes.CAST : {
-                nodes[LEFT].resolveTypes(session, this);
-
-                Type type = nodes[LEFT].dataType;
-
-                if (type != null && !dataType.canConvertFrom(type)) {
-                    throw Error.error(ErrorCode.X_42561);
-                }
-
-                if (nodes[LEFT].opType == OpTypes.VALUE) {
-                    Expression replacement = nodes[LEFT];
+            case OpTypes.LIKE_ARG : {
+                dataType = nodes[LEFT].dataType;
 
+                if (nodes[LEFT].opType == OpTypes.VALUE
+                        && (nodes[RIGHT] == null
+                            || nodes[RIGHT].opType == OpTypes.VALUE)) {
                     setAsConstantValue(session);
 
-                    replacement.dataType  = dataType;
-                    replacement.valueData = valueData;
-
-                    if (parent != null) {
-                        parent.replaceNode(this, replacement);
-                    }
-
-                    break;
-                }
-
-                if (nodes[LEFT].opType == OpTypes.DYNAMIC_PARAM) {
-                    nodes[LEFT].dataType = dataType;
-
-                    if (parent != null && !parent.isSelfAggregate()) {
-//                        parent.replaceNode(this, nodes[LEFT]);
-                    }
-
-                    break;
-                }
-
-                if (dataType.equals(nodes[LEFT].dataType)) {
-
-                    // issues with aggregates
                     break;
                 }
 
                 break;
             }
-            case OpTypes.ZONE_MODIFIER :
-                nodes[LEFT].resolveTypes(session, this);
+            case OpTypes.CAST : {
+                Expression node     = nodes[LEFT];
+                Type       nodeType = node.dataType;
 
+                if (nodeType != null && !dataType.canConvertFrom(nodeType)) {
+                    throw Error.error(ErrorCode.X_42561);
+                }
+
+                if (node.opType == OpTypes.VALUE) {
+                    setAsConstantValue(session);
+
+                    node.dataType  = dataType;
+                    node.valueData = valueData;
+
+                    if (parent != null) {
+                        parent.replaceNode(this, node);
+                    }
+                } else if (nodes[LEFT].opType == OpTypes.DYNAMIC_PARAM) {
+                    node.dataType = dataType;
+                }
+
+                break;
+            }
+            case OpTypes.ZONE_MODIFIER :
                 if (nodes[LEFT].dataType == null) {
                     throw Error.error(ErrorCode.X_42567);
                 }
 
                 if (nodes[RIGHT] != null) {
-                    nodes[RIGHT].resolveTypes(session, this);
-
                     if (nodes[RIGHT].dataType == null) {
                         nodes[RIGHT].dataType =
                             Type.SQL_INTERVAL_HOUR_TO_MINUTE;
@@ -375,7 +402,16 @@ public class ExpressionOp extends Expression {
 
                     if (nodes[RIGHT].dataType.typeCode
                             != Types.SQL_INTERVAL_HOUR_TO_MINUTE) {
-                        throw Error.error(ErrorCode.X_42563);
+                        if (nodes[RIGHT].opType == OpTypes.VALUE) {
+                            nodes[RIGHT].valueData =
+                                Type.SQL_INTERVAL_HOUR_TO_MINUTE.castToType(
+                                    session, nodes[RIGHT].valueData,
+                                    nodes[RIGHT].dataType);
+                            nodes[RIGHT].dataType =
+                                Type.SQL_INTERVAL_HOUR_TO_MINUTE;
+                        } else {
+                            throw Error.error(ErrorCode.X_42563);
+                        }
                     }
                 }
 
@@ -417,19 +453,32 @@ public class ExpressionOp extends Expression {
                 resolveTypesForCaseWhen(session);
                 break;
 
-            case OpTypes.LIMIT :
-                for (int i = 0; i < nodes.length; i++) {
-                    if (nodes[i] != null) {
-                        nodes[i].resolveTypes(session, this);
+            case OpTypes.ALTERNATIVE :
+                break;
 
-                        if (nodes[i].dataType == null) {
-                            nodes[i].dataType = Type.SQL_INTEGER;
-                        }
+            case OpTypes.LIMIT :
+                if (nodes[LEFT] != null) {
+                    if (nodes[LEFT].dataType == null) {
+                        throw Error.error(ErrorCode.X_42567);
+                    }
+
+                    if (!nodes[LEFT].dataType.isIntegralType()) {
+                        throw Error.error(ErrorCode.X_42563);
+                    }
+                }
+
+                if (nodes[RIGHT] != null) {
+                    if (nodes[RIGHT].dataType == null) {
+                        throw Error.error(ErrorCode.X_42567);
+                    }
+
+                    if (!nodes[RIGHT].dataType.isIntegralType()) {
+                        throw Error.error(ErrorCode.X_42563);
                     }
                 }
                 break;
 
-            case OpTypes.ALTERNATIVE :
+            case OpTypes.PREFIX :
                 break;
 
             default :
@@ -456,12 +505,32 @@ public class ExpressionOp extends Expression {
                 expr.nodes[LEFT].dataType = Type.SQL_BOOLEAN;
             }
 
-            expr.nodes[RIGHT].nodes[LEFT].resolveTypes(session, nodes[RIGHT]);
-            expr.nodes[RIGHT].nodes[RIGHT].resolveTypes(session, nodes[RIGHT]);
+            expr.nodes[RIGHT].nodes[LEFT].resolveTypes(session,
+                    expr.nodes[RIGHT]);
+
+            if (expr.nodes[RIGHT].nodes[RIGHT].opType != OpTypes.CASEWHEN) {
+                expr.nodes[RIGHT].nodes[RIGHT].resolveTypes(session,
+                        expr.nodes[RIGHT]);
+            }
 
             expr = expr.nodes[RIGHT].nodes[RIGHT];
         }
 
+        if (exprSubType == OpTypes.CAST) {
+            if (nodes[RIGHT].nodes[RIGHT].dataType != null
+                    && nodes[RIGHT].nodes[RIGHT].dataType
+                       != nodes[RIGHT].nodes[LEFT].dataType) {
+                Type castType = nodes[RIGHT].nodes[RIGHT].dataType;
+
+                if (castType.isCharacterType()) {
+                    castType = Type.SQL_VARCHAR_DEFAULT;
+                }
+
+                nodes[RIGHT].nodes[LEFT] =
+                    new ExpressionOp(nodes[RIGHT].nodes[LEFT], castType);
+            }
+        }
+
         expr = this;
 
         while (expr.opType == OpTypes.CASEWHEN) {
@@ -492,7 +561,7 @@ public class ExpressionOp extends Expression {
             expr = expr.nodes[RIGHT].nodes[RIGHT];
         }
 
-        if (dataType == null) {
+        if (dataType == null || dataType.typeCode == Types.SQL_ALL_TYPES) {
             throw Error.error(ErrorCode.X_42567);
         }
     }
@@ -504,16 +573,207 @@ public class ExpressionOp extends Expression {
             case OpTypes.VALUE :
                 return valueData;
 
-            case OpTypes.SIMPLE_COLUMN : {
-                Object[] data =
-                    (Object[]) session.sessionContext
-                        .rangeIterators[rangePosition].getCurrent();
+            case OpTypes.LIKE_ARG : {
+                boolean hasEscape  = nodes[RIGHT] != null;
+                int     escapeChar = Integer.MAX_VALUE;
 
-                return data[columnIndex];
+                if (dataType.isBinaryType()) {
+                    BinaryData left =
+                        (BinaryData) nodes[LEFT].getValue(session);
+
+                    if (left == null) {
+                        return null;
+                    }
+
+                    if (hasEscape) {
+                        BinaryData right =
+                            (BinaryData) nodes[RIGHT].getValue(session);
+
+                        if (right == null) {
+                            return null;
+                        }
+
+                        if (right.length(session) != 1) {
+                            throw Error.error(ErrorCode.X_2200D);
+                        }
+
+                        escapeChar = right.getBytes()[0];
+                    }
+
+                    byte[]  array       = left.getBytes();
+                    byte[]  newArray    = new byte[array.length];
+                    boolean wasEscape   = false;
+                    int     escapeCount = 0;
+                    int     i           = 0;
+                    int     j           = 0;
+
+                    for (; i < array.length; i++) {
+                        if (array[i] == escapeChar) {
+                            if (wasEscape) {
+                                escapeCount++;
+
+                                newArray[j++] = array[i];
+                                wasEscape     = false;
+
+                                continue;
+                            }
+
+                            wasEscape = true;
+
+                            if (i == array.length - 1) {
+                                throw Error.error(ErrorCode.X_22025);
+                            }
+
+                            continue;
+                        }
+
+                        if (array[i] == '_' || array[i] == '%') {
+                            if (wasEscape) {
+                                escapeCount++;
+
+                                newArray[j++] = array[i];
+                                wasEscape     = false;
+
+                                continue;
+                            }
+
+                            break;
+                        }
+
+                        if (wasEscape) {
+                            throw Error.error(ErrorCode.X_22025);
+                        }
+
+                        newArray[j++] = array[i];
+                    }
+
+                    newArray =
+                        (byte[]) ArrayUtil.resizeArrayIfDifferent(newArray, j);
+
+                    return new BinaryData(newArray, false);
+                } else {
+                    String left = (String) nodes[LEFT].getValue(session);
+
+                    if (left == null) {
+                        return null;
+                    }
+
+                    if (hasEscape) {
+                        String right = (String) nodes[RIGHT].getValue(session);
+
+                        if (right == null) {
+                            return null;
+                        }
+
+                        if (right.length() != 1) {
+                            throw Error.error(ErrorCode.X_22019);
+                        }
+
+                        escapeChar = right.getBytes()[0];
+                    }
+
+                    char[]  array       = left.toCharArray();
+                    char[]  newArray    = new char[array.length];
+                    boolean wasEscape   = false;
+                    int     escapeCount = 0;
+                    int     i           = 0;
+                    int     j           = 0;
+
+                    for (; i < array.length; i++) {
+                        if (array[i] == escapeChar) {
+                            if (wasEscape) {
+                                escapeCount++;
+
+                                newArray[j++] = array[i];
+                                wasEscape     = false;
+
+                                continue;
+                            }
+
+                            wasEscape = true;
+
+                            if (i == array.length - 1) {
+                                throw Error.error(ErrorCode.X_22025);
+                            }
+
+                            continue;
+                        }
+
+                        if (array[i] == '_' || array[i] == '%') {
+                            if (wasEscape) {
+                                escapeCount++;
+
+                                newArray[j++] = array[i];
+                                wasEscape     = false;
+
+                                continue;
+                            }
+
+                            break;
+                        }
+
+                        if (wasEscape) {
+                            throw Error.error(ErrorCode.X_22025);
+                        }
+
+                        newArray[j++] = array[i];
+                    }
+
+                    return new String(newArray, 0, j);
+                }
+            }
+            case OpTypes.SIMPLE_COLUMN : {
+                Object value =
+                    session.sessionContext.rangeIterators[rangePosition]
+                        .getCurrent(columnIndex);
+
+                return value;
             }
             case OpTypes.ORDER_BY :
                 return nodes[LEFT].getValue(session);
 
+            case OpTypes.PREFIX : {
+                if (nodes[LEFT].dataType.isCharacterType()) {
+                    Object        value = nodes[RIGHT].getValue(session);
+
+                    if (value == null) {
+                        return null;
+                    }
+
+                    CharacterType type = (CharacterType) nodes[RIGHT].dataType;
+                    long length =
+                        ((CharacterType) nodes[RIGHT].dataType).size(session,
+                            value);
+
+                    type  = (CharacterType) nodes[LEFT].dataType;
+                    value = nodes[LEFT].getValue(session);
+
+                    if (value == null) {
+                        return null;
+                    }
+
+                    return type.substring(session, value, 0, length, true,
+                                          false);
+                } else {
+                    BinaryData value =
+                        (BinaryData) nodes[RIGHT].getValue(session);
+
+                    if (value == null) {
+                        return null;
+                    }
+
+                    long       length = value.length(session);
+                    BinaryType type   = (BinaryType) nodes[LEFT].dataType;
+
+                    value = (BinaryData) nodes[LEFT].getValue(session);
+
+                    if (value == null) {
+                        return null;
+                    }
+
+                    return type.substring(session, value, 0, length, true);
+                }
+            }
             case OpTypes.CAST : {
                 Object value =
                     dataType.castToType(session,
@@ -526,7 +786,7 @@ public class ExpressionOp extends Expression {
 
                     for (int i = 0; i < constraints.length; i++) {
                         constraints[i].checkCheckConstraint(session, null,
-                                                            value);
+                                                            null, value);
                     }
                 }
 
