@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,7 @@ import org.hsqldb.types.BlobData;
 import org.hsqldb.types.CharacterType;
 import org.hsqldb.types.DTIType;
 import org.hsqldb.types.DateTimeType;
+import org.hsqldb.types.IntervalType;
 import org.hsqldb.types.NumberType;
 import org.hsqldb.types.Type;
 import org.hsqldb.types.Types;
@@ -51,61 +52,61 @@ import org.hsqldb.types.Types;
  * Implementation of SQL standard function calls
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.1.1
  * @since 1.9.0
  */
 public class FunctionSQL extends Expression {
 
-    protected final static int FUNC_POSITION_CHAR                    = 1;     // numeric
-    private final static int   FUNC_POSITION_BINARY                  = 2;
-    private final static int   FUNC_OCCURENCES_REGEX                 = 3;
-    private final static int   FUNC_POSITION_REGEX                   = 4;
-    protected final static int FUNC_EXTRACT                          = 5;
-    protected final static int FUNC_BIT_LENGTH                       = 6;
-    protected final static int FUNC_CHAR_LENGTH                      = 7;
-    protected final static int FUNC_OCTET_LENGTH                     = 8;
-    private final static int   FUNC_CARDINALITY                      = 9;
-    private final static int   FUNC_MAX_CARDINALITY                  = 10;
-    private final static int   FUNC_TRIM_ARRAY                       = 11;
-    private final static int   FUNC_ABS                              = 12;
-    private final static int   FUNC_MOD                              = 13;
-    protected final static int FUNC_LN                               = 14;
-    private final static int   FUNC_EXP                              = 15;
-    private final static int   FUNC_POWER                            = 16;
-    private final static int   FUNC_SQRT                             = 17;
-    private final static int   FUNC_FLOOR                            = 20;
-    private final static int   FUNC_CEILING                          = 21;
-    private final static int   FUNC_WIDTH_BUCKET                     = 22;
-    protected final static int FUNC_SUBSTRING_CHAR                   = 23;    // string
-    private final static int   FUNC_SUBSTRING_REG_EXPR               = 24;
-    private final static int   FUNC_SUBSTRING_REGEX                  = 25;
-    protected final static int FUNC_FOLD_LOWER                       = 26;
-    protected final static int FUNC_FOLD_UPPER                       = 27;
-    private final static int   FUNC_TRANSCODING                      = 28;
-    private final static int   FUNC_TRANSLITERATION                  = 29;
-    private final static int   FUNC_REGEX_TRANSLITERATION            = 30;
-    protected final static int FUNC_TRIM_CHAR                        = 31;
-    final static int           FUNC_OVERLAY_CHAR                     = 32;
-    private final static int   FUNC_CHAR_NORMALIZE                   = 33;
-    private final static int   FUNC_SUBSTRING_BINARY                 = 40;
-    private final static int   FUNC_TRIM_BINARY                      = 41;
-    private final static int   FUNC_OVERLAY_BINARY                   = 42;
-    protected final static int FUNC_CURRENT_DATE                     = 43;    // datetime
-    protected final static int FUNC_CURRENT_TIME                     = 44;
-    protected final static int FUNC_CURRENT_TIMESTAMP                = 50;
-    protected final static int FUNC_LOCALTIME                        = 51;
-    protected final static int FUNC_LOCALTIMESTAMP                   = 52;
-    private final static int   FUNC_CURRENT_CATALOG                  = 53;    // general
-    private final static int   FUNC_CURRENT_DEFAULT_TRANSFORM_GROUP  = 54;
-    private final static int   FUNC_CURRENT_PATH                     = 55;
-    private final static int   FUNC_CURRENT_ROLE                     = 56;
-    private final static int   FUNC_CURRENT_SCHEMA                   = 57;
-    private final static int   FUNC_CURRENT_TRANSFORM_GROUP_FOR_TYPE = 58;
-    private final static int   FUNC_CURRENT_USER                     = 59;
-    private final static int   FUNC_SESSION_USER                     = 60;
-    private final static int   FUNC_SYSTEM_USER                      = 61;
-    protected final static int FUNC_USER                             = 62;
-    private final static int   FUNC_VALUE                            = 63;
+    protected static final int FUNC_POSITION_CHAR                    = 1;     // numeric
+    private static final int   FUNC_POSITION_BINARY                  = 2;
+    private static final int   FUNC_OCCURENCES_REGEX                 = 3;
+    private static final int   FUNC_POSITION_REGEX                   = 4;
+    protected static final int FUNC_EXTRACT                          = 5;
+    protected static final int FUNC_BIT_LENGTH                       = 6;
+    protected static final int FUNC_CHAR_LENGTH                      = 7;
+    protected static final int FUNC_OCTET_LENGTH                     = 8;
+    private static final int   FUNC_CARDINALITY                      = 9;
+    private static final int   FUNC_MAX_CARDINALITY                  = 10;
+    private static final int   FUNC_TRIM_ARRAY                       = 11;
+    private static final int   FUNC_ABS                              = 12;
+    private static final int   FUNC_MOD                              = 13;
+    protected static final int FUNC_LN                               = 14;
+    private static final int   FUNC_EXP                              = 15;
+    private static final int   FUNC_POWER                            = 16;
+    private static final int   FUNC_SQRT                             = 17;
+    private static final int   FUNC_FLOOR                            = 20;
+    private static final int   FUNC_CEILING                          = 21;
+    private static final int   FUNC_WIDTH_BUCKET                     = 22;
+    protected static final int FUNC_SUBSTRING_CHAR                   = 23;    // string
+    private static final int   FUNC_SUBSTRING_REG_EXPR               = 24;
+    private static final int   FUNC_SUBSTRING_REGEX                  = 25;
+    protected static final int FUNC_FOLD_LOWER                       = 26;
+    protected static final int FUNC_FOLD_UPPER                       = 27;
+    private static final int   FUNC_TRANSCODING                      = 28;
+    private static final int   FUNC_TRANSLITERATION                  = 29;
+    private static final int   FUNC_REGEX_TRANSLITERATION            = 30;
+    protected static final int FUNC_TRIM_CHAR                        = 31;
+    static final int           FUNC_OVERLAY_CHAR                     = 32;
+    private static final int   FUNC_CHAR_NORMALIZE                   = 33;
+    private static final int   FUNC_SUBSTRING_BINARY                 = 40;
+    private static final int   FUNC_TRIM_BINARY                      = 41;
+    private static final int   FUNC_OVERLAY_BINARY                   = 42;
+    protected static final int FUNC_CURRENT_DATE                     = 43;    // datetime
+    protected static final int FUNC_CURRENT_TIME                     = 44;
+    protected static final int FUNC_CURRENT_TIMESTAMP                = 50;
+    protected static final int FUNC_LOCALTIME                        = 51;
+    protected static final int FUNC_LOCALTIMESTAMP                   = 52;
+    private static final int   FUNC_CURRENT_CATALOG                  = 53;    // general
+    private static final int   FUNC_CURRENT_DEFAULT_TRANSFORM_GROUP  = 54;
+    private static final int   FUNC_CURRENT_PATH                     = 55;
+    private static final int   FUNC_CURRENT_ROLE                     = 56;
+    private static final int   FUNC_CURRENT_SCHEMA                   = 57;
+    private static final int   FUNC_CURRENT_TRANSFORM_GROUP_FOR_TYPE = 58;
+    private static final int   FUNC_CURRENT_USER                     = 59;
+    private static final int   FUNC_SESSION_USER                     = 60;
+    private static final int   FUNC_SYSTEM_USER                      = 61;
+    protected static final int FUNC_USER                             = 62;
+    private static final int   FUNC_VALUE                            = 63;
 
     //
     static final short[] noParamList             = new short[]{};
@@ -566,9 +567,22 @@ public class FunctionSQL extends Expression {
                     return null;
                 }
 
+                long offset = 0;
+
+                if (nodes.length > 3 && nodes[3] != null) {
+                    Object value = nodes[3].getValue(session);
+
+                    offset = ((Number) value).longValue() - 1;
+
+                    if (offset < 0) {
+                        offset = 0;
+                    }
+                }
+
                 long result =
                     ((CharacterType) nodes[1].dataType).position(
-                        session, data[1], data[0], nodes[0].dataType, 0) + 1;
+                        session, data[1], data[0], nodes[0].dataType,
+                        offset) + 1;
 
                 if (nodes[2] != null
                         && ((Number) nodes[2].valueData).intValue()
@@ -726,8 +740,11 @@ public class FunctionSQL extends Expression {
                 }
 
                 // result type is the same as nodes[1]
-                return ((NumberType) dataType).modulo(data[0], data[1],
-                                                      nodes[0].dataType);
+                Object value = ((NumberType) nodes[0].dataType).modulo(data[0],
+                    data[1], nodes[0].dataType);
+
+                return dataType.convertToType(session, value,
+                                              nodes[0].dataType);
             }
             case FUNC_LN : {
                 if (data[0] == null) {
@@ -762,10 +779,6 @@ public class FunctionSQL extends Expression {
                 double exponent = ((Number) data[1]).doubleValue();
                 double val;
 
-                if (exponent < 0) {
-                    throw Error.error(ErrorCode.X_2201F);
-                }
-
                 if (base == 0) {
                     if (exponent < 0) {
                         throw Error.error(ErrorCode.X_2201F);
@@ -804,7 +817,91 @@ public class FunctionSQL extends Expression {
                 return ((NumberType) dataType).ceiling(data[0]);
             }
             case FUNC_WIDTH_BUCKET : {
-                return null;
+                for (int i = 0; i < data.length; i++) {
+                    if (data[i] == null) {
+                        return null;
+                    }
+                }
+
+                if (((NumberType) nodes[3].dataType).isNegative(data[3])) {
+                    throw Error.error(ErrorCode.X_2201G);
+                }
+
+                int compare = nodes[1].dataType.compare(session, data[1],
+                    data[2]);
+                Type   subType;
+                Object temp;
+                Object temp2;
+
+                if (nodes[0].dataType.isNumberType()) {
+                    subType = nodes[0].dataType;
+                } else {
+                    subType = nodes[0].dataType.getCombinedType(session,
+                            nodes[0].dataType, OpTypes.SUBTRACT);
+                }
+
+                switch (compare) {
+
+                    case 0 :
+                        throw Error.error(ErrorCode.X_2201G);
+                    case -1 : {
+                        if (nodes[0].dataType.compare(
+                                session, data[0], data[1]) < 0) {
+                            return ValuePool.INTEGER_0;
+                        }
+
+                        if (nodes[0].dataType.compare(
+                                session, data[0], data[2]) >= 0) {
+                            return dataType.add(data[3], ValuePool.INTEGER_1,
+                                                Type.SQL_INTEGER);
+                        }
+
+                        temp = subType.subtract(data[0], data[1],
+                                                nodes[0].dataType);
+                        temp2 = subType.subtract(data[2], data[1],
+                                                 nodes[0].dataType);
+
+                        break;
+                    }
+                    case 1 : {
+                        if (nodes[0].dataType.compare(
+                                session, data[0], data[1]) > 0) {
+                            return ValuePool.INTEGER_0;
+                        }
+
+                        if (nodes[0].dataType.compare(
+                                session, data[0], data[2]) <= 0) {
+                            return dataType.add(data[3], ValuePool.INTEGER_1,
+                                                Type.SQL_INTEGER);
+                        }
+
+                        temp = subType.subtract(data[1], data[0],
+                                                nodes[0].dataType);
+                        temp2 = subType.subtract(data[1], data[2],
+                                                 nodes[0].dataType);
+
+                        break;
+                    }
+                    default :
+                        throw Error.runtimeError(ErrorCode.U_S0500, "");
+                }
+
+                Type opType;
+
+                if (subType.typeCode == Types.SQL_DOUBLE) {
+                    opType = subType;
+                } else {
+                    opType = IntervalType.factorType;
+                    temp   = opType.convertToType(session, temp, subType);
+                    temp2  = opType.convertToType(session, temp2, subType);
+                }
+
+                temp = opType.multiply(temp, data[3]);
+                temp = opType.divide(session, temp, temp2);
+                temp = dataType.convertToDefaultType(session, temp);
+
+                return dataType.add(temp, ValuePool.INTEGER_1,
+                                    Type.SQL_INTEGER);
             }
             case FUNC_SUBSTRING_CHAR : {
                 if (data[0] == null || data[1] == null) {
@@ -999,7 +1096,7 @@ public class FunctionSQL extends Expression {
                 byte[] bytes = string.getBytes();
 
                 return ((BinaryType) dataType).trim(session,
-                                                    (BlobData) data[3],
+                                                    (BlobData) data[2],
                                                     bytes[0], leading,
                                                     trailing);
             }
@@ -1041,7 +1138,7 @@ public class FunctionSQL extends Expression {
             */
             case FUNC_CURRENT_ROLE :
                 return session.getRole() == null ? null
-                                                 : session.getRole()
+                                                 : session.getRole().getName()
                                                      .getNameString();
 
             case FUNC_CURRENT_SCHEMA :
@@ -1051,16 +1148,16 @@ public class FunctionSQL extends Expression {
             case FUNC_CURRENT_TRANSFORM_GROUP_FOR_TYPE :
             */
             case FUNC_CURRENT_USER :
-                return session.getUser().getNameString();
+                return session.getUser().getName().getNameString();
 
             case FUNC_SESSION_USER :
-                return session.getUser().getNameString();
+                return session.getUser().getName().getNameString();
 
             case FUNC_SYSTEM_USER :
-                return session.getUser().getNameString();
+                return session.getUser().getName().getNameString();
 
             case FUNC_USER :
-                return session.getUser().getNameString();
+                return session.getUser().getName().getNameString();
 
             case FUNC_VALUE :
                 return session.sessionData.currentValue;
@@ -1138,6 +1235,16 @@ public class FunctionSQL extends Expression {
                     throw Error.error(ErrorCode.X_42563);
                 }
 
+                if (nodes.length > 3 && nodes[3] != null) {
+                    if (nodes[3].isDynamicParam()) {
+                        nodes[3].dataType = Type.SQL_BIGINT;
+                    }
+
+                    if (!nodes[3].dataType.isNumberType()) {
+                        throw Error.error(ErrorCode.X_42563);
+                    }
+                }
+
                 dataType = Type.SQL_BIGINT;
 
                 break;
@@ -1179,6 +1286,10 @@ public class FunctionSQL extends Expression {
                 break;
             }
             case FUNC_CHAR_LENGTH :
+                if (nodes[0].dataType == null) {
+                    nodes[0].dataType = Type.SQL_VARCHAR;
+                }
+
                 if (!nodes[0].dataType.isCharacterType()) {
                     throw Error.error(ErrorCode.X_42563);
                 }
@@ -1247,11 +1358,11 @@ public class FunctionSQL extends Expression {
             }
             case FUNC_MOD : {
                 if (nodes[0].dataType == null) {
-                    nodes[1].dataType = nodes[0].dataType;
+                    nodes[0].dataType = nodes[1].dataType;
                 }
 
                 if (nodes[1].dataType == null) {
-                    nodes[0].dataType = nodes[1].dataType;
+                    nodes[1].dataType = nodes[0].dataType;
                 }
 
                 if (nodes[0].dataType == null) {
@@ -1273,11 +1384,11 @@ public class FunctionSQL extends Expression {
             }
             case FUNC_POWER : {
                 if (nodes[0].dataType == null) {
-                    nodes[1].dataType = nodes[0].dataType;
+                    nodes[0].dataType = nodes[1].dataType;
                 }
 
                 if (nodes[1].dataType == null) {
-                    nodes[0].dataType = nodes[1].dataType;
+                    nodes[1].dataType = nodes[0].dataType;
                 }
 
                 if (nodes[0].dataType == null) {
@@ -1344,16 +1455,28 @@ public class FunctionSQL extends Expression {
                 break;
             }
             case FUNC_WIDTH_BUCKET : {
-                if (nodes[0].dataType == null || nodes[1].dataType == null
-                        || nodes[2].dataType == null
-                        || nodes[3].dataType == null) {
+                nodes[0].dataType = Type.getAggregateType(nodes[0].dataType,
+                        nodes[1].dataType);
+                nodes[0].dataType = Type.getAggregateType(nodes[0].dataType,
+                        nodes[2].dataType);
+
+                if (nodes[0].dataType == null) {
                     throw Error.error(ErrorCode.X_42567);
                 }
 
                 if (!nodes[0].dataType.isNumberType()
-                        || !nodes[1].dataType.isNumberType()
-                        || !nodes[2].dataType.isNumberType()
-                        || !nodes[3].dataType.isIntegralType()) {
+                        && !nodes[0].dataType.isDateTimeType()) {
+                    throw Error.error(ErrorCode.X_42563);
+                }
+
+                nodes[1].dataType = nodes[0].dataType;
+                nodes[2].dataType = nodes[0].dataType;
+
+                if (nodes[3].dataType == null) {
+                    nodes[3].dataType = Type.SQL_INTEGER;
+                }
+
+                if (!nodes[3].dataType.isIntegralType()) {
                     throw Error.error(ErrorCode.X_42563);
                 }
 
@@ -1366,7 +1489,8 @@ public class FunctionSQL extends Expression {
                 if (nodes[0].dataType == null) {
 
                     // in 20.6 parameter not allowed as type cannot be determined as binary or char
-                    throw Error.error(ErrorCode.X_42567);
+                    // throw Error.error(ErrorCode.X_42567);
+                    nodes[0].dataType = Type.SQL_VARCHAR_DEFAULT;
                 }
 
                 if (nodes[1].dataType == null) {
@@ -1396,9 +1520,9 @@ public class FunctionSQL extends Expression {
                     funcType = FUNC_SUBSTRING_CHAR;
 
                     if (dataType.typeCode == Types.SQL_CHAR) {
-                        dataType =
-                            CharacterType.getCharacterType(Types.SQL_VARCHAR,
-                                                           dataType.precision);
+                        dataType = CharacterType.getCharacterType(
+                            Types.SQL_VARCHAR, dataType.precision,
+                            dataType.getCollation());
                     }
                 } else if (dataType.isBinaryType()) {
                     funcType = FUNC_SUBSTRING_BINARY;
@@ -1449,7 +1573,7 @@ public class FunctionSQL extends Expression {
                 }
 
                 if (nodes[2].dataType == null) {
-                    throw Error.error(ErrorCode.X_42567);
+                    nodes[2].dataType = Type.SQL_VARCHAR_DEFAULT;
                 }
 
                 dataType = nodes[2].dataType;
@@ -1458,9 +1582,9 @@ public class FunctionSQL extends Expression {
                     funcType = FUNC_TRIM_CHAR;
 
                     if (dataType.typeCode == Types.SQL_CHAR) {
-                        dataType =
-                            CharacterType.getCharacterType(Types.SQL_VARCHAR,
-                                                           dataType.precision);
+                        dataType = CharacterType.getCharacterType(
+                            Types.SQL_VARCHAR, dataType.precision,
+                            dataType.getCollation());
                     }
 
                     if (nodes[1] == null) {
@@ -1483,14 +1607,17 @@ public class FunctionSQL extends Expression {
             case FUNC_OVERLAY_BINARY : {
                 if (nodes[0].dataType == null) {
                     if (nodes[1].dataType == null) {
-                        throw Error.error(ErrorCode.X_42567);
+                        nodes[0].dataType = nodes[1].dataType =
+                            Type.SQL_VARCHAR_DEFAULT;
+
+                        // throw Error.error(ErrorCode.X_42567);
                     }
 
                     if (nodes[1].dataType.typeCode == Types.SQL_CLOB
                             || nodes[1].dataType.isBinaryType()) {
                         nodes[0].dataType = nodes[1].dataType;
                     } else {
-                        nodes[0].dataType = Type.SQL_VARCHAR;
+                        nodes[0].dataType = Type.SQL_VARCHAR_DEFAULT;
                     }
                 }
 
@@ -1499,7 +1626,7 @@ public class FunctionSQL extends Expression {
                             || nodes[0].dataType.isBinaryType()) {
                         nodes[1].dataType = nodes[0].dataType;
                     } else {
-                        nodes[1].dataType = Type.SQL_VARCHAR;
+                        nodes[1].dataType = Type.SQL_VARCHAR_DEFAULT;
                     }
                 }
 
@@ -1509,15 +1636,23 @@ public class FunctionSQL extends Expression {
 
                     if (nodes[0].dataType.typeCode == Types.SQL_CLOB
                             || nodes[1].dataType.typeCode == Types.SQL_CLOB) {
-                        dataType = CharacterType.getCharacterType(
-                            Types.SQL_CLOB,
-                            nodes[0].dataType.precision
-                            + nodes[1].dataType.precision);
+                        dataType =
+                            CharacterType
+                                .getCharacterType(Types.SQL_CLOB,
+                                                  nodes[0].dataType.precision
+                                                  + nodes[1].dataType
+                                                      .precision, nodes[0]
+                                                      .dataType
+                                                      .getCollation());
                     } else {
-                        dataType = CharacterType.getCharacterType(
-                            Types.SQL_VARCHAR,
-                            nodes[0].dataType.precision
-                            + nodes[1].dataType.precision);
+                        dataType =
+                            CharacterType
+                                .getCharacterType(Types.SQL_VARCHAR,
+                                                  nodes[0].dataType.precision
+                                                  + nodes[1].dataType
+                                                      .precision, nodes[0]
+                                                      .dataType
+                                                      .getCollation());
                     }
                 } else if (nodes[0].dataType.isBinaryType()
                            && nodes[1].dataType.isBinaryType()) {
@@ -1579,7 +1714,7 @@ public class FunctionSQL extends Expression {
             case FUNC_SESSION_USER :
             case FUNC_SYSTEM_USER :
             case FUNC_USER :
-                dataType = SqlInvariants.SQL_IDENTIFIER;
+                dataType = TypeInvariants.SQL_IDENTIFIER;
                 break;
 
             case FUNC_VALUE :
@@ -1935,7 +2070,7 @@ public class FunctionSQL extends Expression {
         return sb.toString();
     }
 
-    public boolean equals(Object other) {
+    public boolean equals(Expression other) {
 
         if (other instanceof FunctionSQL
                 && funcType == ((FunctionSQL) other).funcType) {
