@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,14 +59,13 @@ import org.hsqldb.error.ErrorCode;
  * than all the existing names.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.7.2
  */
 public final class HsqlNameManager {
 
-    public static final String DEFAULT_CATALOG_NAME = "PUBLIC";
-    private static final HsqlNameManager staticManager =
-        new HsqlNameManager(null);
+    public static final String           DEFAULT_CATALOG_NAME = "PUBLIC";
+    private static final HsqlNameManager staticManager = new HsqlNameManager();
 
     static {
         staticManager.serialNumber = Integer.MIN_VALUE;
@@ -86,16 +85,31 @@ public final class HsqlNameManager {
     private int      serialNumber = 1;        // 0 is reserved in lookups
     private int      sysNumber    = 10000;    // avoid name clash in older scripts
     private HsqlName catalogName;
+    private boolean  sqlRegularNames;
+    HsqlName         subqueryTableName;
+
+    public HsqlNameManager() {
+        sqlRegularNames = true;
+    }
 
     public HsqlNameManager(Database database) {
+
         catalogName = new HsqlName(this, DEFAULT_CATALOG_NAME,
                                    SchemaObject.CATALOG, false);
+        sqlRegularNames = database.sqlRegularNames;
+        subqueryTableName = new HsqlName(this, SqlInvariants.SYSTEM_SUBQUERY,
+                                         false, SchemaObject.TABLE);
+        subqueryTableName.schema = SqlInvariants.SYSTEM_SCHEMA_HSQLNAME;
     }
 
     public HsqlName getCatalogName() {
         return catalogName;
     }
 
+    public void setSqlRegularNames(boolean value) {
+        sqlRegularNames = value;
+    }
+
     public static HsqlName newSystemObjectName(String name, int type) {
         return new HsqlName(staticManager, name, type, false);
     }
@@ -187,13 +201,7 @@ public final class HsqlNameManager {
      * Same name string but different objects and serial number
      */
     public HsqlName getSubqueryTableName() {
-
-        HsqlName hsqlName = new HsqlName(this, SqlInvariants.SYSTEM_SUBQUERY,
-                                         false, SchemaObject.TABLE);
-
-        hsqlName.schema = SqlInvariants.SYSTEM_SCHEMA_HSQLNAME;
-
-        return hsqlName;
+        return subqueryTableName;
     }
 
     /**
@@ -227,7 +235,7 @@ public final class HsqlNameManager {
     /**
      * Column index i is 0 based, returns 1 based numbered column.
      */
-    static public HsqlName getAutoColumnName(int i) {
+    public static HsqlName getAutoColumnName(int i) {
 
         if (i < autoColumnNames.length) {
             return autoColumnNames[i];
@@ -239,7 +247,7 @@ public final class HsqlNameManager {
     /**
      * Column index i is 0 based, returns 1 based numbered column.
      */
-    static public String getAutoColumnNameString(int i) {
+    public static String getAutoColumnNameString(int i) {
 
         if (i < autoColumnNames.length) {
             return autoColumnNames[i].name;
@@ -248,7 +256,7 @@ public final class HsqlNameManager {
         return "C" + (i + 1);
     }
 
-    static public String getAutoNoNameColumnString(int i) {
+    public static String getAutoNoNameColumnString(int i) {
 
         if (i < autoColumnNames.length) {
             return autoNoNameColumnNames[i];
@@ -257,7 +265,7 @@ public final class HsqlNameManager {
         return String.valueOf(i);
     }
 
-    static public String getAutoSavepointNameString(long i, int j) {
+    public static String getAutoSavepointNameString(long i, int j) {
 
         StringBuffer sb = new StringBuffer("S");
 
@@ -316,7 +324,7 @@ public final class HsqlNameManager {
         private SimpleName() {}
 
         private SimpleName(String name, boolean isNameQuoted) {
-            this.name         = name;
+            this.name         = new String(name);
             this.isNameQuoted = isNameQuoted;
         }
 
@@ -340,6 +348,10 @@ public final class HsqlNameManager {
                    ? StringConverter.toQuotedString(name, '"', true)
                    : name;
         }
+
+        public String getNameString() {
+            return name;
+        }
     }
 
     public static final class HsqlName extends SimpleName {
@@ -377,8 +389,9 @@ public final class HsqlNameManager {
 
             this(man, type);
 
-            this.name         = this.statementName = name;
-            this.isNameQuoted = isQuoted;
+            this.name          = name;
+            this.statementName = name;
+            this.isNameQuoted  = isQuoted;
 
             if (isNameQuoted) {
                 statementName = StringConverter.toQuotedString(name, '"',
@@ -392,24 +405,32 @@ public final class HsqlNameManager {
 
         public String getSchemaQualifiedStatementName() {
 
-            if (type == SchemaObject.COLUMN) {
-                if (parent == null
-                        || SqlInvariants.SYSTEM_SUBQUERY.equals(parent.name)) {
+            switch (type) {
+
+                case SchemaObject.PARAMETER :
+                case SchemaObject.VARIABLE : {
                     return statementName;
                 }
+                case SchemaObject.COLUMN : {
+                    if (parent == null
+                            || SqlInvariants.SYSTEM_SUBQUERY.equals(
+                                parent.name)) {
+                        return statementName;
+                    }
 
-                StringBuffer sb = new StringBuffer();
+                    StringBuffer sb = new StringBuffer();
 
-                if (schema != null) {
-                    sb.append(schema.getStatementName());
+                    if (schema != null) {
+                        sb.append(schema.getStatementName());
+                        sb.append('.');
+                    }
+
+                    sb.append(parent.getStatementName());
                     sb.append('.');
+                    sb.append(statementName);
+
+                    return sb.toString();
                 }
-
-                sb.append(parent.getStatementName());
-                sb.append('.');
-                sb.append(statementName);
-
-                return sb.toString();
             }
 
             if (schema == null) {
@@ -434,12 +455,13 @@ public final class HsqlNameManager {
 
         public void rename(String name, boolean isquoted) {
 
-            if (name.length() > 128) {
+            if (manager.sqlRegularNames && name.length() > 128) {
                 throw Error.error(ErrorCode.X_42501, name);
             }
 
-            this.name          = name;
-            this.statementName = name;
+            // get rid of the excess
+            this.name          = new String(name);
+            this.statementName = this.name;
             this.isNameQuoted  = isquoted;
 
             if (isNameQuoted) {
