@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,7 +51,7 @@ import org.hsqldb.types.Types;
  */
 public final class NumberSequence implements SchemaObject {
 
-    public final static NumberSequence[] emptyArray = new NumberSequence[]{};
+    public static final NumberSequence[] emptyArray = new NumberSequence[]{};
 
     //
     private HsqlName name;
@@ -88,9 +88,9 @@ public final class NumberSequence implements SchemaObject {
 
     public void setDefaults(HsqlName name, Type type) {
 
-        this.name = name;
-        this.dataType  = type;
-        this.name = name;
+        this.name     = name;
+        this.dataType = type;
+        this.name     = name;
 
         long min;
         long max;
@@ -182,24 +182,11 @@ public final class NumberSequence implements SchemaObject {
 
         StringBuffer sb = new StringBuffer(128);
 
-        if (name == null) {
-            sb.append(Tokens.T_GENERATED).append(' ');
-
-            if (isAlways()) {
-                sb.append(Tokens.T_ALWAYS);
-            } else {
-                sb.append(Tokens.T_BY).append(' ').append(Tokens.T_DEFAULT);
-            }
-
-            sb.append(' ').append(Tokens.T_AS).append(' ').append(
-                Tokens.T_IDENTITY).append(Tokens.T_OPENBRACKET);
-        } else {
-            sb.append(Tokens.T_CREATE).append(' ');
-            sb.append(Tokens.T_SEQUENCE).append(' ');
-            sb.append(getName().getSchemaQualifiedStatementName()).append(' ');
-            sb.append(Tokens.T_AS).append(' ');
-            sb.append(getDataType().getNameString()).append(' ');
-        }
+        sb.append(Tokens.T_CREATE).append(' ');
+        sb.append(Tokens.T_SEQUENCE).append(' ');
+        sb.append(getName().getSchemaQualifiedStatementName()).append(' ');
+        sb.append(Tokens.T_AS).append(' ');
+        sb.append(getDataType().getNameString()).append(' ');
 
         //
         sb.append(Tokens.T_START).append(' ');
@@ -230,6 +217,57 @@ public final class NumberSequence implements SchemaObject {
         return sb.toString();
     }
 
+    public String getSQLColumnDefinition() {
+
+        StringBuffer sb = new StringBuffer(128);
+
+        sb.append(Tokens.T_GENERATED).append(' ');
+
+        if (name == null) {
+            if (isAlways()) {
+                sb.append(Tokens.T_ALWAYS);
+            } else {
+                sb.append(Tokens.T_BY).append(' ').append(Tokens.T_DEFAULT);
+            }
+
+            sb.append(' ').append(Tokens.T_AS).append(' ').append(
+                Tokens.T_IDENTITY).append(Tokens.T_OPENBRACKET);
+
+            //
+            sb.append(Tokens.T_START).append(' ');
+            sb.append(Tokens.T_WITH).append(' ');
+            sb.append(startValue);
+
+            if (getIncrement() != 1) {
+                sb.append(' ').append(Tokens.T_INCREMENT).append(' ');
+                sb.append(Tokens.T_BY).append(' ');
+                sb.append(getIncrement());
+            }
+
+            if (!hasDefaultMinMax()) {
+                sb.append(' ').append(Tokens.T_MINVALUE).append(' ');
+                sb.append(getMinValue());
+                sb.append(' ').append(Tokens.T_MAXVALUE).append(' ');
+                sb.append(getMaxValue());
+            }
+
+            if (isCycle()) {
+                sb.append(' ').append(Tokens.T_CYCLE);
+            }
+
+            if (name == null) {
+                sb.append(Tokens.T_CLOSEBRACKET);
+            }
+        } else {
+            sb.append(Tokens.T_BY).append(' ').append(Tokens.T_DEFAULT);
+            sb.append(' ').append(Tokens.T_AS).append(' ');
+            sb.append(Tokens.T_SEQUENCE).append(' ');
+            sb.append(getName().getSchemaQualifiedStatementName());
+        }
+
+        return sb.toString();
+    }
+
     public long getChangeTimestamp() {
         return 0;
     }
