@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,7 @@ import org.hsqldb.types.Types;
 
 /**
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 public class ParserBase {
@@ -281,7 +281,7 @@ public class ParserBase {
                     break;
             }
 
-            throw Error.error(errorCode);
+            throw Error.error(errorCode, token.getFullString());
         }
 
         if (isRecording) {
@@ -329,6 +329,13 @@ public class ParserBase {
         }
     }
 
+    void checkIsIrregularCharInIdentifier() {
+
+        if (scanner.token.hasIrregularChar) {
+            throw unexpectedToken();
+        }
+    }
+
     boolean isIdentifier() {
         return scanner.token.isUndelimitedIdentifier
                || scanner.token.isDelimitedIdentifier;
@@ -377,7 +384,9 @@ public class ParserBase {
     void checkIsThis(int type) {
 
         if (token.tokenType != type) {
-            throw unexpectedToken();
+            String required = Tokens.getKeyword(type);
+
+            throw unexpectedTokenRequire(required);
         }
     }
 
@@ -525,7 +534,7 @@ public class ParserBase {
         return val;
     }
 
-    Expression readDateTimeIntervalLiteral() {
+    Expression readDateTimeIntervalLiteral(Session session) {
 
         int pos = getPosition();
 
@@ -594,13 +603,18 @@ public class ParserBase {
                     read();
                 }
 
-                if (token.tokenType != Tokens.X_VALUE
-                        || token.dataType.typeCode != Types.SQL_CHAR) {
+                if (token.tokenType != Tokens.X_VALUE) {
                     break;
                 }
 
                 String s = token.tokenString;
 
+                /* INT literal accepted in addition to string literal */
+                if (token.dataType.typeCode != Types.SQL_INTEGER
+                        && token.dataType.typeCode != Types.SQL_CHAR) {
+                    break;
+                }
+
                 read();
 
                 IntervalType dataType = readIntervalType(false);
@@ -731,6 +745,9 @@ public class ParserBase {
         expressionTypeMap.put(Tokens.STDDEV_SAMP, OpTypes.STDDEV_SAMP);
         expressionTypeMap.put(Tokens.VAR_POP, OpTypes.VAR_POP);
         expressionTypeMap.put(Tokens.VAR_SAMP, OpTypes.VAR_SAMP);
+        expressionTypeMap.put(Tokens.ARRAY_AGG, OpTypes.ARRAY_AGG);
+        expressionTypeMap.put(Tokens.GROUP_CONCAT, OpTypes.GROUP_CONCAT);
+        expressionTypeMap.put(Tokens.MEDIAN, OpTypes.MEDIAN);
     }
 
     HsqlException unexpectedToken(String tokenS) {
