@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,8 @@
 
 package org.hsqldb;
 
+import org.hsqldb.HsqlNameManager.HsqlName;
+import org.hsqldb.RangeGroup.RangeGroupSimple;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.ArrayUtil;
@@ -44,7 +46,7 @@ import org.hsqldb.types.Type;
  * Parser for DML statements
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 public class ParserDML extends ParserDQL {
@@ -56,30 +58,30 @@ public class ParserDML extends ParserDQL {
     /**
      * Retrieves an INSERT Statement from this parse context.
      */
-    StatementDMQL compileInsertStatement(RangeVariable[] outerRanges) {
+    StatementDMQL compileInsertStatement(RangeGroup[] rangeGroups) {
 
         read();
         readThis(Tokens.INTO);
 
-        boolean[] columnCheckList;
-        int[]     columnMap;
-        int       colCount;
-        Table     table             = readTableName();
-        boolean   overridingUser    = false;
-        boolean   overridingSystem  = false;
-        boolean   assignsToIdentity = false;
+        boolean[]     columnCheckList;
+        int[]         columnMap;
+        int           colCount;
+        Table         table;
+        RangeVariable range;
+        boolean       overridingUser    = false;
+        boolean       overridingSystem  = false;
+        boolean       assignsToIdentity = false;
 
+        range = readRangeVariableForDataChange(StatementTypes.INSERT);
+
+        range.resolveRangeTableTypes(session, RangeVariable.emptyArray);
+
+        table           = range.getTable();
         columnCheckList = null;
         columnMap       = table.getColumnMap();
         colCount        = table.getColumnCount();
 
-        int position = getPosition();
-
-        if (!table.isInsertable() && !table.isTriggerInsertable()
-                && !session.isProcessingScript) {
-            throw Error.error(ErrorCode.X_42545);
-        }
-
+        int   position  = getPosition();
         Table baseTable = table.isTriggerInsertable() ? table
                                                       : table.getBaseTable();
 
@@ -92,7 +94,7 @@ public class ParserDML extends ParserDQL {
                 Expression insertExpression = new Expression(OpTypes.ROW,
                     new Expression[]{});
 
-                insertExpression = new Expression(OpTypes.TABLE,
+                insertExpression = new Expression(OpTypes.VALUELIST,
                                                   new Expression[]{
                                                       insertExpression });
                 columnCheckList = table.getNewColumnCheckList();
@@ -139,8 +141,9 @@ public class ParserDML extends ParserDQL {
                     }
 
                     OrderedHashSet columnNames = new OrderedHashSet();
+                    boolean        withPrefix  = database.sqlSyntaxOra;
 
-                    readSimpleColumnNames(columnNames, table);
+                    readSimpleColumnNames(columnNames, range, withPrefix);
                     readThis(Tokens.CLOSEBRACKET);
 
                     colCount  = columnNames.size();
@@ -193,8 +196,8 @@ public class ParserDML extends ParserDQL {
                 Expression insertExpressions =
                     XreadContextuallyTypedTable(colCount);
                 HsqlList unresolved =
-                    insertExpressions.resolveColumnReferences(outerRanges,
-                        null);
+                    insertExpressions.resolveColumnReferences(session,
+                        RangeGroup.emptyGroup, rangeGroups, null);
 
                 ExpressionColumn.checkColumnsResolved(unresolved);
                 insertExpressions.resolveTypes(session, null);
@@ -224,7 +227,7 @@ public class ParserDML extends ParserDQL {
                             assignsToIdentity = true;
 
                             if (e.getType() != OpTypes.DEFAULT) {
-                                if (table.identitySequence.isAlways()) {
+                                if (baseTable.identitySequence.isAlways()) {
                                     if (!overridingUser && !overridingSystem) {
                                         throw Error.error(ErrorCode.X_42543);
                                     }
@@ -286,12 +289,12 @@ public class ParserDML extends ParserDQL {
             columnMap = newColumnMap;
         }
 
-        int enforcedDefaultIndex = table.getIdentityColumnIndex();
+        int enforcedDefaultIndex = baseTable.getIdentityColumnIndex();
         int overrideIndex        = -1;
 
         if (enforcedDefaultIndex != -1
                 && ArrayUtil.find(columnMap, enforcedDefaultIndex) > -1) {
-            if (table.identitySequence.isAlways()) {
+            if (baseTable.identitySequence.isAlways()) {
                 if (!overridingUser && !overridingSystem) {
                     throw Error.error(ErrorCode.X_42543);
                 }
@@ -307,11 +310,12 @@ public class ParserDML extends ParserDQL {
         Type[] types = new Type[columnMap.length];
 
         ArrayUtil.projectRow(baseTable.getColumnTypes(), columnMap, types);
+        compileContext.setOuterRanges(rangeGroups);
 
         QueryExpression queryExpression = XreadQueryExpression();
 
         queryExpression.setReturningResult();
-        queryExpression.resolve(session, outerRanges, types);
+        queryExpression.resolve(session, rangeGroups, types);
 
         if (colCount != queryExpression.getColumnCount()) {
             throw Error.error(ErrorCode.X_42546);
@@ -340,88 +344,118 @@ public class ParserDML extends ParserDQL {
         }
     }
 
-    /**
-     * Creates a DELETE-type Statement from this parse context.
-     */
-    StatementDMQL compileDeleteStatement(RangeVariable[] outerRanges) {
+    Statement compileTruncateStatement() {
 
-        Expression condition       = null;
-        boolean    truncate        = false;
-        boolean    restartIdentity = false;
-        int        statementType;
+        boolean         isTable         = false;
+        boolean         withCommit      = false;
+        boolean         noCheck         = false;
+        boolean         restartIdentity = false;
+        HsqlName        objectName      = null;
+        RangeVariable[] rangeVariables  = null;
+        Table           table           = null;
+        HsqlName[]      writeTableNames = null;
+
+        readThis(Tokens.TRUNCATE);
+
+        if (token.tokenType == Tokens.TABLE) {
+            readThis(Tokens.TABLE);
+
+            rangeVariables = new RangeVariable[]{
+                readRangeVariableForDataChange(StatementTypes.TRUNCATE) };
+            table      = rangeVariables[0].getTable();
+            objectName = table.getName();
+            isTable    = true;
+        } else {
+            readThis(Tokens.SCHEMA);
+
+            objectName = readSchemaName();
+        }
 
         switch (token.tokenType) {
 
-            case Tokens.TRUNCATE : {
+            case Tokens.CONTINUE : {
                 read();
-                readThis(Tokens.TABLE);
-
-                truncate      = true;
-                statementType = StatementTypes.TRUNCATE;
+                readThis(Tokens.IDENTITY);
 
                 break;
             }
-            case Tokens.DELETE : {
+            case Tokens.RESTART : {
                 read();
-                readThis(Tokens.FROM);
+                readThis(Tokens.IDENTITY);
 
-                statementType = StatementTypes.DELETE_WHERE;
+                restartIdentity = true;
 
                 break;
             }
-            default :
-                throw unexpectedToken();
         }
 
-        RangeVariable[] rangeVariables = {
-            readSimpleRangeVariable(statementType) };
-        Table table     = rangeVariables[0].getTable();
-        Table baseTable = table.getBaseTable();
+        if (!isTable) {
+            checkIsThis(Tokens.AND);
+        }
 
-        if (truncate) {
-            if (table != baseTable) {
-                throw Error.error(ErrorCode.X_42545);
-            }
+        if (readIfThis(Tokens.AND)) {
+            readThis(Tokens.COMMIT);
 
-            if (table.isTriggerDeletable()) {
+            withCommit = true;
 
-                // redundant
-                throw Error.error(ErrorCode.X_42545);
-            }
+            if (readIfThis(Tokens.NO)) {
+                readThis(Tokens.CHECK);
 
-            switch (token.tokenType) {
-
-                case Tokens.CONTINUE : {
-                    read();
-                    readThis(Tokens.IDENTITY);
-
-                    break;
-                }
-                case Tokens.RESTART : {
-                    read();
-                    readThis(Tokens.IDENTITY);
-
-                    restartIdentity = true;
-
-                    break;
-                }
-            }
-
-            if (table.fkMainConstraints.length > 0) {
-                throw Error.error(ErrorCode.X_23504);
+                noCheck = true;
             }
         }
 
-        if (!truncate && token.tokenType == Tokens.WHERE) {
+        if (isTable) {
+            writeTableNames = new HsqlName[]{ table.getName() };
+        } else {
+            writeTableNames =
+                session.database.schemaManager.getCatalogAndBaseTableNames();
+        }
+
+        if (withCommit) {
+            Object[] args = new Object[] {
+                objectName, restartIdentity, noCheck
+            };
+
+            return new StatementCommand(StatementTypes.TRUNCATE, args, null,
+                                        writeTableNames);
+        }
+
+        Statement cs = new StatementDML(session, table, rangeVariables,
+                                        compileContext, restartIdentity,
+                                        StatementTypes.TRUNCATE);
+
+        return cs;
+    }
+
+    /**
+     * Creates a DELETE-type Statement from this parse context.
+     */
+    Statement compileDeleteStatement(RangeGroup[] rangeGroups) {
+
+        Expression condition       = null;
+        boolean    restartIdentity = false;
+
+        readThis(Tokens.DELETE);
+        readThis(Tokens.FROM);
+
+        RangeVariable[] rangeVariables = null;
+        Table           table          = null;
+
+        rangeVariables = new RangeVariable[]{
+            readRangeVariableForDataChange(StatementTypes.DELETE_WHERE) };
+        table = rangeVariables[0].getTable();
+
+        compileContext.setOuterRanges(rangeGroups);
+
+        if (token.tokenType == Tokens.WHERE) {
             read();
 
             condition = XreadBooleanValueExpression();
 
-            HsqlList unresolved =
-                condition.resolveColumnReferences(outerRanges, null);
-
-            unresolved = Expression.resolveColumnSet(rangeVariables,
-                    rangeVariables.length, unresolved, null);
+            RangeGroup rangeGroup = new RangeGroupSimple(rangeVariables);
+            HsqlList unresolved = condition.resolveColumnReferences(session,
+                rangeGroup, rangeGroups, null);
 
             ExpressionColumn.checkColumnsResolved(unresolved);
             condition.resolveTypes(session, null);
@@ -435,10 +469,10 @@ public class ParserDML extends ParserDQL {
             }
         }
 
-        if (baseTable == null) {
+        Table baseTable = table.isTriggerDeletable() ? table
+                                                     : table.getBaseTable();
 
-            //
-        } else if (table != baseTable) {
+        if (table != baseTable) {
             QuerySpecification baseSelect =
                 ((TableDerived) table).getQueryExpression().getMainSelect();
             RangeVariable[] newRangeVariables =
@@ -491,9 +525,14 @@ public class ParserDML extends ParserDQL {
             rangeVariables = resolver.rangeVariables;
         }
 
-        StatementDMQL cs = new StatementDML(session, table, rangeVariables,
-                                            compileContext, restartIdentity,
-                                            statementType);
+        for (int i = 0; i < rangeVariables.length; i++) {
+            rangeVariables[i].resolveRangeTableTypes(session,
+                    RangeVariable.emptyArray);
+        }
+
+        Statement cs = new StatementDML(session, table, rangeVariables,
+                                        compileContext, restartIdentity,
+                                        StatementTypes.DELETE_WHERE);
 
         return cs;
     }
@@ -501,7 +540,7 @@ public class ParserDML extends ParserDQL {
     /**
      * Creates an UPDATE-type Statement from this parse context.
      */
-    StatementDMQL compileUpdateStatement(RangeVariable[] outerRanges) {
+    StatementDMQL compileUpdateStatement(RangeGroup[] rangeGroups) {
 
         read();
 
@@ -512,9 +551,11 @@ public class ParserDML extends ParserDQL {
         LongDeque      colIndexList = new LongDeque();
         HsqlArrayList  exprList     = new HsqlArrayList();
         RangeVariable[] rangeVariables = {
-            readSimpleRangeVariable(StatementTypes.UPDATE_WHERE) };
-        Table table     = rangeVariables[0].rangeTable;
-        Table baseTable = table.getBaseTable();
+            readRangeVariableForDataChange(StatementTypes.UPDATE_WHERE) };
+        RangeGroup rangeGroup = new RangeGroupSimple(rangeVariables);
+        Table      table      = rangeVariables[0].rangeTable;
+        Table      baseTable  = table.isTriggerUpdatable() ? table
+                                                           : table.getBaseTable();
 
         readThis(Tokens.SET);
         readSetClauseList(rangeVariables, targetSet, colIndexList, exprList);
@@ -528,7 +569,7 @@ public class ParserDML extends ParserDQL {
         targetSet.toArray(targets);
 
         for (int i = 0; i < targets.length; i++) {
-            this.resolveOuterReferencesAndTypes(outerRanges, targets[i]);
+            this.resolveOuterReferencesAndTypes(rangeGroups, targets[i]);
         }
 
         columnCheckList   = table.getColumnCheckList(columnMap);
@@ -543,11 +584,8 @@ public class ParserDML extends ParserDQL {
 
             condition = XreadBooleanValueExpression();
 
-            HsqlList unresolved =
-                condition.resolveColumnReferences(outerRanges, null);
-
-            unresolved = Expression.resolveColumnSet(rangeVariables,
-                    rangeVariables.length, unresolved, null);
+            HsqlList unresolved = condition.resolveColumnReferences(session,
+                rangeGroup, rangeGroups, null);
 
             ExpressionColumn.checkColumnsResolved(unresolved);
             condition.resolveTypes(session, null);
@@ -561,8 +599,8 @@ public class ParserDML extends ParserDQL {
             }
         }
 
-        resolveUpdateExpressions(table, rangeVariables, columnMap,
-                                 updateExpressions, outerRanges);
+        resolveUpdateExpressions(table, rangeGroup, columnMap,
+                                 updateExpressions, rangeGroups);
 
         if (table != baseTable) {
             QuerySpecification baseSelect =
@@ -623,7 +661,12 @@ public class ParserDML extends ParserDQL {
             rangeVariables = resolver.rangeVariables;
         }
 
-        if (baseTable != null && table != baseTable) {
+        for (int i = 0; i < rangeVariables.length; i++) {
+            rangeVariables[i].resolveRangeTableTypes(session,
+                    RangeVariable.emptyArray);
+        }
+
+        if (table != baseTable) {
             int[] baseColumnMap = table.getBaseTableColumnMap();
             int[] newColumnMap  = new int[columnMap.length];
 
@@ -646,11 +689,10 @@ public class ParserDML extends ParserDQL {
         return cs;
     }
 
-    void resolveUpdateExpressions(Table targetTable,
-                                  RangeVariable[] rangeVariables,
+    void resolveUpdateExpressions(Table targetTable, RangeGroup rangeGroup,
                                   int[] columnMap,
                                   Expression[] colExpressions,
-                                  RangeVariable[] outerRanges) {
+                                  RangeGroup[] rangeGroups) {
 
         HsqlList unresolved           = null;
         int      enforcedDefaultIndex = -1;
@@ -691,12 +733,8 @@ public class ParserDML extends ParserDQL {
                             throw Error.error(ErrorCode.X_42544);
                         }
                     } else {
-                        unresolved = expr.resolveColumnReferences(outerRanges,
-                                null);
-                        unresolved =
-                            Expression.resolveColumnSet(rangeVariables,
-                                                        rangeVariables.length,
-                                                        unresolved, null);
+                        unresolved = expr.resolveColumnReferences(session,
+                                rangeGroup, rangeGroups, null);
 
                         ExpressionColumn.checkColumnsResolved(unresolved);
 
@@ -706,14 +744,13 @@ public class ParserDML extends ParserDQL {
                     }
                 }
             } else if (expr.getType() == OpTypes.ROW_SUBQUERY) {
-                unresolved = expr.resolveColumnReferences(outerRanges, null);
-                unresolved = Expression.resolveColumnSet(rangeVariables,
-                        rangeVariables.length, unresolved, null);
+                unresolved = expr.resolveColumnReferences(session, rangeGroup,
+                        rangeGroups, null);
 
                 ExpressionColumn.checkColumnsResolved(unresolved);
                 expr.resolveTypes(session, null);
 
-                int count = expr.subQuery.queryExpression.getColumnCount();
+                int count = expr.table.queryExpression.getColumnCount();
 
                 for (int j = 0; j < count; j++, i++) {
                     if (enforcedDefaultIndex == columnMap[i]) {
@@ -738,10 +775,8 @@ public class ParserDML extends ParserDQL {
                         throw Error.error(ErrorCode.X_42544);
                     }
                 } else {
-                    unresolved = expr.resolveColumnReferences(outerRanges,
-                            null);
-                    unresolved = Expression.resolveColumnSet(rangeVariables,
-                            rangeVariables.length, unresolved, null);
+                    unresolved = expr.resolveColumnReferences(session,
+                            rangeGroup, rangeGroups, null);
 
                     ExpressionColumn.checkColumnsResolved(unresolved);
                     e.resolveTypes(session, null);
@@ -789,13 +824,13 @@ public class ParserDML extends ParserDQL {
             if (token.tokenType == Tokens.SELECT) {
                 rewind(position);
 
-                SubQuery sq = XreadSubqueryBody(false, OpTypes.ROW_SUBQUERY);
+                TableDerived td = XreadSubqueryTableBody(OpTypes.ROW_SUBQUERY);
 
-                if (degree != sq.queryExpression.getColumnCount()) {
+                if (degree != td.queryExpression.getColumnCount()) {
                     throw Error.error(ErrorCode.X_42546);
                 }
 
-                Expression e = new Expression(OpTypes.ROW_SUBQUERY, sq);
+                Expression e = new Expression(OpTypes.ROW_SUBQUERY, td);
 
                 expressions.add(e);
 
@@ -843,10 +878,51 @@ public class ParserDML extends ParserDQL {
         }
     }
 
+    void readGetClauseList(RangeVariable[] rangeVars, OrderedHashSet targets,
+                           LongDeque colIndexList, HsqlArrayList expressions) {
+
+        while (true) {
+            Expression target = XreadTargetSpecification(rangeVars,
+                colIndexList);
+
+            if (!targets.add(target)) {
+                ColumnSchema col = target.getColumn();
+
+                throw Error.error(ErrorCode.X_42579, col.getName().name);
+            }
+
+            readThis(Tokens.EQUALS);
+
+            switch (token.tokenType) {
+
+                case Tokens.ROW_COUNT :
+                case Tokens.MORE :
+                    int columnIndex =
+                        ExpressionColumn.diagnosticsList.getIndex(
+                            token.tokenString);
+                    Expression e =
+                        new ExpressionColumn(OpTypes.DIAGNOSTICS_VARIABLE,
+                                             columnIndex);
+
+                    expressions.add(e);
+                    read();
+                    break;
+            }
+
+            if (token.tokenType == Tokens.COMMA) {
+                read();
+
+                continue;
+            }
+
+            break;
+        }
+    }
+
     /**
      * Retrieves a MERGE Statement from this parse context.
      */
-    StatementDMQL compileMergeStatement(RangeVariable[] outerRanges) {
+    StatementDMQL compileMergeStatement(RangeGroup[] rangeGroups) {
 
         boolean[]     insertColumnCheckList;
         int[]         insertColumnMap = null;
@@ -865,27 +941,34 @@ public class ParserDML extends ParserDQL {
         read();
         readThis(Tokens.INTO);
 
-        targetRange = readSimpleRangeVariable(StatementTypes.MERGE);
+        targetRange = readRangeVariableForDataChange(StatementTypes.MERGE);
         table       = targetRange.rangeTable;
 
         readThis(Tokens.USING);
+        compileContext.setOuterRanges(rangeGroups);
 
         sourceRange = readTableOrSubquery();
 
+        RangeVariable[] targetRanges = new RangeVariable[]{ targetRange };
+
+        sourceRange.resolveRangeTable(session,
+                                      new RangeGroupSimple(targetRanges),
+                                      rangeGroups);;
+        sourceRange.resolveRangeTableTypes(session, targetRanges);
+        compileContext.setOuterRanges(RangeGroup.emptyArray);
+
         // parse ON search conditions
         readThis(Tokens.ON);
 
         mergeCondition = XreadBooleanValueExpression();
 
-        if (mergeCondition.getDataType() != Type.SQL_BOOLEAN) {
-            throw Error.error(ErrorCode.X_42568);
-        }
-
         RangeVariable[] fullRangeVars   = new RangeVariable[] {
             sourceRange, targetRange
         };
         RangeVariable[] sourceRangeVars = new RangeVariable[]{ sourceRange };
         RangeVariable[] targetRangeVars = new RangeVariable[]{ targetRange };
+        RangeGroup      fullRangeGroup  = new RangeGroupSimple(fullRangeVars);
+        RangeGroup sourceRangeGroup = new RangeGroupSimple(sourceRangeVars);
 
         // parse WHEN clause(s) and convert lists to arrays
         insertColumnMap       = table.getColumnMap();
@@ -918,7 +1001,7 @@ public class ParserDML extends ParserDQL {
             updateTargetSet.toArray(targets);
 
             for (int i = 0; i < targets.length; i++) {
-                this.resolveOuterReferencesAndTypes(outerRanges, targets[i]);
+                this.resolveOuterReferencesAndTypes(rangeGroups, targets[i]);
             }
 
             updateExpressions = new Expression[updateList.size()];
@@ -931,7 +1014,8 @@ public class ParserDML extends ParserDQL {
         }
 
         if (updateExpressions.length != 0) {
-            Table baseTable = table.getBaseTable();
+            Table baseTable = table.isTriggerUpdatable() ? table
+                                                         : table.getBaseTable();
 
             baseUpdateColumnMap = updateColumnMap;
 
@@ -942,14 +1026,14 @@ public class ParserDML extends ParserDQL {
                                      updateColumnMap, baseUpdateColumnMap);
             }
 
-            resolveUpdateExpressions(table, sourceRangeVars, updateColumnMap,
-                                     updateExpressions, outerRanges);
+            resolveUpdateExpressions(table, fullRangeGroup, updateColumnMap,
+                                     updateExpressions, rangeGroups);
         }
 
         HsqlList unresolved = null;
 
-        unresolved = mergeCondition.resolveColumnReferences(fullRangeVars,
-                null);
+        unresolved = mergeCondition.resolveColumnReferences(session,
+                fullRangeGroup, rangeGroups, null);
 
         ExpressionColumn.checkColumnsResolved(unresolved);
         mergeCondition.resolveTypes(session, null);
@@ -962,18 +1046,25 @@ public class ParserDML extends ParserDQL {
             throw Error.error(ErrorCode.X_42568);
         }
 
+        fullRangeVars[1].addJoinCondition(mergeCondition);
+
         RangeVariableResolver resolver =
-            new RangeVariableResolver(fullRangeVars, mergeCondition,
-                                      compileContext);
+            new RangeVariableResolver(fullRangeVars, null, compileContext);
 
         resolver.processConditions(session);
 
         fullRangeVars = resolver.rangeVariables;
 
+        for (int i = 0; i < fullRangeVars.length; i++) {
+            fullRangeVars[i].resolveRangeTableTypes(session,
+                    RangeVariable.emptyArray);
+        }
+
         if (insertExpression != null) {
-            unresolved =
-                insertExpression.resolveColumnReferences(sourceRangeVars,
-                    unresolved);
+            unresolved = insertExpression.resolveColumnReferences(session,
+                    sourceRangeGroup, RangeGroup.emptyArray, null);
+            unresolved = Expression.resolveColumnSet(session,
+                    RangeVariable.emptyArray, rangeGroups, unresolved);
 
             ExpressionColumn.checkColumnsResolved(unresolved);
             insertExpression.resolveTypes(session, null);
@@ -1036,7 +1127,10 @@ public class ParserDML extends ParserDQL {
             int brackets = readOpenBrackets();
 
             if (brackets == 1) {
-                readSimpleColumnNames(insertColumnNames, targetRangeVars[0]);
+                boolean withPrefix = database.sqlSyntaxOra;
+
+                readSimpleColumnNames(insertColumnNames, targetRangeVars[0],
+                                      withPrefix);
 
                 columnCount = insertColumnNames.size();
 
@@ -1070,7 +1164,7 @@ public class ParserDML extends ParserDQL {
      */
 
     // to do call argument name and type resolution
-    StatementDMQL compileCallStatement(RangeVariable[] outerRanges,
+    StatementDMQL compileCallStatement(RangeGroup[] rangeGroups,
                                        boolean isStrictlyProcedure) {
 
         read();
@@ -1132,7 +1226,8 @@ public class ParserDML extends ParserDQL {
                             routine.getParameter(i).getParameterMode();
 
                         unresolved =
-                            arguments[i].resolveColumnReferences(outerRanges,
+                            arguments[i].resolveColumnReferences(session,
+                                RangeGroup.emptyGroup, rangeGroups,
                                 unresolved);
 
                         if (paramMode
@@ -1168,27 +1263,23 @@ public class ParserDML extends ParserDQL {
         }
 
         Expression expression = this.XreadValueExpression();
-        HsqlList unresolved = expression.resolveColumnReferences(outerRanges,
-            null);
+        HsqlList unresolved = expression.resolveColumnReferences(session,
+            RangeGroup.emptyGroup, rangeGroups, null);
 
         ExpressionColumn.checkColumnsResolved(unresolved);
         expression.resolveTypes(session, null);
 
-//        expression.paramMode = PARAM_OUT;
         StatementDMQL cs = new StatementProcedure(session, expression,
             compileContext);
 
         return cs;
     }
 
-    void resolveOuterReferencesAndTypes(RangeVariable[] rangeVars,
+    void resolveOuterReferencesAndTypes(RangeGroup[] rangeGroups,
                                         Expression e) {
 
-        HsqlList unresolved = e.resolveColumnReferences(rangeVars,
-            rangeVars.length, null, false);
-
-        unresolved = Expression.resolveColumnSet(rangeVars, rangeVars.length,
-                unresolved, null);
+        HsqlList unresolved = e.resolveColumnReferences(session,
+            RangeGroup.emptyGroup, 0, rangeGroups, null, false);
 
         ExpressionColumn.checkColumnsResolved(unresolved);
         e.resolveTypes(session, null);
