@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,6 +43,7 @@ import org.hsqldb.lib.HsqlList;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.lib.OrderedIntHashSet;
 import org.hsqldb.lib.Set;
+import org.hsqldb.navigator.RowIterator;
 import org.hsqldb.navigator.RowSetNavigatorData;
 import org.hsqldb.navigator.RowSetNavigatorDataTable;
 import org.hsqldb.result.Result;
@@ -55,7 +56,7 @@ import org.hsqldb.types.Types;
  * Implementation of an SQL query expression
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 
@@ -65,7 +66,7 @@ import org.hsqldb.types.Types;
  * - Result metadata for the final result of QueryExpression
  *
  */
-public class QueryExpression {
+public class QueryExpression implements RangeGroup {
 
     public static final int NOUNION       = 0,
                             UNION         = 1,
@@ -92,6 +93,9 @@ public class QueryExpression {
     HsqlList unresolvedExpressions;
 
     //
+    boolean isReferencesResolved;
+    boolean isPartOneResolved;
+    boolean isPartTwoResolved;
     boolean isResolved;
 
     //
@@ -103,12 +107,21 @@ public class QueryExpression {
 
     //
     View    view;
+    boolean isBaseMergeable;
     boolean isMergeable;
     boolean isUpdatable;
     boolean isInsertable;
     boolean isCheckable;
     boolean isTopLevel;
+    boolean isRecursive;
+    boolean isSingleRow;
     boolean acceptsSequences;
+    boolean isCorrelated;
+    boolean isTable;
+    boolean isValueList;
+
+    //
+    TableDerived recursiveTable;
 
     //
     public TableBase resultTable;
@@ -120,6 +133,7 @@ public class QueryExpression {
     //
     CompileContext compileContext;
 
+    //
     QueryExpression(CompileContext compileContext) {
         this.compileContext = compileContext;
         sortAndSlice        = SortAndSlice.noSort;
@@ -134,6 +148,22 @@ public class QueryExpression {
         this.leftQueryExpression = leftQueryExpression;
     }
 
+    public RangeVariable[] getRangeVariables() {
+        return RangeVariable.emptyArray;
+    }
+
+    public void setCorrelated() {
+        isCorrelated = true;
+    }
+
+    public void setSingleRow() {
+        isSingleRow = true;
+    }
+
+    public boolean isRecursive() {
+        return isRecursive;
+    }
+
     void addUnion(QueryExpression queryExpression, int unionType) {
 
         sortAndSlice              = SortAndSlice.noSort;
@@ -160,34 +190,32 @@ public class QueryExpression {
 
         isFullOrder = true;
 
-        if (leftQueryExpression == null) {
-            if (isResolved) {
-                ((QuerySpecification) this).createFullIndex(null);
-            }
-
-            return;
+        if (leftQueryExpression != null) {
+            leftQueryExpression.setFullOrder();
         }
 
-        leftQueryExpression.setFullOrder();
-        rightQueryExpression.setFullOrder();
+        if (rightQueryExpression != null) {
+            rightQueryExpression.setFullOrder();
+        }
     }
 
     public void resolve(Session session) {
 
-        resolveReferences(session);
+        resolveReferences(session, RangeGroup.emptyArray);
         ExpressionColumn.checkColumnsResolved(unresolvedExpressions);
         resolveTypes(session);
     }
 
-    public void resolve(Session session, RangeVariable[] outerRanges,
+    public void resolve(Session session, RangeGroup[] rangeGroups,
                         Type[] targetTypes) {
 
-        resolveReferences(session);
+        resolveReferences(session, rangeGroups);
 
         if (unresolvedExpressions != null) {
             for (int i = 0; i < unresolvedExpressions.size(); i++) {
-                Expression e    = (Expression) unresolvedExpressions.get(i);
-                HsqlList   list = e.resolveColumnReferences(outerRanges, null);
+                Expression e = (Expression) unresolvedExpressions.get(i);
+                HsqlList list = e.resolveColumnReferences(session,
+                    RangeGroup.emptyGroup, rangeGroups, null);
 
                 ExpressionColumn.checkColumnsResolved(list);
             }
@@ -195,23 +223,36 @@ public class QueryExpression {
 
         resolveTypesPartOne(session);
 
-        for (int i = 0; i < unionColumnTypes.length && i < targetTypes.length;
-                i++) {
-            if (unionColumnTypes[i] == null) {
-                unionColumnTypes[i] = targetTypes[i];
+        if (targetTypes != null) {
+            for (int i = 0;
+                    i < unionColumnTypes.length && i < targetTypes.length;
+                    i++) {
+                if (unionColumnTypes[i] == null) {
+                    unionColumnTypes[i] = targetTypes[i];
+                }
             }
         }
 
         resolveTypesPartTwo(session);
+        resolveTypesPartThree(session);
     }
 
-    public void resolveReferences(Session session) {
+    public void resolveReferences(Session session, RangeGroup[] rangeGroups) {
 
-        leftQueryExpression.resolveReferences(session);
-        rightQueryExpression.resolveReferences(session);
+        if (isReferencesResolved) {
+            return;
+        }
+
+        leftQueryExpression.resolveReferences(session, rangeGroups);
+        rightQueryExpression.resolveReferences(session, rangeGroups);
         addUnresolvedExpressions(leftQueryExpression.unresolvedExpressions);
         addUnresolvedExpressions(rightQueryExpression.unresolvedExpressions);
 
+        if (leftQueryExpression.isCorrelated
+                || rightQueryExpression.isCorrelated) {
+            setCorrelated();
+        }
+
         if (!unionCorresponding) {
             columnCount = leftQueryExpression.getColumnCount();
 
@@ -228,6 +269,9 @@ public class QueryExpression {
             ArrayUtil.fillSequence(leftQueryExpression.unionColumnMap);
             resolveColumnRefernecesInUnionOrderBy();
 
+            accessibleColumns    = leftQueryExpression.accessibleColumns;
+            isReferencesResolved = true;
+
             return;
         }
 
@@ -302,6 +346,12 @@ public class QueryExpression {
         unionColumnTypes = new Type[columnCount];
 
         resolveColumnRefernecesInUnionOrderBy();
+
+        accessibleColumns = new boolean[columnCount];
+
+        ArrayUtil.fillArray(accessibleColumns, true);
+
+        isReferencesResolved = true;
     }
 
     /**
@@ -369,12 +419,15 @@ public class QueryExpression {
 
         resolveTypesPartOne(session);
         resolveTypesPartTwo(session);
-
-        isResolved = true;
+        resolveTypesPartThree(session);
     }
 
     void resolveTypesPartOne(Session session) {
 
+        if (isPartOneResolved) {
+            return;
+        }
+
         ArrayUtil.projectRowReverse(leftQueryExpression.unionColumnTypes,
                                     leftQueryExpression.unionColumnMap,
                                     unionColumnTypes);
@@ -389,21 +442,67 @@ public class QueryExpression {
         ArrayUtil.projectRow(rightQueryExpression.unionColumnTypes,
                              rightQueryExpression.unionColumnMap,
                              unionColumnTypes);
+
+        isPartOneResolved = true;
     }
 
     void resolveTypesPartTwo(Session session) {
 
+        if (isPartTwoResolved) {
+            return;
+        }
+
         ArrayUtil.projectRowReverse(leftQueryExpression.unionColumnTypes,
                                     leftQueryExpression.unionColumnMap,
                                     unionColumnTypes);
         leftQueryExpression.resolveTypesPartTwo(session);
+        leftQueryExpression.resolveTypesPartThree(session);
         ArrayUtil.projectRowReverse(rightQueryExpression.unionColumnTypes,
                                     rightQueryExpression.unionColumnMap,
                                     unionColumnTypes);
         rightQueryExpression.resolveTypesPartTwo(session);
+        rightQueryExpression.resolveTypesPartThree(session);
 
         //
-        if (unionCorresponding) {
+        ResultMetaData leftMeta  = leftQueryExpression.getMetaData();
+        ResultMetaData rightMeta = rightQueryExpression.getMetaData();
+
+        for (int i = 0; i < leftQueryExpression.unionColumnMap.length; i++) {
+            int        leftIndex  = leftQueryExpression.unionColumnMap[i];
+            int        rightIndex = rightQueryExpression.unionColumnMap[i];
+            ColumnBase column     = leftMeta.columns[leftIndex];
+            byte leftNullability =
+                leftMeta.columns[leftIndex].getNullability();
+            byte rightNullability =
+                rightMeta.columns[rightIndex].getNullability();
+
+            if (column instanceof ColumnSchema
+                    && rightMeta.columns[rightIndex] instanceof ColumnBase) {
+                column = new ColumnBase();
+
+                column.setType(leftQueryExpression.unionColumnTypes[i]);
+                column.setNullability(leftNullability);
+
+                leftMeta.columns[leftIndex] = column;
+            }
+
+            if (rightNullability == SchemaObject.Nullability
+                    .NULLABLE || (rightNullability == SchemaObject.Nullability
+                        .NULLABLE_UNKNOWN && leftNullability == SchemaObject
+                        .Nullability.NO_NULLS)) {
+                if (column instanceof ColumnSchema) {
+                    column = new ColumnBase();
+
+                    column.setType(leftQueryExpression.unionColumnTypes[i]);
+
+                    leftMeta.columns[leftIndex] = column;
+                }
+
+                column.setNullability(rightNullability);
+            }
+        }
+
+        if (unionCorresponding || isRecursive) {
             resultMetaData = leftQueryExpression.getMetaData().getNewMetaData(
                 leftQueryExpression.unionColumnMap);
 
@@ -426,17 +525,12 @@ public class QueryExpression {
             }
         }
 
-        ResultMetaData meta = getMetaData();
-/*
-        // disallow lobs
-        for (int i = 0, count = meta.getColumnCount(); i < count; i++) {
-            Type dataType = meta.columnTypes[i];
+        isPartTwoResolved = true;
+    }
 
-            if (dataType.isLobType()) {
-                throw Error.error(ErrorCode.X_42534);
-            }
-        }
-*/
+    void resolveTypesPartThree(Session session) {
+        compileContext = null;
+        isResolved     = true;
     }
 
     public Object[] getValues(Session session) {
@@ -453,6 +547,8 @@ public class QueryExpression {
         }
     }
 
+    public void addExtraConditions(Expression e) {}
+
     public Object[] getSingleRowValues(Session session) {
 
         Result r    = getResult(session, 2);
@@ -476,6 +572,10 @@ public class QueryExpression {
 
     Result getResult(Session session, int maxRows) {
 
+        if (isRecursive) {
+            return getResultRecursive(session);
+        }
+
         int    currentMaxRows = unionType == UNION_ALL ? maxRows
                                                        : 0;
         Result first = leftQueryExpression.getResult(session, currentMaxRows);
@@ -500,6 +600,7 @@ public class QueryExpression {
             }
 
             rowSet.copy(navigator, leftQueryExpression.unionColumnMap);
+            navigator.release();
 
             navigator = rowSet;
 
@@ -514,6 +615,7 @@ public class QueryExpression {
             }
 
             rowSet.copy(rightNavigator, rightQueryExpression.unionColumnMap);
+            rightNavigator.release();
 
             rightNavigator = rowSet;
         }
@@ -521,27 +623,27 @@ public class QueryExpression {
         switch (unionType) {
 
             case UNION :
-                navigator.union(rightNavigator);
+                navigator.union(session, rightNavigator);
                 break;
 
             case UNION_ALL :
-                navigator.unionAll(rightNavigator);
+                navigator.unionAll(session, rightNavigator);
                 break;
 
             case INTERSECT :
-                navigator.intersect(rightNavigator);
+                navigator.intersect(session, rightNavigator);
                 break;
 
             case INTERSECT_ALL :
-                navigator.intersectAll(rightNavigator);
+                navigator.intersectAll(session, rightNavigator);
                 break;
 
             case EXCEPT :
-                navigator.except(rightNavigator);
+                navigator.except(session, rightNavigator);
                 break;
 
             case EXCEPT_ALL :
-                navigator.exceptAll(rightNavigator);
+                navigator.exceptAll(session, rightNavigator);
                 break;
 
             default :
@@ -549,12 +651,13 @@ public class QueryExpression {
         }
 
         if (sortAndSlice.hasOrder()) {
-            navigator.sortUnion(sortAndSlice);
+            navigator.sortOrderUnion(session, sortAndSlice);
         }
 
         if (sortAndSlice.hasLimit()) {
-            navigator.trim(sortAndSlice.getLimitStart(session),
-                           sortAndSlice.getLimitCount(session, maxRows));
+            int[] limits = sortAndSlice.getLimits(session, this, maxRows);
+
+            navigator.trim(limits[0], limits[1]);
         }
 
         navigator.reset();
@@ -562,12 +665,69 @@ public class QueryExpression {
         return first;
     }
 
+    Result getResultRecursive(Session session) {
+
+        Result              tempResult;
+        RowSetNavigatorData rowSet = new RowSetNavigatorData(session, this);
+        Result              result = Result.newResult(rowSet);
+
+        recursiveTable.materialise(session);
+
+        RowIterator it = recursiveTable.rowIterator(session);
+
+        rowSet.copy(it, unionColumnMap);
+
+        result.metaData = resultMetaData;
+
+        for (int round = 0; ; round++) {
+            tempResult = rightQueryExpression.getResult(session, 0);
+
+            RowSetNavigatorData tempNavigator =
+                (RowSetNavigatorData) tempResult.getNavigator();
+
+            if (tempNavigator.isEmpty()) {
+                break;
+            }
+
+            switch (unionType) {
+
+                case UNION :
+                    rowSet.union(session, tempNavigator);
+                    break;
+
+                case UNION_ALL :
+                    rowSet.unionAll(session, tempNavigator);
+                    break;
+
+                default :
+                    throw Error.runtimeError(ErrorCode.U_S0500,
+                                             "QueryExpression");
+            }
+
+            recursiveTable.clearAllData(session);
+            tempNavigator.reset();
+            recursiveTable.insertIntoTable(session, tempResult);
+
+            if (round > 256) {
+                throw Error.error(ErrorCode.GENERAL_ERROR);
+            }
+        }
+
+        recursiveTable.clearAllData(session);
+        rowSet.reset();
+
+        return result;
+    }
+
     public OrderedHashSet getSubqueries() {
 
         OrderedHashSet subqueries = leftQueryExpression.getSubqueries();
 
-        return OrderedHashSet.addAll(subqueries,
-                                     rightQueryExpression.getSubqueries());
+        subqueries =
+            OrderedHashSet.addAll(subqueries,
+                                  rightQueryExpression.getSubqueries());
+
+        return subqueries;
     }
 
     public boolean isSingleColumn() {
@@ -597,7 +757,11 @@ public class QueryExpression {
 
         StringBuffer sb;
         String       temp;
-        String       b = ValuePool.spaceString.substring(0, blanks);
+        StringBuffer b = new StringBuffer(blanks);
+
+        for (int i = 0; i < blanks; i++) {
+            b.append(' ');
+        }
 
         sb = new StringBuffer();
 
@@ -648,8 +812,24 @@ public class QueryExpression {
     }
 
     public boolean areColumnsResolved() {
-        return unresolvedExpressions == null
-               || unresolvedExpressions.isEmpty();
+
+        if (unresolvedExpressions == null || unresolvedExpressions.isEmpty()) {
+            return true;
+        }
+
+        for (int i = 0; i < unresolvedExpressions.size(); i++) {
+            Expression e = (Expression) unresolvedExpressions.get(i);
+
+            if (e.getRangeVariable() == null) {
+                return false;
+            }
+
+            if (e.getRangeVariable().rangeType == RangeVariable.TABLE_RANGE) {
+                return false;
+            }
+        }
+
+        return true;
     }
 
     String[] getColumnNames() {
@@ -699,6 +879,18 @@ public class QueryExpression {
         return set;
     }
 
+    OrderedHashSet collectRangeVariables(RangeVariable[] rangeVars,
+                                         OrderedHashSet set) {
+
+        set = leftQueryExpression.collectRangeVariables(rangeVars, set);
+
+        if (rightQueryExpression != null) {
+            set = rightQueryExpression.collectRangeVariables(rangeVars, set);
+        }
+
+        return set;
+    }
+
     public void collectObjectNames(Set set) {
 
         leftQueryExpression.collectObjectNames(set);
@@ -720,8 +912,8 @@ public class QueryExpression {
      */
     public void setView(View view) {
 
-        this.isUpdatable      = true;
         this.view             = view;
+        this.isUpdatable      = true;
         this.acceptsSequences = true;
         this.isTopLevel       = true;
     }
@@ -747,17 +939,15 @@ public class QueryExpression {
         mainIndex = resultTable.getPrimaryIndex();
 
         if (sortAndSlice.hasOrder()) {
-            orderIndex = resultTable.createAndAddIndexStructure(session, null,
-                    sortAndSlice.sortOrder, sortAndSlice.sortDescending,
-                    sortAndSlice.sortNullsLast, false, false, false);
+            orderIndex = sortAndSlice.getNewIndex(session, resultTable);
         }
 
         int[] fullCols = new int[columnCount];
 
         ArrayUtil.fillSequence(fullCols);
 
-        fullIndex = resultTable.createAndAddIndexStructure(session, null,
-                fullCols, null, null, false, false, false);
+        fullIndex = resultTable.createAndAddIndexStructure(null, fullCols,
+                null, null, false, false, false);
         resultTable.fullIndex = fullIndex;
     }
 
@@ -776,7 +966,8 @@ public class QueryExpression {
         try {
             resultTable = new TableDerived(session.database, tableName,
                                            tableType, unionColumnTypes,
-                                           columnList, null, null);
+                                           columnList,
+                                           ValuePool.emptyIntArray);
         } catch (Exception e) {}
     }
 
@@ -823,7 +1014,8 @@ public class QueryExpression {
             HashMappedList list    = new HashMappedList();
 
             for (int i = 0; i < unionColumnMap.length; i++) {
-                ColumnSchema column = (ColumnSchema) columns.get(i);
+                ColumnSchema column =
+                    (ColumnSchema) columns.get(unionColumnMap[i]);
 
                 list.add(column.getName().name, column);
             }
@@ -912,10 +1104,10 @@ public class QueryExpression {
                        other.rightQueryExpression));
     }
 
-    public void replaceColumnReference(RangeVariable range,
-                                       Expression[] list) {
-        leftQueryExpression.replaceColumnReference(range, list);
-        rightQueryExpression.replaceColumnReference(range, list);
+    public void replaceColumnReferences(RangeVariable range,
+                                        Expression[] list) {
+        leftQueryExpression.replaceColumnReferences(range, list);
+        rightQueryExpression.replaceColumnReferences(range, list);
     }
 
     public void replaceRangeVariables(RangeVariable[] ranges,
