@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,7 @@ package org.hsqldb;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
+import java.sql.ResultSet;
 
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.HsqlNameManager.SimpleName;
@@ -43,9 +44,11 @@ import org.hsqldb.jdbc.JDBCResultSet;
 import org.hsqldb.lib.HashMappedList;
 import org.hsqldb.lib.HsqlArrayList;
 import org.hsqldb.lib.OrderedHashSet;
+import org.hsqldb.persist.HsqlDatabaseProperties;
 import org.hsqldb.result.Result;
 import org.hsqldb.rights.Grantee;
 import org.hsqldb.store.BitMap;
+import org.hsqldb.store.ValuePool;
 import org.hsqldb.types.RowType;
 import org.hsqldb.types.Type;
 import org.hsqldb.types.Types;
@@ -55,26 +58,26 @@ import org.hsqldb.types.Types;
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  *
- * @version 1.9.0
+ * @version 2.0.1
  * @since 1.9.0
  */
-public class Routine implements SchemaObject {
+public class Routine implements SchemaObject, RangeGroup, Cloneable {
 
-    public final static int NO_SQL       = 1;
-    public final static int CONTAINS_SQL = 2;
-    public final static int READS_SQL    = 3;
-    public final static int MODIFIES_SQL = 4;
+    public static final int NO_SQL       = 1;
+    public static final int CONTAINS_SQL = 2;
+    public static final int READS_SQL    = 3;
+    public static final int MODIFIES_SQL = 4;
 
     //
-    public final static int LANGUAGE_JAVA = 1;
-    public final static int LANGUAGE_SQL  = 2;
+    public static final int LANGUAGE_JAVA = 1;
+    public static final int LANGUAGE_SQL  = 2;
 
     //
     public static final int PARAM_STYLE_JAVA = 1;
     public static final int PARAM_STYLE_SQL  = 2;
 
     //
-    final static Routine[] emptyArray = new Routine[]{};
+    static final Routine[] emptyArray = new Routine[]{};
 
     //
     RoutineSchema    routineSchema;
@@ -92,7 +95,8 @@ public class Routine implements SchemaObject {
     boolean          isDeterministic;
     boolean          isNullInputOutput;
     boolean          isNewSavepointLevel = true;
-    boolean          isPSM;
+    int              maxDynamicResults   = 0;
+    boolean          isRecursive;
     boolean          returnsTable;
     Statement        statement;
 
@@ -107,8 +111,7 @@ public class Routine implements SchemaObject {
 
     //
     HashMappedList  parameterList = new HashMappedList();
-    int             scopeVariableCount;
-    RangeVariable[] ranges;
+    RangeVariable[] ranges        = RangeVariable.emptyArray;
 
     //
     int variableCount;
@@ -126,7 +129,8 @@ public class Routine implements SchemaObject {
         routineType = type;
         returnType  = Type.SQL_ALL_TYPES;
         ranges = new RangeVariable[]{
-            new RangeVariable(parameterList, false) };
+            new RangeVariable(parameterList, null, false,
+                              RangeVariable.PARAMETER_RANGE) };
     }
 
     public Routine(Table table, RangeVariable[] ranges, int impact,
@@ -150,6 +154,11 @@ public class Routine implements SchemaObject {
     }
 
     public HsqlName getSchemaName() {
+
+        if (routineType == SchemaObject.TRIGGER) {
+            return triggerTable.getSchemaName();
+        }
+
         return name.schema;
     }
 
@@ -188,6 +197,27 @@ public class Routine implements SchemaObject {
     }
 
     public String getSQL() {
+        return getDefinitionSQL(true);
+    }
+
+    public String getSQLAlter() {
+
+        StringBuffer sb = new StringBuffer();
+
+        sb.append(Tokens.T_ALTER).append(' ').append(Tokens.T_SPECIFIC);
+        sb.append(' ').append(Tokens.T_ROUTINE).append(' ');
+        sb.append(specificName.getSchemaQualifiedStatementName());
+        sb.append(' ').append(Tokens.T_BODY);
+        sb.append(' ').append(statement.getSQL());
+
+        return sb.toString();
+    }
+
+    public String getSQLDeclaration() {
+        return getDefinitionSQL(false);
+    }
+
+    private String getDefinitionSQL(boolean withBody) {
 
         StringBuffer sb = new StringBuffer();
 
@@ -288,8 +318,34 @@ public class Routine implements SchemaObject {
 
             sb.append(' ').append(Tokens.T_SAVEPOINT).append(' ');
             sb.append(Tokens.T_LEVEL).append(' ');
+
+            if (maxDynamicResults != 0) {
+                sb.append(' ').append(Tokens.T_DYNAMIC).append(' ');
+                sb.append(Tokens.T_RESULT).append(' ').append(Tokens.T_SETS);
+                sb.append(' ').append(maxDynamicResults).append(' ');
+            }
         }
 
+        if (language == LANGUAGE_JAVA) {
+            sb.append(Tokens.T_EXTERNAL).append(' ').append(Tokens.T_NAME);
+            sb.append(' ').append('\'').append(methodName).append('\'');
+        } else {
+            if (withBody) {
+                sb.append(statement.getSQL());
+            } else {
+                sb.append(Tokens.T_SIGNAL).append(' ');
+                sb.append(Tokens.T_SQLSTATE).append(' ');
+                sb.append('\'').append("45000").append('\'');
+            }
+        }
+
+        return sb.toString();
+    }
+
+    public String getSQLBodyDefinition() {
+
+        StringBuffer sb = new StringBuffer();
+
         if (language == LANGUAGE_JAVA) {
             sb.append(Tokens.T_EXTERNAL).append(' ').append(Tokens.T_NAME);
             sb.append(' ').append('\'').append(methodName).append('\'');
@@ -300,6 +356,15 @@ public class Routine implements SchemaObject {
         return sb.toString();
     }
 
+    public String getExternalName() {
+
+        if (language == LANGUAGE_JAVA) {
+            return methodName;
+        } else {
+            return null;
+        }
+    }
+
     public long getChangeTimestamp() {
         return 0;
     }
@@ -317,7 +382,6 @@ public class Routine implements SchemaObject {
 
     public void setLanguage(int lang) {
         language = lang;
-        isPSM    = language == LANGUAGE_SQL;
     }
 
     public int getLanguage() {
@@ -325,7 +389,7 @@ public class Routine implements SchemaObject {
     }
 
     boolean isPSM() {
-        return isPSM;
+        return language == LANGUAGE_SQL;
     }
 
     public void setDataImpact(int impact) {
@@ -396,6 +460,10 @@ public class Routine implements SchemaObject {
         specificName = name;
     }
 
+    public int getMaxDynamicResults() {
+        return maxDynamicResults;
+    }
+
     public void setName(HsqlName name) {
         this.name = name;
     }
@@ -424,6 +492,10 @@ public class Routine implements SchemaObject {
         isNewSavepointLevel = value;
     }
 
+    public void setMaxDynamicResults(int value) {
+        maxDynamicResults = value;
+    }
+
     public void setParameterStyle(int style) {
         parameterStyle = style;
     }
@@ -469,17 +541,17 @@ public class Routine implements SchemaObject {
 
         if (language == Routine.LANGUAGE_SQL) {
             if (dataImpact == NO_SQL) {
-                throw Error.error(ErrorCode.X_42604);
+                throw Error.error(ErrorCode.X_42604, "CONTAINS SQL");
             }
 
             if (parameterStyle == PARAM_STYLE_JAVA) {
-                throw Error.error(ErrorCode.X_42604);
+                throw Error.error(ErrorCode.X_42604, "PARAMETER STYLE");
             }
         }
 
         if (language == Routine.LANGUAGE_SQL) {
             if (parameterStyle != 0 && parameterStyle != PARAM_STYLE_SQL) {
-                throw Error.error(ErrorCode.X_42604);
+                throw Error.error(ErrorCode.X_42604, "PARAMETER STYLE");
             }
         }
 
@@ -498,33 +570,6 @@ public class Routine implements SchemaObject {
             }
         }
 
-        if (statement != null) {
-            statement.resolve(session);
-
-            if (dataImpact == CONTAINS_SQL) {
-                checkNoSQLData(session.database, statement.getReferences());
-            }
-        }
-
-        if (methodName != null && javaMethod == null) {
-            boolean[] hasConnection = new boolean[1];
-
-            javaMethod = getMethod(methodName, this, hasConnection,
-                                   returnsTable);
-
-            if (javaMethod == null) {
-                throw Error.error(ErrorCode.X_46103);
-            }
-
-            javaMethodWithConnection = hasConnection[0];
-
-            String className = javaMethod.getDeclaringClass().getName();
-
-            if (className.equals("java.lang.Math")) {
-                isLibraryRoutine = true;
-            }
-        }
-
         if (isAggregate) {
             if (parameterTypes.length != 4) {
                 throw Error.error(ErrorCode.X_42610);
@@ -552,6 +597,35 @@ public class Routine implements SchemaObject {
             }
         }
 
+        resolveReferences(session);
+    }
+
+    void resolveReferences(Session session) {
+
+        if (statement != null) {
+            statement.resolve(session);
+            checkSQLData(session);
+        }
+
+        if (methodName != null && javaMethod == null) {
+            boolean[] hasConnection = new boolean[1];
+
+            javaMethod = getMethod(methodName, this, hasConnection,
+                                   returnsTable);
+
+            if (javaMethod == null) {
+                throw Error.error(ErrorCode.X_46103);
+            }
+
+            javaMethodWithConnection = hasConnection[0];
+
+            String className = javaMethod.getDeclaringClass().getName();
+
+            if (className.equals("java.lang.Math")) {
+                isLibraryRoutine = true;
+            }
+        }
+
         setReferences();
     }
 
@@ -562,16 +636,80 @@ public class Routine implements SchemaObject {
         for (int i = 0; i < parameterTypes.length; i++) {
             ColumnSchema param = (ColumnSchema) parameterList.get(i);
 
-            set.addAll(param.getReferences());
+            OrderedHashSet refs = param.getReferences();
+
+            if (refs != null) {
+                set.addAll(refs);
+            }
         }
 
         if (statement != null) {
             set.addAll(statement.getReferences());
         }
 
+        isRecursive = false;
+
+        if (set.contains(getSpecificName())) {
+            set.remove(this.getSpecificName());
+
+            isRecursive = true;
+        }
+
         references = set;
     }
 
+    void checkSQLData(Session session) {
+
+        OrderedHashSet set = statement.getReferences();
+
+        for (int i = 0; i < set.size(); i++) {
+            HsqlName name = (HsqlName) set.get(i);
+
+            if (name.type == SchemaObject.SPECIFIC_ROUTINE) {
+                Routine routine =
+                    (Routine) session.database.schemaManager.getSchemaObject(
+                        name);
+
+                if (routine.dataImpact == Routine.READS_SQL) {
+                    if (dataImpact == Routine.CONTAINS_SQL) {
+                        throw Error.error(ErrorCode.X_42608,
+                                          Tokens.T_READS + ' ' + Tokens.T_SQL);
+                    }
+                } else if (routine.dataImpact == Routine.MODIFIES_SQL) {
+                    if (dataImpact == Routine.CONTAINS_SQL
+                            || dataImpact == Routine.READS_SQL) {
+                        throw Error.error(ErrorCode.X_42608,
+                                          Tokens.T_MODIFIES + ' '
+                                          + Tokens.T_SQL);
+                    }
+                }
+            }
+        }
+
+        if (dataImpact == Routine.CONTAINS_SQL
+                || dataImpact == Routine.READS_SQL) {
+            HsqlName[] names = statement.getTableNamesForWrite();
+
+            for (int i = 0; i < names.length; i++) {
+                if (names[i].schema != SqlInvariants.MODULE_HSQLNAME) {
+                    throw Error.error(ErrorCode.X_42608,
+                                      Tokens.T_MODIFIES + ' ' + Tokens.T_SQL);
+                }
+            }
+        }
+
+        if (dataImpact == Routine.CONTAINS_SQL) {
+            HsqlName[] names = statement.getTableNamesForRead();
+
+            for (int i = 0; i < names.length; i++) {
+                if (names[i].schema != SqlInvariants.MODULE_HSQLNAME) {
+                    throw Error.error(ErrorCode.X_42608,
+                                      Tokens.T_READS + ' ' + Tokens.T_SQL);
+                }
+            }
+        }
+    }
+
     public boolean isTrigger() {
         return routineType == SchemaObject.TRIGGER;
     }
@@ -619,10 +757,14 @@ public class Routine implements SchemaObject {
         return parameterList.getIndex(name);
     }
 
-    public RangeVariable[] getParameterRangeVariables() {
+    public RangeVariable[] getRangeVariables() {
         return ranges;
     }
 
+    public void setCorrelated() {
+        //
+    }
+
     public int getVariableCount() {
         return variableCount;
     }
@@ -649,14 +791,55 @@ public class Routine implements SchemaObject {
         return statement.getTableNamesForWrite();
     }
 
+    public void resetAlteredRoutineSettings() {
+
+        if (isPSM()) {
+            methodName               = null;
+            javaMethod               = null;
+            javaMethodWithConnection = false;
+            parameterStyle           = PARAM_STYLE_SQL;
+
+            if (dataImpact == NO_SQL) {
+                dataImpact = CONTAINS_SQL;
+            }
+        } else {
+            statement     = null;
+            references    = null;
+            variableCount = 0;
+            ranges        = RangeVariable.emptyArray;
+        }
+    }
+
+    public void setAsAlteredRoutine(Routine routine) {
+
+        language          = routine.language;
+        dataImpact        = routine.dataImpact;
+        parameterStyle    = routine.parameterStyle;
+        isDeterministic   = routine.isDeterministic;
+        isNullInputOutput = routine.isNullInputOutput;
+        maxDynamicResults = routine.maxDynamicResults;
+        isRecursive       = routine.isRecursive;
+        javaMethod        = routine.javaMethod;
+
+        //
+        isRecursive              = routine.isRecursive;
+        javaMethodWithConnection = routine.javaMethodWithConnection;
+        methodName               = routine.methodName;
+        statement                = routine.statement;
+        references               = routine.references;
+        variableCount            = routine.variableCount;
+        ranges                   = routine.ranges;
+    }
+
     Object[] convertArgsToJava(Session session, Object[] callArguments) {
 
         int      extraArg = javaMethodWithConnection ? 1
                                                      : 0;
-        Object[] data     = new Object[callArguments.length + extraArg];
+        Object[] data     = new Object[javaMethod.getParameterTypes().length];
         Type[]   types    = getParameterTypes();
+        int      i        = 0;
 
-        for (int i = 0; i < callArguments.length; i++) {
+        for (; i < types.length; i++) {
             Object       value = callArguments[i];
             ColumnSchema param = getParameter(i);
 
@@ -673,6 +856,10 @@ public class Routine implements SchemaObject {
             }
         }
 
+        for (; i + extraArg < data.length; i++) {
+            data[i + extraArg] = new java.sql.ResultSet[1];
+        }
+
         return data;
     }
 
@@ -682,8 +869,9 @@ public class Routine implements SchemaObject {
         int    extraArg = javaMethodWithConnection ? 1
                                                    : 0;
         Type[] types    = getParameterTypes();
+        int    i        = 0;
 
-        for (int i = 0; i < callArguments.length; i++) {
+        for (; i < types.length; i++) {
             Object       value = data[i + extraArg];
             ColumnSchema param = getParameter(i);
 
@@ -693,11 +881,49 @@ public class Routine implements SchemaObject {
 
             callArguments[i] = types[i].convertJavaToSQL(session, value);
         }
+
+        Result head = null;
+        for (; i + extraArg < data.length; i++) {
+            ResultSet rs = ((ResultSet[]) data[i + extraArg])[0];
+
+            if (rs != null) {
+                if (rs instanceof JDBCResultSet) {
+                    Result r    = ((JDBCResultSet) rs).result;
+
+                    if (head == null) {
+                        callArguments[i] = r;
+                        head = r;
+                    } else {
+                        head.addChainedResult(r);
+                    }
+                } else {
+                    Error.error(ErrorCode.X_46000, "ResultSet not native");
+                }
+            }
+        }
+    }
+
+    public Result invokeJavaMethodDirect(Object[] data) {
+
+        Result result;
+
+        try {
+            Object returnValue = javaMethod.invoke(null, data);
+
+            returnValue = returnType.convertJavaToSQL(null, returnValue);
+            result      = Result.newPSMResult(returnValue);
+        } catch (Throwable t) {
+            result = Result.newErrorResult(
+                Error.error(t, ErrorCode.X_46000, getName().name), null);
+        }
+
+        return result;
     }
 
     Result invokeJavaMethod(Session session, Object[] data) {
 
-        Result result;
+        Result   result;
+        HsqlName oldSessionSchema = session.getCurrentSchemaHsqlName();
 
         try {
             if (dataImpact == Routine.NO_SQL) {
@@ -710,6 +936,8 @@ public class Routine implements SchemaObject {
                 session.sessionContext.isReadOnly = Boolean.TRUE;
             }
 
+            session.setCurrentSchemaHsqlName(getSchemaName());
+
             Object returnValue = javaMethod.invoke(null, data);
 
             if (returnsTable()) {
@@ -728,18 +956,84 @@ public class Routine implements SchemaObject {
             }
         } catch (InvocationTargetException e) {
             result = Result.newErrorResult(
-                Error.error(ErrorCode.X_46000, getName().name), null);
+                Error.error(e, ErrorCode.X_46000, getName().name), null);
         } catch (IllegalAccessException e) {
             result = Result.newErrorResult(
-                Error.error(ErrorCode.X_46000, getName().name), null);
+                Error.error(e, ErrorCode.X_46000, getName().name), null);
         } catch (Throwable e) {
             result = Result.newErrorResult(
-                Error.error(ErrorCode.X_46000, getName().name), null);
+                Error.error(e, ErrorCode.X_46000, getName().name), null);
+        }
+
+        session.setCurrentSchemaHsqlName(oldSessionSchema);
+
+        return result;
+    }
+
+    public Result invoke(Session session, Object[] data,
+                         Object[] aggregateData, boolean push) {
+
+        Result result;
+
+        if (push) {
+            session.sessionContext.push();
+        }
+
+        if (isPSM()) {
+            try {
+                session.sessionContext.routineArguments = data;
+                session.sessionContext.routineVariables =
+                    ValuePool.emptyObjectArray;
+
+                if (variableCount > 0) {
+                    session.sessionContext.routineVariables =
+                        new Object[variableCount];
+                }
+
+                result = statement.execute(session);
+
+                if (aggregateData != null) {
+                    for (int i = 0; i < aggregateData.length; i++) {
+                        aggregateData[i] = data[i + 1];
+                    }
+                }
+            } catch (Throwable e) {
+                result = Result.newErrorResult(e);
+            }
+        } else {
+            if (isAggregate) {
+                data = convertArgsToJava(session, data);
+            }
+
+            result = invokeJavaMethod(session, data);
+
+            if (isAggregate) {
+                Object[] callResult = new Object[data.length];
+
+                convertArgsToSQL(session, callResult, data);
+
+                for (int i = 0; i < aggregateData.length; i++) {
+                    aggregateData[i] = callResult[i + 1];
+                }
+            }
+        }
+
+        if (push) {
+            session.sessionContext.pop();
         }
 
         return result;
     }
 
+    public Routine duplicate() {
+
+        try {
+            return (Routine) super.clone();
+        } catch (CloneNotSupportedException e) {
+            throw Error.runtimeError(ErrorCode.U_S0500, "Type");
+        }
+    }
+
     static Method getMethod(String name, Routine routine,
                             boolean[] hasConnection, boolean returnsTable) {
 
@@ -763,6 +1057,7 @@ public class Routine implements SchemaObject {
 
             Method  method = methods[i];
             Class[] params = method.getParameterTypes();
+            int     matchedParamCount;
 
             if (params.length > 0
                     && params[0].equals(java.sql.Connection.class)) {
@@ -770,7 +1065,20 @@ public class Routine implements SchemaObject {
                 hasConnection[0] = true;
             }
 
-            if (params.length - offset != routine.parameterTypes.length) {
+            matchedParamCount = params.length - offset;
+
+            if (routine.isProcedure()) {
+                for (int j = offset; j < params.length; j++) {
+                    if (params[j].isArray()
+                            && java.sql.ResultSet.class.isAssignableFrom(params[j].getComponentType())) {
+                        matchedParamCount = j - offset;
+
+                        break;
+                    }
+                }
+            }
+
+            if (matchedParamCount != routine.parameterTypes.length) {
                 continue;
             }
 
@@ -787,7 +1095,17 @@ public class Routine implements SchemaObject {
                     continue;
                 }
 
-                if (methodReturnType.typeCode != routine.returnType.typeCode) {
+                if (!routine.returnType.canBeAssignedFrom(methodReturnType)) {
+                    continue;
+                }
+
+                if (!methodReturnType.isLobType()
+                        && (methodReturnType.isBinaryType()
+                            || methodReturnType.isCharacterType())) {
+
+                    //
+                } else if (methodReturnType.typeCode
+                           != routine.returnType.typeCode) {
                     continue;
                 }
             }
@@ -876,6 +1194,10 @@ public class Routine implements SchemaObject {
         Class    cl;
         Method[] methods = null;
 
+        if (!HsqlDatabaseProperties.supportsJavaMethod(name)) {
+            throw Error.error(ErrorCode.X_42501, className);
+        }
+
         try {
             cl = Class.forName(className, true,
                                Thread.currentThread().getContextClassLoader());
@@ -885,7 +1207,7 @@ public class Routine implements SchemaObject {
             } catch (Throwable t) {
                 throw Error.error(t, ErrorCode.X_42501,
                                   ErrorCode.M_Message_Pair, new Object[] {
-                    t.getMessage(), className
+                    t.toString(), className
                 });
             }
         }
@@ -895,7 +1217,7 @@ public class Routine implements SchemaObject {
         } catch (Throwable t) {
             throw Error.error(t, ErrorCode.X_42501, ErrorCode.M_Message_Pair,
                               new Object[] {
-                t.getMessage(), className
+                t.toString(), className
             });
         }
 
@@ -903,6 +1225,7 @@ public class Routine implements SchemaObject {
 
         for (i = 0; i < methods.length; i++) {
             int    offset    = 0;
+            int    endIndex  = Integer.MAX_VALUE;
             Method method    = methods[i];
             int    modifiers = method.getModifiers();
 
@@ -931,6 +1254,28 @@ public class Routine implements SchemaObject {
 
                             break;
                         }
+
+                        if (java.sql.ResultSet.class.isAssignableFrom(param)) {
+                            if (endIndex > j) {
+                                endIndex = j;
+                            }
+                        }
+                    }
+
+                    if (j >= endIndex) {
+                        if (java.sql.ResultSet.class.isAssignableFrom(param)) {
+                            continue;
+                        } else {
+                            method = null;
+
+                            break;
+                        }
+                    }
+                } else {
+                    if (j > endIndex) {
+                        method = null;
+
+                        break;
                     }
                 }
 
@@ -1022,7 +1367,7 @@ public class Routine implements SchemaObject {
         Type methodReturnType =
             Types.getParameterSQLType(method.getReturnType());
 
-        routine.javaMethodWithConnection = offset == 1;;
+        routine.javaMethodWithConnection = offset == 1;
 
         routine.setReturnType(methodReturnType);
         routine.resolve(session);
@@ -1043,27 +1388,4 @@ public class Routine implements SchemaObject {
             session.database.schemaManager.addSchemaObject(routines[i]);
         }
     }
-
-    static void checkNoSQLData(Database database, OrderedHashSet set) {
-
-        for (int i = 0; i < set.size(); i++) {
-            HsqlName name = (HsqlName) set.get(i);
-
-            if (name.type == SchemaObject.SPECIFIC_ROUTINE) {
-                Routine routine =
-                    (Routine) database.schemaManager.getSchemaObject(name);
-
-                if (routine.dataImpact == Routine.READS_SQL) {
-                    throw Error.error(ErrorCode.X_42608,
-                                      Tokens.T_READS + " " + Tokens.T_SQL);
-                } else if (routine.dataImpact == Routine.MODIFIES_SQL) {
-                    throw Error.error(ErrorCode.X_42608,
-                                      Tokens.T_MODIFIES + " " + Tokens.T_SQL);
-                } else if (name.type == SchemaObject.TABLE) {
-                    throw Error.error(ErrorCode.X_42608,
-                                      Tokens.T_READS + " " + Tokens.T_SQL);
-                }
-            }
-        }
-    }
 }
