@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,7 @@ import org.hsqldb.persist.PersistentStore;
  * Represents the chain of insert / delete / rollback / commit actions on a row.
  *
  * @author Fred Toussi (fredt@users dot sourceforge dot net)
- * @version 2.0.0
+ * @version 2.2.9
  * @since 2.0.0
  */
 public class RowAction extends RowActionBase {
@@ -50,7 +50,7 @@ public class RowAction extends RowActionBase {
     final TableBase       table;
     final PersistentStore store;
     Row                   memoryRow;
-    int                   rowId;
+    long                  rowId;
     boolean               isMemory;
     RowAction             updatedAction;
 
@@ -103,7 +103,7 @@ public class RowAction extends RowActionBase {
         this.type            = type;
         this.actionTimestamp = session.actionTimestamp;
         this.table           = table;
-        this.store           = session.sessionData.getRowStore(table);
+        this.store           = table.getRowStore(session);
         this.isMemory        = row.isMemory();
         this.memoryRow       = row;
         this.rowId           = row.getPos();
@@ -137,28 +137,60 @@ public class RowAction extends RowActionBase {
             RowActionBase action = this;
 
             while (true) {
-                if (action.type == ACTION_INSERT) {
-                    if (action.commitTimestamp == 0
-                            && session != action.session) {
-                        throw Error.runtimeError(ErrorCode.U_S0500,
-                                                 "RowAction");
+                if (action.rolledback) {
+                    if (action.next == null) {
+                        break;
                     }
-                } else if (action.type == ACTION_DELETE) {
-                    if (session != action.session) {
-                        session.tempSet.add(action.session);
 
-                        return null;
+                    action = action.next;
+
+                    continue;
+                }
+
+                switch (action.type) {
+
+                    case ACTION_INSERT : {
+                        if (action.commitTimestamp == 0
+                                && session != action.session) {
+                            throw Error.runtimeError(ErrorCode.U_S0500,
+                                                     "RowAction");
+                        }
+
+                        break;
                     }
-                } else if (action.type == ACTION_REF) {
-                    if (session != action.session
-                            && action.commitTimestamp == 0) {
-                        if (colMap == null
-                                || ArrayUtil.haveCommonElement(
-                                    colMap, action.changeColumnMap)) {
-                            session.tempSet.add(action.session);
+                    case ACTION_DELETE_FINAL :
+                    case ACTION_DELETE : {
+                        if (session != action.session) {
+                            if (action.commitTimestamp == 0) {
+                                if (!session.tempSet.isEmpty()) {
+                                    session.tempSet.clear();
+                                }
+
+                                session.tempSet.add(action);
+                            }
 
                             return null;
                         }
+
+                        break;
+                    }
+                    case ACTION_REF : {
+                        if (session != action.session
+                                && action.commitTimestamp == 0) {
+                            if (colMap == null
+                                    || ArrayUtil.haveCommonElement(
+                                        colMap, action.changeColumnMap)) {
+                                if (!session.tempSet.isEmpty()) {
+                                    session.tempSet.clear();
+                                }
+
+                                session.tempSet.add(action);
+
+                                return null;
+                            }
+                        }
+
+                        break;
                     }
                 }
 
@@ -210,7 +242,11 @@ public class RowAction extends RowActionBase {
                     if (action.changeColumnMap == null
                             || ArrayUtil.haveCommonElement(
                                 colMap, action.changeColumnMap)) {
-                        session.tempSet.add(action.session);
+                        if (!session.tempSet.isEmpty()) {
+                            session.tempSet.clear();
+                        }
+
+                        session.tempSet.add(action);
 
                         return false;
                     }
@@ -421,7 +457,7 @@ public class RowAction extends RowActionBase {
                 }
 
                 if (action.commitTimestamp == 0) {
-                    set.add(action.session);
+                    set.add(action);
                 } else if (action.commitTimestamp > commitTimestamp) {
                     commitTimestamp = action.commitTimestamp;
                 }
@@ -536,11 +572,11 @@ public class RowAction extends RowActionBase {
         return actionType;
     }
 
-    public synchronized int getPos() {
+    public synchronized long getPos() {
         return rowId;
     }
 
-    synchronized void setPos(int pos) {
+    synchronized void setPos(long pos) {
         rowId = pos;
     }
 
@@ -780,11 +816,10 @@ public class RowAction extends RowActionBase {
                     if (mode == TransactionManager.ACTION_READ) {
                         actionType = action.ACTION_DELETE;
                     } else if (mode == TransactionManager.ACTION_DUP) {
-                        if (action.changeColumnMap == null) {
-                            actionType = ACTION_INSERT;
-                        } else {
-                            actionType = ACTION_DELETE;
-                        }
+                        actionType = ACTION_INSERT;
+
+                        session.tempSet.clear();
+                        session.tempSet.add(action);
                     } else if (mode == TransactionManager.ACTION_REF) {
                         actionType = ACTION_DELETE;
                     }
@@ -804,17 +839,22 @@ public class RowAction extends RowActionBase {
                 continue;
             } else if (action.commitTimestamp < threshold) {
                 if (action.type == ACTION_DELETE) {
-                    if (actionType == ACTION_INSERT) {
-                        actionType = action.type;
-                    } else {
-                        actionType = action.type;
-                    }
+                    actionType = ACTION_DELETE;
                 } else if (action.type == ACTION_INSERT) {
-                    actionType = action.type;
+                    actionType = ACTION_INSERT;
                 }
             } else {
                 if (action.type == ACTION_INSERT) {
-                    actionType = ACTION_DELETE;
+                    if (mode == TransactionManager.ACTION_READ) {
+                        actionType = action.ACTION_DELETE;
+                    } else if (mode == TransactionManager.ACTION_DUP) {
+                        actionType = ACTION_INSERT;
+
+                        session.tempSet.clear();
+                        session.tempSet.add(action);
+                    } else if (mode == TransactionManager.ACTION_REF) {
+                        actionType = ACTION_DELETE;
+                    }
                 }
             }
 
@@ -859,24 +899,25 @@ public class RowAction extends RowActionBase {
         return updatedAction;
     }
 
-    synchronized String dump() {
+    public synchronized String describe(Session session) {
 
         StringBuilder sb     = new StringBuilder();
         RowActionBase action = this;
 
         do {
-            if (action == this && memoryRow != null) {
-                sb.append("" + memoryRow.getId());
+            if (action == this) {
+                sb.append(this.rowId).append(' ');
             }
 
-            sb.append(" " + action.session.getId() + ' ' + action.type + ' '
-                      + action.actionTimestamp + ' ' + action.commitTimestamp);
+            sb.append(action.session.getId()).append(' ');
+            sb.append(action.type).append(' ').append(action.actionTimestamp);
+            sb.append(' ').append(action.commitTimestamp);
 
             if (action.commitTimestamp != 0) {
                 if (action.rolledback) {
-                    sb.append("r");
+                    sb.append('r');
                 } else {
-                    sb.append("c");
+                    sb.append('c');
                 }
             }
 
