@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,7 +62,7 @@ import org.hsqldb.types.Types;
  * Scans for SQL tokens.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 public class Scanner {
@@ -88,7 +88,7 @@ public class Scanner {
 
     */
     //J-
-    final static char[] specials = new char[] {
+    static final char[] specials = new char[] {
         '"',
         '%',
         '&',
@@ -116,7 +116,7 @@ public class Scanner {
         '{',
         '}'
     };
-    final static String[] multi = new String[] {
+    static final String[] multi = new String[] {
         "??(",
         "??)",
         "<>",
@@ -131,7 +131,7 @@ public class Scanner {
         "*/",
     };
 
-    final static char[] whitespace = {
+    static final char[] whitespace = {
         // SQL extras
         0x9,
         0xA,
@@ -166,7 +166,7 @@ public class Scanner {
     };
 
 //J+
-    final static OrderedIntHashSet whiteSpaceSet = new OrderedIntHashSet(32);
+    static final OrderedIntHashSet whiteSpaceSet = new OrderedIntHashSet(32);
 
     static {
         for (int i = 0; i < whitespace.length; i++) {
@@ -816,11 +816,23 @@ public class Scanner {
                 token.isDelimiter = false;
         }
 
+        boolean hasPreSpace = scanWhitespace();
+
         c = charAt(currentPosition);
 
         if (c == '.') {
+            if (hasPreSpace) {
+                int cNext = charAt(currentPosition + 1);
+
+                if (cNext >= '0' && cNext <= '9') {
+                    return;
+                }
+            }
+
             currentPosition++;
 
+            scanWhitespace();
+
             c = charAt(currentPosition);
 
             if (c == '*') {
@@ -843,9 +855,10 @@ public class Scanner {
             return false;
         }
 
-        char start = sqlString.charAt(currentPosition);
+        char    start     = sqlString.charAt(currentPosition);
+        boolean irregular = start == '_' || start == '$';
 
-        if (!Character.isLetter(start)) {
+        if (!irregular && !Character.isLetter(start)) {
             token.tokenString = Character.toString(start);
             token.tokenType   = Tokens.X_UNKNOWN_TOKEN;
             token.isMalformed = true;
@@ -858,6 +871,12 @@ public class Scanner {
         for (; i < limit; i++) {
             char c = sqlString.charAt(i);
 
+            if (c == '$') {
+                irregular = true;
+
+                continue;
+            }
+
             if (c == '_' || Character.isLetterOrDigit(c)) {
                 continue;
             }
@@ -913,6 +932,10 @@ public class Scanner {
             }
         }
 
+        if (irregular) {
+            token.hasIrregularChar = true;
+        }
+
         return true;
     }
 
@@ -1412,15 +1435,15 @@ public class Scanner {
             case '?' :
                 if (charAt(currentPosition + 1) == '?') {
                     if (charAt(currentPosition + 2) == '(') {
-                        token.tokenString = Tokens.T_OPENBRACKET;
-                        token.tokenType   = Tokens.OPENBRACKET;
+                        token.tokenString = Tokens.T_LEFTBRACKET;
+                        token.tokenType   = Tokens.LEFTBRACKET;
                         currentPosition   += 3;
                         token.isDelimiter = true;
 
                         return;
                     } else if (charAt(currentPosition + 2) == ')') {
-                        token.tokenString = Tokens.T_CLOSEBRACKET;
-                        token.tokenType   = Tokens.CLOSEBRACKET;
+                        token.tokenString = Tokens.T_RIGHTBRACKET;
+                        token.tokenType   = Tokens.RIGHTBRACKET;
                         currentPosition   += 3;
                         token.isDelimiter = true;
 
@@ -1702,6 +1725,8 @@ public class Scanner {
                  * identifier chain must not have catalog identifier
                  * character set specification to be included in the token.dataType
                  */
+                int startPosition = currentPosition;
+
                 currentPosition++;
 
                 scanIdentifierChain();
@@ -1710,8 +1735,7 @@ public class Scanner {
                     return;
                 }
 
-                if (token.tokenType != Tokens.X_IDENTIFIER
-                        || token.namePrePrefix != null) {
+                if (token.tokenType != Tokens.X_IDENTIFIER) {
 
                     /** @todo 1.9.0 - review message malformed character set identifier */
                     token.tokenType   = Tokens.X_MALFORMED_STRING;
@@ -1720,12 +1744,21 @@ public class Scanner {
                     return;
                 }
 
-                token.charsetSchema = token.namePrefix;
-                token.charsetName   = token.tokenString;
-
                 scanSeparator();
 
                 if (charAt(currentPosition) == '\'') {
+                    if (token.namePrePrefix != null) {
+
+                        /** @todo 1.9.0 - review message malformed character set identifier */
+                        token.tokenType   = Tokens.X_MALFORMED_STRING;
+                        token.isMalformed = true;
+
+                        return;
+                    }
+
+                    token.charsetSchema = token.namePrefix;
+                    token.charsetName   = token.tokenString;
+
                     scanCharacterString();
 
                     token.tokenType = Tokens.X_VALUE;
@@ -1734,9 +1767,12 @@ public class Scanner {
                     token.isDelimiter = true;
 
                     return;
-                }
-                break;
+                } else {
+                    position(startPosition);
+                    resetState();
 
+                    break;
+                }
             case '0' :
             case '1' :
             case '2' :
@@ -1756,19 +1792,26 @@ public class Scanner {
         }
 
         scanIdentifierChain();
+        setIdentifierProperties();
+    }
+
+    private void setIdentifierProperties() {
 
         if (token.tokenType == Tokens.X_IDENTIFIER) {
             token.isUndelimitedIdentifier = true;
-            token.tokenType = Tokens.getKeywordID(token.tokenString,
-                                                  Tokens.X_IDENTIFIER);
 
-            if (token.tokenType == Tokens.X_IDENTIFIER) {
-                token.tokenType = Tokens.getNonKeywordID(token.tokenString,
-                        Tokens.X_IDENTIFIER);
-            } else {
-                token.isReservedIdentifier = true;
-                token.isCoreReservedIdentifier =
-                    Tokens.isCoreKeyword(token.tokenType);
+            if (token.namePrefix == null) {
+                token.tokenType = Tokens.getKeywordID(token.tokenString,
+                                                      Tokens.X_IDENTIFIER);
+
+                if (token.tokenType == Tokens.X_IDENTIFIER) {
+                    token.tokenType = Tokens.getNonKeywordID(token.tokenString,
+                            Tokens.X_IDENTIFIER);
+                } else {
+                    token.isReservedIdentifier = true;
+                    token.isCoreReservedIdentifier =
+                        Tokens.isCoreKeyword(token.tokenType);
+                }
             }
         } else if (token.tokenType == Tokens.X_DELIMITED_IDENTIFIER) {
             token.isDelimitedIdentifier = true;
@@ -2025,7 +2068,7 @@ public class Scanner {
                         throw Error.error(ErrorCode.X_22007);
                     }
                 } else {
-                    if (currentDigits != 2) {
+                    if (currentDigits == 0 || currentDigits > 2) {
                         throw Error.error(ErrorCode.X_22007);
                     }
                 }
@@ -2092,8 +2135,9 @@ public class Scanner {
 
     public Object newInterval(String s, IntervalType type) {
 
-        intervalPosition = 0;
-        intervalString   = s;
+        intervalPosition  = 0;
+        fractionPrecision = 0;
+        intervalString    = s;
 
         boolean negate   = scanIntervalSign();
         long    units    = scanIntervalValue(type);
@@ -2193,7 +2237,7 @@ public class Scanner {
                         throw Error.error(ErrorCode.X_22015);
                     }
 
-                    if (currentDigits != 2) {
+                    if (currentDigits == 0 || currentDigits > 2) {
                         throw Error.error(ErrorCode.X_22006);
                     }
 
