@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,55 +31,60 @@
 
 package org.hsqldb;
 
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.IOException;
 import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.io.Reader;
 
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.lib.CharArrayWriter;
 import org.hsqldb.lib.CountdownInputStream;
 import org.hsqldb.lib.HashMap;
+import org.hsqldb.lib.HsqlByteArrayOutputStream;
 import org.hsqldb.lib.Iterator;
+import org.hsqldb.lib.LongDeque;
 import org.hsqldb.lib.LongKeyHashMap;
 import org.hsqldb.lib.LongKeyLongValueHashMap;
-import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.lib.ReaderInputStream;
 import org.hsqldb.navigator.RowSetNavigator;
 import org.hsqldb.navigator.RowSetNavigatorClient;
-import org.hsqldb.persist.DataFileCacheSession;
 import org.hsqldb.persist.PersistentStore;
 import org.hsqldb.persist.PersistentStoreCollectionSession;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultLob;
 import org.hsqldb.result.ResultProperties;
 import org.hsqldb.types.BlobData;
-import org.hsqldb.types.ClobData;
-import org.hsqldb.types.LobData;
 import org.hsqldb.types.BlobDataID;
+import org.hsqldb.types.ClobData;
 import org.hsqldb.types.ClobDataID;
+import org.hsqldb.types.LobData;
 
 /*
  * Session semi-persistent data structures
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.7
  * @since 1.9.0
  */
 public class SessionData {
 
-    private final Database           database;
-    private final Session            session;
-    PersistentStoreCollectionSession persistentStoreCollection;
+    private final Database                  database;
+    private final Session                   session;
+    public PersistentStoreCollectionSession persistentStoreCollection;
 
     // large results
-    LongKeyHashMap       resultMap;
-    DataFileCacheSession resultCache;
+    LongKeyHashMap resultMap;
 
     // VALUE
     Object currentValue;
 
     // SEQUENCE
-    HashMap        sequenceMap;
-    OrderedHashSet sequenceUpdateSet;
+    HashMap sequenceMap;
+    HashMap sequenceUpdateMap;
 
     public SessionData(Database database, Session session) {
 
@@ -89,28 +94,6 @@ public class SessionData {
             new PersistentStoreCollectionSession(session);
     }
 
-    // transitional feature
-    public PersistentStore getRowStore(TableBase table) {
-
-        if (table.tableType == TableBase.SYSTEM_TABLE) {
-            if (session.isAdmin()) {
-                return table.store;
-            }
-
-            return persistentStoreCollection.getStore(table);
-        }
-
-        if (table.store != null) {
-            return table.store;
-        }
-
-        if (table.isSessionBased) {
-            return persistentStoreCollection.getStore(table);
-        }
-
-        return database.persistentStoreCollection.getStore(table);
-    }
-
     public PersistentStore getSubqueryRowStore(TableBase table) {
 
         PersistentStore store = persistentStoreCollection.getStore(table);
@@ -125,7 +108,11 @@ public class SessionData {
 
         try {
             PersistentStore store = session.database.logger.newStore(session,
-                persistentStoreCollection, table, isCached);
+                persistentStoreCollection, table);
+
+            if (!isCached) {
+                store.setMemory(true);
+            }
 
             return store;
         } catch (HsqlException e) {}
@@ -227,6 +214,9 @@ public class SessionData {
             }
 
             resultMap.put(result.getResultId(), result);
+
+            result.rsProperties =
+                ResultProperties.addIsHeld(result.rsProperties, true);
         }
 
         if (copy) {
@@ -238,9 +228,14 @@ public class SessionData {
 
     Result getDataResultSlice(long id, int offset, int count) {
 
-        RowSetNavigatorClient navigator = getRowSetSlice(id, offset, count);
+        Result          result = (Result) resultMap.get(id);
+        RowSetNavigator source = result.getNavigator();
 
-        return Result.newDataRowsResult(navigator);
+        if (offset + count > source.getSize()) {
+            count = source.getSize() - offset;
+        }
+
+        return Result.newDataRowsResult(result, offset, count);
     }
 
     Result getDataResult(long id) {
@@ -266,7 +261,9 @@ public class SessionData {
 
         Result result = (Result) resultMap.remove(id);
 
-        result.getNavigator().close();
+        if (result != null) {
+            result.getNavigator().release();
+        }
     }
 
     public void closeAllNavigators() {
@@ -280,7 +277,7 @@ public class SessionData {
         while (it.hasNext()) {
             Result result = (Result) it.next();
 
-            result.getNavigator().close();
+            result.getNavigator().release();
         }
 
         resultMap.clear();
@@ -298,59 +295,34 @@ public class SessionData {
             Result result = (Result) it.next();
 
             if (!ResultProperties.isHoldable(result.rsProperties)) {
-                result.getNavigator().close();
+                result.getNavigator().release();
                 it.remove();
             }
         }
-
-        resultMap.clear();
     }
 
-    public DataFileCacheSession getResultCache() {
+    // lob creation
+    boolean hasLobOps;
+    long    firstNewLobID;
 
-        if (resultCache == null) {
-            String path = database.logger.getTempDirectoryPath();
-
-            if (path == null) {
-                return null;
-            }
-
-            try {
-                resultCache =
-                    new DataFileCacheSession(database,
-                                             path + "/session_"
-                                             + Long.toString(session.getId()));
-
-                resultCache.open(false);
-            } catch (Throwable t) {
-                return null;
-            }
-        }
-
-        return resultCache;
+    public void registerNewLob(long lobID) {
+        firstNewLobID = lobID;
+        hasLobOps     = true;
     }
 
-    synchronized void closeResultCache() {
+    public void clearLobOps() {
+        firstNewLobID = 0;
+        hasLobOps     = false;
+    }
 
-        if (resultCache != null) {
-            try {
-                resultCache.close(false);
-            } catch (HsqlException e) {}
-
-            resultCache = null;
-        }
+    public long getFirstLobID() {
+        return firstNewLobID;
     }
 
     // lobs in results
     LongKeyLongValueHashMap resultLobs = new LongKeyLongValueHashMap();
 
     // lobs in transaction
-    boolean hasLobOps;
-
-    public void addToCreatedLobs(long lobID) {
-        hasLobOps = true;
-    }
-
     public void adjustLobUsageCount(Object value, int adjust) {
 
         if (session.isProcessingLog || session.isProcessingScript) {
@@ -361,7 +333,8 @@ public class SessionData {
             return;
         }
 
-        database.lobManager.adjustUsageCount(((LobData) value).getId(),
+        database.lobManager.adjustUsageCount(session,
+                                             ((LobData) value).getId(),
                                              adjust);
 
         hasLobOps = true;
@@ -374,6 +347,10 @@ public class SessionData {
             return;
         }
 
+        if (table.isTemp) {
+            return;
+        }
+
         if (session.isProcessingLog || session.isProcessingScript) {
             return;
         }
@@ -386,7 +363,8 @@ public class SessionData {
                     continue;
                 }
 
-                database.lobManager.adjustUsageCount(((LobData) value).getId(),
+                database.lobManager.adjustUsageCount(session,
+                                                     ((LobData) value).getId(),
                                                      adjust);
 
                 hasLobOps = true;
@@ -400,62 +378,167 @@ public class SessionData {
     public void allocateLobForResult(ResultLob result,
                                      InputStream inputStream) {
 
-        long                 resultLobId = result.getLobID();
-        CountdownInputStream countStream;
+        try {
+            CountdownInputStream countStream;
 
-        switch (result.getSubType()) {
+            switch (result.getSubType()) {
 
-            case ResultLob.LobResultTypes.REQUEST_CREATE_BYTES : {
-                long blobId;
-                long blobLength = result.getBlockLength();
+                case ResultLob.LobResultTypes.REQUEST_CREATE_BYTES : {
+                    long blobId;
+                    long blobLength = result.getBlockLength();
 
-                if (inputStream == null) {
-                    blobId      = resultLobId;
-                    inputStream = result.getInputStream();
-                } else {
-                    BlobData blob = session.createBlob(blobLength);
+                    if (blobLength < 0) {
 
-                    blobId = blob.getId();
+                        // embedded session + unknown lob length
+                        allocateBlobSegments(result, result.getInputStream());
 
-                    resultLobs.put(resultLobId, blobId);
-                }
-
-                countStream = new CountdownInputStream(inputStream);
-
-                countStream.setCount(blobLength);
-                database.lobManager.setBytesForNewBlob(
-                    blobId, countStream, result.getBlockLength());
-
-                break;
-            }
-            case ResultLob.LobResultTypes.REQUEST_CREATE_CHARS : {
-                long clobId;
-                long clobLength = result.getBlockLength();
-
-                if (inputStream == null) {
-                    clobId = resultLobId;
-
-                    if (result.getReader() != null) {
-                        inputStream =
-                            new ReaderInputStream(result.getReader());
-                    } else {
-                        inputStream = result.getInputStream();
+                        break;
                     }
-                } else {
-                    ClobData clob = session.createClob(clobLength);
 
-                    clobId = clob.getId();
+                    if (inputStream == null) {
 
-                    resultLobs.put(resultLobId, clobId);
+                        // embedded session + known lob length
+                        blobId      = result.getLobID();
+                        inputStream = result.getInputStream();
+                    } else {
+
+                        // server session + known or unknown lob length
+                        BlobData blob = session.createBlob(blobLength);
+
+                        blobId = blob.getId();
+
+                        resultLobs.put(result.getLobID(), blobId);
+                    }
+
+                    countStream = new CountdownInputStream(inputStream);
+
+                    countStream.setCount(blobLength);
+                    database.lobManager.setBytesForNewBlob(
+                        blobId, countStream, result.getBlockLength());
+
+                    break;
                 }
+                case ResultLob.LobResultTypes.REQUEST_CREATE_CHARS : {
+                    long clobId;
+                    long clobLength = result.getBlockLength();
 
-                countStream = new CountdownInputStream(inputStream);
+                    if (clobLength < 0) {
 
-                countStream.setCount(clobLength * 2);
-                database.lobManager.setCharsForNewClob(
-                    clobId, countStream, result.getBlockLength());
+                        // embedded session + unknown lob length
+                        allocateClobSegments(result, result.getReader());
 
-                break;
+                        break;
+                    }
+
+                    if (inputStream == null) {
+                        clobId = result.getLobID();
+
+                        // embedded session + known lob length
+                        if (result.getReader() != null) {
+                            inputStream =
+                                new ReaderInputStream(result.getReader());
+                        } else {
+                            inputStream = result.getInputStream();
+                        }
+                    } else {
+
+                        // server session + known or unknown lob length
+                        ClobData clob = session.createClob(clobLength);
+
+                        clobId = clob.getId();
+
+                        resultLobs.put(result.getLobID(), clobId);
+                    }
+
+                    countStream = new CountdownInputStream(inputStream);
+
+                    countStream.setCount(clobLength * 2);
+                    database.lobManager.setCharsForNewClob(
+                        clobId, countStream, result.getBlockLength(), false);
+
+                    break;
+                }
+                case ResultLob.LobResultTypes.REQUEST_SET_BYTES : {
+
+                    // server session + unknown lob length
+                    long   blobId     = resultLobs.get(result.getLobID());
+                    long   dataLength = result.getBlockLength();
+                    byte[] byteArray  = result.getByteArray();
+                    Result actionResult = database.lobManager.setBytes(blobId,
+                        result.getOffset(), byteArray, (int) dataLength);
+
+                    break;
+                }
+                case ResultLob.LobResultTypes.REQUEST_SET_CHARS : {
+
+                    // server session + unknown lob length
+                    long   clobId     = resultLobs.get(result.getLobID());
+                    long   dataLength = result.getBlockLength();
+                    char[] charArray  = result.getCharArray();
+                    Result actionResult = database.lobManager.setChars(clobId,
+                        result.getOffset(), charArray);
+
+                    break;
+                }
+            }
+        } catch (Throwable e) {
+            resultLobs.clear();
+
+            throw Error.error(ErrorCode.GENERAL_ERROR, e);
+        }
+    }
+
+    private void allocateBlobSegments(ResultLob result,
+                                      InputStream stream) throws IOException {
+
+        //
+        long currentOffset = result.getOffset();
+        int  bufferLength  = session.getStreamBlockSize();
+        HsqlByteArrayOutputStream byteArrayOS =
+            new HsqlByteArrayOutputStream(bufferLength);
+
+        while (true) {
+            byteArrayOS.reset();
+            byteArrayOS.write(stream, bufferLength);
+
+            byte[] byteArray = byteArrayOS.getBuffer();
+            Result actionResult =
+                database.lobManager.setBytes(result.getLobID(), currentOffset,
+                                             byteArray, byteArrayOS.size());
+
+            currentOffset += byteArrayOS.size();
+
+            if (byteArrayOS.size() < bufferLength) {
+                return;
+            }
+        }
+    }
+
+    private void allocateClobSegments(ResultLob result,
+                                      Reader reader) throws IOException {
+
+        long            currentOffset = result.getOffset();
+        int             bufferLength  = session.getStreamBlockSize();
+        CharArrayWriter charWriter    = new CharArrayWriter(bufferLength);
+
+        while (true) {
+            charWriter.reset();
+            charWriter.write(reader, bufferLength);
+
+            char[] charArray = charWriter.getBuffer();
+
+            if (charWriter.size() < bufferLength) {
+                charArray = charWriter.toCharArray();
+            }
+
+            Result actionResult =
+                database.lobManager.setChars(result.getLobID(), currentOffset,
+                                             charArray);
+
+            currentOffset += charWriter.size();
+
+            if (charWriter.size() < bufferLength) {
+                return;
             }
         }
     }
@@ -484,19 +567,111 @@ public class SessionData {
         for (int i = 0; i < data.length; i++) {
             if (data[i] instanceof BlobDataID) {
                 BlobData blob = (BlobDataID) data[i];
-                long     id   = resultLobs.get(blob.getId());
+                long     id   = blob.getId();
+
+                if (id < 0) {
+                    id = resultLobs.get(id);
+                }
 
                 data[i] = database.lobManager.getBlob(id);
+
+                // handle invalid id;
             } else if (data[i] instanceof ClobDataID) {
                 ClobData clob = (ClobDataID) data[i];
-                long     id   = resultLobs.get(clob.getId());
+                long     id   = clob.getId();
+
+                if (id < 0) {
+                    id = resultLobs.get(id);
+                }
 
                 data[i] = database.lobManager.getClob(id);
+
+                // handle invalid id;
             }
         }
     }
 
-    //
+    ClobData createClobFromFile(String filename, String encoding) {
+
+        session.checkAdmin();
+
+        filename = database.logger.getSecurePath(filename);
+
+        if (filename == null) {
+            throw (Error.error(ErrorCode.ACCESS_IS_DENIED, filename));
+        }
+
+        File    file   = new File(filename);
+        boolean exists = file.exists();
+
+        if (!exists) {
+            throw Error.error(ErrorCode.FILE_IO_ERROR);
+        }
+
+        long        fileLength = file.length();
+        InputStream is         = null;
+
+        try {
+            ClobData clob = session.createClob(fileLength);
+
+            is = new FileInputStream(file);
+
+            Reader reader = new InputStreamReader(is, encoding);
+
+            is = new ReaderInputStream(reader);
+
+            database.lobManager.setCharsForNewClob(clob.getId(), is,
+                                                   fileLength, true);
+
+            return clob;
+        } catch (IOException e) {
+            throw Error.error(ErrorCode.FILE_IO_ERROR, e.toString());
+        } finally {
+            try {
+                is.close();
+            } catch (Exception e) {}
+        }
+    }
+
+    BlobData createBlobFromFile(String filename) {
+
+        session.checkAdmin();
+
+        filename = database.logger.getSecurePath(filename);
+
+        if (filename == null) {
+            throw (Error.error(ErrorCode.ACCESS_IS_DENIED, filename));
+        }
+
+        File    file   = new File(filename);
+        boolean exists = file.exists();
+
+        if (!exists) {
+            throw Error.error(ErrorCode.FILE_IO_ERROR);
+        }
+
+        long        fileLength = file.length();
+        InputStream is         = null;
+
+        try {
+            BlobData blob = session.createBlob(fileLength);
+
+            is = new FileInputStream(file);
+
+            database.lobManager.setBytesForNewBlob(blob.getId(), is,
+                                                   fileLength);
+
+            return blob;
+        } catch (IOException e) {
+            throw Error.error(ErrorCode.FILE_IO_ERROR);
+        } finally {
+            try {
+                is.close();
+            } catch (Exception e) {}
+        }
+    }
+
+    // sequences
     public void startRowProcessing() {
 
         if (sequenceMap != null) {
@@ -508,7 +683,7 @@ public class SessionData {
 
         if (sequenceMap == null) {
             sequenceMap       = new HashMap();
-            sequenceUpdateSet = new OrderedHashSet();
+            sequenceUpdateMap = new HashMap();
         }
 
         HsqlName key   = sequence.getName();
@@ -518,9 +693,14 @@ public class SessionData {
             value = sequence.getValueObject();
 
             sequenceMap.put(key, value);
-            sequenceUpdateSet.add(sequence);
+            sequenceUpdateMap.put(sequence, value);
         }
 
         return value;
     }
+
+    public Object getSequenceCurrent(NumberSequence sequence) {
+        return sequenceUpdateMap == null ? null
+                                         : sequenceUpdateMap.get(sequence);
+    }
 }
