@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,23 +46,34 @@ import org.hsqldb.navigator.RowSetNavigatorDataChange;
 import org.hsqldb.persist.PersistentStore;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultConstants;
+import org.hsqldb.result.ResultMetaData;
 import org.hsqldb.types.Type;
+import org.hsqldb.types.Types;
 
 /**
  * Implementation of Statement for DML statements.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.0.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 
 // support for MERGE statement originally contributed by Justin Spadea (jzs9783@users dot sourceforge.net)
 public class StatementDML extends StatementDMQL {
 
-    Expression[]  targets;
-    Expression    updatableTableCheck;
-    RangeVariable checkRangeVariable;
-    boolean       isTruncate;
+    Expression[] targets;
+    boolean      isTruncate;
+
+    //
+    boolean        isSimpleInsert;
+    int            generatedType;
+    ResultMetaData generatedInputMetaData;
+
+    /** column indexes for generated values */
+    int[] generatedIndexes;
+
+    /** ResultMetaData for generated values */
+    ResultMetaData generatedResultMetaData;
 
     public StatementDML(int type, int group, HsqlName schemaName) {
         super(type, group, schemaName);
@@ -116,9 +127,9 @@ public class StatementDML extends StatementDMQL {
         this.updateCheckColumns   = checkColumns;
         this.targetRangeVariables = rangeVars;
 
+        setupChecks();
         setDatabseObjects(session, compileContext);
         checkAccessRights(session);
-        setupChecks();
         targetRangeVariables[0].addAllColumns();
     }
 
@@ -137,9 +148,9 @@ public class StatementDML extends StatementDMQL {
         this.targets     = targets;
         this.sourceTable = targetRangeVars[0].rangeTable;
         this.targetTable = targetRangeVars[1].rangeTable;
-        this.baseTable   = targetTable.getBaseTable() == null ? targetTable
-                                                              : targetTable
-                                                              .getBaseTable();
+        this.baseTable   = targetTable.isTriggerUpdatable() ? targetTable
+                                                            : targetTable
+                                                            .getBaseTable();
         this.insertCheckColumns   = checkColumns;
         this.insertColumnMap      = insertColMap;
         this.updateColumnMap      = updateColMap;
@@ -148,9 +159,9 @@ public class StatementDML extends StatementDMQL {
         this.targetRangeVariables = targetRangeVars;
         this.condition            = mergeCondition;
 
+        setupChecks();
         setDatabseObjects(session, compileContext);
         checkAccessRights(session);
-        setupChecks();
     }
 
     /**
@@ -169,7 +180,8 @@ public class StatementDML extends StatementDMQL {
                     .getMainSelect();
 
             this.updatableTableCheck = select.checkQueryCondition;
-            this.checkRangeVariable  = select.rangeVariables[0];
+            this.checkRangeVariable =
+                select.rangeVariables[select.rangeVariables.length - 1];
         }
     }
 
@@ -199,6 +211,10 @@ public class StatementDML extends StatementDMQL {
                 throw Error.runtimeError(ErrorCode.U_S0500, "StatementDML");
         }
 
+        session.sessionContext
+            .diagnosticsVariables[ExpressionColumn.idx_row_count] =
+                Integer.valueOf(result.getUpdateCount());
+
         return result;
     }
 
@@ -211,15 +227,40 @@ public class StatementDML extends StatementDMQL {
             for (int i = 0; i < baseTable.fkConstraints.length; i++) {
                 Constraint constraint = baseTable.fkConstraints[i];
 
-                if (type == StatementTypes.UPDATE_WHERE
-                        || type == StatementTypes.MERGE) {
-                    if (ArrayUtil.haveCommonElement(constraint.getRefColumns(),
-                                                    updateColumnMap)) {
+                switch (type) {
+
+                    case StatementTypes.UPDATE_WHERE : {
+                        if (ArrayUtil.haveCommonElement(
+                                constraint.getRefColumns(), updateColumnMap)) {
+                            set.add(baseTable.fkConstraints[i].getMain()
+                                .getName());
+                        }
+
+                        break;
+                    }
+                    case StatementTypes.INSERT : {
                         set.add(
                             baseTable.fkConstraints[i].getMain().getName());
+
+                        break;
+                    }
+                    case StatementTypes.MERGE : {
+                        if (updateColumnMap != null) {
+                            if (ArrayUtil.haveCommonElement(
+                                    constraint.getRefColumns(),
+                                    updateColumnMap)) {
+                                set.add(baseTable.fkConstraints[i].getMain()
+                                    .getName());
+                            }
+                        }
+
+                        if (insertExpression != null) {
+                            set.add(baseTable.fkConstraints[i].getMain()
+                                .getName());
+                        }
+
+                        break;
                     }
-                } else if (type == StatementTypes.INSERT) {
-                    set.add(baseTable.fkConstraints[i].getMain().getName());
                 }
             }
 
@@ -269,7 +310,9 @@ public class StatementDML extends StatementDMQL {
 
             if (type == StatementTypes.UPDATE_WHERE
                     || type == StatementTypes.MERGE) {
-                baseTable.collectFKWriteLocks(updateColumnMap, set);
+                if (updateExpressions.length != 0) {
+                    baseTable.collectFKWriteLocks(updateColumnMap, set);
+                }
             } else if (type == StatementTypes.DELETE_WHERE) {
                 baseTable.collectFKWriteLocks(null, set);
             }
@@ -278,6 +321,117 @@ public class StatementDML extends StatementDMQL {
         }
     }
 
+    /**
+     * @todo - fredt - low priority - this does not work with different prepare calls
+     * with the same SQL statement, but different generated column requests
+     * To fix, add comment encapsulating the generated column list to SQL
+     * to differentiate between the two invocations
+     */
+    public void setGeneratedColumnInfo(int generate, ResultMetaData meta) {
+
+        // also supports INSERT_SELECT
+        if (type != StatementTypes.INSERT) {
+            return;
+        }
+
+        int idColIndex = baseTable.getIdentityColumnIndex();
+
+        generatedType          = generate;
+        generatedInputMetaData = meta;
+
+        switch (generate) {
+
+            case ResultConstants.RETURN_NO_GENERATED_KEYS :
+                return;
+
+            case ResultConstants.RETURN_GENERATED_KEYS_COL_INDEXES :
+                generatedIndexes = meta.getGeneratedColumnIndexes();
+
+                for (int i = 0; i < generatedIndexes.length; i++) {
+                    if (generatedIndexes[i] < 0
+                            || generatedIndexes[i]
+                               >= baseTable.getColumnCount()) {
+                        throw Error.error(ErrorCode.X_42501);
+                    }
+                }
+                break;
+
+            case ResultConstants.RETURN_GENERATED_KEYS :
+                if (baseTable.hasGeneratedColumn()) {
+                    if (idColIndex >= 0) {
+                        int generatedCount =
+                            ArrayUtil.countTrueElements(baseTable.colGenerated)
+                            + 1;
+
+                        generatedIndexes = new int[generatedCount];
+
+                        for (int i = 0, j = 0;
+                                i < baseTable.colGenerated.length; i++) {
+                            if (baseTable.colGenerated[i] || i == idColIndex) {
+                                generatedIndexes[j++] = i;
+                            }
+                        }
+                    } else {
+                        generatedIndexes = ArrayUtil.booleanArrayToIntIndexes(
+                            baseTable.colGenerated);
+                    }
+                } else if (idColIndex >= 0) {
+                    generatedIndexes = new int[]{ idColIndex };
+                } else {
+                    return;
+                }
+                break;
+
+            case ResultConstants.RETURN_GENERATED_KEYS_COL_NAMES :
+                String[] columnNames = meta.getGeneratedColumnNames();
+
+                generatedIndexes = baseTable.getColumnIndexes(columnNames);
+
+                for (int i = 0; i < generatedIndexes.length; i++) {
+                    if (generatedIndexes[i] < 0) {
+                        throw Error.error(ErrorCode.X_42501, columnNames[0]);
+                    }
+                }
+                break;
+        }
+
+        generatedResultMetaData =
+            ResultMetaData.newResultMetaData(generatedIndexes.length);
+
+        for (int i = 0; i < generatedIndexes.length; i++) {
+            ColumnSchema column = baseTable.getColumn(generatedIndexes[i]);
+
+            generatedResultMetaData.columns[i] = column;
+        }
+
+        generatedResultMetaData.prepareData();
+
+        isSimpleInsert = false;
+    }
+
+    Object[] getGeneratedColumns(Object[] data) {
+
+        if (generatedIndexes == null) {
+            return null;
+        }
+
+        Object[] values = new Object[generatedIndexes.length];
+
+        for (int i = 0; i < generatedIndexes.length; i++) {
+            values[i] = data[generatedIndexes[i]];
+        }
+
+        return values;
+    }
+
+    public boolean hasGeneratedColumns() {
+        return generatedIndexes != null;
+    }
+
+    public ResultMetaData generatedResultMetaData() {
+        return generatedResultMetaData;
+    }
+
     void getTriggerTableNames(OrderedHashSet set, boolean write) {
 
         for (int i = 0; i < baseTable.triggerList.length; i++) {
@@ -333,12 +487,23 @@ public class StatementDML extends StatementDMQL {
      */
     Result executeUpdateStatement(Session session) {
 
-        int                       count          = 0;
-        Expression[]              colExpressions = updateExpressions;
-        RowSetNavigatorDataChange rowset = new RowSetNavigatorDataChange();
-        Type[]                    colTypes       = baseTable.getColumnTypes();
+        int          count          = 0;
+        Expression[] colExpressions = updateExpressions;
+        RowSetNavigatorDataChange rowset =
+            session.sessionContext.getRowSetDataChange();
+        Type[] colTypes = baseTable.getColumnTypes();
         RangeIterator it = RangeVariable.getIterator(session,
             targetRangeVariables);
+        Result          resultOut          = null;
+        RowSetNavigator generatedNavigator = null;
+
+        if (generatedIndexes != null) {
+            resultOut = Result.newUpdateCountResult(generatedResultMetaData,
+                    0);
+            generatedNavigator = resultOut.getChainedResult().getNavigator();
+        }
+
+        session.sessionContext.rownum = 1;
 
         while (it.next()) {
             session.sessionData.startRowProcessing();
@@ -362,8 +527,11 @@ public class StatementDML extends StatementDMQL {
             }
 
             rowset.addRow(session, row, newData, colTypes, updateColumnMap);
+
+            session.sessionContext.rownum++;
         }
 
+        rowset.endMainDataSet();
         it.release();
 /* debug 190
         if (rowset.size() == 0) {
@@ -377,15 +545,27 @@ public class StatementDML extends StatementDMQL {
 //* debug 190 */
         rowset.beforeFirst();
 
-        count = update(session, baseTable, rowset);
+        count = update(session, baseTable, rowset, generatedNavigator);
 
-        if (count == 1) {
-            return Result.updateOneResult;
-        } else if (count == 0) {
-            return Result.updateZeroResult;
+        if (resultOut == null) {
+            if (count == 1) {
+                return Result.updateOneResult;
+            } else if (count == 0) {
+                session.addWarning(HsqlException.noDataCondition);
+
+                return Result.updateZeroResult;
+            }
+
+            return new Result(ResultConstants.UPDATECOUNT, count);
+        } else {
+            resultOut.setUpdateCount(count);
+
+            if (count == 0) {
+                session.addWarning(HsqlException.noDataCondition);
+            }
+
+            return resultOut;
         }
-
-        return new Result(ResultConstants.UPDATECOUNT, count);
     }
 
     static Object[] getUpdatedData(Session session, Expression[] targets,
@@ -436,7 +616,7 @@ public class StatementDML extends StatementDMQL {
                 for (int j = 0; j < values.length; j++, i++) {
                     int colIndex = columnMap[i];
                     Type colType =
-                        expr.subQuery.queryExpression.getMetaData()
+                        expr.table.queryExpression.getMetaData()
                             .columnTypes[j];
 
                     data[colIndex] = colTypes[colIndex].convertToType(session,
@@ -502,7 +682,7 @@ public class StatementDML extends StatementDMQL {
 
         // rowset for update operation
         RowSetNavigatorDataChange updateRowSet =
-            new RowSetNavigatorDataChange();
+            session.sessionContext.getRowSetDataChange();
         RangeVariable[] joinRangeIterators = targetRangeVariables;
 
         // populate insert and update lists
@@ -545,6 +725,9 @@ public class StatementDML extends StatementDMQL {
             // row matches!
             if (updateExpressions.length != 0) {
                 Row row = it.getCurrentRow();    // this is always the second iterator
+
+                session.sessionData.startRowProcessing();
+
                 Object[] data = getUpdatedData(session, targets, baseTable,
                                                updateColumnMap,
                                                updateExpressions, colTypes,
@@ -563,6 +746,8 @@ public class StatementDML extends StatementDMQL {
             }
         }
 
+        updateRowSet.endMainDataSet();
+
         for (int i = 0; i < joinRangeIterators.length; i++) {
             rangeIterators[i].reset();
         }
@@ -570,7 +755,8 @@ public class StatementDML extends StatementDMQL {
         // run the transaction as a whole, updating and inserting where needed
         // update any matched rows
         if (updateExpressions.length != 0) {
-            count = update(session, baseTable, updateRowSet);
+            count = update(session, baseTable, updateRowSet,
+                           generatedNavigator);
         }
 
         // insert any non-matched rows
@@ -590,10 +776,20 @@ public class StatementDML extends StatementDMQL {
                 return Result.updateOneResult;
             }
 
+            if (count == 0) {
+                session.addWarning(HsqlException.noDataCondition);
+
+                return Result.updateZeroResult;
+            }
+
             return new Result(ResultConstants.UPDATECOUNT, count);
         } else {
             resultOut.setUpdateCount(count);
 
+            if (count == 0) {
+                session.addWarning(HsqlException.noDataCondition);
+            }
+
             return resultOut;
         }
     }
@@ -624,6 +820,8 @@ public class StatementDML extends StatementDMQL {
         while (newData.hasNext()) {
             Object[] data = (Object[]) newData.getNext();
 
+            // for identity using global sequence
+            session.sessionData.startRowProcessing();
             baseTable.insertSingleRow(session, store, data, null);
 
             if (checkIterator != null) {
@@ -716,8 +914,6 @@ public class StatementDML extends StatementDMQL {
                 if (baseTable.colDefaults[colIndex] != null) {
                     data[colIndex] =
                         baseTable.colDefaults[colIndex].getValue(session);
-
-                    continue;
                 }
 
                 continue;
@@ -726,7 +922,28 @@ public class StatementDML extends StatementDMQL {
             Object value = e.getValue(session);
             Type   type  = colTypes[colIndex];
 
-            if (colTypes[colIndex] != e.dataType) {
+            if (session.database.sqlSyntaxMys
+                    || session.database.sqlSyntaxPgs) {
+                try {
+                    value = type.convertToType(session, value, e.dataType);
+                } catch (HsqlException ex) {
+                    if (type.typeCode == Types.SQL_DATE) {
+                        value = Type.SQL_TIMESTAMP.convertToType(session,
+                                value, e.dataType);
+                        value = type.convertToType(session, value,
+                                                   Type.SQL_TIMESTAMP);
+                    } else if (type.typeCode == Types.SQL_TIMESTAMP) {
+                        value = Type.SQL_DATE.convertToType(session, value,
+                                                            e.dataType);
+                        value = type.convertToType(session, value,
+                                                   Type.SQL_DATE);
+                    } else {
+                        throw ex;
+                    }
+                }
+            } else {
+
+                // DYNAMIC_PARAM and PARAMETER expressions may have wider values
                 value = type.convertToType(session, value, e.dataType);
             }
 
@@ -756,7 +973,8 @@ public class StatementDML extends StatementDMQL {
      * @return int
      */
     int update(Session session, Table table,
-               RowSetNavigatorDataChange navigator) {
+               RowSetNavigatorDataChange navigator,
+               RowSetNavigator generatedNavigator) {
 
         int rowCount = navigator.getSize();
 
@@ -766,6 +984,9 @@ public class StatementDML extends StatementDMQL {
 
             Object[] data = navigator.getCurrentChangedData();
 
+            // for identity using global sequence
+            session.sessionData.startRowProcessing();
+
             /**
              * @todo 1.9.0 - make optional using database property -
              * this means the identity column can be set to null to force
@@ -781,7 +1002,9 @@ public class StatementDML extends StatementDMQL {
             HashSet path = session.sessionContext.getConstraintPath();
 
             for (int i = 0; i < rowCount; i++) {
-                Row      row  = navigator.getNextRow();
+                navigator.next();
+
+                Row      row  = navigator.getCurrentRow();
                 Object[] data = navigator.getCurrentChangedData();
 
                 performReferentialActions(session, table, navigator, row,
@@ -792,12 +1015,16 @@ public class StatementDML extends StatementDMQL {
             navigator.beforeFirst();
         }
 
-        for (int i = 0; i < navigator.getSize(); i++) {
-            Row      row            = navigator.getNextRow();
+        while (navigator.next()) {
+            Row      row            = navigator.getCurrentRow();
             Object[] data           = navigator.getCurrentChangedData();
             int[]    changedColumns = navigator.getCurrentChangedColumns();
             Table    currentTable   = ((Table) row.getTable());
 
+            if (currentTable instanceof TableDerived) {
+                currentTable = ((TableDerived) currentTable).view;
+            }
+
             if (currentTable.triggerLists[Trigger.UPDATE_BEFORE_ROW].length
                     > 0) {
                 currentTable.fireTriggers(session, Trigger.UPDATE_BEFORE_ROW,
@@ -812,8 +1039,8 @@ public class StatementDML extends StatementDMQL {
 
         navigator.beforeFirst();
 
-        for (int i = 0; i < navigator.getSize(); i++) {
-            Row   row            = navigator.getNextRow();
+        while (navigator.next()) {
+            Row   row            = navigator.getCurrentRow();
             Table currentTable   = ((Table) row.getTable());
             int[] changedColumns = navigator.getCurrentChangedColumns();
 
@@ -822,8 +1049,8 @@ public class StatementDML extends StatementDMQL {
 
         navigator.beforeFirst();
 
-        for (int i = 0; i < navigator.getSize(); i++) {
-            Row             row          = navigator.getNextRow();
+        while (navigator.next()) {
+            Row             row          = navigator.getCurrentRow();
             Object[]        data         = navigator.getCurrentChangedData();
             Table           currentTable = ((Table) row.getTable());
             int[] changedColumns = navigator.getCurrentChangedColumns();
@@ -836,6 +1063,12 @@ public class StatementDML extends StatementDMQL {
             Row newRow = currentTable.insertSingleRow(session, store, data,
                 changedColumns);
 
+            if (generatedNavigator != null) {
+                Object[] generatedValues = getGeneratedColumns(data);
+
+                generatedNavigator.add(generatedValues);
+            }
+
 //            newRow.rowAction.updatedAction = row.rowAction;
         }
 
@@ -845,8 +1078,8 @@ public class StatementDML extends StatementDMQL {
         boolean hasAfterRowTriggers =
             table.triggerLists[Trigger.UPDATE_AFTER_ROW].length > 0;
 
-        for (int i = 0; i < navigator.getSize(); i++) {
-            Row      row            = navigator.getNextRow();
+        while (navigator.next()) {
+            Row      row            = navigator.getCurrentRow();
             Table    currentTable   = ((Table) row.getTable());
             Object[] changedData    = navigator.getCurrentChangedData();
             int[]    changedColumns = navigator.getCurrentChangedColumns();
@@ -871,8 +1104,8 @@ public class StatementDML extends StatementDMQL {
         navigator.beforeFirst();
 
         if (hasAfterRowTriggers) {
-            for (int i = 0; i < navigator.getSize(); i++) {
-                Row      row            = navigator.getNextRow();
+            while (navigator.next()) {
+                Row      row            = navigator.getCurrentRow();
                 Object[] changedData    = navigator.getCurrentChangedData();
                 int[]    changedColumns = navigator.getCurrentChangedColumns();
                 Table    currentTable   = ((Table) row.getTable());
@@ -909,19 +1142,27 @@ public class StatementDML extends StatementDMQL {
         int count = 0;
         RangeIterator it = RangeVariable.getIterator(session,
             targetRangeVariables);
-        RowSetNavigatorDataChange navigator = new RowSetNavigatorDataChange();
+        RowSetNavigatorDataChange rowset =
+            session.sessionContext.getRowSetDataChange();
+
+        session.sessionContext.rownum = 1;
 
         while (it.next()) {
             Row currentRow = it.getCurrentRow();
 
-            navigator.addRow(currentRow);
+            rowset.addRow(currentRow);
+
+            session.sessionContext.rownum++;
         }
 
         it.release();
+        rowset.endMainDataSet();
 
-        if (navigator.getSize() > 0) {
-            count = delete(session, baseTable, navigator);
+        if (rowset.getSize() > 0) {
+            count = delete(session, baseTable, rowset);
         } else {
+            session.addWarning(HsqlException.noDataCondition);
+
             return Result.updateZeroResult;
         }
 
@@ -934,8 +1175,24 @@ public class StatementDML extends StatementDMQL {
 
     Result executeDeleteTruncateStatement(Session session) {
 
-        PersistentStore store = targetTable.getRowStore(session);
-        RowIterator     it    = targetTable.getPrimaryIndex().firstRow(store);
+        PersistentStore store   = targetTable.getRowStore(session);
+        RowIterator     it = targetTable.getPrimaryIndex().firstRow(store);
+        boolean         hasData = it.hasNext();
+
+        for (int i = 0; i < targetTable.fkMainConstraints.length; i++) {
+            if (targetTable.fkMainConstraints[i].getRef() != targetTable) {
+                HsqlName tableName =
+                    targetTable.fkMainConstraints[i].getRef().getName();
+                Table refTable =
+                    session.database.schemaManager.getUserTable(session,
+                        tableName);
+
+                if (!refTable.isEmpty(session)) {
+                    throw Error.error(ErrorCode.X_23504,
+                                      refTable.getName().name);
+                }
+            }
+        }
 
         try {
             while (it.hasNext()) {
@@ -951,6 +1208,10 @@ public class StatementDML extends StatementDMQL {
             it.release();
         }
 
+        if (!hasData) {
+            session.addWarning(HsqlException.noDataCondition);
+        }
+
         return Result.updateOneResult;
     }
 
@@ -981,14 +1242,16 @@ public class StatementDML extends StatementDMQL {
             navigator.beforeFirst();
         }
 
-        while (navigator.hasNext()) {
-            navigator.next();
-
+        while (navigator.next()) {
             Row      row            = navigator.getCurrentRow();
             Object[] changedData    = navigator.getCurrentChangedData();
             int[]    changedColumns = navigator.getCurrentChangedColumns();
             Table    currentTable   = ((Table) row.getTable());
 
+            if (currentTable instanceof TableDerived) {
+                currentTable = ((TableDerived) currentTable).view;
+            }
+
             if (changedData == null) {
                 currentTable.fireTriggers(session, Trigger.DELETE_BEFORE_ROW,
                                           row.getData(), null, null);
@@ -1007,8 +1270,8 @@ public class StatementDML extends StatementDMQL {
 
         boolean hasUpdate = false;
 
-        for (int i = 0; i < navigator.getSize(); i++) {
-            Row      row          = navigator.getNextRow();
+        while (navigator.next()) {
+            Row      row          = navigator.getCurrentRow();
             Object[] data         = navigator.getCurrentChangedData();
             Table    currentTable = ((Table) row.getTable());
 
@@ -1022,8 +1285,8 @@ public class StatementDML extends StatementDMQL {
         navigator.beforeFirst();
 
         if (hasUpdate) {
-            for (int i = 0; i < navigator.getSize(); i++) {
-                Row             row          = navigator.getNextRow();
+            while (navigator.next()) {
+                Row             row          = navigator.getCurrentRow();
                 Object[]        data = navigator.getCurrentChangedData();
                 Table           currentTable = ((Table) row.getTable());
                 int[] changedColumns = navigator.getCurrentChangedColumns();
@@ -1048,9 +1311,7 @@ public class StatementDML extends StatementDMQL {
             table.triggerLists[Trigger.DELETE_AFTER_ROW].length > 0;
 
         if (rowCount != navigator.getSize()) {
-            while (navigator.hasNext()) {
-                navigator.next();
-
+            while (navigator.next()) {
                 Row      row            = navigator.getCurrentRow();
                 Object[] changedData    = navigator.getCurrentChangedData();
                 int[]    changedColumns = navigator.getCurrentChangedColumns();
@@ -1093,9 +1354,7 @@ public class StatementDML extends StatementDMQL {
         }
 
         if (hasAfterRowTriggers) {
-            while (navigator.hasNext()) {
-                navigator.next();
-
+            while (navigator.next()) {
                 Row      row          = navigator.getCurrentRow();
                 Object[] changedData  = navigator.getCurrentChangedData();
                 Table    currentTable = ((Table) row.getTable());
@@ -1225,7 +1484,20 @@ public class StatementDML extends StatementDMQL {
 
                     case SchemaObject.ReferentialAction.CASCADE : {
                         if (delete) {
-                            if (navigator.addRow(refRow)) {
+                            boolean result;
+
+                            try {
+                                result = navigator.addRow(refRow);
+                            } catch (HsqlException e) {
+                                String[] info = getConstraintInfo(c);
+
+                                refiterator.release();
+
+                                throw Error.error(null, ErrorCode.X_27000,
+                                                  ErrorCode.CONSTRAINT, info);
+                            }
+
+                            if (result) {
                                 performReferentialActions(session,
                                                           c.core.refTable,
                                                           navigator, refRow,
@@ -1276,18 +1548,17 @@ public class StatementDML extends StatementDMQL {
                         break;
                     }
                     case SchemaObject.ReferentialAction.NO_ACTION :
-                    case SchemaObject.ReferentialAction.RESTRICT : {
                         if (navigator.containsDeletedRow(refRow)) {
                             continue;
                         }
 
+                    // fall through
+                    case SchemaObject.ReferentialAction.RESTRICT : {
                         int errorCode = c.core.deleteAction
                                         == SchemaObject.ReferentialAction
                                             .NO_ACTION ? ErrorCode.X_23504
                                                        : ErrorCode.X_23001;
-                        String[] info = new String[] {
-                            c.core.refName.name, c.core.refTable.getName().name
-                        };
+                        String[] info = getConstraintInfo(c);
 
                         refiterator.release();
 
@@ -1298,9 +1569,25 @@ public class StatementDML extends StatementDMQL {
                         continue;
                 }
 
-                refData = navigator.addRow(session, refRow, refData,
-                                           table.getColumnTypes(),
-                                           c.core.refCols);
+                try {
+                    refData =
+                        navigator.addRow(session, refRow, refData,
+                                         c.core.refTable.getColumnTypes(),
+                                         c.core.refCols);
+                } catch (HsqlException e) {
+                    String[] info = getConstraintInfo(c);
+
+                    refiterator.release();
+
+                    throw Error.error(null, ErrorCode.X_27000,
+                                      ErrorCode.CONSTRAINT, info);
+                }
+
+                if (refData == null) {
+
+                    // happens only with enforceDeleteOrUpdate=false and updated row is already deleted
+                    continue;
+                }
 
                 if (!path.add(c)) {
                     continue;
@@ -1315,4 +1602,15 @@ public class StatementDML extends StatementDMQL {
             refiterator.release();
         }
     }
+
+    static String[] getConstraintInfo(Constraint c) {
+
+        return new String[] {
+            c.core.refName.name, c.core.refTable.getName().name
+        };
+    }
+
+    public void clearStructures(Session session) {
+        session.sessionContext.clearStructures(this);
+    }
 }
