@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,7 @@
 package org.hsqldb;
 
 import org.hsqldb.HsqlNameManager.HsqlName;
+import org.hsqldb.HsqlNameManager.SimpleName;
 import org.hsqldb.ParserDQL.CompileContext;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
@@ -41,14 +42,15 @@ import org.hsqldb.lib.HashSet;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultMetaData;
+import org.hsqldb.rights.Grantee;
 import org.hsqldb.store.ValuePool;
 
 /**
  * Statement implementation for DML and base DQL statements.
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.7
  * @since 1.7.2
  */
 
@@ -102,12 +104,23 @@ public abstract class StatementDMQL extends Statement {
     boolean[] insertCheckColumns;
     boolean[] updateCheckColumns;
 
+    /**
+     * VIEW check
+     */
+    Expression    updatableTableCheck;
+    RangeVariable checkRangeVariable;
+
     /**
      * Select to be evaluated when this is an INSERT_SELECT or
      * SELECT statement
      */
     QueryExpression queryExpression;
 
+    /**
+     * Name of cursor
+     */
+    SimpleName cursorName;
+
     /**
      * Parse-order array of Expression objects, all of type PARAMETER ,
      * involved in some way in any INSERT_XXX, UPDATE, DELETE, SELECT or
@@ -115,16 +128,6 @@ public abstract class StatementDMQL extends Statement {
      */
     ExpressionColumn[] parameters;
 
-    /**
-     * int[] contains column indexes for generated values
-     */
-    int[] generatedIndexes;
-
-    /**
-     * ResultMetaData for generated values
-     */
-    ResultMetaData generatedResultMetaData;
-
     /**
      * ResultMetaData for parameters
      */
@@ -133,7 +136,7 @@ public abstract class StatementDMQL extends Statement {
     /**
      * Subqueries inverse usage depth order
      */
-    SubQuery[] subqueries = SubQuery.emptySubqueryArray;
+    TableDerived[] subqueries = TableDerived.emptyArray;
 
     /**
      * Total number of RangeIterator objects used
@@ -162,6 +165,14 @@ public abstract class StatementDMQL extends Statement {
         }
     }
 
+    public void setCursorName(SimpleName name) {
+        cursorName = name;
+    }
+
+    public SimpleName getCursorName() {
+        return cursorName;
+    }
+
     public Result execute(Session session) {
 
         Result result;
@@ -183,14 +194,16 @@ public abstract class StatementDMQL extends Statement {
             }
 
             result = getResult(session);
+
+            clearStructures(session);
         } catch (Throwable t) {
+            clearStructures(session);
+
             result = Result.newErrorResult(t, null);
 
             result.getException().setStatementType(group, type);
         }
 
-        session.sessionContext.clearStructures(this);
-
         return result;
     }
 
@@ -236,29 +249,6 @@ public abstract class StatementDMQL extends Statement {
 
     abstract void collectTableNamesForWrite(OrderedHashSet set);
 
-    Object[] getGeneratedColumns(Object[] data) {
-
-        if (generatedIndexes == null) {
-            return null;
-        }
-
-        Object[] values = new Object[generatedIndexes.length];
-
-        for (int i = 0; i < generatedIndexes.length; i++) {
-            values[i] = data[generatedIndexes[i]];
-        }
-
-        return values;
-    }
-
-    public boolean hasGeneratedColumns() {
-        return generatedIndexes != null;
-    }
-
-    public ResultMetaData generatedResultMetaData() {
-        return generatedResultMetaData;
-    }
-
     boolean[] getInsertOrUpdateColumnCheckList() {
 
         switch (type) {
@@ -281,32 +271,25 @@ public abstract class StatementDMQL extends Statement {
         return null;
     }
 
-    private void setParameters() {
-
-        for (int i = 0; i < parameters.length; i++) {
-            parameters[i].parameterIndex = i;
-        }
-    }
-
     void materializeSubQueries(Session session) {
 
         HashSet subqueryPopFlags = new HashSet();
 
         for (int i = 0; i < subqueries.length; i++) {
-            SubQuery sq = subqueries[i];
+            TableDerived td = subqueries[i];
 
-            // VIEW working tables may be reused in a single query but they are filled only once
-            if (!subqueryPopFlags.add(sq)) {
+            if (!subqueryPopFlags.add(td)) {
                 continue;
             }
 
-            if (!sq.isCorrelated()) {
-                sq.materialise(session);
+            if (!td.isCorrelated()) {
+                td.materialise(session);
             }
         }
     }
 
-    SubQuery[] getSubqueries(Session session) {
+
+    TableDerived[] getSubqueries(Session session) {
 
         OrderedHashSet subQueries = null;
 
@@ -338,20 +321,22 @@ public abstract class StatementDMQL extends Statement {
             subQueries = OrderedHashSet.addAll(subQueries, set);
         }
 
-        if (subQueries == null || subQueries.size() == 0) {
-            return SubQuery.emptySubqueryArray;
+        if (updatableTableCheck != null) {
+            OrderedHashSet set = updatableTableCheck.getSubqueries();
+
+            subQueries = OrderedHashSet.addAll(subQueries, set);
         }
 
-        SubQuery[] subQueryArray = new SubQuery[subQueries.size()];
+        if (subQueries == null || subQueries.size() == 0) {
+            return TableDerived.emptyArray;
+        }
+
+        TableDerived[] subQueryArray = new TableDerived[subQueries.size()];
 
         subQueries.toArray(subQueryArray);
         ArraySort.sort(subQueryArray, 0, subQueryArray.length,
                        subQueryArray[0]);
 
-        for (int i = 0; i < subQueryArray.length; i++) {
-            subQueryArray[i].prepareTable(session);
-        }
-
         return subQueryArray;
     }
 
@@ -359,12 +344,11 @@ public abstract class StatementDMQL extends Statement {
 
         parameters = compileContext.getParameters();
 
-        setParameters();
         setParameterMetaData();
 
         subqueries         = getSubqueries(session);
         rangeIteratorCount = compileContext.getRangeVarCount();
-        rangeVariables     = compileContext.getRangeVariables();
+        rangeVariables     = compileContext.getAllRangeVariables();
         sequences          = compileContext.getSequences();
         routines           = compileContext.getRoutines();
 
@@ -403,17 +387,19 @@ public abstract class StatementDMQL extends Statement {
     void checkAccessRights(Session session) {
 
         if (targetTable != null && !targetTable.isTemp()) {
-            if (targetTable.getOwner().isSystem()) {
+            if (!session.isProcessingScript) {
+                targetTable.checkDataReadOnly();
+            }
+
+            Grantee owner = targetTable.getOwner();
+
+            if (owner != null && owner.isSystem()) {
                 if (!session.getUser().isSystem()) {
                     throw Error.error(ErrorCode.X_42501,
                                       targetTable.getName().name);
                 }
             }
 
-            if (!session.isProcessingScript) {
-                targetTable.checkDataReadOnly();
-            }
-
             session.checkReadWrite();
         }
 
@@ -567,6 +553,10 @@ public abstract class StatementDMQL extends Statement {
                                                   + (i + 1);
             parameterMetaData.columnTypes[idx] = parameters[i].dataType;
 
+            if (parameters[i].dataType == null) {
+                throw Error.error(ErrorCode.X_42567);
+            }
+
             byte parameterMode = SchemaObject.ParameterModes.PARAM_IN;
 
             if (parameters[i].column != null
@@ -709,9 +699,15 @@ public abstract class StatementDMQL extends Statement {
         sb.append("SUBQUERIES[");
 
         for (int i = 0; i < subqueries.length; i++) {
-            sb.append("\n[level=").append(subqueries[i].level).append('\n');
+            sb.append("\n[level=").append(subqueries[i].depth).append('\n');
 
-            if (subqueries[i].queryExpression != null) {
+            if (subqueries[i].queryExpression == null) {
+                for (int j = 0; j < blanks; j++) {
+                    sb.append(' ');
+                }
+
+                sb.append("value expression");
+            } else {
                 sb.append(subqueries[i].queryExpression.describe(session,
                         blanks));
             }
@@ -808,11 +804,15 @@ public abstract class StatementDMQL extends Statement {
 
     public void resolve(Session session) {}
 
-    public RangeVariable[] getRangeVariables() {
-        return rangeVariables;
-    }
-
-    public final boolean isCatalogChange() {
+    public final boolean isCatalogLock() {
         return false;
     }
+
+    public boolean isCatalogChange() {
+        return false;
+    }
+
+    public void clearStructures(Session session) {
+        session.sessionContext.clearStructures(this);
+    }
 }
