@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,7 @@ package org.hsqldb;
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.index.Index;
 import org.hsqldb.lib.HsqlArrayList;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.navigator.RowIterator;
@@ -43,13 +44,14 @@ import org.hsqldb.rights.GranteeManager;
 import org.hsqldb.rights.Right;
 import org.hsqldb.store.ValuePool;
 import org.hsqldb.types.Charset;
+import org.hsqldb.types.Collation;
 import org.hsqldb.types.Type;
 
 /**
  * Implementation of Statement for DDL statements.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.1.1
  * @since 1.9.0
  */
 public class StatementSchema extends Statement {
@@ -59,25 +61,23 @@ public class StatementSchema extends Statement {
     boolean  isSchemaDefinition;
     Token[]  statementTokens;
 
-    StatementSchema() {
+    StatementSchema(int type, int group) {
 
-        super(StatementTypes.CREATE_SCHEMA,
-              StatementTypes.X_SQL_SCHEMA_DEFINITION);
+        super(type, group);
 
         isTransactionStatement = true;
     }
 
-    StatementSchema(String sql, int type, HsqlName readName,
-                    HsqlName writeName) {
-        this(sql, type, null, readName, writeName);
+    StatementSchema(String sql, int type) {
+        this(sql, type, null, (HsqlName[]) null, null);
     }
 
     StatementSchema(String sql, int type, Object[] args) {
-        this(sql, type, args, null, null);
+        this(sql, type, args, (HsqlName[]) null, null);
     }
 
-    StatementSchema(String sql, int type, Object[] args, HsqlName readName,
-                    HsqlName writeName) {
+    StatementSchema(String sql, int type, Object[] args, HsqlName[] readName,
+                    HsqlName[] writeName) {
 
         super(type);
 
@@ -88,12 +88,12 @@ public class StatementSchema extends Statement {
             arguments = args;
         }
 
-        if (readName != null && readName != writeName) {
-            readTableNames = new HsqlName[]{ readName };
+        if (readName != null) {
+            readTableNames = readName;
         }
 
         if (writeName != null) {
-            writeTableNames = new HsqlName[]{ writeName };
+            writeTableNames = writeName;
         }
 
         switch (type) {
@@ -103,6 +103,7 @@ public class StatementSchema extends Statement {
                 break;
 
             case StatementTypes.ALTER_DOMAIN :
+            case StatementTypes.ALTER_INDEX :
             case StatementTypes.ALTER_ROUTINE :
             case StatementTypes.ALTER_SEQUENCE :
             case StatementTypes.ALTER_TYPE :
@@ -283,7 +284,7 @@ public class StatementSchema extends Statement {
 
         try {
             if (isLogged) {
-                session.database.logger.writeToLog(session, sql);
+                session.database.logger.writeOtherStatement(session, sql);
             }
         } catch (Throwable e) {
             return Result.newErrorResult(e, sql);
@@ -384,6 +385,26 @@ public class StatementSchema extends Statement {
                     return Result.newErrorResult(e, sql);
                 }
             }
+            case StatementTypes.ALTER_INDEX : {
+                Table    table        = (Table) arguments[0];
+                int[]    indexColumns = (int[]) arguments[1];
+                HsqlName name         = (HsqlName) arguments[2];
+                Index    index;
+
+                try {
+                    index =
+                        (Index) session.database.schemaManager.getSchemaObject(
+                            name);
+
+                    TableWorks tableWorks = new TableWorks(session, table);
+
+                    tableWorks.alterIndex(index, indexColumns);
+
+                    break;
+                } catch (HsqlException e) {
+                    return Result.newErrorResult(e, sql);
+                }
+            }
             case StatementTypes.ALTER_SEQUENCE : {
                 try {
                     NumberSequence sequence = (NumberSequence) arguments[0];
@@ -399,19 +420,223 @@ public class StatementSchema extends Statement {
                 }
             }
             case StatementTypes.ALTER_DOMAIN :
-            case StatementTypes.ALTER_ROUTINE :
-            case StatementTypes.ALTER_TYPE :
-            case StatementTypes.ALTER_TABLE :
-            case StatementTypes.ALTER_TRANSFORM : {
                 try {
-                    session.parser.reset(sql);
-                    session.parser.read();
-                    session.parser.processAlter();
+                    int  subType = ((Integer) arguments[0]).intValue();
+                    Type domain  = (Type) arguments[1];
+
+                    switch (subType) {
+
+                        case StatementTypes.ADD_CONSTRAINT : {
+                            Constraint c = (Constraint) arguments[2];
+
+                            session.database.schemaManager
+                                .checkSchemaObjectNotExists(c.getName());
+                            domain.userTypeModifier.addConstraint(c);
+                            session.database.schemaManager.addSchemaObject(c);
+
+                            break;
+                        }
+                        case StatementTypes.ADD_DEFAULT : {
+                            Expression e = (Expression) arguments[2];
+
+                            domain.userTypeModifier.setDefaultClause(e);
+
+                            break;
+                        }
+                        case StatementTypes.DROP_CONSTRAINT : {
+                            HsqlName name = (HsqlName) arguments[2];
+
+                            session.database.schemaManager.removeSchemaObject(
+                                name);
+
+                            break;
+                        }
+                        case StatementTypes.DROP_DEFAULT : {
+                            domain.userTypeModifier.removeDefaultClause();
+
+                            break;
+                        }
+                    }
 
                     break;
                 } catch (HsqlException e) {
                     return Result.newErrorResult(e, sql);
                 }
+            case StatementTypes.ALTER_TABLE :
+                try {
+                    int   subType = ((Integer) arguments[0]).intValue();
+                    Table table   = (Table) arguments[1];
+
+                    switch (subType) {
+
+                        case StatementTypes.ADD_CONSTRAINT : {
+                            Constraint c = (Constraint) arguments[2];
+
+                            switch (c.getConstraintType()) {
+
+                                case SchemaObject.ConstraintTypes
+                                        .PRIMARY_KEY : {
+                                    TableWorks tableWorks =
+                                        new TableWorks(session, table);
+
+                                    tableWorks.addPrimaryKey(c);
+
+                                    break;
+                                }
+                                case SchemaObject.ConstraintTypes.UNIQUE : {
+                                    TableWorks tableWorks =
+                                        new TableWorks(session, table);
+
+                                    tableWorks.addUniqueConstraint(c);
+
+                                    break;
+                                }
+                                case SchemaObject.ConstraintTypes
+                                        .FOREIGN_KEY : {
+                                    TableWorks tableWorks =
+                                        new TableWorks(session, table);
+
+                                    tableWorks.addForeignKey(c);
+
+                                    break;
+                                }
+                                case SchemaObject.ConstraintTypes.CHECK : {
+                                    TableWorks tableWorks =
+                                        new TableWorks(session, table);
+
+                                    tableWorks.addCheckConstraint(c);
+
+                                    break;
+                                }
+                            }
+
+                            break;
+                        }
+                        case StatementTypes.ADD_COLUMN : {
+                            ColumnSchema  column = (ColumnSchema) arguments[2];
+                            int colIndex = ((Integer) arguments[3]).intValue();
+                            HsqlArrayList list = (HsqlArrayList) arguments[4];
+                            TableWorks tableWorks = new TableWorks(session,
+                                                                   table);
+
+                            tableWorks.addColumn(column, colIndex, list);
+
+                            break;
+                        }
+                        case StatementTypes.ALTER_COLUMN_TYPE : {
+                            ColumnSchema column = (ColumnSchema) arguments[2];
+                            Type         type   = (Type) arguments[3];
+                            ColumnSchema newCol = column.duplicate();
+
+                            newCol.setType(type);
+
+                            TableWorks tw = new TableWorks(session, table);
+
+                            tw.retypeColumn(column, newCol);
+
+                            break;
+                        }
+                        case StatementTypes.ALTER_COLUMN_TYPE_IDENTITY : {
+                            ColumnSchema column = (ColumnSchema) arguments[2];
+                            Type         type   = (Type) arguments[3];
+                            NumberSequence sequence =
+                                (NumberSequence) arguments[4];
+                            ColumnSchema newCol = column.duplicate();
+
+                            newCol.setType(type);
+                            newCol.setIdentity(sequence);
+
+                            TableWorks tw = new TableWorks(session, table);
+
+                            tw.retypeColumn(column, newCol);
+
+                            break;
+                        }
+                        case StatementTypes.ALTER_COLUMN_SEQUENCE : {
+                            ColumnSchema column = (ColumnSchema) arguments[2];
+                            int columnIndex =
+                                ((Integer) arguments[3]).intValue();
+                            NumberSequence sequence =
+                                (NumberSequence) arguments[4];
+
+                            if (column.isIdentity()) {
+                                column.getIdentitySequence().reset(sequence);
+                            } else {
+                                column.setIdentity(sequence);
+                                table.setColumnTypeVars(columnIndex);
+                            }
+
+                            break;
+                        }
+                        case StatementTypes.ALTER_COLUMN_NULL : {
+                            ColumnSchema column = (ColumnSchema) arguments[2];
+                            boolean nullable =
+                                ((Boolean) arguments[3]).booleanValue();
+                            TableWorks tw = new TableWorks(session, table);
+
+                            tw.setColNullability(column, nullable);
+
+                            break;
+                        }
+                        case StatementTypes.ALTER_COLUMN_DEFAULT : {
+                            ColumnSchema column = (ColumnSchema) arguments[2];
+                            int columnIndex =
+                                ((Integer) arguments[3]).intValue();
+                            Expression e  = (Expression) arguments[4];
+                            TableWorks tw = new TableWorks(session, table);
+
+                            tw.setColDefaultExpression(columnIndex, e);
+
+                            break;
+                        }
+                        case StatementTypes.ALTER_COLUMN_DROP_DEFAULT : {
+                            ColumnSchema column = (ColumnSchema) arguments[2];
+                            int columnIndex =
+                                ((Integer) arguments[3]).intValue();
+                            TableWorks tw = new TableWorks(session, table);
+
+                            tw.setColDefaultExpression(columnIndex, null);
+                            table.setColumnTypeVars(columnIndex);
+
+                            break;
+                        }
+                        case StatementTypes.ALTER_COLUMN_DROP_GENERATED : {
+                            ColumnSchema column = (ColumnSchema) arguments[2];
+                            int columnIndex =
+                                ((Integer) arguments[3]).intValue();
+
+                            column.setIdentity(null);
+                            table.setColumnTypeVars(columnIndex);
+
+                            break;
+                        }
+                    }
+
+                    break;
+                } catch (HsqlException e) {
+                    return Result.newErrorResult(e, sql);
+                }
+            case StatementTypes.ALTER_ROUTINE : {
+                Routine routine = (Routine) arguments[0];
+
+                try {
+                    routine.resolveReferences(session);
+
+                    Routine oldRoutine =
+                        (Routine) schemaManager.getSchemaObject(
+                            routine.getSpecificName());
+
+                    schemaManager.replaceReferences(oldRoutine, routine);
+                    oldRoutine.setAsAlteredRoutine(routine);
+
+                    break;
+                } catch (HsqlException e) {
+                    return Result.newErrorResult(e, sql);
+                }
+            }
+            case StatementTypes.ALTER_TYPE :
+            case StatementTypes.ALTER_TRANSFORM : {
+                throw Error.runtimeError(ErrorCode.U_S0500, "StatementSchema");
             }
             case StatementTypes.ALTER_VIEW : {
                 View view = (View) arguments[0];
@@ -432,8 +657,7 @@ public class StatementSchema extends Statement {
                     view.compile(session, null);
 
                     OrderedHashSet dependents =
-                        schemaManager.getReferencingObjectNames(
-                            oldView.getName());
+                        schemaManager.getReferencesTo(oldView.getName());
 
                     if (dependents.getCommonElementCount(view.getReferences())
                             > 0) {
@@ -476,7 +700,6 @@ public class StatementSchema extends Statement {
 
                     checkSchemaUpdateAuthorisation(session,
                                                    table.getSchemaName());
-                    session.commit(false);
 
                     TableWorks tableWorks = new TableWorks(session, table);
 
@@ -668,7 +891,6 @@ public class StatementSchema extends Statement {
                     switch (schemaObject.getType()) {
 
                         case SchemaObject.CHARSET :
-                            System.out.println("grant charset!");
                             break;
 
                         case SchemaObject.VIEW :
@@ -759,7 +981,17 @@ public class StatementSchema extends Statement {
                 }
             }
             case StatementTypes.CREATE_COLLATION : {
-                return Result.updateZeroResult;
+                Collation collation = (Collation) arguments[0];
+
+                try {
+                    setOrCheckObjectName(session, null, collation.getName(),
+                                         true);
+                    schemaManager.addSchemaObject(collation);
+
+                    break;
+                } catch (HsqlException e) {
+                    return Result.newErrorResult(e, sql);
+                }
             }
             case StatementTypes.CREATE_ROLE : {
                 try {
@@ -780,12 +1012,13 @@ public class StatementSchema extends Statement {
                 String   password = (String) arguments[1];
                 Grantee  grantor  = (Grantee) arguments[2];
                 boolean  admin    = ((Boolean) arguments[3]).booleanValue();
+                boolean  isDigest = ((Boolean) arguments[4]).booleanValue();
 
                 try {
                     session.checkAdmin();
                     session.checkDDLWrite();
                     session.database.getUserManager().createUser(name,
-                            password);
+                            password, isDigest);
 
                     if (admin) {
                         session.database.getGranteeManager().grant(name.name,
@@ -822,7 +1055,7 @@ public class StatementSchema extends Statement {
                         if (session.isProcessingScript()
                                 && session.database.getProperties()
                                     .isVersion18()) {
-                            session.setSchema(schema.getName().name);
+                            session.setCurrentSchemaHsqlName(schema.getName());
                         }
                     }
 
@@ -905,12 +1138,17 @@ public class StatementSchema extends Statement {
                 Table         table              = (Table) arguments[0];
                 HsqlArrayList tempConstraints = (HsqlArrayList) arguments[1];
                 StatementDMQL statement = (StatementDMQL) arguments[2];
+                Boolean       ifNotExists        = (Boolean) arguments[3];
                 HsqlArrayList foreignConstraints = null;
 
                 try {
                     setOrCheckObjectName(session, null, table.getName(), true);
                 } catch (HsqlException e) {
-                    return Result.newErrorResult(e, sql);
+                    if (ifNotExists != null && ifNotExists.booleanValue()) {
+                        return Result.updateZeroResult;
+                    } else {
+                        return Result.newErrorResult(e, sql);
+                    }
                 }
 
                 try {
@@ -918,7 +1156,7 @@ public class StatementSchema extends Statement {
                         foreignConstraints = new HsqlArrayList();
                     }
 
-                    if (tempConstraints != null) {
+                    if (tempConstraints.size() != 0) {
                         table =
                             ParserDDL.addTableConstraintDefinitions(session,
                                 table, tempConstraints, foreignConstraints,
@@ -1120,8 +1358,7 @@ public class StatementSchema extends Statement {
                 break;
 
             default :
-                throw Error.runtimeError(ErrorCode.U_S0500,
-                                         "CompiledStateemntSchema");
+                throw Error.runtimeError(ErrorCode.U_S0500, "StatementSchema");
         }
 
         return Result.updateZeroResult;
@@ -1145,8 +1382,7 @@ public class StatementSchema extends Statement {
         Type domain =
             (Type) session.database.schemaManager.getSchemaObject(name);
         OrderedHashSet set =
-            session.database.schemaManager.getReferencingObjectNames(
-                domain.getName());
+            session.database.schemaManager.getReferencesTo(domain.getName());
 
         if (!cascade && set.size() > 0) {
             HsqlName objectName = (HsqlName) set.get(0);
@@ -1241,7 +1477,8 @@ public class StatementSchema extends Statement {
                 cascade);
     }
 
-    void checkSchemaUpdateAuthorisation(Session session, HsqlName schema) {
+    static void checkSchemaUpdateAuthorisation(Session session,
+            HsqlName schema) {
 
         if (session.isProcessingLog) {
             return;
@@ -1319,4 +1556,8 @@ public class StatementSchema extends Statement {
     public String describe(Session session) {
         return sql;
     }
+
+    public Object[] getArguments() {
+        return arguments;
+    }
 }
