@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,7 +35,6 @@ import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.HsqlArrayList;
-import org.hsqldb.navigator.RowIterator;
 import org.hsqldb.result.Result;
 import org.hsqldb.rights.Grantee;
 import org.hsqldb.rights.User;
@@ -47,11 +46,36 @@ import org.hsqldb.types.Type;
  * Implementation of Statement for SQL session statements.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.6
  * @since 1.9.0
  */
 public class StatementSession extends Statement {
 
+    public final static StatementSession commitNoChainStatement =
+        new StatementSession(StatementTypes.COMMIT_WORK,
+                             new Object[]{ Boolean.FALSE });
+    public final static StatementSession rollbackNoChainStatement =
+        new StatementSession(StatementTypes.ROLLBACK_WORK,
+                             new Object[]{ Boolean.FALSE });
+    public final static StatementSession commitAndChainStatement =
+        new StatementSession(StatementTypes.COMMIT_WORK,
+                             new Object[]{ Boolean.TRUE });
+    public final static StatementSession rollbackAndChainStatement =
+        new StatementSession(StatementTypes.ROLLBACK_WORK,
+                             new Object[]{ Boolean.TRUE });
+
+    static {
+        commitNoChainStatement.sql   = Tokens.T_COMMIT;
+        commitAndChainStatement.sql  = Tokens.T_COMMIT + ' ' + Tokens.T_CHAIN;
+        rollbackNoChainStatement.sql = Tokens.T_ROLLBACK;
+        rollbackAndChainStatement.sql = Tokens.T_ROLLBACK + ' '
+                                        + Tokens.T_CHAIN;
+        commitNoChainStatement.compileTimestamp    = Long.MAX_VALUE;
+        commitAndChainStatement.compileTimestamp   = Long.MAX_VALUE;
+        rollbackNoChainStatement.compileTimestamp  = Long.MAX_VALUE;
+        rollbackAndChainStatement.compileTimestamp = Long.MAX_VALUE;
+    }
+
     Expression[] expressions;
     Object[]     parameters;
 
@@ -61,6 +85,7 @@ public class StatementSession extends Statement {
 
         this.expressions       = args;
         isTransactionStatement = false;
+        isLogged               = false;
 
         switch (type) {
 
@@ -110,7 +135,7 @@ public class StatementSession extends Statement {
             case StatementTypes.ALLOCATE_DESCRIPTOR :
             case StatementTypes.DEALLOCATE_DESCRIPTOR :
             case StatementTypes.DEALLOCATE_PREPARE :
-                group = StatementTypes.X_DYNAMIC;
+                group = StatementTypes.X_SQL_DYNAMIC;
                 break;
 
             //
@@ -137,7 +162,7 @@ public class StatementSession extends Statement {
             case StatementTypes.PREPARABLE_DYNAMIC_DELETE_CURSOR :
             case StatementTypes.PREPARABLE_DYNAMIC_UPDATE_CURSOR :
             case StatementTypes.PREPARE :
-                group = StatementTypes.X_DYNAMIC;
+                group = StatementTypes.X_SQL_DYNAMIC;
                 break;
 
             // logged by session
@@ -146,6 +171,7 @@ public class StatementSession extends Statement {
                 break;
 
             //
+            case StatementTypes.SET_COLLATION :
             case StatementTypes.SET_CONNECTION :
             case StatementTypes.SET_CONSTRAINT :
             case StatementTypes.SET_DESCRIPTOR :
@@ -154,10 +180,14 @@ public class StatementSession extends Statement {
             case StatementTypes.SET_SESSION_RESULT_MAX_ROWS :
             case StatementTypes.SET_SESSION_RESULT_MEMORY_ROWS :
             case StatementTypes.SET_SESSION_AUTOCOMMIT :
-            case StatementTypes.SET_SESSION_SQL_IGNORECASE :
                 group = StatementTypes.X_HSQLDB_SESSION;
                 break;
 
+            case StatementTypes.SET_SESSION_SQL_IGNORECASE :
+                isLogged = true;
+                group    = StatementTypes.X_HSQLDB_SESSION;
+                break;
+
             // logged by session if necessary
             case StatementTypes.COMMIT_WORK :
             case StatementTypes.RELEASE_SAVEPOINT :
@@ -218,7 +248,7 @@ public class StatementSession extends Statement {
 
         try {
             if (isLogged) {
-                session.database.logger.writeToLog(session, sql);
+                session.database.logger.writeOtherStatement(session, sql);
             }
         } catch (Throwable e) {
             return Result.newErrorResult(e, sql);
@@ -246,7 +276,7 @@ public class StatementSession extends Statement {
             //
             case StatementTypes.COMMIT_WORK : {
                 try {
-                    boolean chain = parameters != null;
+                    boolean chain = ((Boolean) parameters[0]).booleanValue();
 
                     session.commit(chain);
 
@@ -371,9 +401,9 @@ public class StatementSession extends Statement {
                         if (single && data != null && data[0] != null) {
                             value = data[0];
 
-                            result.getNavigator().close();
+                            result.getNavigator().release();
                         } else {
-                            result.getNavigator().close();
+                            result.getNavigator().release();
 
                             return Result.newErrorResult(
                                 Error.error(ErrorCode.X_22009), sql);
@@ -458,7 +488,7 @@ public class StatementSession extends Statement {
                     schema =
                         session.database.schemaManager.getSchemaHsqlName(name);
 
-                    session.setSchema(schema.name);
+                    session.setCurrentSchemaHsqlName(schema);
 
                     return Result.updateZeroResult;
                 } catch (HsqlException e) {
@@ -497,12 +527,19 @@ public class StatementSession extends Statement {
                         throw Error.error(ErrorCode.X_28501);
                     }
 
+                    // override the actual SQL at runtime
                     sql = userObject.getConnectUserSQL();
 
                     if (userObject == session.getGrantee()) {
                         return Result.updateZeroResult;
                     }
 
+                    if (password == null && !session.isProcessingLog()
+                            && userObject.isAdmin()
+                            && !session.getGrantee().isAdmin()) {
+                        throw Error.error(ErrorCode.X_28000);
+                    }
+
                     if (session.getGrantee().canChangeAuthorisation()) {
                         session.setUser((User) userObject);
                         session.setRole(null);
@@ -629,14 +666,18 @@ public class StatementSession extends Statement {
                 StatementDMQL statement       = (StatementDMQL) parameters[2];
 
                 try {
-                    if (tempConstraints != null) {
+                    if (tempConstraints.size() != 0) {
                         table =
                             ParserDDL.addTableConstraintDefinitions(session,
                                 table, tempConstraints, null, false);
                     }
 
                     table.compile(session, null);
-                    session.addSessionTable(table);
+                    session.sessionContext.addSessionTable(table);
+
+                    if (table.hasLobColumn) {
+                        throw Error.error(ErrorCode.X_42534);
+                    }
 
                     if (statement != null) {
                         Result result = statement.execute(session);
@@ -644,18 +685,6 @@ public class StatementSession extends Statement {
                         table.insertIntoTable(session, result);
                     }
 
-                    if (table.hasLobColumn) {
-                        RowIterator it = table.rowIterator(session);
-
-                        while (it.hasNext()) {
-                            Row      row  = it.getNextRow();
-                            Object[] data = row.getData();
-
-                            session.sessionData.adjustLobUsageCount(table,
-                                    data, 1);
-                        }
-                    }
-
                     return Result.updateZeroResult;
                 } catch (HsqlException e) {
                     return Result.newErrorResult(e, sql);
@@ -664,7 +693,8 @@ public class StatementSession extends Statement {
             case StatementTypes.DROP_TABLE : {
                 HsqlName name     = (HsqlName) parameters[0];
                 Boolean  ifExists = (Boolean) parameters[1];
-                Table    table    = session.findSessionTable(name.name);
+                Table table =
+                    session.sessionContext.findSessionTable(name.name);
 
                 if (table == null) {
                     if (ifExists.booleanValue()) {
@@ -674,7 +704,7 @@ public class StatementSession extends Statement {
                     }
                 }
 
-                session.dropSessionTable(name.name);
+                session.sessionContext.dropSessionTable(name.name);
 
                 return Result.updateZeroResult;
             }
@@ -692,6 +722,10 @@ public class StatementSession extends Statement {
         return sql;
     }
 
+    public boolean isCatalogLock() {
+        return false;
+    }
+
     public boolean isCatalogChange() {
         return false;
     }
