@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,23 +44,25 @@ import org.hsqldb.types.Type;
  * The  base of all HSQLDB table implementations.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.7
  * @since 1.7.2
  */
 public class TableBase {
 
     // types of table
-    public static final int SYSTEM_TABLE     = 1;
-    public static final int SYSTEM_SUBQUERY  = 2;
-    public static final int TEMP_TABLE       = 3;
-    public static final int MEMORY_TABLE     = 4;
-    public static final int CACHED_TABLE     = 5;
-    public static final int TEMP_TEXT_TABLE  = 6;
-    public static final int TEXT_TABLE       = 7;
-    public static final int VIEW_TABLE       = 8;
-    public static final int RESULT_TABLE     = 9;
-    public static final int TRANSITION_TABLE = 10;
-    public static final int FUNCTION_TABLE   = 11;
+    public static final int INFO_SCHEMA_TABLE = 1;
+    public static final int SYSTEM_SUBQUERY   = 2;
+    public static final int TEMP_TABLE        = 3;
+    public static final int MEMORY_TABLE      = 4;
+    public static final int CACHED_TABLE      = 5;
+    public static final int TEMP_TEXT_TABLE   = 6;
+    public static final int TEXT_TABLE        = 7;
+    public static final int VIEW_TABLE        = 8;
+    public static final int RESULT_TABLE      = 9;
+    public static final int TRANSITION_TABLE  = 10;
+    public static final int FUNCTION_TABLE    = 11;
+    public static final int SYSTEM_TABLE      = 12;
+    public static final int CHANGE_SET_TABLE  = 13;
 
     //
     public static final int SCOPE_STATEMENT   = 21;
@@ -98,12 +100,13 @@ public class TableBase {
     protected boolean isCached;
     protected boolean isText;
     boolean           isView;
+    protected boolean isWithDataSource;
     public boolean    isSessionBased;
     protected boolean isSchemaBased;
     protected boolean isLogged;
     private boolean   isTransactional = true;
     boolean           hasLobColumn;
-
+    long              dataTimestamp;
     //
     TableBase() {}
 
@@ -135,7 +138,7 @@ public class TableBase {
         copy.persistenceId    = database.persistentStoreCollection.getNextId();
         copy.database         = database;
         copy.colTypes         = colTypes;
-        copy.columnCount      = colTypes.length;
+        copy.columnCount      = columnCount;
         copy.primaryKeyCols   = primaryKeyCols;
         copy.primaryKeyTypes  = primaryKeyTypes;
         copy.indexList        = indexList;
@@ -161,7 +164,7 @@ public class TableBase {
 
     public final RowIterator rowIterator(Session session) {
 
-        PersistentStore store = session.sessionData.getRowStore(this);
+        PersistentStore store = getRowStore(session);
 
         return getPrimaryIndex().firstRow(session, store);
     }
@@ -286,7 +289,7 @@ public class TableBase {
         for (int i = 0; i < indexList.length; i++) {
             Index index     = indexList[i];
             int[] cols      = index.getColumns();
-            int   colsCount = index.getVisibleColumns();
+            int   colsCount = index.getColumnCount();
 
             if (colsCount == 0) {
                 continue;
@@ -377,20 +380,19 @@ public class TableBase {
             false);
 
         try {
-            addIndex(null, newIndex);
+            addIndex(newIndex);
         } catch (HsqlException e) {}
     }
 
-    public final Index createAndAddIndexStructure(Session session,
-            HsqlName name, int[] columns, boolean[] descending,
-            boolean[] nullsLast, boolean unique, boolean constraint,
-            boolean forward) {
+    public final Index createAndAddIndexStructure(HsqlName name,
+            int[] columns, boolean[] descending, boolean[] nullsLast,
+            boolean unique, boolean constraint, boolean forward) {
 
         Index newindex = createIndexStructure(name, columns, descending,
                                               nullsLast, unique, constraint,
                                               forward);
 
-        addIndex(session, newindex);
+        addIndex(newindex);
 
         return newindex;
     }
@@ -441,7 +443,7 @@ public class TableBase {
         }
     }
 
-    final void addIndex(Session session, Index index) {
+    final void addIndex(Index index) {
 
         int i = 0;
 
@@ -500,15 +502,15 @@ public class TableBase {
                                    boolean[] nullsLast, boolean unique,
                                    boolean constraint, boolean forward) {
 
-        Index newIndex = createAndAddIndexStructure(session, name, columns,
-            descending, nullsLast, unique, constraint, forward);
+        Index newIndex = createAndAddIndexStructure(name, columns, descending,
+            nullsLast, unique, constraint, forward);
 
         return newIndex;
     }
 
     public void clearAllData(Session session) {
 
-        PersistentStore store = session.sessionData.getRowStore(this);
+        PersistentStore store = getRowStore(session);
 
         store.removeAll();
     }
@@ -528,17 +530,19 @@ public class TableBase {
             return true;
         }
 
-        PersistentStore store = session.sessionData.getRowStore(this);
+        PersistentStore store = getRowStore(session);
 
         return getIndex(0).isEmpty(store);
     }
 
-    public int getRowCount(PersistentStore store) {
-        return getPrimaryIndex().size(null, store);
+    public PersistentStore getRowStore(Session session) {
+
+        return store == null
+               ? session.sessionData.persistentStoreCollection.getStore(this)
+               : store;
     }
 
-    public PersistentStore getRowStore(Session session) {
-        return store == null ? session.sessionData.getRowStore(this)
-                             : store;
+    public void setDataTimestamp(long timestamp) {
+        // no op
     }
 }
