@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,30 +45,30 @@ import org.hsqldb.rights.Grantee;
 // fredt@users 20030727 - signature and other alterations
 // fredt@users 20040430 - changes by mattshaw@users to allow termination of the
 // trigger thread -
+// fredt@users - updated for v. 2.x
 
 /**
  *  Represents an HSQLDB Trigger definition. <p>
  *
- *  Provides services regarding HSLDB Trigger execution and metadata. <p>
+ *  Provides services regarding HSQLDB Trigger execution and metadata. <p>
  *
  *  Development of the trigger implementation sponsored by Logicscope
  *  Realisations Ltd
  *
- * @author Peter Hudson - Logicscope Realisations Ltd
- * @version  1.7.0 (1.0.0.3)
- *      Revision History: 1.0.0.1 First release in hsqldb 1.61
- *      1.0.0.2 'nowait' support to prevent deadlock 1.0.0.3 multiple row
- *      queue for each trigger
+ * @author Peter Hudson (peterhudson@users dot sourceforge.net)
+ * @version  2.0.1
+ * @since hsqldb 1.61
  */
 public class TriggerDef implements Runnable, SchemaObject {
 
-    static final int OLD_ROW   = 0;
-    static final int NEW_ROW   = 1;
-    static final int OLD_TABLE = 2;
-    static final int NEW_TABLE = 3;
-    static final int BEFORE    = 4;
-    static final int AFTER     = 5;
-    static final int INSTEAD   = 6;
+    static final int OLD_ROW     = 0;
+    static final int NEW_ROW     = 1;
+    static final int RANGE_COUNT = 2;
+    static final int OLD_TABLE   = 2;
+    static final int NEW_TABLE   = 3;
+    static final int BEFORE      = 4;
+    static final int AFTER       = 5;
+    static final int INSTEAD     = 6;
 
     //
     static final int NUM_TRIGGER_OPS = 3;                      // {ins,del,upd}
@@ -82,7 +82,6 @@ public class TriggerDef implements Runnable, SchemaObject {
     boolean                   hasTransitionTables;
     boolean                   hasTransitionRanges;
     String                    conditionSQL;
-    String                    procedureSQL;
     Routine                   routine;
     int[]                     updateColumns;
 
@@ -91,6 +90,7 @@ public class TriggerDef implements Runnable, SchemaObject {
     long             changeTimestamp;
     int              actionTiming;
     int              operationType;
+    boolean          isSystem;
     boolean          forEachRow;
     boolean          nowait;                                   // block or overwrite if queue full
     int              maxRowsQueued;                            // max size of queue of pending triggers
@@ -193,8 +193,8 @@ public class TriggerDef implements Runnable, SchemaObject {
                                                : condition;
         this.updateColumns = updateColumns;
         this.conditionSQL  = conditionSQL;
-        hasTransitionRanges = transitions[OLD_ROW] != null
-                              || transitions[NEW_ROW] != null;
+        hasTransitionRanges = rangeVars[OLD_ROW] != null
+                              || rangeVars[NEW_ROW] != null;
         hasTransitionTables = transitions[OLD_TABLE] != null
                               || transitions[NEW_TABLE] != null;
 
@@ -299,17 +299,19 @@ public class TriggerDef implements Runnable, SchemaObject {
         if (hasTransitionRanges || hasTransitionTables) {
             sb.append(Tokens.T_REFERENCING).append(' ');
 
-            if (transitions[OLD_ROW] != null) {
+            if (rangeVars[OLD_ROW] != null) {
                 sb.append(Tokens.T_OLD).append(' ').append(Tokens.T_ROW);
                 sb.append(' ').append(Tokens.T_AS).append(' ');
-                sb.append(transitions[OLD_ROW].getName().statementName);
+                sb.append(
+                    rangeVars[OLD_ROW].getTableAlias().getStatementName());
                 sb.append(' ');
             }
 
-            if (transitions[NEW_ROW] != null) {
+            if (rangeVars[NEW_ROW] != null) {
                 sb.append(Tokens.T_NEW).append(' ').append(Tokens.T_ROW);
                 sb.append(' ').append(Tokens.T_AS).append(' ');
-                sb.append(transitions[NEW_ROW].getName().statementName);
+                sb.append(
+                    rangeVars[NEW_ROW].getTableAlias().getStatementName());
                 sb.append(' ');
             }
 
@@ -383,6 +385,10 @@ public class TriggerDef implements Runnable, SchemaObject {
         }
     }
 
+    public boolean isSystem() {
+        return isSystem;
+    }
+
     public boolean isForEachRow() {
         return forEachRow;
     }
@@ -392,7 +398,8 @@ public class TriggerDef implements Runnable, SchemaObject {
     }
 
     public String getProcedureSQL() {
-        return procedureSQL;
+        return routine == null ? null
+                               : routine.getSQLBodyDefinition();
     }
 
     public int[] getUpdateColumnIndexes() {
@@ -409,16 +416,16 @@ public class TriggerDef implements Runnable, SchemaObject {
 
     public String getOldTransitionRowName() {
 
-        return transitions[OLD_ROW] == null ? null
-                                            : transitions[OLD_ROW].getName()
-                                            .name;
+        return rangeVars[OLD_ROW] == null ? null
+                                          : rangeVars[OLD_ROW].getTableAlias()
+                                              .name;
     }
 
     public String getNewTransitionRowName() {
 
-        return transitions[NEW_ROW] == null ? null
-                                            : transitions[NEW_ROW].getName()
-                                            .name;
+        return rangeVars[NEW_ROW] == null ? null
+                                          : rangeVars[NEW_ROW].getTableAlias()
+                                              .name;
     }
 
     public String getOldTransitionTableName() {
@@ -534,6 +541,10 @@ public class TriggerDef implements Runnable, SchemaObject {
                 }
             }
         }
+
+        try {
+            thread.setContextClassLoader(null);
+        } catch (Throwable t) {}
     }
 
     /**
@@ -602,8 +613,14 @@ public class TriggerDef implements Runnable, SchemaObject {
     synchronized void pushPair(Session session, Object[] row1, Object[] row2) {
 
         if (maxRowsQueued == 0) {
-            trigger.fire(triggerType, name.name, table.getName().name, row1,
-                         row2);
+            session.getInternalConnection();
+
+            try {
+                trigger.fire(triggerType, name.name, table.getName().name,
+                             row1, row2);
+            } finally {
+                session.releaseInternalConnection();
+            }
 
             return;
         }
