@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@ import org.hsqldb.lib.FrameworkLogger;
 import org.hsqldb.lib.RCData;
 import org.hsqldb.cmdline.sqltool.Token;
 
-/* $Id: SqlTool.java 3481 2010-02-26 18:05:06Z fredt $ */
+/* $Id: SqlTool.java 4720 2011-11-08 00:10:09Z unsaved $ */
 
 /**
  * A command-line JDBC SQL tool supporting both interactive and
@@ -69,7 +69,7 @@ import org.hsqldb.cmdline.sqltool.Token;
  * @see #objectMain(String[])
  * @see SqlFile
  * @see org.hsqldb.sample.SqlFileEmbedder
- * @version $Revision: 3481 $, $Date: 2010-02-26 13:05:06 -0500 (Fri, 26 Feb 2010) $
+ * @version $Revision: 4720 $, $Date: 2011-11-08 00:10:09 +0000 (Tue, 08 Nov 2011) $
  * @author Blaine Simpson (blaine dot simpson at admc dot com)
  */
 public class SqlTool {
@@ -77,10 +77,14 @@ public class SqlTool {
             FrameworkLogger.getLog(SqlTool.class);
     public static final String DEFAULT_RCFILE =
         System.getProperty("user.home") + "/sqltool.rc";
-    // N.b. the following is static!
-    private static String  revnum =
-            "$Revision: 3481 $".substring("$Revision: ".length(),
-            "$Revision: 3481 $".length() - 2);
+    // N.b. the following are static!
+    private static final String revString = "$Revision: 4720 $";
+    private static final int revStringLength = revString.length();
+    private static final String  revnum =
+            (revStringLength - " $".length() > "$Revision: ".length())
+            ?  revString.substring("$Revision: ".length(),
+                    revStringLength - " $".length())
+            : "<UNTRACKED>";
 
     public static final int SQLTOOLERR_EXITVAL = 1;
     public static final int SYNTAXERR_EXITVAL = 11;
@@ -192,7 +196,8 @@ public class SqlTool {
      * @param lowerCaseKeys Set to <code>true</code> if the map keys should be
      *        converted to lower case
      */
-    private static void varParser(String varString, Map<String, String> varMap,
+    private static void varParser(String inVarString,
+                                  Map<String, String> varMap,
                                   boolean lowerCaseKeys)
                                   throws PrivateException {
 
@@ -204,13 +209,17 @@ public class SqlTool {
             throw new IllegalArgumentException(
                     "varMap is null in SqlTool.varParser call");
         }
-        if (varString == null) {
+        if (inVarString == null) {
             throw new IllegalArgumentException(
-                    "varString is null in SqlTool.varParser call");
+                    "inVarString is null in SqlTool.varParser call");
         }
+        boolean escapesPresent = inVarString.indexOf("\\,") > -1;
+        String  varString = escapesPresent ?
+                inVarString.replace("\\,", "\u0002")
+                : inVarString;
 
         for (String token : varString.split("\\s*,\\s*")) {
-            equals     = token.indexOf('=');
+            equals = token.indexOf('=');
 
             if (equals < 1) {
                 throw new PrivateException(
@@ -219,6 +228,9 @@ public class SqlTool {
 
             var = token.substring(0, equals).trim();
             val = token.substring(equals + 1).trim();
+            if (escapesPresent) {
+                val = val.replace("\u0002", ",");
+            }
 
             if (var.length() < 1) {
                 throw new PrivateException(
@@ -284,7 +296,6 @@ public class SqlTool {
         String  sqlText          = null;
         String  driver           = null;
         String  targetDb         = null;
-        String  varSettings      = null;
         boolean debug            = false;
         File[]  scriptFiles      = null;
         int     i                = -1;
@@ -306,11 +317,12 @@ public class SqlTool {
         String  parameter;
         SqlFile[] sqlFiles       = null;
         Connection conn          = null;
-        Map<String, String> userVars = null;
+        Map<String, String> userVars = new HashMap<String, String>();
 
         try { // Try block to GC tmpReader
         try { // Try block for BadCmdline
-            while ((i + 1 < arg.length) && arg[i + 1].startsWith("--")) {
+            while ((i + 1 < arg.length))
+            if (arg[i + 1].startsWith("--")) {
                 i++;
 
                 if (arg[i].length() == 2) {
@@ -347,65 +359,151 @@ public class SqlTool {
                     coeOverride = Boolean.valueOf(
                             arg[i].substring("--continueonerr=".length()));
                 } else if (parameter.equals("list")) {
+                    if (listMode) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     listMode = true;
                 } else if (parameter.equals("rcfile")) {
                     if (++i == arg.length) {
                         throw bcl;
                     }
+                    if (rcFile != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
 
                     rcFile = arg[i];
                 } else if (parameter.startsWith("rcfile=")) {
+                    if (rcFile != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     rcFile = arg[i].substring("--rcfile=".length());
                 } else if (parameter.equals("setvar")) {
                     if (++i == arg.length) {
                         throw bcl;
                     }
 
-                    varSettings = arg[i];
+                    try {
+                        varParser(arg[i], userVars, false);
+                    } catch (PrivateException pe) {
+                        throw new SqlToolException(
+                                RCERR_EXITVAL, pe.getMessage());
+                    }
                 } else if (parameter.startsWith("setvar=")) {
-                    varSettings = arg[i].substring("--setvar=".length());
+                    try {
+                        varParser(arg[i].substring("--setvar=".length()),
+                                userVars, false);
+                    } catch (PrivateException pe) {
+                        throw new SqlToolException(
+                                RCERR_EXITVAL, pe.getMessage());
+                    }
                 } else if (parameter.equals("sql")) {
                     noinput = true;    // but turn back on if file "-" specd.
 
                     if (++i == arg.length) {
                         throw bcl;
                     }
+                    if (sqlText != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
 
                     sqlText = arg[i];
                 } else if (parameter.startsWith("sql=")) {
                     noinput = true;    // but turn back on if file "-" specd.
+                    if (sqlText != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     sqlText = arg[i].substring("--sql=".length());
                 } else if (parameter.equals("debug")) {
+                    if (debug) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     debug = true;
                 } else if (parameter.equals("noautofile")) {
+                    if (noautoFile) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     noautoFile = true;
                 } else if (parameter.equals("autocommit")) {
+                    if (autoCommit) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     autoCommit = true;
                 } else if (parameter.equals("stdinput")) {
                     noinput          = false;
+                    if (stdinputOverride != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     stdinputOverride = Boolean.TRUE;
                 } else if (parameter.equals("noinput")) {
                     noinput          = true;
+                    if (stdinputOverride != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     stdinputOverride = Boolean.FALSE;
                 } else if (parameter.equals("driver")) {
                     if (++i == arg.length) {
                         throw bcl;
                     }
+                    if (driver != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
 
                     driver = arg[i];
                 } else if (parameter.startsWith("driver=")) {
+                    if (driver != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     driver = arg[i].substring("--driver=".length());
                 } else if (parameter.equals("inlinerc")) {
                     if (++i == arg.length) {
                         throw bcl;
                     }
+                    if (rcParams != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
 
                     rcParams = arg[i];
                 } else if (parameter.startsWith("inlinerc=")) {
+                    if (rcParams != null) {
+                        throw new SqlToolException(SYNTAXERR_EXITVAL,
+                                SqltoolRB.SqlTool_params_redundant.getString());
+                    }
                     rcParams = arg[i].substring("--inlinerc=".length());
                 } else {
                     throw bcl;
                 }
+            } else if (arg[i + 1].startsWith("-P")
+                    || arg[i + 1].startsWith("-p")) {
+                i++;
+                boolean sepSwitch = arg[i].length() < 3;
+                if (sepSwitch) {
+                    if (++i == arg.length) {
+                        throw bcl;
+                    }
+                }
+
+                int equalAt = arg[i].indexOf('=');
+                if (equalAt < (sepSwitch ? 1 : 3)) {
+                    throw new SqlToolException(RCERR_EXITVAL,
+                            "Specified var assignment contains no '='");
+                }
+                userVars.put(arg[i].substring(sepSwitch ? 0 : 2, equalAt),
+                        arg[i].substring(equalAt + 1));
+            } else {
+                break;
             }
 
             if (!listMode && rcParams == null && ++i != arg.length) {
@@ -456,7 +554,7 @@ public class SqlTool {
                 // I.e., if there are any SQL files specified.
                 scriptFiles = new File[arg.length - i - 1
                         + ((stdinputOverride == null
-                                ||!stdinputOverride.booleanValue()) ? 0 : 1)];
+                                || !stdinputOverride.booleanValue()) ? 0 : 1)];
 
                 if (debug) {
                     System.err.println("scriptFiles has "
@@ -584,11 +682,11 @@ public class SqlTool {
         File[] singleNullFileArray = { null };
         File   autoFile            = null;
 
-        if (interactive &&!noautoFile) {
+        if (interactive && !noautoFile) {
             autoFile = new File(System.getProperty("user.home")
                                 + "/auto.sql");
 
-            if ((!autoFile.isFile()) ||!autoFile.canRead()) {
+            if ((!autoFile.isFile()) || !autoFile.canRead()) {
                 autoFile = null;
             }
         }
@@ -613,13 +711,6 @@ public class SqlTool {
 
         sqlFiles = new SqlFile[numFiles];
 
-        if (varSettings != null) try {
-            userVars = new HashMap<String, String>();
-            varParser(varSettings, userVars, false);
-        } catch (PrivateException pe) {
-            throw new SqlToolException(RCERR_EXITVAL, pe.getMessage());
-        }
-
         // We print version before execing this one.
         int interactiveFileIndex = -1;
         String encoding = (conData == null) ? null : conData.charset;
@@ -632,8 +723,8 @@ public class SqlTool {
             }
 
             if (tmpReader != null) {
-                sqlFiles[fileIndex++] =
-                    new SqlFile(tmpReader, "--sql", System.out, null, false);
+                sqlFiles[fileIndex++] = new SqlFile(
+                        tmpReader, "--sql", System.out, null, false, null);
             }
 
             for (File scriptFile : scriptFiles) {
@@ -664,7 +755,7 @@ public class SqlTool {
         try {
             for (SqlFile sqlFile : sqlFiles) {
                 if (conn != null) sqlFile.setConnection(conn);
-                if (userVars != null) sqlFile.addUserVars(userVars);
+                if (userVars.size() > 0) sqlFile.addUserVars(userVars);
                 if (macros != null) sqlFile.addMacros(macros);
                 if (coeOverride != null)
                     sqlFile.setContinueOnError(coeOverride.booleanValue());
