@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,6 +37,7 @@ import org.hsqldb.Database;
 import org.hsqldb.Session;
 import org.hsqldb.Table;
 import org.hsqldb.lib.IntValueHashMap;
+import org.hsqldb.persist.PersistentStore;
 
 // fredt@users - 1.7.2 - structural modifications to allow inheritance
 // boucherB@users 20020305 - completed inheritance work, including final access
@@ -52,8 +53,8 @@ import org.hsqldb.lib.IntValueHashMap;
  * This class has been developed from scratch to replace the previous
  * DatabaseInformation implementations. <p>
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
- * @version 1.9.0
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
+ * @version 2.0.1
  * @since 1.7.2
  */
 public class DatabaseInformation {
@@ -77,84 +78,86 @@ public class DatabaseInformation {
 
     // HSQLDB-specific
     protected static final int SYSTEM_CACHEINFO             = 15;
-    protected static final int SYSTEM_COMMENTS              = 16;
-    protected static final int SYSTEM_CONNECTION_PROPERTIES = 17;
-    protected static final int SYSTEM_PROPERTIES            = 18;
-    protected static final int SYSTEM_SESSIONINFO           = 19;
-    protected static final int SYSTEM_SESSIONS              = 20;
-    protected static final int SYSTEM_TEXTTABLES            = 21;
+    protected static final int SYSTEM_COLUMN_SEQUENCE_USAGE = 16;
+    protected static final int SYSTEM_COMMENTS              = 17;
+    protected static final int SYSTEM_CONNECTION_PROPERTIES = 18;
+    protected static final int SYSTEM_PROPERTIES            = 19;
+    protected static final int SYSTEM_SESSIONINFO           = 20;
+    protected static final int SYSTEM_SESSIONS              = 21;
+    protected static final int SYSTEM_TEXTTABLES            = 22;
 
     // SQL 200n tables
-    protected static final int ADMINISTRABLE_ROLE_AUTHORIZATIONS = 22;
-    protected static final int APPLICABLE_ROLES                  = 23;
-    protected static final int ASSERTIONS                        = 24;
-    protected static final int AUTHORIZATIONS                    = 25;
-    protected static final int CHARACTER_SETS                    = 26;
-    protected static final int CHECK_CONSTRAINT_ROUTINE_USAGE    = 27;
-    protected static final int CHECK_CONSTRAINTS                 = 28;
-    protected static final int COLLATIONS                        = 29;
-    protected static final int COLUMN_COLUMN_USAGE               = 30;
-    protected static final int COLUMN_DOMAIN_USAGE               = 31;
-    protected static final int COLUMN_PRIVILEGES                 = 32;
-    protected static final int COLUMN_UDT_USAGE                  = 33;
-    protected static final int COLUMNS                           = 34;
-    protected static final int CONSTRAINT_COLUMN_USAGE           = 35;
-    protected static final int CONSTRAINT_TABLE_USAGE            = 36;
-    protected static final int DATA_TYPE_PRIVILEGES              = 37;
-    protected static final int DOMAIN_CONSTRAINTS                = 38;
-    protected static final int DOMAINS                           = 39;
-    protected static final int ENABLED_ROLES                     = 40;
-    protected static final int INFORMATION_SCHEMA_CATALOG_NAME   = 41;
-    protected static final int JAR_JAR_USAGE                     = 42;
-    protected static final int JARS                              = 43;
-    protected static final int KEY_COLUMN_USAGE                  = 44;
-    protected static final int METHOD_SPECIFICATIONS             = 45;
-    protected static final int MODULE_COLUMN_USAGE               = 46;
-    protected static final int MODULE_PRIVILEGES                 = 47;
-    protected static final int MODULE_TABLE_USAGE                = 48;
-    protected static final int MODULES                           = 49;
-    protected static final int PARAMETERS                        = 50;
-    protected static final int REFERENTIAL_CONSTRAINTS           = 51;
-    protected static final int ROLE_AUTHORIZATION_DESCRIPTORS    = 52;
-    protected static final int ROLE_COLUMN_GRANTS                = 53;
-    protected static final int ROLE_MODULE_GRANTS                = 54;
-    protected static final int ROLE_ROUTINE_GRANTS               = 55;
-    protected static final int ROLE_TABLE_GRANTS                 = 56;
-    protected static final int ROLE_UDT_GRANTS                   = 57;
-    protected static final int ROLE_USAGE_GRANTS                 = 58;
-    protected static final int ROUTINE_COLUMN_USAGE              = 59;
-    protected static final int ROUTINE_JAR_USAGE                 = 60;
-    protected static final int ROUTINE_PRIVILEGES                = 61;
-    protected static final int ROUTINE_ROUTINE_USAGE             = 62;
-    protected static final int ROUTINE_SEQUENCE_USAGE            = 63;
-    protected static final int ROUTINE_TABLE_USAGE               = 64;
-    protected static final int ROUTINES                          = 65;
-    protected static final int SCHEMATA                          = 66;
-    protected static final int SEQUENCES                         = 67;
-    protected static final int SQL_FEATURES                      = 68;
-    protected static final int SQL_IMPLEMENTATION_INFO           = 69;
-    protected static final int SQL_PACKAGES                      = 70;
-    protected static final int SQL_PARTS                         = 71;
-    protected static final int SQL_SIZING                        = 72;
-    protected static final int SQL_SIZING_PROFILES               = 73;
-    protected static final int TABLE_CONSTRAINTS                 = 74;
-    protected static final int TABLE_PRIVILEGES                  = 75;
-    protected static final int TABLES                            = 76;
-    protected static final int TRANSLATIONS                      = 77;
-    protected static final int TRIGGER_COLUMN_USAGE              = 78;
-    protected static final int TRIGGER_ROUTINE_USAGE             = 79;
-    protected static final int TRIGGER_SEQUENCE_USAGE            = 80;
-    protected static final int TRIGGER_TABLE_USAGE               = 81;
-    protected static final int TRIGGERED_UPDATE_COLUMNS          = 82;
-    protected static final int TRIGGERS                          = 83;
-    protected static final int TYPE_JAR_USAGE                    = 84;
-    protected static final int UDT_PRIVILEGES                    = 85;
-    protected static final int USAGE_PRIVILEGES                  = 86;
-    protected static final int USER_DEFINED_TYPES                = 87;
-    protected static final int VIEW_COLUMN_USAGE                 = 88;
-    protected static final int VIEW_ROUTINE_USAGE                = 89;
-    protected static final int VIEW_TABLE_USAGE                  = 90;
-    protected static final int VIEWS                             = 91;
+    protected static final int ADMINISTRABLE_ROLE_AUTHORIZATIONS = 23;
+    protected static final int APPLICABLE_ROLES                  = 24;
+    protected static final int ASSERTIONS                        = 25;
+    protected static final int AUTHORIZATIONS                    = 26;
+    protected static final int CHARACTER_SETS                    = 27;
+    protected static final int CHECK_CONSTRAINT_ROUTINE_USAGE    = 28;
+    protected static final int CHECK_CONSTRAINTS                 = 29;
+    protected static final int COLLATIONS                        = 30;
+    protected static final int COLUMN_COLUMN_USAGE               = 31;
+    protected static final int COLUMN_DOMAIN_USAGE               = 32;
+    protected static final int COLUMN_PRIVILEGES                 = 33;
+    protected static final int COLUMN_UDT_USAGE                  = 34;
+    protected static final int COLUMNS                           = 35;
+    protected static final int CONSTRAINT_COLUMN_USAGE           = 36;
+    protected static final int CONSTRAINT_TABLE_USAGE            = 37;
+    protected static final int DATA_TYPE_PRIVILEGES              = 38;
+    protected static final int DOMAIN_CONSTRAINTS                = 39;
+    protected static final int DOMAINS                           = 40;
+    protected static final int ELEMENT_TYPES                     = 41;
+    protected static final int ENABLED_ROLES                     = 42;
+    protected static final int INFORMATION_SCHEMA_CATALOG_NAME   = 43;
+    protected static final int JAR_JAR_USAGE                     = 44;
+    protected static final int JARS                              = 45;
+    protected static final int KEY_COLUMN_USAGE                  = 46;
+    protected static final int METHOD_SPECIFICATIONS             = 47;
+    protected static final int MODULE_COLUMN_USAGE               = 48;
+    protected static final int MODULE_PRIVILEGES                 = 49;
+    protected static final int MODULE_TABLE_USAGE                = 50;
+    protected static final int MODULES                           = 51;
+    protected static final int PARAMETERS                        = 52;
+    protected static final int REFERENTIAL_CONSTRAINTS           = 53;
+    protected static final int ROLE_AUTHORIZATION_DESCRIPTORS    = 54;
+    protected static final int ROLE_COLUMN_GRANTS                = 55;
+    protected static final int ROLE_MODULE_GRANTS                = 56;
+    protected static final int ROLE_ROUTINE_GRANTS               = 57;
+    protected static final int ROLE_TABLE_GRANTS                 = 58;
+    protected static final int ROLE_UDT_GRANTS                   = 59;
+    protected static final int ROLE_USAGE_GRANTS                 = 60;
+    protected static final int ROUTINE_COLUMN_USAGE              = 61;
+    protected static final int ROUTINE_JAR_USAGE                 = 62;
+    protected static final int ROUTINE_PRIVILEGES                = 63;
+    protected static final int ROUTINE_ROUTINE_USAGE             = 64;
+    protected static final int ROUTINE_SEQUENCE_USAGE            = 65;
+    protected static final int ROUTINE_TABLE_USAGE               = 66;
+    protected static final int ROUTINES                          = 67;
+    protected static final int SCHEMATA                          = 68;
+    protected static final int SEQUENCES                         = 69;
+    protected static final int SQL_FEATURES                      = 70;
+    protected static final int SQL_IMPLEMENTATION_INFO           = 71;
+    protected static final int SQL_PACKAGES                      = 72;
+    protected static final int SQL_PARTS                         = 73;
+    protected static final int SQL_SIZING                        = 74;
+    protected static final int SQL_SIZING_PROFILES               = 75;
+    protected static final int TABLE_CONSTRAINTS                 = 76;
+    protected static final int TABLE_PRIVILEGES                  = 77;
+    protected static final int TABLES                            = 78;
+    protected static final int TRANSLATIONS                      = 79;
+    protected static final int TRIGGER_COLUMN_USAGE              = 80;
+    protected static final int TRIGGER_ROUTINE_USAGE             = 81;
+    protected static final int TRIGGER_SEQUENCE_USAGE            = 82;
+    protected static final int TRIGGER_TABLE_USAGE               = 83;
+    protected static final int TRIGGERED_UPDATE_COLUMNS          = 84;
+    protected static final int TRIGGERS                          = 85;
+    protected static final int TYPE_JAR_USAGE                    = 86;
+    protected static final int UDT_PRIVILEGES                    = 87;
+    protected static final int USAGE_PRIVILEGES                  = 88;
+    protected static final int USER_DEFINED_TYPES                = 89;
+    protected static final int VIEW_COLUMN_USAGE                 = 90;
+    protected static final int VIEW_ROUTINE_USAGE                = 91;
+    protected static final int VIEW_TABLE_USAGE                  = 92;
+    protected static final int VIEWS                             = 93;
 
     /** system table names strictly in order of their ids */
     protected static final String[] sysTableNames = {
@@ -176,7 +179,8 @@ public class DatabaseInformation {
 
         // HSQLDB-specific
         "SYSTEM_CACHEINFO",                                      //
-        "SYSTEM_COMMENTS",
+        "SYSTEM_COLUMN_SEQUENCE_USAGE",                          //
+        "SYSTEM_COMMENTS",                                       //
         "SYSTEM_CONNECTION_PROPERTIES",                          //
         "SYSTEM_PROPERTIES",                                     //
         "SYSTEM_SESSIONINFO",                                    //
@@ -202,6 +206,7 @@ public class DatabaseInformation {
         "DATA_TYPE_PRIVILEGES",                                  //
         "DOMAIN_CONSTRAINTS",                                    //
         "DOMAINS",                                               //
+        "ELEMENT_TYPES",                                         //
         "ENABLED_ROLES",                                         //
         "INFORMATION_SCHEMA_CATALOG_NAME",                       //
         "JAR_JAR_USAGE",                                         //
@@ -352,10 +357,16 @@ public class DatabaseInformation {
      * @return a table corresponding to the name and session arguments, or
      *      <code>null</code> if there is no such table to be produced
      */
-    public synchronized Table getSystemTable(Session session, String name) {
+    public Table getSystemTable(Session session, String name) {
         return null;
     }
 
+    /**
+     * Sets the store for the given session, populates the store if necessary.
+     */
+    public void setStore(Session session, Table table,
+                                      PersistentStore store) {}
+
     /**
      * Switches this table producer between producing empty (surrogate)
      * or contentful tables. <p>
@@ -363,7 +374,7 @@ public class DatabaseInformation {
      * @param withContent if true, then produce contentful tables, else
      *        produce emtpy (surrogate) tables
      */
-    public synchronized final void setWithContent(boolean withContent) {
+    public final void setWithContent(boolean withContent) {
         this.withContent = withContent;
     }
 }
