@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,14 +35,15 @@ package org.hsqldb.error;
  * SQL error codes.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.3.0
  * @since 1.9.0
  */
 public interface ErrorCode {
 
     // neutral placeholder strings
-    int TOKEN_REQUIRED = 1;                        // $$ required: $$
-    int CONSTRAINT     = 2;                        // $$ table: $$
+    int TOKEN_REQUIRED    = 1;                     // $$ required: $$
+    int CONSTRAINT        = 2;                     // $$ table: $$
+    int COLUMN_CONSTRAINT = 3;                     // $$ table: $$ column: $$
 
     //
     int M_ERROR_IN_BINARY_SCRIPT_1 = 21;           // \u0020table $$ row count error : $$ read, needed $$
@@ -94,10 +95,13 @@ public interface ErrorCode {
     int X_S0522 = 321;                             // invalid statemnet - text table required
     int X_S0531 = 331;                             // encode / decode error:
 
+    // HSQLDB authentication
+    int PASSWORD_COMPLEXITY = 391;                 // S1000 password complexity
+
     // HSQLDB Server
     int SERVER_TRANSFER_CORRUPTED    = 401;        // S1000 Transfer corrupted
     int SERVER_DATABASE_DISCONNECTED = 402;        // S0504 database disconnected
-    int SERVER_VERSIONS_INCOMPATIBLE = 403;        // S0504 Client version '$$' incompatible.
+    int SERVER_VERSIONS_INCOMPATIBLE = 403;        // S0504 Client driver version greater than '$$' is required ....
     int SERVER_UNKNOWN_CLIENT = 404;               // S0504 Network client is not a HSQLDB JDBC driver
     int SERVER_HTTP_NOT_HSQL_PROTOCOL = 405;       // S0504 Client using HSQLDB http protocol instead of hsql
     int SERVER_INCOMPLETE_HANDSHAKE_READ = 406;    // S0504 Incomplete read of handshaking bytes
@@ -115,7 +119,7 @@ public interface ErrorCode {
     int LOCK_FILE_ACQUISITION_FAILURE = 451;       // S1000 Database lock acquisition failure
     int FILE_IO_ERROR               = 452;         // S1000 File input/output error
     int WRONG_DATABASE_FILE_VERSION = 453;         // S1000 unsupported database file version
-    int SHUTDOWN_REQUIRED           = 454;         // S1000 old version database must be shutdown
+    int DATA_FILE_BACKUP_MISMATCH   = 454;         // S1000 Data file modified, no backup
     int DATABASE_IS_READONLY        = 455;         // S1000 The database is in read only mode
     int DATA_IS_READONLY            = 456;         // S1000 The table data is read only
     int ACCESS_IS_DENIED            = 457;         // S1000 Access is denied
@@ -131,6 +135,7 @@ public interface ErrorCode {
     int GENERAL_IO_ERROR     = 467;                // S1000 IO error
     int DATA_FILE_IS_FULL    = 468;                // S1000 Data file size limit is reached
     int DATA_FILE_IN_USE     = 469;                // S1000 Data file is in use
+    int BACKUP_ERROR         = 470;                // S1000 Backup error
 
     // HSQLDB Text tables
     int TEXT_TABLE_UNKNOWN_DATA_SOURCE = 481;      // S1000 The table's data source for has not been defined
@@ -185,7 +190,7 @@ public interface ErrorCode {
     // HSQLDB
     int X_07501 = 1251;                            // statement is closed
     int X_07502 = 1252;                            // statement is invalid
-    int X_07503 = 1253;                            // statement generates a result set
+    int X_07503 = 1253;                            // statement does not generate a row count
     int X_07504 = 1254;                            // statement does not generate a result set
     int X_07505 = 1255;                            // statement is in batch mode
     int X_07506 = 1256;                            // statement is not in batch mode
@@ -525,7 +530,7 @@ public interface ErrorCode {
     int X_42531 = 5531;                            // default expression needed
     int X_42532 = 5532;                            // primary key already exist
     int X_42533 = 5533;                            // is referenced by FOREIGN KEY constraint
-    int X_42534 = 5534;                            // column of LOB type cannot be used in operation
+    int X_42534 = 5534;                            // column of LOB or ARRAY type cannot be used in operation
 
     // other definition issues
     int X_42535 = 5535;                            // not an identity column
@@ -556,7 +561,7 @@ public interface ErrorCode {
     int X_42563 = 5563;                            // incompatible data type in operation
     int X_42564 = 5564;                            // row column count mismatch
     int X_42565 = 5565;                            // row expression not allowed
-    int X_42566 = 5566;                            // interval qualifier missing
+    int X_42566 = 5566;                            // interval qualifier missing or not valid
     int X_42567 = 5567;                            // data type cast needed for parameter or null literal
     int X_42568 = 5568;                            // data type of expression is not boolean
     int X_42569 = 5569;                            // quoted identifier required
