@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,10 +36,11 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 
 import org.hsqldb.ColumnBase;
+import org.hsqldb.SessionInterface;
+import org.hsqldb.error.ErrorCode;
 import org.hsqldb.navigator.RowSetNavigatorClient;
 import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultMetaData;
-import org.hsqldb.store.ValuePool;
 import org.hsqldb.types.Type;
 
 /**
@@ -81,9 +82,11 @@ import org.hsqldb.types.Type;
  * All methods on the <code>Array</code> interface must be fully implemented if the
  * JDBC driver supports the data type.
  *
- * @since 1.2
+ * @author Fred Toussi (fredt@users dot sourceforge.net)
+ * @version 2.0.1
+ * @since JDK 1.2, HSQLDB 2.0
  */
-public class JDBCArray implements Array{
+public class JDBCArray implements Array {
 
     /**
      * Retrieves the SQL type name of the elements in
@@ -100,9 +103,12 @@ public class JDBCArray implements Array{
      * to access the type name
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
     public String getBaseTypeName() throws SQLException {
+
+        checkClosed();
+
         return elementType.getNameString();
     }
 
@@ -117,9 +123,12 @@ public class JDBCArray implements Array{
      * to access the base type
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
     public int getBaseType() throws SQLException {
+
+        checkClosed();
+
         return elementType.getJDBCTypeCode();
     }
 
@@ -154,13 +163,16 @@ public class JDBCArray implements Array{
      * access the array
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
     public Object getArray() throws SQLException {
+
+        checkClosed();
+
         Object[] array = new Object[data.length];
 
-        for (int i = 0; i <data.length; i++) {
-            array[i] = elementType.convertSQLToJava(connection.sessionProxy, data[i]);
+        for (int i = 0; i < data.length; i++) {
+            array[i] = elementType.convertSQLToJava(sessionProxy, data[i]);
         }
 
         return array;
@@ -201,10 +213,10 @@ public class JDBCArray implements Array{
      *                         access the array
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
-    public Object getArray(java.util.Map<String, Class<?>> map) throws
-        SQLException {
+    public Object getArray(java.util.Map<String,
+            Class<?>> map) throws SQLException {
         return getArray();
     }
 
@@ -220,7 +232,7 @@ public class JDBCArray implements Array{
      * a base type that maps to a primitive data type, then it is
      * implementation-defined whether the array returned is an array of
      * that primitive data type or an array of <code>Object</code>.
-
+     *
      * <!-- end generic documentation -->
      * <!-- start release-specific documentation -->
      * <div class="ReleaseSpecificDocumentation">
@@ -240,18 +252,21 @@ public class JDBCArray implements Array{
      * access the array
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
     public Object getArray(long index, int count) throws SQLException {
 
+        checkClosed();
 
         if (!JDBCClobClient.isInLimits(data.length, index - 1, count)) {
             throw Util.outOfRangeArgument();
         }
+
         Object[] slice = new Object[count];
 
         for (int i = 0; i < count; i++) {
-            slice[i] = elementType.convertSQLToJava(connection.sessionProxy, data[(int) index + i - 1]);
+            slice[i] = elementType.convertSQLToJava(sessionProxy,
+                    data[(int) index + i - 1]);
         }
 
         return slice;
@@ -302,10 +317,11 @@ public class JDBCArray implements Array{
      * access the array
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
-    public Object getArray(long index, int count, java.util.Map<String, Class<?>>
-        map) throws SQLException {
+    public Object getArray(long index, int count,
+                           java.util.Map<String,
+                               Class<?>> map) throws SQLException {
         return getArray(index, count);
     }
 
@@ -330,9 +346,12 @@ public class JDBCArray implements Array{
      * access the array
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
     public ResultSet getResultSet() throws SQLException {
+
+        checkClosed();
+
         Result result = this.newColumnResult(0, data.length);
 
         return new JDBCResultSet(connection, result, result.metaData);
@@ -365,10 +384,10 @@ public class JDBCArray implements Array{
      * access the array
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
-    public ResultSet getResultSet(java.util.Map<String, Class<?>> map) throws
-        SQLException {
+    public ResultSet getResultSet(java.util.Map<String,
+            Class<?>> map) throws SQLException {
         return getResultSet();
     }
 
@@ -399,9 +418,12 @@ public class JDBCArray implements Array{
      * access the array
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
     public ResultSet getResultSet(long index, int count) throws SQLException {
+
+        checkClosed();
+
         Result result = this.newColumnResult(index - 1, count);
 
         return new JDBCResultSet(connection, result, result.metaData);
@@ -440,25 +462,26 @@ public class JDBCArray implements Array{
      * access the array
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.2
+     * @since JDK 1.2
      */
     public ResultSet getResultSet(long index, int count,
-                                  java.util.Map<String, Class<?>> map) throws
-        SQLException {
+                                  java.util.Map<String,
+                                      Class<?>> map) throws SQLException {
         return getResultSet(index, count);
-
     }
 
     /**
      * Returns a string representation in the form <code>ARRAY[..., ...]</code>
      */
     public String toString() {
+
         if (arrayType == null) {
             arrayType = Type.getDefaultArrayType(elementType.typeCode);
         }
 
         return arrayType.convertToString(data);
     }
+
     /**
      * This method frees the <code>Array</code> object and releases the resources that
      * it holds. The object is invalid once the <code>free</code>
@@ -474,43 +497,59 @@ public class JDBCArray implements Array{
      * the Array's resources
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
-     * @since 1.6
+     * @since JDK 1.6
      */
-    public void free() throws SQLException {}
+    public void free() throws SQLException {
+
+        if (!closed) {
+            closed       = true;
+            connection   = null;
+            sessionProxy = null;
+        }
+    }
 
     //-------------
-    Type           arrayType;
-    Type           elementType;
-    Object[]       data;
-    JDBCConnection connection;
+    volatile boolean closed;
+    Type             arrayType;
+    Type             elementType;
+    Object[]         data;
+    JDBCConnection   connection;
+    SessionInterface sessionProxy;
+
+    public JDBCArray(Object[] data, Type type, Type arrayType,
+                     SessionInterface session) {
+
+        this(data, type, arrayType, session.getJDBCConnection());
+
+        this.sessionProxy = session;
+    }
 
     /**
-     * Constructors reject unsupported types.
+     * Constructor should reject unsupported types.
      */
-    JDBCArray(Object[] data, Type type, JDBCConnection connection) throws SQLException {
+    JDBCArray(Object[] data, Type type,
+              JDBCConnection connection) throws SQLException {
         this(data, type, null, connection);
     }
 
+    JDBCArray(Object[] data, Type type, Type arrayType,
+              JDBCConnection connection) {
 
-    JDBCArray(Object[] data, Type type, Type arrayType, JDBCConnection connection) throws SQLException {
-
-        if (type.isArrayType() || type.isLobType() || type.isRowType() ) {
-            throw Util.notSupported();
-        }
-
-        this.data        = data;
-        this.elementType = type;
-        this.arrayType   = arrayType;
-        this.connection  = connection;
+        this.data         = data;
+        this.elementType  = type;
+        this.arrayType    = arrayType;
+        this.connection   = connection;
+        this.sessionProxy = connection.sessionProxy;
     }
 
-    Object[] getArrayInternal() {
+    public Object[] getArrayInternal() {
         return data;
     }
 
-    private Result newColumnResult(long position, int count) throws SQLException {
+    private Result newColumnResult(long position,
+                                   int count) throws SQLException {
 
-        if(!JDBCClobClient.isInLimits(data.length, position, count)) {
+        if (!JDBCClobClient.isInLimits(data.length, position, count)) {
             throw Util.outOfRangeArgument();
         }
 
@@ -542,11 +581,12 @@ public class JDBCArray implements Array{
 
         RowSetNavigatorClient navigator = new RowSetNavigatorClient();
 
-        for (int i = (int) position; i < count; i++) {
+        for (int i = (int) position; i < position + count; i++) {
             Object[] rowData = new Object[2];
 
-            rowData[0] = ValuePool.getInt(i + 1);
+            rowData[0] = Integer.valueOf(i + 1);
             rowData[1] = data[i];
+
             navigator.add(rowData);
         }
 
@@ -556,4 +596,11 @@ public class JDBCArray implements Array{
 
         return result;
     }
+
+    private void checkClosed() throws SQLException {
+
+        if (closed) {
+            throw Util.sqlException(ErrorCode.X_07501);
+        }
+    }
 }
