@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,9 +38,11 @@ import java.io.StringReader;
 import java.sql.Clob;
 import java.sql.SQLException;
 
+import org.hsqldb.error.ErrorCode;
+import org.hsqldb.lib.KMPSearchAlgorithm;
 import org.hsqldb.lib.java.JavaSystem;
 
-/* $Id: JDBCClob.java 3481 2010-02-26 18:05:06Z fredt $ */
+/* $Id: JDBCClob.java 4167 2011-03-29 21:41:14Z fredt $ */
 
 // boucherb@users 2004-03/04-xx - doc 1.7.2 - javadocs updated; methods put in
 //                                            correct (historical, interface
@@ -106,7 +108,7 @@ import org.hsqldb.lib.java.JavaSystem;
  * data (as implemented in this class).  In particular, when the product is built
  * under JDK 1.6+ and the Clob instance is constructed as a result of calling
  * JDBCConnection.createClob(), then the resulting Clob instance is initially
- * disconnected (is not bound to the tranaction scope of the vending Connection
+ * disconnected (is not bound to the transaction scope of the vending Connection
  * object), the data is contained directly and all interface methods for
  * updating the CLOB value are supported for local use until the first
  * invocation of free(); otherwise, an HSQLDB Clob's implementation is
@@ -137,12 +139,7 @@ public class JDBCClob implements Clob {
      * @since JDK 1.2, HSQLDB 1.7.2
      */
     public long length() throws SQLException {
-
-        final String ldata = data;
-
-        checkValid(ldata);
-
-        return ldata.length();
+        return getData().length();
     }
 
     /**
@@ -173,17 +170,19 @@ public class JDBCClob implements Clob {
      * required to survive for any non-trivial duration.  It is left up to the
      * client to decide how to handle the trade-off (whether to make an isolated
      * copy of the returned substring or risk that more memory remains allocated
-     * than is absolutely reuired).
+     * than is absolutely required).
      * </div>
      * <!-- end release-specific documentation -->
      *
      * @param pos the first character of the substring to be extracted.
      *            The first character is at position 1.
-     * @param length the number of consecutive characters to be copied
+     * @param length the number of consecutive characters to be copied;
+     * JDBC 4.1[ the value for length must be 0 or greater]
      * @return a <code>String</code> that is the specified substring in
      *         the <code>CLOB</code> value designated by this <code>Clob</code> object
      * @exception SQLException if there is an error accessing the
-     *            <code>CLOB</code> value or if pos is less than 1
+     *            <code>CLOB</code> value; if pos is less than 1 JDBC 4.1[or length is
+     * less than 0]
      * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
      * this method
      * @since JDK 1.2, HSQLDB 1.7.2
@@ -191,11 +190,8 @@ public class JDBCClob implements Clob {
     public String getSubString(long pos,
                                final int length) throws SQLException {
 
-        final String ldata = data;
-
-        checkValid(ldata);
-
-        final int dlen = ldata.length();
+        final String data = getData();
+        final int    dlen = data.length();
 
         if (pos < MIN_POS || pos > dlen) {
             Util.outOfRangeArgument("pos: " + pos);
@@ -206,8 +202,8 @@ public class JDBCClob implements Clob {
             throw Util.outOfRangeArgument("length: " + length);
         }
 
-        return (pos == 0 && length == dlen) ? ldata
-                : ldata.substring((int) pos, (int) pos + length);
+        return (pos == 0 && length == dlen) ? data
+                : data.substring((int) pos, (int) pos + length);
     }
 
     /**
@@ -225,17 +221,12 @@ public class JDBCClob implements Clob {
      * @since JDK 1.2, HSQLDB 1.7.2
      */
     public java.io.Reader getCharacterStream() throws SQLException {
-
-        final String ldata = data;
-
-        checkValid(ldata);
-
-        return new StringReader(ldata);
+        return new StringReader(getData());
     }
 
     /**
      * Retrieves the <code>CLOB</code> value designated by this <code>Clob</code>
-     * object as an ascii stream.
+     * object as an ASCII stream.
      *
      * @return a <code>java.io.InputStream</code> object containing the
      *         <code>CLOB</code> data
@@ -248,10 +239,8 @@ public class JDBCClob implements Clob {
      */
     public java.io.InputStream getAsciiStream() throws SQLException {
 
-        checkValid(data);
-
         try {
-            return new ByteArrayInputStream(data.getBytes("US-ASCII"));
+            return new ByteArrayInputStream(getData().getBytes("US-ASCII"));
         } catch (IOException e) {
             return null;
         }
@@ -277,9 +266,7 @@ public class JDBCClob implements Clob {
     public long position(final String searchstr,
                          long start) throws SQLException {
 
-        final String ldata = data;
-
-        checkValid(ldata);
+        final String data = getData();
 
         if (start < MIN_POS) {
             throw Util.outOfRangeArgument("start: " + start);
@@ -289,10 +276,11 @@ public class JDBCClob implements Clob {
             return -1;
         }
 
-        final int pos = ldata.indexOf(searchstr, (int) --start);
+        final int position = KMPSearchAlgorithm.search(data, searchstr, null,
+            (int) start);
 
-        return (pos < 0) ? -1
-                         : pos + 1;
+        return (position == -1) ? -1
+                                : position + 1;
     }
 
     /**
@@ -315,9 +303,7 @@ public class JDBCClob implements Clob {
     public long position(final Clob searchstr,
                          long start) throws SQLException {
 
-        final String ldata = data;
-
-        checkValid(ldata);
+        final String data = getData();
 
         if (start < MIN_POS) {
             throw Util.outOfRangeArgument("start: " + start);
@@ -327,7 +313,7 @@ public class JDBCClob implements Clob {
             return -1;
         }
 
-        final long dlen  = ldata.length();
+        final long dlen  = data.length();
         final long sslen = searchstr.length();
 
         start--;
@@ -341,18 +327,19 @@ public class JDBCClob implements Clob {
         }
 
         // by now, we know sslen and start are both < Integer.MAX_VALUE
-        String s;
+        String pattern;
 
         if (searchstr instanceof JDBCClob) {
-            s = ((JDBCClob) searchstr).data();
+            pattern = ((JDBCClob) searchstr).data();
         } else {
-            s = searchstr.getSubString(1L, (int) sslen);
+            pattern = searchstr.getSubString(1L, (int) sslen);
         }
 
-        final int pos = ldata.indexOf(s, (int) start);
+        final int position = KMPSearchAlgorithm.search(data, pattern, null,
+            (int) start);
 
-        return (pos < 0) ? -1
-                         : pos + 1;
+        return (position == -1) ? -1
+                                : position + 1;
     }
 
     //---------------------------- jdbc 3.0 -----------------------------------
@@ -364,7 +351,7 @@ public class JDBCClob implements Clob {
      * in the <code>Clob</code> object starting at the position
      * <code>pos</code>.  If the end of the <code>Clob</code> value is reached
      * while writing the given string, then the length of the <code>Clob</code>
-     * value will be increased to accomodate the extra characters.
+     * value will be increased to accommodate the extra characters.
      * <p>
      * <b>Note:</b> If the value specified for <code>pos</code>
      * is greater then the length+1 of the <code>CLOB</code> value then the
@@ -382,7 +369,7 @@ public class JDBCClob implements Clob {
      * result of calling JDBCConnection.createClob(), this operation affects
      * only the client-side value; it has no effect upon a value stored in the
      * database because JDBCConnection.createClob() constructs disconnected,
-     * initially empty Clob instances. To propogate the Clob value to a database
+     * initially empty Clob instances. To propagate the Clob value to a database
      * in this case, it is required to supply the Clob instance to an updating
      * or inserting setXXX method of a Prepared or Callable Statement, or to
      * supply the Clob instance to an updateXXX method of an updateable
@@ -434,7 +421,7 @@ public class JDBCClob implements Clob {
      * in the <code>Clob</code> object starting at the position
      * <code>pos</code>.  If the end of the <code>Clob</code> value is reached
      * while writing the given string, then the length of the <code>Clob</code>
-     * value will be increased to accomodate the extra characters.
+     * value will be increased to accommodate the extra characters.
      * <p>
      * <b>Note:</b> If the value specified for <code>pos</code>
      * is greater then the length+1 of the <code>CLOB</code> value then the
@@ -452,7 +439,7 @@ public class JDBCClob implements Clob {
      * result of calling JDBCConnection.createClob(), this operation affects
      * only the client-side value; it has no effect upon a value stored in a
      * database because JDBCConnection.createClob() constructs disconnected,
-     * initially empty Clob instances. To propogate the Clob value to a database
+     * initially empty Clob instances. To propagate the Clob value to a database
      * in this case, it is required to supply the Clob instance to an updating
      * or inserting setXXX method of a Prepared or Callable Statement, or to
      * supply the Clob instance to an updateXXX method of an updateable
@@ -499,15 +486,13 @@ public class JDBCClob implements Clob {
     public int setString(long pos, String str, int offset,
                          int len) throws SQLException {
 
-        if (!this.createdByConnection) {
+        if (!m_createdByConnection) {
 
             /** @todo - better error message */
             throw Util.notSupported();
         }
 
-        String ldata = this.data;
-
-        checkValid(ldata);
+        String data = getData();
 
         if (str == null) {
             throw Util.nullArgument("str");
@@ -527,47 +512,43 @@ public class JDBCClob implements Clob {
             throw Util.outOfRangeArgument("pos: " + pos);
         }
 
-        final int    dlen = ldata.length();
+        final int    dlen = data.length();
         final int    ipos = (int) (pos - 1);
         StringBuffer sb;
 
         if (ipos > dlen - len) {
             sb = new StringBuffer(ipos + len);
 
-            sb.append(ldata.substring(0, ipos));
+            sb.append(data.substring(0, ipos));
 
-            ldata = null;
+            data = null;
 
             sb.append(str.substring(offset, offset + len));
 
             str = null;
         } else {
-            sb    = new StringBuffer(ldata);
-            ldata = null;
+            sb   = new StringBuffer(data);
+            data = null;
 
             for (int i = ipos, j = 0; j < len; i++, j++) {
                 sb.setCharAt(i, str.charAt(offset + j));
             }
             str = null;
         }
-
-        // paranoia, in case somone free'd us during the copies.
-        checkValid(this.data);
-
-        this.data = sb.toString();
+        setData(sb.toString());
 
         return len;
     }
 
     /**
-     * Retrieves a stream to be used to write Ascii characters to the
+     * Retrieves a stream to be used to write ASCII characters to the
      * <code>CLOB</code> value that this <code>Clob</code> object represents,
      * starting at position <code>pos</code>.  Characters written to the stream
      * will overwrite the existing characters
      * in the <code>Clob</code> object starting at the position
      * <code>pos</code>.  If the end of the <code>Clob</code> value is reached
      * while writing characters to the stream, then the length of the <code>Clob</code>
-     * value will be increased to accomodate the extra characters.
+     * value will be increased to accommodate the extra characters.
      * <p>
      * <b>Note:</b> If the value specified for <code>pos</code>
      * is greater than the length of the <code>CLOB</code> value, then the
@@ -585,7 +566,7 @@ public class JDBCClob implements Clob {
      * result of calling JDBCConnection.createClob(), this operation affects
      * only the client-side value; it has no effect upon a value stored in a
      * database because JDBCConnection.createClob() constructs disconnected,
-     * initially empty Clob instances. To propogate the Clob value to a database
+     * initially empty Clob instances. To propagate the Clob value to a database
      * in this case, it is required to supply the Clob instance to an updating
      * or inserting setXXX method of a Prepared or Callable Statement, or to
      * supply the Clob instance to an updateXXX method of an updateable
@@ -631,12 +612,12 @@ public class JDBCClob implements Clob {
     public java.io.OutputStream setAsciiStream(
             final long pos) throws SQLException {
 
-        if (!this.createdByConnection) {
+        if (!m_createdByConnection) {
 
             /** @todo - Better error message */
             throw Util.notSupported();
         }
-        checkValid(this.data);
+        checkClosed();
 
         if (pos < MIN_POS || pos > MAX_POS) {
             throw Util.outOfRangeArgument("pos: " + pos);
@@ -666,7 +647,7 @@ public class JDBCClob implements Clob {
      * in the <code>Clob</code> object starting at the position
      * <code>pos</code>.  If the end of the <code>Clob</code> value is reached
      * while writing characters to the stream, then the length of the <code>Clob</code>
-     * value will be increased to accomodate the extra characters.
+     * value will be increased to accommodate the extra characters.
      * <p>
      * <b>Note:</b> If the value specified for <code>pos</code>
      * is greater then the length+1 of the <code>CLOB</code> value then the
@@ -684,7 +665,7 @@ public class JDBCClob implements Clob {
      * result of calling JDBCConnection.createClob(), this operation affects
      * only the client-side value; it has no effect upon a value stored in a
      * database because JDBCConnection.createClob() constructs disconnected,
-     * initially empty Clob instances. To propogate the Clob value to a database
+     * initially empty Clob instances. To propagate the Clob value to a database
      * in this case, it is required to supply the Clob instance to an updating
      * or inserting setXXX method of a Prepared or Callable Statement, or to
      * supply the Clob instance to an updateXXX method of an updateable
@@ -731,12 +712,12 @@ public class JDBCClob implements Clob {
     public java.io.Writer setCharacterStream(
             final long pos) throws SQLException {
 
-        if (!this.createdByConnection) {
+        if (!m_createdByConnection) {
 
             /** @todo - better error message */
             throw Util.notSupported();
         }
-        checkValid(this.data);
+        checkClosed();
 
         if (pos < MIN_POS || pos > MAX_POS) {
             throw Util.outOfRangeArgument("pos: " + pos);
@@ -750,8 +731,6 @@ public class JDBCClob implements Clob {
                     JDBCClob.this.setString(pos, toString());
                 } catch (SQLException se) {
                     throw JavaSystem.toIOException(se);
-                } finally {
-                    super.close();
                 }
             }
         };
@@ -778,7 +757,7 @@ public class JDBCClob implements Clob {
      * result of calling JDBCConnection.createClob(), this operation affects
      * only the client-side value; it has no effect upon a value stored in a
      * database because JDBCConnection.createClob() constructs disconnected,
-     * initially empty Blob instances. To propogate the truncated clob value to
+     * initially empty Blob instances. To propagate the truncated Clob value to
      * a database in this case, it is required to supply the Clob instance to
      * an updating or inserting setXXX method of a Prepared or Callable
      * Statement, or to supply the Blob instance to an updateXXX method of an
@@ -804,11 +783,8 @@ public class JDBCClob implements Clob {
      */
     public void truncate(final long len) throws SQLException {
 
-        final String ldata = this.data;
-
-        this.checkValid(ldata);
-
-        final long dlen = ldata.length();
+        final String data = getData();
+        final long   dlen = data.length();
 
         if (len == dlen) {
 
@@ -818,7 +794,7 @@ public class JDBCClob implements Clob {
         } else {
 
             // no need to get rid of slack
-            data = ldata.substring(0, (int) len);
+            setData(data.substring(0, (int) len));
         }
     }
 
@@ -841,8 +817,9 @@ public class JDBCClob implements Clob {
      * this method
      * @since JDK 1.6, HSQLDB 2.0
      */
-    public void free() throws SQLException {
-        this.data = null;
+    public synchronized void free() throws SQLException {
+        m_closed = true;
+        m_data   = null;
     }
 
     /**
@@ -874,8 +851,9 @@ public class JDBCClob implements Clob {
     // ---------------------- internal implementation --------------------------
     private static final long MIN_POS = 1L;
     private static final long MAX_POS = 1L + (long) Integer.MAX_VALUE;
-    private volatile String   data;
-    private final boolean     createdByConnection;
+    private boolean           m_closed;
+    private String            m_data;
+    private final boolean     m_createdByConnection;
 
     /**
      * Constructs a new JDBCClob object wrapping the given character
@@ -886,7 +864,7 @@ public class JDBCClob implements Clob {
      * As such (in the interest of efficiency) this object maintains a reference
      * to the given String object rather than making a copy and so it is
      * gently suggested (in the interest of effective memory management) that
-     * extenal clients using this constructor either take pause to consider
+     * external clients using this constructor either take pause to consider
      * the implications or at least take care to provide a String object whose
      * internal character buffer is not much larger than required to represent
      * the value.
@@ -896,37 +874,40 @@ public class JDBCClob implements Clob {
      */
     public JDBCClob(final String data) throws SQLException {
 
-        this.init(data);
-
-        this.createdByConnection = false;
+        if (data == null) {
+            throw Util.nullArgument();
+        }
+        m_data                = data;
+        m_createdByConnection = false;
     }
 
     protected JDBCClob() {
-        this.data                = "";
-        this.createdByConnection = true;
+        m_data                = "";
+        m_createdByConnection = true;
     }
 
-    protected void init(String data) throws SQLException {
+    protected synchronized void checkClosed() throws SQLException {
 
-        if (data == null) {
-            throw Util.nullArgument("data");
-        }
-        this.data = data;
-    }
-
-    protected void checkValid(final Object data) {
-
-        if (data == null) {
-            throw new RuntimeException("null data");
+        if (m_closed) {
+            throw Util.sqlException(ErrorCode.X_07501);
         }
     }
 
     protected String data() throws SQLException {
+        return getData();
+    }
 
-        final String ldata = data;
+    private synchronized String getData() throws SQLException {
 
-        checkValid(ldata);
+        checkClosed();
 
-        return ldata;
+        return m_data;
+    }
+
+    private synchronized void setData(String data) throws SQLException {
+
+        checkClosed();
+
+        m_data = data;
     }
 }
