@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,12 +38,11 @@ import java.sql.SQLException;
 
 import org.hsqldb.persist.HsqlDatabaseProperties;
 import org.hsqldb.result.ResultMetaData;
-import org.hsqldb.types.CharacterType;
 import org.hsqldb.types.DateTimeType;
+import org.hsqldb.types.IntervalType;
 import org.hsqldb.types.Type;
-import org.hsqldb.types.Types;
 
-/* $Id: JDBCParameterMetaData.java 3481 2010-02-26 18:05:06Z fredt $ */
+/* $Id: JDBCParameterMetaData.java 4141 2011-03-14 01:35:49Z fredt $ */
 
 /** @todo 1.9.0 - implement internal support for INOUT, OUT return parameter */
 
@@ -67,8 +66,8 @@ import org.hsqldb.types.Types;
  * types and properties for each parameter marker in a <code>CallableStatement</code>
  * object.
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
- * @version 2.0
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
+ * @version 2.0.1
  * @since JDK 1.4, HSQLDB 1.7.2
  * @revised JDK 1.6, HSQLDB 2.0
  */
@@ -337,7 +336,7 @@ public class JDBCParameterMetaData
 
     /** The number of parameters in the described statement */
     int             parameterCount;
-    private boolean translateDTIType;
+    private boolean translateTTIType;
 
     /**
      * Creates a new instance of JDBCParameterMetaData. <p>
@@ -352,8 +351,8 @@ public class JDBCParameterMetaData
         parameterCount = rmd.getColumnCount();
 
         if (conn.clientProperties != null) {
-            translateDTIType = conn.clientProperties.isPropertyTrue(
-                HsqlDatabaseProperties.jdbc_translate_dti_types);
+            translateTTIType = conn.clientProperties.isPropertyTrue(
+                HsqlDatabaseProperties.jdbc_translate_tti_types);
         }
     }
 
@@ -364,10 +363,9 @@ public class JDBCParameterMetaData
      */
     private Type translateType(Type type) {
 
-        if (this.translateDTIType) {
+        if (this.translateTTIType) {
             if (type.isIntervalType()) {
-                type = new CharacterType(Types.SQL_VARCHAR,
-                        type.displaySize());
+                type = ((IntervalType) type).getCharacterType();
             } else if (type.isDateTimeTypeWithZone()) {
                 type = ((DateTimeType) type).getDateTimeTypeWithoutZone();
             }
