@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,9 +42,10 @@ import org.hsqldb.HsqlException;
 import org.hsqldb.StatementTypes;
 import org.hsqldb.navigator.RowSetNavigator;
 import org.hsqldb.result.Result;
+import org.hsqldb.result.ResultConstants;
 import org.hsqldb.result.ResultProperties;
 
-/* $Id: JDBCStatement.java 3529 2010-03-16 14:23:19Z fredt $ */
+/* $Id: JDBCStatement.java 4879 2012-01-06 23:44:17Z fredt $ */
 
 // fredt@users   20020320 - patch 1.7.0 - JDBC 2 support and error trapping
 //
@@ -130,9 +131,9 @@ import org.hsqldb.result.ResultProperties;
  * </div>
  * <!-- end release-specific documentation -->
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.0
+ * @version 2.0.1
  * @revised JDK 1.6, HSQLDB 2.0
  * @see JDBCConnection#createStatement
  * @see JDBCResultSet
@@ -635,7 +636,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         fetchResult(sql, StatementTypes.RETURN_ANY,
                     JDBCStatementBase.NO_GENERATED_KEYS, null, null);
 
-        return resultIn.isData();
+        return currentResultSet != null;
     }
 
     /**
@@ -1205,10 +1206,25 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <div class="ReleaseSpecificDocumentation">
      * <h3>HSQLDB-Specific Information:</h3> <p>
      *
-     * Supported in version 2.0 <p>
+     * Starting with version 2.0, HSQLDB supports this feature with single-row
+     * and multi-row insert, update and merge statements. <p>
+     *
+     * This method returns a result set only if
+     * the executeUpdate methods that was used is one of the three methods that
+     * have the extra parameter indicating return of generated keys<p>
+     *
+     * If the executeUpaged method did not specify the columns which represent
+     * the auto-generated keys the IDENTITY column or GENERATED column(s) of the
+     * table are returned.<p>
+     *
+     * The executeUpdate methods with column indexes or column names return the
+     * post-insert or post-update values of the specified columns, whether the
+     * columns are generated or not. This allows values that have been modified
+     * by execution of triggers to be returned.<p>
      *
      * If column names or indexes provided by the user in the executeUpdate()
-     * method calls are not correct, an empty result is returned.
+     * method calls do not correspond to table columns (incorrect names or
+     * indexes larger than the coloum count), an empty result is returned.
      *
      * </div>
      * <!-- end release-specific documentation -->
@@ -1242,7 +1258,10 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <div class="ReleaseSpecificDocumentation">
      * <h3>HSQLDB-Specific Information:</h3> <p>
      *
-     * Starting with version 2.0, HSQLDB supports this feature.
+     * Starting with version 2.0, HSQLDB supports returning generated columns
+     * with single-row and multi-row INSERT, UPDATE and MERGE statements. <p>
+     * If the table has an IDENTITY or GENERATED column(s) the values for these
+     * columns are returned in the next call to getGeneratedKeys().
      *
      * </div>
      * <!-- end release-specific documentation -->
@@ -1302,9 +1321,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <div class="ReleaseSpecificDocumentation">
      * <h3>HSQLDB-Specific Information:</h3> <p>
      *
-     * Starting with version 2.0, HSQLDB supports this feature with single-row and
-     * multi-row inserts. <p>
-     * CHECKME: possibly change method depending to final GENERATED column support.
+     * Starting with version 2.0, HSQLDB supports returning generated columns
+     * with single-row and multi-row INSERT, UPDATE and MERGE statements. <p>
+     * The columnIndexes may specify any set of columns of the table.
      *
      * </div>
      * <!-- end release-specific documentation -->
@@ -1336,7 +1355,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
             throw Util.invalidArgument("columnIndexes");
         }
         fetchResult(sql, StatementTypes.RETURN_COUNT,
-                    Statement.RETURN_GENERATED_KEYS, columnIndexes, null);
+                    ResultConstants.RETURN_GENERATED_KEYS_COL_INDEXES,
+                    columnIndexes, null);
 
         return resultIn.getUpdateCount();
     }
@@ -1357,7 +1377,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <div class="ReleaseSpecificDocumentation">
      * <h3>HSQLDB-Specific Information:</h3> <p>
      *
-     * Starting with 2.0, HSQLDB supports this feature.
+     * Starting with version 2.0, HSQLDB supports returning generated columns
+     * with single-row and multi-row INSERT, UPDATE and MERGE statements. <p>
+     * The columnNames may specify any set of columns of the table.
      *
      * </div>
      * <!-- end release-specific documentation -->
@@ -1388,7 +1410,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
             throw Util.invalidArgument("columnIndexes");
         }
         fetchResult(sql, StatementTypes.RETURN_COUNT,
-                    Statement.RETURN_GENERATED_KEYS, null, columnNames);
+                    ResultConstants.RETURN_GENERATED_KEYS_COL_NAMES, null,
+                    columnNames);
 
         return resultIn.getUpdateCount();
     }
@@ -1524,7 +1547,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
             throw Util.invalidArgument("columnIndexes");
         }
         fetchResult(sql, StatementTypes.RETURN_ANY,
-                    Statement.RETURN_GENERATED_KEYS, columnIndexes, null);
+                    ResultConstants.RETURN_GENERATED_KEYS_COL_INDEXES,
+                    columnIndexes, null);
 
         return resultIn.isData();
     }
@@ -1590,7 +1614,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
             throw Util.invalidArgument("columnIndexes");
         }
         fetchResult(sql, StatementTypes.RETURN_ANY,
-                    Statement.RETURN_GENERATED_KEYS, null, columnNames);
+                    ResultConstants.RETURN_GENERATED_KEYS_COL_NAMES, null,
+                    columnNames);
 
         return resultIn.isData();
     }
@@ -1761,10 +1786,10 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      */
     JDBCStatement(JDBCConnection c, int props) {
 
-        resultOut    = Result.newExecuteDirectRequest();
-        connection   = c;
+        resultOut             = Result.newExecuteDirectRequest();
+        connection            = c;
         connectionIncarnation = connection.incarnation;
-        rsProperties = props;
+        rsProperties          = props;
     }
 
     /**
@@ -1804,5 +1829,13 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         if (resultIn.isError()) {
             throw Util.sqlException(resultIn);
         }
+
+        if (resultIn.isData()) {
+            currentResultSet = new JDBCResultSet(connection, this, resultIn,
+                    resultIn.metaData);
+        } else if (resultIn.getStatementType()
+                   == StatementTypes.RETURN_RESULT) {
+            getMoreResults();
+        }
     }
 }
