@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.result.Result;
 
-/* $Id: Util.java 3481 2010-02-26 18:05:06Z fredt $ */
+/* $Id: Util.java 4413 2011-09-17 10:38:53Z fredt $ */
 
 // boucherb@users - 20060523 - patch 1.9.0 - removed some unused imports
 // Revision 1.16  2006/07/12 11:53:53  boucherb
@@ -60,9 +60,9 @@ import org.hsqldb.result.Result;
  * Provides driver constants and a gateway from internal HsqlExceptions to
  * external SQLExceptions.
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.0.1
  * @since 1.7.2
  */
 public class Util {
@@ -75,8 +75,12 @@ public class Util {
 
 //#else
 /*
-        throw new SQLException(e.getMessage(), e.getSQLState(),
-                               e.getErrorCode());
+        SQLException se = new SQLException(e.getMessage(), e.getSQLState(),
+                                           e.getErrorCode());
+
+        se.initCause(e);
+
+        throw se;
 */
 
 //#endif JAVA6
@@ -90,8 +94,14 @@ public class Util {
 
 //#else
 /*
-        throw new SQLException(r.getMainString(), r.getSubString(),
-                               r.getErrorCode());
+        SQLException se = new SQLException(r.getMainString(),
+                                           r.getSubString(), r.getErrorCode());
+
+        if (r.getException() != null) {
+            se.initCause(r.getException());
+        }
+
+        throw se;
 */
 
 //#endif JAVA6
@@ -105,8 +115,12 @@ public class Util {
 
 //#else
 /*
-        return new SQLException(e.getMessage(), e.getSQLState(),
-                                e.getErrorCode());
+        SQLException se = new SQLException(e.getMessage(), e.getSQLState(),
+                                           e.getErrorCode());
+
+        se.initCause(e);
+
+        return se;
 */
 
 //#endif JAVA6
@@ -121,8 +135,14 @@ public class Util {
 
 //#else
 /*
-        return new SQLException(e.getMessage(), e.getSQLState(),
-                                e.getErrorCode());
+        SQLException se = new SQLException(e.getMessage(), e.getSQLState(),
+                                           e.getErrorCode());
+
+        if (cause != null) {
+            se.initCause(cause);
+        }
+
+        return se;
 */
 
 //#endif JAVA6
@@ -141,7 +161,7 @@ public class Util {
     }
 
     public static final SQLException sqlException(int id, String message,
-            Exception cause) {
+            Throwable cause) {
         return sqlException(Error.error(id, message), cause);
     }
 
@@ -154,7 +174,19 @@ public class Util {
     }
 
     static final SQLException notSupported() {
+
+//#ifdef JAVA6
+        HsqlException e = Error.error(ErrorCode.X_0A000);
+
+        return new SQLFeatureNotSupportedException(e.getMessage(),
+                e.getSQLState(), -ErrorCode.X_0A000);
+
+//#else
+/*
         return sqlException(Error.error(ErrorCode.X_0A000));
+*/
+
+//#endif JAVA6
     }
 
     static SQLException notUpdatableColumn() {
@@ -194,16 +226,37 @@ public class Util {
                               r.getErrorCode());
     }
 
-    public static SQLException sqlException(Result r) {
+    public static SQLException sqlException(Throwable t) {
 
 //#ifdef JAVA6
-        return new SQLException(r.getMainString(), r.getSubString(),
-                                r.getErrorCode(), r.getException());
+        return new SQLNonTransientConnectionException(t);
 
 //#else
 /*
-        return new SQLException(r.getMainString(), r.getSubString(),
-                                r.getErrorCode());
+        return new SQLException(t.getMessage(),
+                                Error.getStateString(ErrorCode.GENERAL_ERROR),
+                                ErrorCode.GENERAL_ERROR);
+*/
+
+//#endif JAVA6
+    }
+
+    public static SQLException sqlException(Result r) {
+
+//#ifdef JAVA6
+        return sqlException(r.getMainString(), r.getSubString(),
+                            r.getErrorCode(), r.getException());
+
+//#else
+/*
+        SQLException se = new SQLException(r.getMainString(),
+                                           r.getSubString(), r.getErrorCode());
+
+        if (r.getException() != null) {
+            se.initCause(r.getException());
+        }
+
+        return se;
 */
 
 //#endif JAVA6
