@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,6 +42,8 @@ import java.io.FileInputStream;
 import java.util.Properties;
 import java.io.IOException;
 import java.util.HashMap;
+import java.util.Set;
+import java.util.HashSet;
 import java.lang.reflect.Method;
 
 /**
@@ -75,107 +77,224 @@ import java.lang.reflect.Method;
  *   logger.finer("Doing something log-worthy");
  * </PRE> </CODE>
  *
+ * <p/>
+ * The system level property <code>hsqldb.reconfig_logging=false</code> is
+ * required to avoid configuration of java.util.logging. Otherwise
+ * configuration takes place.<p/>
+ *
  * @author Blaine Simpson (blaine dot simpson at admc dot com)
- * @version 1.9.0
+ * @version 2.2.8
  * @since 1.9.0
  */
 public class FrameworkLogger {
 
-    static private Map     loggerInstances  = new HashMap();
-    static private Map     jdkToLog4jLevels = new HashMap();
-    static private Method  log4jGetLogger;
-    static private Method  log4jLogMethod;
-    private Object         log4jLogger;
-    private Logger         jdkLogger;
-    static private boolean haveLoadedOurDefault;
-    static private ConsoleHandler  consoleHandler = new ConsoleHandler();
+    /*
+     * FrameworkLogger coders:  It would be convenient to be able to log
+     * states and such at debug level in this class.
+     * I tentatively think that using a logger instance early in the static
+     * lifecycle is too risky, possibly using the underlying plumbing before
+     * the application has had a chance to customize, and perhaps before
+     * classloaders have been re-prioritized, etc.
+     * Could be that it all works out ok, but make sure you consider all
+     * situations before logging with FrameworkLogger instances here.
+     * This is one reason why there are a couple uses of System.err below.
+     */
+
+    /**
+     * Utility method for integrators.
+     * Returns a string representation of the active Logger instance keys.
+     * <p>
+     * Not named similar to 'toString' to avoid ambiguity with instance method
+     * toString.
+     * </p>
+     */
+    public static String report() {
+
+        return new StringBuilder().append(loggerInstances.size()).append(
+            " logger instances:  ").append(
+            loggerInstances.keySet()).toString();
+    }
+
+    static private Map    loggerInstances  = new HashMap();
+    static private Map    jdkToLog4jLevels = new HashMap();
+    static private Method log4jGetLogger;
+    static private Method log4jLogMethod;
+    private Object        log4jLogger;
+    private Logger        jdkLogger;
+
     // No need for more than one static, since we have only one console
+    static private boolean noopMode;    // If true, then logging calls do nothing
 
     static {
-        reconfigure();
+        try {
+            reconfigure();
+        } catch (java.lang.SecurityException e) {}
+    }
+
+    /**
+     * Frees Logger(s), if any, with the specified category, or that begins with
+     * the specified prefix + dot.
+     * <p>
+     * Note that as of today, this depends on the underlying logging framework
+     * implementation to release the underlying Logger instances.
+     * JUL in Sun's JVM uses weak references, so that should be fine.
+     * Log4j as of today seems to use strong references (and no API hooks to
+     * free anything), so this method will probably have little benefit for
+     * Log4j.
+     * </p>
+     */
+    public static synchronized void clearLoggers(String prefixToZap) {
+
+        Set                targetKeys = new HashSet();
+        java.util.Iterator it         = loggerInstances.keySet().iterator();
+        String             k;
+        String             dottedPrefix = prefixToZap + '.';
+
+        while (it.hasNext()) {
+            k = (String) it.next();
+
+            if (k.equals(prefixToZap) || k.startsWith(dottedPrefix)) {
+                targetKeys.add(k);
+            }
+        }
+
+        loggerInstances.keySet().removeAll(targetKeys);
     }
 
     static void reconfigure() {
+
+        noopMode = false;
+
         Class log4jLoggerClass = null;
+
         loggerInstances.clear();
 
+        log4jLoggerClass = null;
+        log4jGetLogger   = null;
+        log4jLogMethod   = null;
+
         try {
             log4jLoggerClass = Class.forName("org.apache.log4j.Logger");
         } catch (Exception e) {
-            log4jLoggerClass = null;
+
+            // The class will only load successfully if Log4j thinks it is
+            // in usable state.
+            // Intentionally empty.
         }
 
-        if (log4jLoggerClass == null) try {
-            log4jGetLogger = null;
-            log4jLogMethod = null;
+        // Try log4j first so we can fall back to JUL if anything goes wrong.
+        if (log4jLoggerClass != null) {
+            try {
+                if (jdkToLog4jLevels.size() < 1) {
+                    Method log4jToLevel = Class.forName(
+                        "org.apache.log4j.Level").getMethod(
+                        "toLevel", new Class[]{ String.class });
+
+                    jdkToLog4jLevels.put(Level.ALL,
+                                         log4jToLevel.invoke(null,
+                                             new Object[]{ "ALL" }));
+                    jdkToLog4jLevels.put(Level.FINER,
+                                         log4jToLevel.invoke(null,
+                                             new Object[]{ "DEBUG" }));
+                    jdkToLog4jLevels.put(Level.WARNING,
+                                         log4jToLevel.invoke(null,
+                                             new Object[]{ "ERROR" }));
+                    jdkToLog4jLevels.put(Level.SEVERE,
+                                         log4jToLevel.invoke(null,
+                                             new Object[]{ "FATAL" }));
+                    jdkToLog4jLevels.put(Level.INFO,
+                                         log4jToLevel.invoke(null,
+                                             new Object[]{ "INFO" }));
+                    jdkToLog4jLevels.put(Level.OFF,
+                                         log4jToLevel.invoke(null,
+                                             new Object[]{ "OFF" }));
+                    jdkToLog4jLevels.put(Level.FINEST,
+                                         log4jToLevel.invoke(null,
+                                             new Object[]{ "TRACE" }));
+                    jdkToLog4jLevels.put(Level.WARNING,
+                                         log4jToLevel.invoke(null,
+                                             new Object[]{ "WARN" }));
+                }
+
+                log4jLogMethod = log4jLoggerClass.getMethod("log",
+                        new Class[] {
+                    String.class, Class.forName("org.apache.log4j.Priority"),
+                    Object.class, Throwable.class
+                });
+                log4jGetLogger = log4jLoggerClass.getMethod("getLogger",
+                        new Class[]{ String.class });
+
+                // This last object is what we toggle on to generate either
+                // Log4j or Jdk Logger objects (to wrap).
+                return;    // Success for Log4j
+            } catch (Exception e) {
+
+                // This is an unexpected  problem, because our Log4j try block will
+                // only be attempted if Log4j itself initialized (even if it
+                // successfully initialized with warnings due to bad config).
+                try {
+                    System.err.println(
+                        "<clinit> failure "
+                        + "instantiating configured Log4j system: " + e);
+
+                    // It's possible we don't have write access to System.err.
+                } catch (Throwable t) {
+
+                    // Intentionally empty.  We tried our best to report problem,
+                    // but don't want to throw and prevent JUL from working.
+                }
+            }
+        }
+
+        log4jLoggerClass = null;
+        log4jLogMethod   = null;
+        log4jGetLogger   = null;
+
+        String propVal = System.getProperty("hsqldb.reconfig_logging");
+
+        if (propVal != null && propVal.equalsIgnoreCase("false")) {
+            return;
+        }
+
+        try {
             LogManager lm = LogManager.getLogManager();
-            if (haveLoadedOurDefault || isDefaultJdkConfig()) {
-                haveLoadedOurDefault = true;
+
+            if (isDefaultJdkConfig()) {
+                lm.reset();
+
+                ConsoleHandler consoleHandler = new ConsoleHandler();
+
                 consoleHandler.setFormatter(
-                        new BasicTextJdkLogFormatter(false));
+                    new BasicTextJdkLogFormatter(false));
                 consoleHandler.setLevel(Level.INFO);
                 lm.readConfiguration(
-                        FrameworkLogger.class.getResourceAsStream(
+                    FrameworkLogger.class.getResourceAsStream(
                         "/org/hsqldb/resources/jdklogging-default.properties"));
+
                 Logger cmdlineLogger = Logger.getLogger("org.hsqldb.cmdline");
+
                 cmdlineLogger.addHandler(consoleHandler);
                 cmdlineLogger.setUseParentHandlers(false);
             } else {
-                // Do not intervene.  Use JDK logging exactly as configured by
-                // user.
+
+                // Do not intervene.  Use JDK logging exactly as configured
+                // by user.
                 lm.readConfiguration();
-                // The only bad thing about doing this is that if the app has
-                // programmatically changed the logging config after starting
-                // the program but before using FrameworkLogger, we will
-                // clobber those customizations.
+
+                // The only bad thing about doing this is that if the app
+                // has programmatically changed the logging config after
+                // starting the program but before using FrameworkLogger,
+                // we will clobber those customizations.
+                // Set sys srop 'hsqldb.reconfig_logging' to false to
+                // prevent this.
             }
         } catch (Exception e) {
-            throw new RuntimeException(
-                "<clinit> failure initializing JDK logging system", e);
-        } else try {
-            Method log4jToLevel = Class.forName(
-                "org.apache.log4j.Level").getMethod(
-                "toLevel", new Class[]{ String.class });
+            noopMode = true;
 
-            jdkToLog4jLevels.put(Level.ALL,
-                                 log4jToLevel.invoke(null,
-                                     new Object[]{ "ALL" }));
-            jdkToLog4jLevels.put(Level.FINER,
-                                 log4jToLevel.invoke(null,
-                                     new Object[]{ "DEBUG" }));
-            jdkToLog4jLevels.put(Level.WARNING,
-                                 log4jToLevel.invoke(null,
-                                     new Object[]{ "ERROR" }));
-            jdkToLog4jLevels.put(Level.SEVERE,
-                                 log4jToLevel.invoke(null,
-                                     new Object[]{ "FATAL" }));
-            jdkToLog4jLevels.put(Level.INFO,
-                                 log4jToLevel.invoke(null,
-                                     new Object[]{ "INFO" }));
-            jdkToLog4jLevels.put(Level.OFF,
-                                 log4jToLevel.invoke(null,
-                                     new Object[]{ "OFF" }));
-            jdkToLog4jLevels.put(Level.FINEST,
-                                 log4jToLevel.invoke(null,
-                                     new Object[]{ "TRACE" }));
-            jdkToLog4jLevels.put(Level.WARNING,
-                                 log4jToLevel.invoke(null,
-                                     new Object[]{ "WARN" }));
-
-            log4jLogMethod = log4jLoggerClass.getMethod("log",
-                    new Class[] {
-                String.class, Class.forName("org.apache.log4j.Priority"),
-                Object.class, Throwable.class
-            });
-
-            log4jGetLogger = log4jLoggerClass.getMethod("getLogger",
-                    new Class[]{ String.class });
-
-            // This last object is what we toggle on to generate either
-            // Log4j or Jdk Logger objects (to wrap).
-        } catch (Exception e) {
-            throw new RuntimeException(
-                "<clinit> failure instantiating present Log4j system", e);
+            System.err.println(
+                "<clinit> failure initializing JDK logging system.  "
+                + "Continuing without Application logging.");
+            e.printStackTrace();
         }
     }
 
@@ -184,14 +303,17 @@ public class FrameworkLogger {
      */
     private FrameworkLogger(String s) {
 
-        if (log4jGetLogger == null) {
-            jdkLogger = Logger.getLogger(s);
-        } else {
-            try {
-                log4jLogger = log4jGetLogger.invoke(null, new Object[]{ s });
-            } catch (Exception e) {
-                throw new RuntimeException(
-                    "Failed to instantiate Log4j Logger", e);
+        if (!noopMode) {
+            if (log4jGetLogger == null) {
+                jdkLogger = Logger.getLogger(s);
+            } else {
+                try {
+                    log4jLogger = log4jGetLogger.invoke(null,
+                                                        new Object[]{ s });
+                } catch (Exception e) {
+                    throw new RuntimeException(
+                        "Failed to instantiate Log4j Logger", e);
+                }
             }
         }
 
@@ -218,14 +340,24 @@ public class FrameworkLogger {
      * associated FileHander using the supplied String identifier.
      */
     public static FrameworkLogger getLog(Class c, String contextId) {
-        return (contextId == null)
-                ?  getLog(c)
-                :  getLog(contextId + '.' + c.getName());
+        return (contextId == null) ? getLog(c)
+                                   : getLog(contextId + '.' + c.getName());
     }
 
     /**
-     * Alternative entry-point into this logging sytem, for cases where
-     * your want to share a single logger instance among multiple classes,
+     * This method just defers to the getLog(String) method unless default
+     * (no local configuration) JDK logging is being used;
+     * In that case, this method assures that the returned logger has an
+     * associated FileHander using the supplied String identifier.
+     */
+    public static FrameworkLogger getLog(String baseId, String contextId) {
+        return (contextId == null) ? getLog(baseId)
+                                   : getLog(contextId + '.' + baseId);
+    }
+
+    /**
+     * Alternative entry-point into this logging system, for cases where
+     * you want to share a single logger instance among multiple classes,
      * or you want to use multiple logger instances from a single class.
      *
      * @see #getLog(Class)
@@ -261,8 +393,12 @@ public class FrameworkLogger {
      * If you don't know that you want to do this, then you should not use
      * this method.
      */
-    public void privlog(Level level, String message,
-    Throwable t, int revertMethods, Class skipClass) {
+    public void privlog(Level level, String message, Throwable t,
+                        int revertMethods, Class skipClass) {
+
+        if (noopMode) {
+            return;
+        }
 
         if (log4jLogger == null) {
             StackTraceElement elements[] = new Throwable().getStackTrace();
@@ -288,10 +424,14 @@ public class FrameworkLogger {
     }
 
     public void enduserlog(Level level, String message) {
+
         /* This method is SqlTool-specific, which is where this class began at.
          * Need to move this back there, but it needs access to the logging
          * structures private to this class.  Thinking...
          */
+        if (noopMode) {
+            return;
+        }
 
         if (log4jLogger == null) {
             String c = FrameworkLogger.class.getName();
@@ -302,8 +442,7 @@ public class FrameworkLogger {
             try {
                 log4jLogMethod.invoke(log4jLogger, new Object[] {
                     FrameworkLogger.class.getName(),
-                    jdkToLog4jLevels.get(level),
-                    message, null
+                    jdkToLog4jLevels.get(level), message, null
                 });
 
                 // Test where SqlFile correct here.
@@ -443,41 +582,56 @@ public class FrameworkLogger {
      * If the JRE-provided config file is not in the expected place, then
      * we return false.
      */
-    static public boolean isDefaultJdkConfig() {
+    public static boolean isDefaultJdkConfig() {
+
         File globalCfgFile = new File(System.getProperty("java.home"),
-                "lib/logging.properties");
+                                      "lib/logging.properties");
+
         if (!globalCfgFile.isFile()) {
             return false;
         }
+
         FileInputStream fis = null;
-        LogManager lm = LogManager.getLogManager();
+        LogManager      lm  = LogManager.getLogManager();
+
         try {
             fis = new FileInputStream(globalCfgFile);
+
             Properties defaultProps = new Properties();
+
             defaultProps.load(fis);
+
             Enumeration names = defaultProps.propertyNames();
-            int i = 0;
-            String name;
-            String liveVal;
+            int         i     = 0;
+            String      name;
+            String      liveVal;
+
             while (names.hasMoreElements()) {
                 i++;
-                name = (String) names.nextElement();
+
+                name    = (String) names.nextElement();
                 liveVal = lm.getProperty(name);
+
                 if (liveVal == null) {
                     return false;
                 }
+
                 if (!lm.getProperty(name).equals(liveVal)) {
                     return false;
                 }
             }
+
             return true;
         } catch (IOException ioe) {
             return false;
         } finally {
-            if (fis != null) try {
-                fis.close();
-            } catch (IOException ioe) {
-                // Intentional no-op
+            if (fis != null) {
+                try {
+                    fis.close();
+                } catch (IOException ioe) {
+
+                    // Intentional no-op
+                }
             }
         }
     }
