@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,7 +42,7 @@ import java.sql.SQLException;
 import java.util.Properties;
 import java.util.StringTokenizer;
 
-/* $Id: RCData.java 3481 2010-02-26 18:05:06Z fredt $ */
+/* $Id: RCData.java 4254 2011-05-15 02:24:14Z unsaved $ */
 
 /**
  * Manages all the details we need to connect up to JDBC database(s),
@@ -327,20 +327,26 @@ public class RCData {
                                            SQLException {
 
         // Local vars to satisfy compiler warnings
-        String curDriver = curDriverIn;
-        String curTrustStore = curTrustStoreIn;
+        String curDriver = null;
+        String curTrustStore = null;
 
         Properties sysProps = System.getProperties();
 
-        if (curDriver == null) {
+        if (curDriverIn == null) {
 
             // If explicit driver not specified
             curDriver = ((driver == null) ? DEFAULT_JDBC_DRIVER
                                           : driver);
+        } else {
+            curDriver = expandSysPropVars(curDriverIn);
         }
 
-        if (curTrustStore == null && truststore != null) {
-            curTrustStore = truststore;
+        if (curTrustStoreIn == null) {
+            if (truststore != null) {
+                curTrustStore = expandSysPropVars(truststore);
+            }
+        } else {
+            curTrustStore = expandSysPropVars(curTrustStoreIn);
         }
 
         if (curTrustStore == null) {
@@ -354,7 +360,7 @@ public class RCData {
         try {
             urlString = expandSysPropVars(url);
         } catch (IllegalArgumentException iae) {
-            throw new MalformedURLException(iae.getMessage() + " for URL '"
+            throw new MalformedURLException(iae.toString() + " for URL '"
                                             + url + "'");
         }
 
@@ -363,7 +369,7 @@ public class RCData {
         if (username != null) try {
             userString = expandSysPropVars(username);
         } catch (IllegalArgumentException iae) {
-            throw new MalformedURLException(iae.getMessage()
+            throw new MalformedURLException(iae.toString()
                                             + " for user name '" + username
                                             + "'");
         }
@@ -373,7 +379,7 @@ public class RCData {
         if (password != null) try {
             passwordString = expandSysPropVars(password);
         } catch (IllegalArgumentException iae) {
-            throw new MalformedURLException(iae.getMessage()
+            throw new MalformedURLException(iae.toString()
                                             + " for password");
         }
 
@@ -393,7 +399,7 @@ public class RCData {
         // Debug:
         // System.err.println("TI set to " + ti + "\nPOST: "
         // + SqlTool.tiToString(c.getTransactionIsolation()));
-        
+
         return c;
     }
 
