@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,6 +31,8 @@
 
 package org.hsqldb.lib;
 
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.PropertyResourceBundle;
 import java.util.Map;
 import java.util.HashMap;
@@ -45,7 +47,7 @@ import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 
 
-/* $Id: RefCapablePropertyResourceBundle.java 3481 2010-02-26 18:05:06Z fredt $ */
+/* $Id: RefCapablePropertyResourceBundle.java 4141 2011-03-14 01:35:49Z fredt $ */
 
 /**
  * Just like PropertyResourceBundle, except keys mapped to nothing in the
@@ -330,7 +332,7 @@ public class RefCapablePropertyResourceBundle {
      *           Otherwise returns a copy of inString, with all \n's
      *           transformed to the platform's line separators.
      */
-    static public String toNativeLs(String inString) {
+    public static String toNativeLs(String inString) {
         return LS.equals("\n") ? inString : inString.replaceAll("\\Q\n", LS);
     }
 
@@ -383,13 +385,19 @@ public class RefCapablePropertyResourceBundle {
      */
     private InputStream getMostSpecificStream(
             String key, String l, String c, String v) {
-        String filePath = baseName.replace('.', '/') + '/' + key
+        final String filePath = baseName.replace('.', '/') + '/' + key
                 + ((l == null) ? "" : ("_" + l))
                 + ((c == null) ? "" : ("_" + c))
                 + ((v == null) ? "" : ("_" + v))
                 + ".text";
         // System.err.println("Seeking " + filePath);
-        InputStream is = loader.getResourceAsStream(filePath);
+        InputStream is = (InputStream) AccessController.doPrivileged(
+            new PrivilegedAction() {
+
+            public InputStream run() {
+                return loader.getResourceAsStream(filePath);
+            }
+        });
         // N.b.  If were using Class.getRes... instead of ClassLoader.getRes...
         // we would need to prefix the path with "/".
         return (is == null && l != null)
