@@ -1,7 +1,7 @@
 /*
  * For work developed by the HSQL Development Group:
  *
- * Copyright (c) 2001-2010, The HSQL Development Group
+ * Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -72,8 +72,6 @@ package org.hsqldb.lib;
 
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
-import java.io.StringWriter;
 import java.io.UTFDataFormatException;
 
 import org.hsqldb.store.BitMap;
@@ -86,7 +84,7 @@ import org.hsqldb.store.BitMap;
  *
  * @author Thomas Mueller (Hypersonic SQL Group)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.0.1
  * @since 1.7.2
  */
 public class StringConverter {
@@ -633,35 +631,29 @@ public class StringConverter {
     }
 
     /**
-     * Using a Reader and a Writer, returns a String from an InputStream.
+     * Using an output stream, returns a String from an InputStream.
      *
-     * Method based on Hypersonic Code
-     *
-     * @param x InputStream to read from
+     * @param is InputStream to read from
+     * @param encoding character encoding of the string
      * @throws IOException
      * @return a Java string
      */
-    public static String inputStreamToString(InputStream x,
+    public static String inputStreamToString(InputStream is,
             String encoding) throws IOException {
 
-        InputStreamReader in        = new InputStreamReader(x, encoding);
-        StringWriter      writer    = new StringWriter();
-        int               blocksize = 8 * 1024;
-        char[]            buffer    = new char[blocksize];
+        HsqlByteArrayOutputStream baOS = new HsqlByteArrayOutputStream(1024);
 
-        for (;;) {
-            int read = in.read(buffer);
+        while (true) {
+            int c = is.read();
 
-            if (read == -1) {
+            if (c == -1) {
                 break;
             }
 
-            writer.write(buffer, 0, read);
+            baOS.write(c);
         }
 
-        writer.close();
-
-        return writer.toString();
+        return new String(baOS.getBuffer(), 0, baOS.size(), encoding);
     }
 
 // fredt@users 20020130 - patch 497872 by Nitin Chauhan - use byte[] of exact size
@@ -731,4 +723,127 @@ public class StringConverter {
 
         return count;
     }
+
+    /**
+     * Converts the string to an HTML representation in the ASCII character set.
+     *
+     * The string is treated as an SQL statement.
+     *
+     * @param b the byte array
+     * @return UUID string form
+     */
+    public static void stringToHtmlBytes(HsqlByteArrayOutputStream b,
+                                         String s) {
+
+        if (s == null) {
+            return;
+        }
+
+        final int len = s.length();
+        char[]    chars;
+
+        if (len == 0) {
+            return;
+        }
+
+        chars = s.toCharArray();
+
+        b.ensureRoom(len);
+
+        for (int i = 0; i < len; i++) {
+            char c = chars[i];
+
+            if (c > 0x007f || c == '"' || c == '&' || c == '<' || c == '>') {
+                int codePoint = Character.codePointAt(chars, i);
+
+                if (Character.charCount(codePoint) == 2) {
+                    i++;
+                }
+
+                b.ensureRoom(16);
+                b.writeNoCheck('&');
+                b.writeNoCheck('#');
+                b.writeBytes(String.valueOf(codePoint));
+                b.writeNoCheck(';');
+            } else if (c < 0x0020 ) {
+                b.writeNoCheck(' ');
+            } else {
+                b.writeNoCheck(c);
+            }
+        }
+    }
+
+    /**
+     * Returns a string representation in UUID form from a binary string.
+     *
+     * UUID string is composed of 8-4-4-4-12 hexadecimal characters.
+     *
+     * @param b the byte array
+     * @return UUID string form
+     */
+    public static String toStringUUID(byte[] b) {
+
+        char[] chars = new char[36];
+        int    hexIndex;
+
+        if (b == null) {
+            return null;
+        }
+
+        if (b.length != 16) {
+            throw new NumberFormatException();
+        }
+
+        for (int i = 0, j = 0; i < b.length; ) {
+            hexIndex   = (b[i] & 0xf0) >> 4;
+            chars[j++] = (char) HEXBYTES[hexIndex];
+            hexIndex   = b[i] & 0xf;
+            chars[j++] = (char) HEXBYTES[hexIndex];
+
+            i++;
+
+            if (i >= 4 && i <= 10 && (i % 2) == 0) {
+                chars[j++] = '-';
+            }
+        }
+
+        return new String(chars);
+    }
+
+    /**
+     * Returns a byte[] representation in UUID form from a UUID string.
+     *
+     * @param s the UUID string
+     * @return byte array
+     */
+    public static byte[] toBinaryUUID(String s) {
+
+        byte[] bytes = new byte[16];
+
+        if (s == null) {
+            return null;
+        }
+
+        if (s.length() != 36) {
+            throw new NumberFormatException();
+        }
+
+        for (int i = 0, j = 0; i < bytes.length; ) {
+            char c    = s.charAt(j++);
+            int  high = getNibble(c);
+
+            c        = s.charAt(j++);
+            bytes[i] = (byte) ((high << 4) + getNibble(c));
+
+            i++;
+
+            if (i >= 4 && i <= 10 && (i % 2) == 0) {
+                c = s.charAt(j++);
+
+                if (c != '-') {}
+            }
+        }
+
+        return bytes;
+    }
 }
