@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,18 +47,15 @@ import org.hsqldb.lib.StringUtil;
  * Manages a .properties file for a database.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.7.0
  */
 public class HsqlDatabaseProperties extends HsqlProperties {
 
-    // We are using persist.Logger-instance-specific FrameworkLogger
-    // because it is Database-instance specific.
-    // If add any static level logging, should instantiate a standard,
-    // context-agnostic FrameworkLogger for that purpose.
     private static final String hsqldb_method_class_names =
         "hsqldb.method_class_names";
     private static HashSet accessibleJavaMethodNames;
+    private static boolean allowFullPath;
 
     static {
         try {
@@ -73,6 +70,14 @@ public class HsqlDatabaseProperties extends HsqlProperties {
                     accessibleJavaMethodNames.add(names[i]);
                 }
             }
+
+            prop = System.getProperty(hsqldb_method_class_names);
+
+            if (prop != null) {
+                if (Boolean.valueOf(prop)) {
+                    allowFullPath = true;
+                }
+            }
         } catch (Exception e) {}
     }
 
@@ -88,8 +93,6 @@ public class HsqlDatabaseProperties extends HsqlProperties {
      * asterisk.<p>
      *
      * All methods of java.lang.Math are always accessible.
-     *
-     *
      */
     public static boolean supportsJavaMethod(String name) {
 
@@ -131,10 +134,12 @@ public class HsqlDatabaseProperties extends HsqlProperties {
     // db files modified
     public static final int     FILES_NOT_MODIFIED = 0;
     public static final int     FILES_MODIFIED     = 1;
-    public static final int     FILES_NEW          = 2;
+    public static final int     FILES_MODIFIED_NEW = 2;
+    public static final int     FILES_NEW          = 3;
     private static final String MODIFIED_NO        = "no";
     private static final String MODIFIED_YES       = "yes";
-    private static final String MODIFIED_NEW       = "no-new-files";
+    private static final String MODIFIED_YES_NEW   = "yes-new-files";
+    private static final String MODIFIED_NO_NEW    = "no-new-files";
 
     // allowed property metadata
     private static final HashMap dbMeta   = new HashMap(67);
@@ -142,13 +147,13 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
     // versions
     public static final String VERSION_STRING_1_8_0 = "1.8.0";
-    public static final String THIS_VERSION         = "2.0.0";
-    public static final String THIS_FULL_VERSION    = "2.0.0";
+    public static final String THIS_VERSION         = "2.2.9";
+    public static final String THIS_FULL_VERSION    = "2.2.9";
     public static final String THIS_CACHE_VERSION   = "2.0.0";
     public static final String PRODUCT_NAME         = "HSQL Database Engine";
     public static final int    MAJOR                = 2,
-                               MINOR                = 0,
-                               REVISION             = 0;
+                               MINOR                = 2,
+                               REVISION             = 9;
 
     /**
      * system properties supported by HSQLDB
@@ -159,18 +164,22 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         "hsqldb.max_char_or_varchar_display_size";
 
     //
-    public static final String hsqldb_inc_backup = "hsqldb.incremental_backup";
+    public static final String hsqldb_inc_backup = "hsqldb.inc_backup";
 
     //
     public static final String  hsqldb_version  = "version";
     public static final String  hsqldb_readonly = "readonly";
     private static final String hsqldb_modified = "modified";
 
+    //
+    public static final String hsqldb_cache_version = "hsqldb.cache_version";
+
     //
     public static final String runtime_gc_interval = "runtime.gc_interval";
 
     //
     public static final String url_ifexists        = "ifexists";
+    public static final String url_create          = "create";
     public static final String url_default_schema  = "default_schema";
     public static final String url_check_props     = "check_props";
     public static final String url_get_column_name = "get_column_name";
@@ -186,16 +195,20 @@ public class HsqlDatabaseProperties extends HsqlProperties {
     public static final String url_crypt_key      = "crypt_key";
     public static final String url_crypt_type     = "crypt_type";
     public static final String url_crypt_provider = "crypt_provider";
+    public static final String url_crypt_lobs     = "crypt_lobs";
 
     //
-    public static final String hsqldb_tx             = "hsqldb.tx";
-    public static final String hsqldb_tx_level       = "hsqldb.tx_level";
+    public static final String hsqldb_tx       = "hsqldb.tx";
+    public static final String hsqldb_tx_level = "hsqldb.tx_level";
+    public static final String hsqldb_tx_conflict_rollback =
+        "hsqldb.tx_conflict_rollback";
     public static final String hsqldb_applog         = "hsqldb.applog";
+    public static final String hsqldb_sqllog         = "hsqldb.sqllog";
     public static final String hsqldb_lob_file_scale = "hsqldb.lob_file_scale";
     public static final String hsqldb_cache_file_scale =
         "hsqldb.cache_file_scale";
-    public static final String hsqldb_cache_free_count_scale =
-        "hsqldb.cache_free_count_scale";
+    public static final String hsqldb_cache_free_count =
+        "hsqldb.cache_free_count";
     public static final String hsqldb_cache_rows = "hsqldb.cache_rows";
     public static final String hsqldb_cache_size = "hsqldb.cache_size";
     public static final String hsqldb_default_table_type =
@@ -206,7 +219,7 @@ public class HsqlDatabaseProperties extends HsqlProperties {
     public static final String hsqldb_log_data       = "hsqldb.log_data";
     public static final String hsqldb_log_size       = "hsqldb.log_size";
     public static final String hsqldb_nio_data_file  = "hsqldb.nio_data_file";
-    public static final String hsqldb_max_nio_scale  = "hsqldb.max_nio_scale";
+    public static final String hsqldb_nio_max_size   = "hsqldb.nio_max_size";
     public static final String hsqldb_script_format  = "hsqldb.script_format";
     public static final String hsqldb_temp_directory = "hsqldb.temp_directory";
     public static final String hsqldb_result_max_memory_rows =
@@ -214,23 +227,45 @@ public class HsqlDatabaseProperties extends HsqlProperties {
     public static final String hsqldb_write_delay = "hsqldb.write_delay";
     public static final String hsqldb_write_delay_millis =
         "hsqldb.write_delay_millis";
+    public static final String hsqldb_full_log_replay =
+        "hsqldb.full_log_replay";
+    public static final String hsqldb_large_data = "hsqldb.large_data";
 
     //
-    public static final String sql_ref_integrity     = "sql.ref_integrity";
+    public static final String sql_ref_integrity       = "sql.ref_integrity";
     public static final String sql_compare_in_locale = "sql.compare_in_locale";
-    public static final String sql_enforce_size      = "sql.enforce_size";
+    public static final String sql_enforce_size        = "sql.enforce_size";
     public static final String sql_enforce_strict_size =
-        "sql.enforce_strict_size";
-    public static final String sql_enforce_refs = "sql.enforce_refs";
+        "sql.enforce_strict_size";    // synonym for sql_enforce_size
+    public static final String sql_enforce_refs  = "sql.enforce_refs";
     public static final String sql_enforce_names = "sql.enforce_names";
-    public static final String jdbc_translate_dti_types =
-        "jdbc.translate_dti_types";
+    public static final String sql_regular_names = "sql.regular_names";
+    public static final String sql_enforce_types = "sql.enforce_types";
+    public static final String sql_enforce_tdcd  = "sql.enforce_tdc_delete";
+    public static final String sql_enforce_tdcu  = "sql.enforce_tdc_update";
+    public static final String sql_concat_nulls  = "sql.concat_nulls";
+    public static final String sql_nulls_first   = "sql.nulls_first";
+    public static final String sql_unique_nulls  = "sql.unique_nulls";
+    public static final String sql_convert_trunc = "sql.convert_trunc";
+    public static final String sql_avg_scale     = "sql.avg_scale";
+    public static final String sql_double_nan    = "sql.double_nan";
+    public static final String sql_syntax_db2    = "sql.syntax_db2";
+    public static final String sql_syntax_mss    = "sql.syntax_mss";
+    public static final String sql_syntax_mys    = "sql.syntax_mys";
+    public static final String sql_syntax_ora    = "sql.syntax_ora";
+    public static final String sql_syntax_pgs    = "sql.syntax_pgs";
+    public static final String jdbc_translate_tti_types =
+        "jdbc.translate_tti_types";
     public static final String sql_identity_is_pk = "sql.identity_is_pk";
+    public static final String sql_longvar_is_lob = "sql.longvar_is_lob";
+    public static final String sql_pad_space      = "sql.pad_space";
 
     //
     public static final String textdb_cache_scale = "textdb.cache_scale";
     public static final String textdb_cache_size_scale =
         "textdb.cache_size_scale";
+    public static final String textdb_cache_rows = "textdb.cache_rows";
+    public static final String textdb_cache_size = "textdb.cache_size";
     public static final String textdb_all_quoted = "textdb.all_quoted";
     public static final String textdb_allow_full_path =
         "textdb.allow_full_path";
@@ -246,10 +281,10 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         // text table defaults
         textMeta.put(textdb_allow_full_path,
                      HsqlProperties.getMeta(textdb_allow_full_path,
-                                            SYSTEM_PROPERTY, false));
+                                            SYSTEM_PROPERTY, allowFullPath));
         textMeta.put(textdb_quoted,
                      HsqlProperties.getMeta(textdb_quoted, SQL_PROPERTY,
-                                            false));
+                                            true));
         textMeta.put(textdb_all_quoted,
                      HsqlProperties.getMeta(textdb_all_quoted, SQL_PROPERTY,
                                             false));
@@ -271,6 +306,12 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         textMeta.put(textdb_cache_size_scale,
                      HsqlProperties.getMeta(textdb_cache_size_scale,
                                             SQL_PROPERTY, 10, 6, 20));
+        textMeta.put(textdb_cache_rows,
+                     HsqlProperties.getMeta(textdb_cache_rows, SQL_PROPERTY,
+                                            1000, 100, 1000000));
+        textMeta.put(textdb_cache_size,
+                     HsqlProperties.getMeta(textdb_cache_size, SQL_PROPERTY,
+                                            100, 10, 1000000));
         dbMeta.putAll(textMeta);
 
         // string defaults for protected props
@@ -280,6 +321,9 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         dbMeta.put(hsqldb_modified,
                    HsqlProperties.getMeta(hsqldb_modified, FILE_PROPERTY,
                                           null));
+        dbMeta.put(hsqldb_cache_version,
+                   HsqlProperties.getMeta(hsqldb_cache_version, FILE_PROPERTY,
+                                          null));
 
         // boolean defaults for protected props
         dbMeta.put(hsqldb_readonly,
@@ -289,20 +333,6 @@ public class HsqlDatabaseProperties extends HsqlProperties {
                    HsqlProperties.getMeta(hsqldb_files_readonly,
                                           FILE_PROPERTY, false));
 
-        // integral defaults for protected props
-        dbMeta.put(hsqldb_lob_file_scale,
-                   HsqlProperties.getMeta(hsqldb_lob_file_scale,
-                                          FILE_PROPERTY, 32, new int[] {
-            1, 2, 4, 8, 16, 32
-        }));
-
-        // this property is normally either 1 or 8 - 8 for new databases
-        dbMeta.put(hsqldb_cache_file_scale,
-                   HsqlProperties.getMeta(hsqldb_cache_file_scale,
-                                          FILE_PROPERTY, 8, new int[] {
-            1, 8, 16, 32, 64, 128
-        }));
-
         // string defaults for user defined props
         dbMeta.put(hsqldb_tx,
                    HsqlProperties.getMeta(hsqldb_tx, SQL_PROPERTY, "LOCKS"));
@@ -317,8 +347,11 @@ public class HsqlDatabaseProperties extends HsqlProperties {
                                           SQL_PROPERTY, "MEMORY"));
 
         // boolean defaults for user defined props
-        dbMeta.put(jdbc_translate_dti_types,
-                   HsqlProperties.getMeta(jdbc_translate_dti_types,
+        dbMeta.put(hsqldb_tx_conflict_rollback,
+                   HsqlProperties.getMeta(hsqldb_tx_conflict_rollback,
+                                          SQL_PROPERTY, true));
+        dbMeta.put(jdbc_translate_tti_types,
+                   HsqlProperties.getMeta(jdbc_translate_tti_types,
                                           SQL_PROPERTY, true));
         dbMeta.put(hsqldb_inc_backup,
                    HsqlProperties.getMeta(hsqldb_inc_backup, SQL_PROPERTY,
@@ -332,73 +365,136 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         dbMeta.put(hsqldb_nio_data_file,
                    HsqlProperties.getMeta(hsqldb_nio_data_file, SQL_PROPERTY,
                                           true));
-
-        // char padding to size and exception if data is too long
-        dbMeta.put(sql_identity_is_pk,
-                   HsqlProperties.getMeta(sql_identity_is_pk, SQL_PROPERTY,
-                                          false));
+        dbMeta.put(hsqldb_full_log_replay,
+                   HsqlProperties.getMeta(hsqldb_full_log_replay,
+                                          SQL_PROPERTY, false));
         dbMeta.put(sql_ref_integrity,
                    HsqlProperties.getMeta(sql_ref_integrity, SQL_PROPERTY,
                                           true));
-        dbMeta.put(sql_enforce_refs,
-                   HsqlProperties.getMeta(sql_enforce_refs, SQL_PROPERTY,
-                                          false));
-        dbMeta.put(sql_enforce_size,
-                   HsqlProperties.getMeta(sql_enforce_size, SQL_PROPERTY,
-                                          true));
-        dbMeta.put(sql_enforce_strict_size,
-                   HsqlProperties.getMeta(sql_enforce_strict_size,
-                                          SQL_PROPERTY, true));
 
         // SQL reserved words not allowed as some identifiers
         dbMeta.put(sql_enforce_names,
                    HsqlProperties.getMeta(sql_enforce_names, SQL_PROPERTY,
                                           false));
+        dbMeta.put(sql_regular_names,
+                   HsqlProperties.getMeta(sql_regular_names, SQL_PROPERTY,
+                                          true));
+        dbMeta.put(sql_enforce_refs,
+                   HsqlProperties.getMeta(sql_enforce_refs, SQL_PROPERTY,
+                                          false));
+
+        // char padding to size and exception if data is too long
+        dbMeta.put(sql_enforce_size,
+                   HsqlProperties.getMeta(sql_enforce_size, SQL_PROPERTY,
+                                          true));
+        dbMeta.put(sql_enforce_types,
+                   HsqlProperties.getMeta(sql_enforce_types, SQL_PROPERTY,
+                                          false));
+        dbMeta.put(sql_enforce_tdcd,
+                   HsqlProperties.getMeta(sql_enforce_tdcd, SQL_PROPERTY,
+                                          true));
+        dbMeta.put(sql_enforce_tdcu,
+                   HsqlProperties.getMeta(sql_enforce_tdcu, SQL_PROPERTY,
+                                          true));
+        dbMeta.put(sql_concat_nulls,
+                   HsqlProperties.getMeta(sql_concat_nulls, SQL_PROPERTY,
+                                          true));
+        dbMeta.put(sql_nulls_first,
+                   HsqlProperties.getMeta(sql_nulls_first, SQL_PROPERTY,
+                                          true));
+        dbMeta.put(sql_unique_nulls,
+                   HsqlProperties.getMeta(sql_unique_nulls, SQL_PROPERTY,
+                                          true));
+        dbMeta.put(sql_convert_trunc,
+                   HsqlProperties.getMeta(sql_convert_trunc, SQL_PROPERTY,
+                                          true));
+        dbMeta.put(sql_avg_scale,
+                   HsqlProperties.getMeta(sql_avg_scale, SQL_PROPERTY, 0, 0,
+                                          10));
+        dbMeta.put(sql_double_nan,
+                   HsqlProperties.getMeta(sql_double_nan, SQL_PROPERTY, true));
+        dbMeta.put(sql_syntax_db2,
+                   HsqlProperties.getMeta(sql_syntax_db2, SQL_PROPERTY,
+                                          false));
+        dbMeta.put(sql_syntax_mss,
+                   HsqlProperties.getMeta(sql_syntax_mss, SQL_PROPERTY,
+                                          false));
+        dbMeta.put(sql_syntax_mys,
+                   HsqlProperties.getMeta(sql_syntax_mys, SQL_PROPERTY,
+                                          false));
+        dbMeta.put(sql_syntax_ora,
+                   HsqlProperties.getMeta(sql_syntax_ora, SQL_PROPERTY,
+                                          false));
+        dbMeta.put(sql_syntax_pgs,
+                   HsqlProperties.getMeta(sql_syntax_pgs, SQL_PROPERTY,
+                                          false));
         dbMeta.put(sql_compare_in_locale,
                    HsqlProperties.getMeta(sql_compare_in_locale, SQL_PROPERTY,
                                           false));
+        dbMeta.put(sql_identity_is_pk,
+                   HsqlProperties.getMeta(sql_identity_is_pk, SQL_PROPERTY,
+                                          false));
+        dbMeta.put(sql_longvar_is_lob,
+                   HsqlProperties.getMeta(sql_longvar_is_lob, SQL_PROPERTY,
+                                          false));
         dbMeta.put(hsqldb_write_delay,
                    HsqlProperties.getMeta(hsqldb_write_delay, SQL_PROPERTY,
                                           true));
         dbMeta.put(hsqldb_write_delay_millis,
                    HsqlProperties.getMeta(hsqldb_write_delay_millis,
-                                          SQL_PROPERTY, 500, 20, 10000));
+                                          SQL_PROPERTY, 500, 0, 10000));
 
-        // integral defaults for user-defined set props
+        // integral defaults for user-defined props - sets
         dbMeta.put(hsqldb_applog,
                    HsqlProperties.getMeta(hsqldb_applog, SQL_PROPERTY, 0, 0,
-                                          2));
+                                          3));
+        dbMeta.put(hsqldb_sqllog,
+                   HsqlProperties.getMeta(hsqldb_sqllog, SQL_PROPERTY, 0, 0,
+                                          3));
         dbMeta.put(hsqldb_script_format,
                    HsqlProperties.getMeta(hsqldb_script_format, SQL_PROPERTY,
                                           0, new int[] {
             0, 1, 3
         }));
+        dbMeta.put(hsqldb_lob_file_scale,
+                   HsqlProperties.getMeta(hsqldb_lob_file_scale, SQL_PROPERTY,
+                                          32, new int[] {
+            1, 2, 4, 8, 16, 32
+        }));
 
-        // integral defaults for user defined range props
+        // this property is normally 8 - or 1 for old databases from early versions
+        dbMeta.put(hsqldb_cache_file_scale,
+                   HsqlProperties.getMeta(hsqldb_cache_file_scale,
+                                          SQL_PROPERTY, 32, new int[] {
+            1, 8, 16, 32, 64, 128, 256, 512, 1024
+        }));
+
+        // integral defaults for user defined props - ranges
         dbMeta.put(hsqldb_log_size,
                    HsqlProperties.getMeta(hsqldb_log_size, SQL_PROPERTY, 50,
-                                          0, 1000));
+                                          0, 4 * 1024));
         dbMeta.put(hsqldb_defrag_limit,
                    HsqlProperties.getMeta(hsqldb_defrag_limit, SQL_PROPERTY,
-                                          20, 0, 100));
+                                          0, 0, 100));
         dbMeta.put(runtime_gc_interval,
                    HsqlProperties.getMeta(runtime_gc_interval, SQL_PROPERTY,
                                           0, 0, 1000000));
         dbMeta.put(hsqldb_cache_size,
                    HsqlProperties.getMeta(hsqldb_cache_size, SQL_PROPERTY,
-                                          10000, 100, 1000000));
+                                          10000, 100, 4 * 1024 * 1024));
         dbMeta.put(hsqldb_cache_rows,
                    HsqlProperties.getMeta(hsqldb_cache_rows, SQL_PROPERTY,
-                                          50000, 100, 1000000));
-        dbMeta.put(hsqldb_cache_free_count_scale,
-                   HsqlProperties.getMeta(hsqldb_cache_free_count_scale,
-                                          SQL_PROPERTY, 9, 6, 12));
+                                          50000, 100, 4 * 1024 * 1024));
+        dbMeta.put(hsqldb_cache_free_count,
+                   HsqlProperties.getMeta(hsqldb_cache_free_count,
+                                          SQL_PROPERTY, 512, 0, 4096));
         dbMeta.put(hsqldb_result_max_memory_rows,
                    HsqlProperties.getMeta(hsqldb_result_max_memory_rows,
-                                          SQL_PROPERTY, 0, 0, 1000000));
-        dbMeta.put(hsqldb_max_nio_scale,
-                   HsqlProperties.getMeta(hsqldb_max_nio_scale, SQL_PROPERTY,
-                                          28, 24, 31));
+                                          SQL_PROPERTY, 0, 0,
+                                          4 * 1024 * 1024));
+        dbMeta.put(hsqldb_nio_max_size,
+                   HsqlProperties.getMeta(hsqldb_nio_max_size, SQL_PROPERTY,
+                                          256, 64, 262144));
     }
 
     private Database database;
@@ -417,7 +513,7 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
         // version of a new database
         setProperty(hsqldb_version, THIS_VERSION);
-        setProperty(hsqldb_modified, "no-new-files");
+        setProperty(hsqldb_modified, MODIFIED_NO_NEW);
 
         // OOo related code
         if (database.logger.isStoredFileAccess()) {
@@ -428,7 +524,7 @@ public class HsqlDatabaseProperties extends HsqlProperties {
             setProperty(hsqldb_nio_data_file, false);
             setProperty(hsqldb_lock_file, true);
             setProperty(hsqldb_default_table_type, "cached");
-            setProperty(jdbc_translate_dti_types, true);
+            setProperty(jdbc_translate_tti_types, true);
         }
 
         // OOo end
@@ -451,7 +547,7 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         } catch (Throwable t) {
             throw Error.error(t, ErrorCode.FILE_IO_ERROR,
                               ErrorCode.M_LOAD_SAVE_PROPERTIES, new Object[] {
-                t.getMessage(), fileName
+                t.toString(), fileName
             });
         }
 
@@ -469,21 +565,17 @@ public class HsqlDatabaseProperties extends HsqlProperties {
             throw Error.error(ErrorCode.WRONG_DATABASE_FILE_VERSION);
         }
 
-        check = version.substring(0, 5).compareTo(THIS_VERSION);
-
-        // do not open if the database belongs to a later (future) version
-        if (check > 0) {
-            throw Error.error(ErrorCode.WRONG_DATABASE_FILE_VERSION);
-        }
-
-        // do not open modified databases of compatible earlier versions
-        if (check < 0) {
-            if (!MODIFIED_NO.equals(getStringProperty(hsqldb_modified))) {
-                throw Error.error(ErrorCode.SHUTDOWN_REQUIRED);
+        // do not open databases of 1.8 versions if script format is not compatible
+        if (check == 0) {
+            if (getIntegerProperty(hsqldb_script_format) != 0) {
+                throw Error.error(ErrorCode.WRONG_DATABASE_FILE_VERSION);
             }
         }
 
-        if (getIntegerProperty(hsqldb_script_format) != 0) {
+        check = version.substring(0, 2).compareTo(THIS_VERSION);
+
+        // do not open if the database belongs to a later (future) version (3.x)
+        if (check > 0) {
             throw Error.error(ErrorCode.WRONG_DATABASE_FILE_VERSION);
         }
 
@@ -501,7 +593,22 @@ public class HsqlDatabaseProperties extends HsqlProperties {
             HsqlProperties props = new HsqlProperties(dbMeta,
                 database.getPath(), database.logger.getFileAccess(), false);
 
+            if (getIntegerProperty(hsqldb_script_format) == 3) {
+                props.setProperty(hsqldb_script_format, 3);
+            }
+
             props.setProperty(hsqldb_version, THIS_VERSION);
+
+            if (database.logger.isStoredFileAccess()) {
+                if (!database.logger.isNewStoredFileAccess()) {
+
+// when jar is used with embedded databases in AOO 3.4 and recent(2012) LO this
+// line can be uncommented to circumvent hard-coded check in OOo code in
+// drivers/hsqldb/HDriver.cxx
+//                    props.setProperty(hsqldb_version, VERSION_STRING_1_8_0);
+                }
+            }
+
             props.setProperty(hsqldb_modified, getProperty(hsqldb_modified));
             props.save(fileName + ".properties" + ".new");
             fa.renameElement(fileName + ".properties" + ".new",
@@ -511,13 +618,19 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
             throw Error.error(t, ErrorCode.FILE_IO_ERROR,
                               ErrorCode.M_LOAD_SAVE_PROPERTIES, new Object[] {
-                t.getMessage(), fileName
+                t.toString(), fileName
             });
         }
     }
 
     void filterLoadedProperties() {
 
+        String val = stringProps.getProperty(sql_enforce_strict_size);
+
+        if (val != null) {
+            stringProps.setProperty(sql_enforce_size, val);
+        }
+
         Enumeration en = stringProps.propertyNames();
 
         while (en.hasMoreElements()) {
@@ -542,20 +655,30 @@ public class HsqlDatabaseProperties extends HsqlProperties {
             return;
         }
 
+        String val = p.getProperty(sql_enforce_strict_size);
+
+        if (val != null) {
+            p.setProperty(sql_enforce_size, val);
+            p.removeProperty(sql_enforce_strict_size);
+        }
+
         strict = p.isPropertyTrue(url_check_props, false);
 
         for (Enumeration e = p.propertyNames(); e.hasMoreElements(); ) {
-            String   propertyName = (String) e.nextElement();
-            Object[] row          = (Object[]) dbMeta.get(propertyName);
-            boolean  valid        = false;
-            boolean  validVal     = false;
+            String   propertyName  = (String) e.nextElement();
+            String   propertyValue = p.getProperty(propertyName);
+            boolean  valid         = false;
+            boolean  validVal      = false;
+            String   error         = null;
+            Object[] meta          = (Object[]) dbMeta.get(propertyName);
 
-            if (row != null
-                    && ((Integer) row[HsqlProperties.indexType]).intValue()
+            if (meta != null
+                    && ((Integer) meta[HsqlProperties.indexType]).intValue()
                        == SQL_PROPERTY) {
                 valid = true;
-                validVal = setDatabaseProperty(propertyName,
-                                               p.getProperty(propertyName));
+                error = HsqlProperties.validateProperty(propertyName,
+                        propertyValue, meta);
+                validVal = error == null;
             }
 
             if (propertyName.startsWith("sql.")
@@ -570,6 +693,17 @@ public class HsqlDatabaseProperties extends HsqlProperties {
                 }
             }
         }
+
+        for (Enumeration e = p.propertyNames(); e.hasMoreElements(); ) {
+            String   propertyName = (String) e.nextElement();
+            Object[] meta         = (Object[]) dbMeta.get(propertyName);
+
+            if (meta != null
+                    && ((Integer) meta[HsqlProperties.indexType]).intValue()
+                       == SQL_PROPERTY) {
+                setDatabaseProperty(propertyName, p.getProperty(propertyName));
+            }
+        }
     }
 
     public Set getUserDefinedPropertyData() {
@@ -647,16 +781,14 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         }
 
         // OOo end
-        return 10000;
+        return 500;
     }
 
 //---------------------
 // new properties to review / persist
-    public final static int NO_MESSAGE = 1;
+    public static final int NO_MESSAGE = 1;
 
     public int getErrorLevel() {
-
-        //      return 0;
         return NO_MESSAGE;
     }
 
@@ -667,12 +799,25 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 //------------------------
     public void setDBModified(int mode) {
 
-        String value = MODIFIED_NO;
+        String value;
 
-        if (mode == FILES_MODIFIED) {
-            value = MODIFIED_YES;
-        } else if (mode == FILES_NEW) {
-            value = MODIFIED_NEW;
+        switch (mode) {
+
+            case FILES_NOT_MODIFIED :
+                value = MODIFIED_NO;
+                break;
+
+            case FILES_MODIFIED :
+                value = MODIFIED_YES;
+                break;
+
+            case FILES_MODIFIED_NEW :
+                value = MODIFIED_YES_NEW;
+                break;
+
+            default :
+                throw Error.runtimeError(ErrorCode.U_S0500,
+                                         "HsqlDatabaseProperties");
         }
 
         stringProps.put(hsqldb_modified, value);
@@ -681,11 +826,13 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
     public int getDBModified() {
 
-        String value = getStringProperty("modified");
+        String value = getStringProperty(hsqldb_modified);
 
         if (MODIFIED_YES.equals(value)) {
             return FILES_MODIFIED;
-        } else if (MODIFIED_NEW.equals(value)) {
+        } else if (MODIFIED_YES_NEW.equals(value)) {
+            return FILES_MODIFIED_NEW;
+        } else if (MODIFIED_NO_NEW.equals(value)) {
             return FILES_NEW;
         }
 
@@ -704,6 +851,39 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         return stringProps.getProperty(key);
     }
 
+    /** for all types of property apart from system props */
+    public String getPropertyString(String key) {
+
+        Object[] metaData = (Object[]) dbMeta.get(key);
+
+        if (metaData == null) {
+            throw Error.error(ErrorCode.X_42555, key);
+        }
+
+        String prop = stringProps.getProperty(key);
+        boolean isSystem =
+            ((Integer) metaData[HsqlProperties.indexType]).intValue()
+            == SYSTEM_PROPERTY;
+
+        if (prop == null && isSystem) {
+            try {
+                prop = System.getProperty(key);
+            } catch (SecurityException e) {}
+        }
+
+        if (prop == null) {
+            Object value = metaData[HsqlProperties.indexDefaultValue];
+
+            if (value == null) {
+                return null;
+            }
+
+            return String.valueOf(value);
+        }
+
+        return prop;
+    }
+
     public boolean isPropertyTrue(String key) {
 
         Boolean  value;
@@ -715,13 +895,17 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
         value = (Boolean) metaData[HsqlProperties.indexDefaultValue];
 
-        String prop = stringProps.getProperty(key);
+        String prop = null;
         boolean isSystem =
             ((Integer) metaData[HsqlProperties.indexType]).intValue()
             == SYSTEM_PROPERTY;
 
-        if (prop == null && isSystem) {
-            prop = System.getProperty(key);
+        if (isSystem) {
+            try {
+                prop = System.getProperty(key);
+            } catch (SecurityException e) {}
+        } else {
+            prop = stringProps.getProperty(key);
         }
 
         if (prop != null) {
@@ -780,10 +964,12 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
     public String getClientPropertiesAsString() {
 
-        if (isPropertyTrue(jdbc_translate_dti_types)) {
-            StringBuffer sb = new StringBuffer(jdbc_translate_dti_types);
+        if (isPropertyTrue(jdbc_translate_tti_types)) {
+            StringBuffer sb = new StringBuffer(jdbc_translate_tti_types);
 
             sb.append('=').append(true);
+
+            return sb.toString();
         }
 
         return "";
@@ -792,8 +978,9 @@ public class HsqlDatabaseProperties extends HsqlProperties {
     public boolean isVersion18() {
 
         String version =
-            getStringProperty(HsqlDatabaseProperties.hsqldb_version);
+            getProperty(HsqlDatabaseProperties.hsqldb_cache_version,
+                        THIS_VERSION);
 
-        return version.substring(0, 4).equals("1.8.");
+        return version.substring(0, 4).equals("1.7.");
     }
 }
