@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -352,7 +352,7 @@ import org.hsqldb.lib.StringConverter;
  * <tt>NIOLockFile</tt> to the hsqldb.jar if such features are reported
  * present. <p>
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
  * @version 1.8.1.2
  * @since 1.7.2
  */
@@ -1405,7 +1405,7 @@ public class LockFile {
      *      acquisition of a cooperative lock condition using the specified
      *      <tt>path</tt>, else <tt>false</tt>
      */
-    public final static boolean isLocked(final String path) {
+    public static final boolean isLocked(final String path) {
 
         boolean locked = true;
 
