@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,6 @@ import org.hsqldb.lib.Iterator;
 import org.hsqldb.lib.MultiValueHashMap;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.lib.Set;
-import org.hsqldb.lib.WrapperIterator;
 import org.hsqldb.types.Type;
 
 /**
@@ -71,11 +70,11 @@ import org.hsqldb.types.Type;
  * and method "isAdminDirect()) to mean this Grantee has admin priv
  * directly.
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  * @author Blaine Simpson (blaine dot simpson at admc dot com)
  *
- * @version 1.9.0
+ * @version 2.0.1
  * @since 1.8.0
  */
 public class Grantee implements SchemaObject {
@@ -107,7 +106,7 @@ public class Grantee implements SchemaObject {
     private MultiValueHashMap directRightsMap;
 
     /** contains righs granted direct, or via roles, expept those of PUBLIC */
-    private HashMap fullRightsMap;
+    HashMap fullRightsMap;
 
     /** These are the DIRECT roles.  Each of these may contain nested roles */
     OrderedHashSet roles;
@@ -182,14 +181,6 @@ public class Grantee implements SchemaObject {
         return 0;
     }
 
-    public String getNameString() {
-        return granteeName.name;
-    }
-
-    public String getStatementName() {
-        return granteeName.statementName;
-    }
-
     public boolean isRole() {
         return isRole;
     }
@@ -198,47 +189,6 @@ public class Grantee implements SchemaObject {
         return isSystem;
     }
 
-    /**
-     * Retrieves the map object that represents the rights that have been
-     * granted on database objects.  <p>
-     *
-     * The map has keys and values with the following interpretation: <P>
-     *
-     * <UL>
-     * <LI> The keys are generally (but not limited to) objects having
-     *      an attribute or value equal to the name of an actual database
-     *      object.
-     *
-     * <LI> Specifically, the keys act as database object identifiers.
-     *
-     * <LI> The values are Right objects.
-     * </UL>
-     */
-    public MultiValueHashMap getRights() {
-
-        // necessary to create the script
-        return directRightsMap;
-    }
-
-    /**
-     * Grant a role
-     */
-    public void grant(Grantee role) {
-        roles.add(role);
-    }
-
-    /**
-     * Revoke a direct role only
-     */
-    public void revoke(Grantee role) {
-
-        if (!hasRoleDirect(role)) {
-            throw Error.error(ErrorCode.X_0P503, role.getNameString());
-        }
-
-        roles.remove(role);
-    }
-
     /**
      * Gets direct roles, not roles nested within them.
      */
@@ -246,31 +196,6 @@ public class Grantee implements SchemaObject {
         return roles;
     }
 
-    String getAllRolesAsString() {
-        return roleMapToString(getAllRoles());
-    }
-
-    public String getDirectRolesAsString() {
-        return roleMapToString(roles);
-    }
-
-    public String roleMapToString(OrderedHashSet roles) {
-
-        StringBuffer sb = new StringBuffer();
-
-        for (int i = 0; i < roles.size(); i++) {
-            if (sb.length() > 0) {
-                sb.append(',');
-            }
-
-            Grantee role = (Grantee) roles.get(i);
-
-            sb.append(role.getStatementName());
-        }
-
-        return sb.toString();
-    }
-
     /**
      * Gets direct and indirect roles.
      */
@@ -303,6 +228,47 @@ public class Grantee implements SchemaObject {
         return set;
     }
 
+    public boolean isAccessible(HsqlName name, int privilegeType) {
+
+        if (isFullyAccessibleByRole(name)) {
+            return true;
+        }
+
+        Right right = (Right) fullRightsMap.get(name);
+
+        if (right == null) {
+            return false;
+        }
+
+        return right.canAccess(privilegeType);
+    }
+
+    /**
+     * returns true if grantee has any privilege (to any column) of the object
+     */
+    public boolean isAccessible(SchemaObject object) {
+        return isAccessible(object.getName());
+    }
+
+    public boolean isAccessible(HsqlName name) {
+
+        if (isFullyAccessibleByRole(name)) {
+            return true;
+        }
+
+        Right right = (Right) fullRightsMap.get(name);
+
+        if (right != null && !right.isEmpty()) {
+            return true;
+        }
+
+        if (!isPublic) {
+            return granteeManager.publicRole.isAccessible(name);
+        }
+
+        return false;
+    }
+
     /**
      * Adds to given Set this.sName plus all roles and nested roles.
      *
@@ -325,19 +291,7 @@ public class Grantee implements SchemaObject {
         return set;
     }
 
-    /**
-     * returns a map with grantee name keys and sets of granted roles as value
-     */
-    public void addAllRoles(HashMap map) {
-
-        for (int i = 0; i < roles.size(); i++) {
-            Grantee role = (Grantee) roles.get(i);
-
-            map.put(role.granteeName.name, role.roles);
-        }
-    }
-
-    public boolean hasRoleDirect(Grantee role) {
+    private boolean hasRoleDirect(Grantee role) {
         return roles.contains(role);
     }
 
@@ -354,8 +308,9 @@ public class Grantee implements SchemaObject {
     void grant(HsqlName name, Right right, Grantee grantor,
                boolean withGrant) {
 
-        final Right grantableRights = grantor.getAllGrantableRights(name);
-        Right       existingRight   = null;
+        final Right grantableRights =
+            ((Grantee) grantor).getAllGrantableRights(name);
+        Right existingRight = null;
 
         if (right == Right.fullRights) {
             if (grantableRights.isEmpty()) {
@@ -399,10 +354,10 @@ public class Grantee implements SchemaObject {
             }
         }
 
-        if (!grantor.isSystem) {
+        if (!grantor.isSystem()) {
 
             // based on assumption that there is no need to access
-            grantor.grantedRightsMap.put(name, existingRight);
+            ((Grantee) grantor).grantedRightsMap.put(name, existingRight);
         }
 
         updateAllRights();
@@ -449,7 +404,7 @@ public class Grantee implements SchemaObject {
 
         if (right.isFull) {
             directRightsMap.remove(name, existing);
-            grantor.grantedRightsMap.remove(name, existing);
+            ((Grantee) grantor).grantedRightsMap.remove(name, existing);
             updateAllRights();
 
             return;
@@ -459,7 +414,7 @@ public class Grantee implements SchemaObject {
 
         if (existing.isEmpty()) {
             directRightsMap.remove(name, existing);
-            grantor.grantedRightsMap.remove(name, existing);
+            ((Grantee) grantor).grantedRightsMap.remove(name, existing);
         }
 
         updateAllRights();
@@ -493,16 +448,22 @@ public class Grantee implements SchemaObject {
         isAdmin = false;
     }
 
-    public OrderedHashSet getColumnsForAllPrivileges(Table table) {
+    public OrderedHashSet getColumnsForAllPrivileges(SchemaObject object) {
 
-        if (isFullyAccessibleByRole(table.getName())) {
-            return table.getColumnNameSet();
+        if (object instanceof Table) {
+            Table table = (Table) object;
+
+            if (isFullyAccessibleByRole(table.getName())) {
+                return table.getColumnNameSet();
+            }
+
+            Right right = (Right) fullRightsMap.get(table.getName());
+
+            return right == null ? Right.emptySet
+                                 : right.getColumnsForAllRights(table);
         }
 
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        return right == null ? Right.emptySet
-                             : right.getColumnsForAllRights(table);
+        return Right.emptySet;
     }
 
     public OrderedHashSet getAllDirectPrivileges(SchemaObject object) {
@@ -515,7 +476,13 @@ public class Grantee implements SchemaObject {
             return set;
         }
 
-        Iterator rights = directRightsMap.get(object.getName());
+        HsqlName name = object.getName();
+
+        if (object instanceof Routine) {
+            name = ((Routine) object).getSpecificName();
+        }
+
+        Iterator rights = directRightsMap.get(name);
 
         if (rights.hasNext()) {
             OrderedHashSet set = new OrderedHashSet();
@@ -532,7 +499,13 @@ public class Grantee implements SchemaObject {
 
     public OrderedHashSet getAllGrantedPrivileges(SchemaObject object) {
 
-        Iterator rights = grantedRightsMap.get(object.getName());
+        HsqlName name = object.getName();
+
+        if (object instanceof Routine) {
+            name = ((Routine) object).getSpecificName();
+        }
+
+        Iterator rights = grantedRightsMap.get(name);
 
         if (rights.hasNext()) {
             OrderedHashSet set = new OrderedHashSet();
@@ -554,94 +527,118 @@ public class Grantee implements SchemaObject {
      * This is done by checking that a mapping exists in the rights map
      * from the dbobject argument. Otherwise, it throws.
      */
-    public void checkSelect(Table table, boolean[] checkList) {
+    public void checkSelect(SchemaObject object, boolean[] checkList) {
 
-        if (isFullyAccessibleByRole(table.getName())) {
-            return;
+        if (object instanceof Table) {
+            Table table = (Table) object;
+
+            if (isFullyAccessibleByRole(table.getName())) {
+                return;
+            }
+
+            Right right = (Right) fullRightsMap.get(table.getName());
+
+            if (right != null && right.canSelect(table, checkList)) {
+                return;
+            }
         }
 
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        if (right != null && right.canSelect(table, checkList)) {
-            return;
-        }
-
-        throw Error.error(ErrorCode.X_42501, table.getName().name);
+        throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkInsert(Table table, boolean[] checkList) {
+    public void checkInsert(SchemaObject object, boolean[] checkList) {
 
-        if (isFullyAccessibleByRole(table.getName())) {
-            return;
+        if (object instanceof Table) {
+            Table table = (Table) object;
+
+            if (isFullyAccessibleByRole(table.getName())) {
+                return;
+            }
+
+            Right right = (Right) fullRightsMap.get(table.getName());
+
+            if (right != null && right.canInsert(table, checkList)) {
+                return;
+            }
         }
 
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        if (right != null && right.canInsert(table, checkList)) {
-            return;
-        }
-
-        throw Error.error(ErrorCode.X_42501, table.getName().name);
+        throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkUpdate(Table table, boolean[] checkList) {
+    public void checkUpdate(SchemaObject object, boolean[] checkList) {
 
-        if (isFullyAccessibleByRole(table.getName())) {
-            return;
+        if (object instanceof Table) {
+            Table table = (Table) object;
+
+            if (isFullyAccessibleByRole(table.getName())) {
+                return;
+            }
+
+            Right right = (Right) fullRightsMap.get(table.getName());
+
+            if (right != null && right.canUpdate(table, checkList)) {
+                return;
+            }
         }
 
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        if (right != null && right.canUpdate(table, checkList)) {
-            return;
-        }
-
-        throw Error.error(ErrorCode.X_42501, table.getName().name);
+        throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkReferences(Table table, boolean[] checkList) {
+    public void checkReferences(SchemaObject object, boolean[] checkList) {
 
-        if (isFullyAccessibleByRole(table.getName())) {
-            return;
+        if (object instanceof Table) {
+            Table table = (Table) object;
+
+            if (isFullyAccessibleByRole(table.getName())) {
+                return;
+            }
+
+            Right right = (Right) fullRightsMap.get(table.getName());
+
+            if (right != null && right.canReference(table, checkList)) {
+                return;
+            }
         }
 
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        if (right != null && right.canReference(table, checkList)) {
-            return;
-        }
-
-        throw Error.error(ErrorCode.X_42501, table.getName().name);
+        throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkTrigger(Table table, boolean[] checkList) {
+    public void checkTrigger(SchemaObject object, boolean[] checkList) {
 
-        if (isFullyAccessibleByRole(table.getName())) {
-            return;
+        if (object instanceof Table) {
+            Table table = (Table) object;
+
+            if (isFullyAccessibleByRole(table.getName())) {
+                return;
+            }
+
+            Right right = (Right) fullRightsMap.get(table.getName());
+
+            if (right != null && right.canReference(table, checkList)) {
+                return;
+            }
         }
 
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        if (right != null && right.canReference(table, checkList)) {
-            return;
-        }
-
-        throw Error.error(ErrorCode.X_42501, table.getName().name);
+        throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
-    public void checkDelete(Table table) {
+    public void checkDelete(SchemaObject object) {
 
-        if (isFullyAccessibleByRole(table.getName())) {
-            return;
+        if (object instanceof Table) {
+            Table table = (Table) object;
+
+            if (isFullyAccessibleByRole(table.getName())) {
+                return;
+            }
+
+            Right right = (Right) fullRightsMap.get(table.getName());
+
+            if (right != null && right.canDelete()) {
+                return;
+            }
         }
 
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        if (right != null && right.canDelete()) {
-            return;
-        }
-
-        throw Error.error(ErrorCode.X_42501, table.getName().name);
+        throw Error.error(ErrorCode.X_42501, object.getName().name);
     }
 
     public void checkAccess(SchemaObject object) {
@@ -720,16 +717,20 @@ public class Grantee implements SchemaObject {
 
     public boolean isFullyAccessibleByRole(HsqlName name) {
 
+        Grantee owner;
+
         if (isAdmin) {
             return true;
         }
 
-        if (name.schema == null) {
+        if (name.type == SchemaObject.SCHEMA) {
+            owner = name.owner;
+        } else if (name.schema == null) {
             return false;
+        } else {
+            owner = name.schema.owner;
         }
 
-        Grantee owner = name.schema.owner;
-
         if (owner == this) {
             return true;
         }
@@ -781,6 +782,63 @@ public class Grantee implements SchemaObject {
         return isPublic;
     }
 
+    /**
+     * Iteration of all visible grantees, including self. <p>
+     *
+     * For grantees with admin, this is all grantees.
+     * For regular grantees, this is self plus all roles granted directly
+     * or indirectly
+     */
+    public OrderedHashSet visibleGrantees() {
+
+        OrderedHashSet grantees = new OrderedHashSet();
+        GranteeManager gm       = granteeManager;
+
+        if (isAdmin()) {
+            grantees.addAll(gm.getGrantees());
+        } else {
+            grantees.add(this);
+
+            Iterator it = getAllRoles().iterator();
+
+            while (it.hasNext()) {
+                grantees.add(it.next());
+            }
+        }
+
+        return grantees;
+    }
+
+    public boolean hasNonSelectTableRight(SchemaObject table) {
+
+        if (isFullyAccessibleByRole(table.getName())) {
+            return true;
+        }
+
+        Right right = (Right) fullRightsMap.get(table.getName());
+
+        if (right == null) {
+            return false;
+        }
+
+        return right.canAcesssNonSelect();
+    }
+
+    public boolean hasColumnRights(SchemaObject table, int[] columnMap) {
+
+        if (isFullyAccessibleByRole(table.getName())) {
+            return true;
+        }
+
+        Right right = (Right) fullRightsMap.get(table.getName());
+
+        if (right == null) {
+            return false;
+        }
+
+        return right.canAccess((Table) table, columnMap);
+    }
+
     /**
      * Violates naming convention (for backward compatibility).
      * Should be "setAdminDirect(boolean").
@@ -902,7 +960,7 @@ public class Grantee implements SchemaObject {
     /**
      * Full or partial rights are added to existing
      */
-    void addToFullRights(MultiValueHashMap map) {
+    private void addToFullRights(MultiValueHashMap map) {
 
         Iterator it = map.keySet().iterator();
 
@@ -935,106 +993,10 @@ public class Grantee implements SchemaObject {
         }
     }
 
-    /**
-     * Iteration of all visible grantees, including self. <p>
-     *
-     * For grantees with admin, this is all grantees.
-     * For regular grantees, this is self plus all roles granted directly
-     * or indirectly
-     */
-    public Set visibleGrantees() {
-
-        HashSet        grantees = new HashSet();
-        GranteeManager gm       = granteeManager;
-
-        if (isAdmin()) {
-            grantees.addAll(gm.getGrantees());
-        } else {
-            grantees.add(this);
-
-            Iterator it = getAllRoles().iterator();
-
-            while (it.hasNext()) {
-                grantees.add(it.next());
-            }
-        }
-
-        return grantees;
-    }
-
-    /**
-     * Set of all non-reserved visible grantees, including self. <p>
-     *
-     * For grantees with admin, this is all grantees.
-     * For regular grantees, this is self plus all roles granted directly
-     * or indirectly. <P>
-     *
-     * @param andPublic when <tt>true</tt> retains the reserved PUBLIC grantee
-     */
-    public Set nonReservedVisibleGrantees(boolean andPublic) {
-
-        Set            grantees = visibleGrantees();
-        GranteeManager gm       = granteeManager;
-
-        grantees.remove(gm.dbaRole);
-        grantees.remove(GranteeManager.systemAuthorisation);
-
-        if (!andPublic) {
-            grantees.remove(gm.publicRole);
-        }
-
-        return grantees;
-    }
-
-    public boolean hasNonSelectTableRight(Table table) {
-
-        if (isFullyAccessibleByRole(table.getName())) {
-            return true;
-        }
-
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        if (right == null) {
-            return false;
-        }
-
-        return right.isFull || right.isFullDelete || right.isFullInsert
-               || right.isFullUpdate || right.isFullReferences
-               || right.isFullTrigger;
-    }
-
-    public boolean hasTableRight(Table table) {
-
-        if (isFullyAccessibleByRole(table.getName())) {
-            return true;
-        }
-
-        Right right = (Right) fullRightsMap.get(table.getName());
-
-        if (right == null) {
-            return false;
-        }
-
-        return right.isFull || right.isFullDelete || right.isFullInsert
-               || right.isFullUpdate || right.isFullReferences
-               || right.isFullTrigger || right.isFullSelect;
-    }
-
-    public Iterator getAllDirectFullRights(SchemaObject object) {
-
-        Grantee owner = object.getOwner();
-
-        if (owner == this) {
-            return new WrapperIterator(ownerRights);
-        }
-
-        return directRightsMap.get(object.getName());
-    }
-
-    public Right getAllGrantableRights(HsqlName name) {
+    Right getAllGrantableRights(HsqlName name) {
 
         if (isAdmin) {
-            return name.schema.owner.ownerRights;
+            return ((Grantee) name.schema.owner).ownerRights;
         }
 
         if (name.schema.owner == this) {
@@ -1042,7 +1004,7 @@ public class Grantee implements SchemaObject {
         }
 
         if (roles.contains(name.schema.owner)) {
-            return name.schema.owner.ownerRights;
+            return ((Grantee) name.schema.owner).ownerRights;
         }
 
         OrderedHashSet set = getAllRoles();
@@ -1062,58 +1024,76 @@ public class Grantee implements SchemaObject {
                                                               .grantableRights;
     }
 
-    public boolean isAccessible(HsqlName name, int privilegeType) {
+    /**
+     * Retrieves the map object that represents the rights that have been
+     * granted on database objects.  <p>
+     *
+     * The map has keys and values with the following interpretation: <P>
+     *
+     * <UL>
+     * <LI> The keys are generally (but not limited to) objects having
+     *      an attribute or value equal to the name of an actual database
+     *      object.
+     *
+     * <LI> Specifically, the keys act as database object identifiers.
+     *
+     * <LI> The values are Right objects.
+     * </UL>
+     */
+    private MultiValueHashMap getRights() {
 
-        if (isFullyAccessibleByRole(name)) {
-            return true;
-        }
-
-        Right right = (Right) fullRightsMap.get(name);
-
-        if (right == null) {
-            return false;
-        }
-
-        return right.canAccess(privilegeType);
+        // necessary to create the script
+        return directRightsMap;
     }
 
     /**
-     * returns true if grantee has any privilege (to any column) of the object
+     * Grant a role
      */
-    public boolean isAccessible(SchemaObject object) {
-        return isAccessible(object.getName());
+    void grant(Grantee role) {
+        roles.add(role);
     }
 
-    public boolean isAccessible(HsqlName name) {
+    /**
+     * Revoke a direct role only
+     */
+    void revoke(Grantee role) {
 
-        if (isFullyAccessibleByRole(name)) {
-            return true;
+        if (!hasRoleDirect(role)) {
+            throw Error.error(ErrorCode.X_0P503,
+                              role.getName().getNameString());
         }
 
-        Right right = (Right) fullRightsMap.get(name);
-
-        if (right != null && !right.isEmpty()) {
-            return true;
-        }
-
-        if (!isPublic) {
-            return granteeManager.publicRole.isAccessible(name);
-        }
-
-        return false;
+        roles.remove(role);
     }
 
-    public HsqlArrayList getRightsSQL() {
+    private String roleMapToString(OrderedHashSet roles) {
+
+        StringBuffer sb = new StringBuffer();
+
+        for (int i = 0; i < roles.size(); i++) {
+            if (sb.length() > 0) {
+                sb.append(',');
+            }
+
+            Grantee role = (Grantee) roles.get(i);
+
+            sb.append(role.getName().getStatementName());
+        }
+
+        return sb.toString();
+    }
+
+    HsqlArrayList getRightsSQL() {
 
         HsqlArrayList list       = new HsqlArrayList();
-        String        roleString = getDirectRolesAsString();
+        String        roleString = roleMapToString(roles);
 
         if (roleString.length() != 0) {
             StringBuffer sb = new StringBuffer(128);
 
             sb.append(Tokens.T_GRANT).append(' ').append(roleString);
             sb.append(' ').append(Tokens.T_TO).append(' ');
-            sb.append(getStatementName());
+            sb.append(getName().getStatementName());
             list.add(sb.toString());
         }
 
@@ -1234,7 +1214,7 @@ public class Grantee implements SchemaObject {
                 }
 
                 sb.append(' ').append(Tokens.T_TO).append(' ');
-                sb.append(getStatementName());
+                sb.append(getName().getStatementName());
                 list.add(sb.toString());
             }
         }
