@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,11 +55,11 @@ import org.hsqldb.lib.Set;
  * Administrative privileges.
  *
  *
- * @author Campbell Boucher-Burnett (boucherb@users dot sourceforge.net)
+ * @author Campbell Boucher-Burnet (boucherb@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  * @author Blaine Simpson (blaine dot simpson at admc dot com)
  *
- * @version 1.9.0
+ * @version 2.0.1
  * @since 1.8.0
  * @see Grantee
  */
@@ -219,12 +219,14 @@ public class GranteeManager {
         }
 
         HsqlName name = dbObject.getName();
+
         if (dbObject instanceof Routine) {
             name = ((Routine) dbObject).getSpecificName();
         }
 
         if (!grantor.isGrantable(dbObject, right)) {
-            throw Error.error(ErrorCode.X_0L000, grantor.getNameString());
+            throw Error.error(ErrorCode.X_0L000,
+                              grantor.getName().getNameString());
         }
 
         if (grantor.isAdmin()) {
@@ -260,7 +262,8 @@ public class GranteeManager {
         }
 
         if (!granted) {
-            throw Error.error(ErrorCode.X_0L000, grantor.getNameString());
+            throw Error.error(ErrorCode.X_0L000,
+                              grantor.getName().getNameString());
         }
     }
 
@@ -277,6 +280,10 @@ public class GranteeManager {
             if (isImmutable(name)) {
                 throw Error.error(ErrorCode.X_28502, name);
             }
+
+            if (grantee instanceof User && ((User) grantee).isExternalOnly) {
+                throw Error.error(ErrorCode.X_28000, name);
+            }
         }
     }
 
@@ -317,7 +324,8 @@ public class GranteeManager {
         }
 
         if (!grantor.isGrantable(role)) {
-            throw Error.error(ErrorCode.X_0L000, grantor.getNameString());
+            throw Error.error(ErrorCode.X_0L000,
+                              grantor.getName().getNameString());
         }
 
         grantee.grant(role);
@@ -361,7 +369,8 @@ public class GranteeManager {
             }
 
             if (!grantor.isAdmin()) {
-                throw Error.error(ErrorCode.X_0L000, grantor.getNameString());
+                throw Error.error(ErrorCode.X_0L000,
+                                  grantor.getName().getNameString());
             }
         }
     }
@@ -417,6 +426,7 @@ public class GranteeManager {
         }
 
         HsqlName name = dbObject.getName();
+
         if (dbObject instanceof Routine) {
             name = ((Routine) dbObject).getSpecificName();
         }
@@ -431,7 +441,7 @@ public class GranteeManager {
 
         for (int i = 0; i < granteeList.size(); i++) {
             String  granteeName = (String) granteeList.get(i);
-            Grantee g    = get(granteeName);
+            Grantee g           = get(granteeName);
 
             if (g == null) {
                 throw Error.error(ErrorCode.X_28501, granteeName);
@@ -444,7 +454,7 @@ public class GranteeManager {
 
         for (int i = 0; i < granteeList.size(); i++) {
             String  granteeName = (String) granteeList.get(i);
-            Grantee g    = get(granteeName);
+            Grantee g           = get(granteeName);
 
             g.revoke(dbObject, rights, grantor, grantOption);
             g.updateAllRights();
@@ -577,6 +587,11 @@ public class GranteeManager {
             throw Error.error(ErrorCode.X_28503, name.name);
         }
 
+        if (SqlInvariants.isLobsSchemaName(name.name)
+                || SqlInvariants.isSystemSchemaName(name.name)) {
+            throw Error.error(ErrorCode.X_28502, name.name);
+        }
+
         Grantee g = new Grantee(name, this);
 
         g.isRole = true;
@@ -593,6 +608,11 @@ public class GranteeManager {
             throw Error.error(ErrorCode.X_28503, name.name);
         }
 
+        if (SqlInvariants.isLobsSchemaName(name.name)
+                || SqlInvariants.isSystemSchemaName(name.name)) {
+            throw Error.error(ErrorCode.X_28502, name.name);
+        }
+
         User g = new User(name, this);
 
         map.put(name.name, g);
@@ -719,7 +739,8 @@ public class GranteeManager {
             Grantee grantee = (Grantee) it.next();
 
             // ADMIN_ROLE_NAME is not persisted
-            if (!GranteeManager.isReserved(grantee.getNameString())) {
+            if (!GranteeManager.isReserved(
+                    grantee.getName().getNameString())) {
                 list.add(grantee.getSQL());
             }
         }
@@ -731,7 +752,15 @@ public class GranteeManager {
             Grantee grantee = (Grantee) it.next();
 
             if (grantee instanceof User) {
+                if (((User) grantee).isExternalOnly) {
+                    continue;
+                }
+
                 list.add(grantee.getSQL());
+
+                if (((User) grantee).isLocalOnly) {
+                    list.add(((User) grantee).getLocalUserSQL());
+                }
             }
         }
 
@@ -749,13 +778,17 @@ public class GranteeManager {
 
         while (grantees.hasNext()) {
             Grantee grantee = (Grantee) grantees.next();
-            String  name    = grantee.getNameString();
+            String  name    = grantee.getName().getNameString();
 
             // _SYSTEM user, DBA Role grants not persisted
             if (GranteeManager.isImmutable(name)) {
                 continue;
             }
 
+            if (grantee instanceof User && ((User) grantee).isExternalOnly) {
+                continue;
+            }
+
             HsqlArrayList subList = grantee.getRightsSQL();
 
             list.addAll(subList);
