@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -30,28 +30,25 @@
 
 
 /*
- * $Id: PgType.java 3481 2010-02-26 18:05:06Z fredt $
+ * $Id: PgType.java 4141 2011-03-14 01:35:49Z fredt $
  */
 
 package org.hsqldb.server;
 
+import java.sql.SQLException;
+
+import org.hsqldb.HsqlException;
+import org.hsqldb.Session;
+import org.hsqldb.jdbc.Util;
 import org.hsqldb.types.Type;
 import org.hsqldb.types.Types;
-import org.hsqldb.types.NumberType;
-import org.hsqldb.types.BooleanType;
-import org.hsqldb.types.CharacterType;
-import org.hsqldb.types.DateTimeType;
-import org.hsqldb.Session;
-import org.hsqldb.types.Types;
-import java.sql.SQLException;
-import java.io.Serializable;
-import org.hsqldb.types.JavaObjectData;
-import org.hsqldb.HsqlException;
-import org.hsqldb.types.BinaryData;
-import org.hsqldb.error.Error;
-import org.hsqldb.error.ErrorCode;
-import org.hsqldb.jdbc.Util;
 
+/**
+ * Postgres types.
+ *
+ * @author Blaine Simpson (blaine dot simpson at admc dot com)
+ * @since 1.9.0
+ */
 public class PgType {
     private int oid;
     private int typeWidth = -1;
@@ -132,7 +129,7 @@ public class PgType {
                        ? -1 : lpConstraintObject.intValue();
     }
 
-    static public PgType getPgType(Type hType, boolean directColumn)
+    public static PgType getPgType(Type hType, boolean directColumn)
     throws RecoverableOdbcFailure {
         switch (hType.typeCode) {
             case Types.TINYINT:
