@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,7 +53,6 @@ import org.hsqldb.lib.IntKeyHashMap;
 import org.hsqldb.lib.Iterator;
 import org.hsqldb.lib.StopWatch;
 import org.hsqldb.lib.StringUtil;
-import org.hsqldb.lib.WrapperIterator;
 import org.hsqldb.lib.java.JavaSystem;
 import org.hsqldb.persist.HsqlDatabaseProperties;
 import org.hsqldb.persist.HsqlProperties;
@@ -118,8 +117,9 @@ import org.hsqldb.result.ResultConstants;
  * | --silent        | true|false  | true     | false => display all queries |
  * | --trace         | true|false  | false    | display JDBC trace messages  |
  * | --tls           | true|false  | false    | TLS/SSL (secure) sockets     |
- * | --no_system_exit| true|false  | false    | do not issue System.exit()  |
+ * | --no_system_exit| true|false  | false    | do not issue System.exit()   |
  * | --remote_open   | true|false  | false    | can open databases remotely  |
+ * | --props         | filepath    |          | file path of properties file |
  * +-----------------+-------------+----------+------------------------------+
  * </pre>
  *
@@ -231,7 +231,7 @@ import org.hsqldb.result.ResultConstants;
  * is started as part of a larger framework. <p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.7.2
  *
  * @jmx.mbean
@@ -285,6 +285,7 @@ public class Server implements HsqlSocketRequestHandler {
     private PrintWriter        logWriter;
     private PrintWriter        errWriter;
     private ServerAcl          acl = null;    // null means no access tests
+    private volatile boolean   isShuttingDown;
 
 //
 
@@ -316,6 +317,13 @@ public class Server implements HsqlSocketRequestHandler {
         }
     }
 
+    /**
+     * Returns thread object for "HSQLDB Server" thread
+     */
+    public Thread getServerThread() {
+        return serverThread;
+    }
+
     /**
      * Creates a new Server instance handling HSQL protocol connections.
      */
@@ -345,10 +353,10 @@ public class Server implements HsqlSocketRequestHandler {
      *
      * @param running if true, ensure the server is running, else ensure the
      *      server is not running
-     * @throws RuntimeException if the supplied value does not match the
+     * @throws HsqlException if the supplied value does not match the
      *      current running status
      */
-    public void checkRunning(boolean running) throws RuntimeException {
+    public void checkRunning(boolean running) {
 
         int     state;
         boolean error;
@@ -364,7 +372,7 @@ public class Server implements HsqlSocketRequestHandler {
             String msg = "server is " + (running ? "not "
                                                  : "") + "running";
 
-            throw new RuntimeException(msg);
+            throw Error.error(ErrorCode.GENERAL_ERROR, msg);
         }
 
         printWithThread("checkRunning(" + running + ") exited");
@@ -379,18 +387,21 @@ public class Server implements HsqlSocketRequestHandler {
      */
     public synchronized void signalCloseAllServerConnections() {
 
-        Iterator it;
+        Iterator           it;
+        ServerConnection[] array;
 
         printWithThread("signalCloseAllServerConnections() entered");
 
         synchronized (serverConnSet) {
 
             // snapshot
-            it = new WrapperIterator(serverConnSet.toArray(null));
+            array = new ServerConnection[serverConnSet.size()];
+
+            serverConnSet.toArray(array);
         }
 
-        for (; it.hasNext(); ) {
-            ServerConnection sc = (ServerConnection) it.next();
+        for (int i = 0; i < array.length; i++) {
+            ServerConnection sc = array[i];
 
             printWithThread("Closing " + sc);
 
@@ -420,7 +431,7 @@ public class Server implements HsqlSocketRequestHandler {
     public String getAddress() {
 
         return socket == null
-               ? serverProperties.getProperty(ServerConstants.SC_KEY_ADDRESS)
+               ? serverProperties.getProperty(ServerProperties.sc_key_address)
                : socket.getInetAddress().getHostAddress();
     }
 
@@ -453,7 +464,7 @@ public class Server implements HsqlSocketRequestHandler {
     public String getDatabaseName(int index, boolean asconfigured) {
 
         if (asconfigured) {
-            return serverProperties.getProperty(ServerConstants.SC_KEY_DBNAME
+            return serverProperties.getProperty(ServerProperties.sc_key_dbname
                                                 + "." + index);
         } else if (getState() == ServerConstants.SERVER_STATE_ONLINE) {
             return (dbAlias == null || index < 0 || index >= dbAlias.length)
@@ -494,8 +505,8 @@ public class Server implements HsqlSocketRequestHandler {
     public String getDatabasePath(int index, boolean asconfigured) {
 
         if (asconfigured) {
-            return serverProperties.getProperty(ServerConstants.SC_KEY_DATABASE
-                                                + "." + index);
+            return serverProperties.getProperty(
+                ServerProperties.sc_key_database + "." + index);
         } else if (getState() == ServerConstants.SERVER_STATE_ONLINE) {
             return (dbPath == null || index < 0 || index >= dbPath.length)
                    ? null
@@ -562,8 +573,10 @@ public class Server implements HsqlSocketRequestHandler {
      *  description="At which ServerSocket listens for connections"
      */
     public int getPort() {
-        return serverProperties.getIntegerProperty(ServerConstants.SC_KEY_PORT,
-                ServerConfiguration.getDefaultPort(serverProtocol, isTls()));
+
+        return serverProperties.getIntegerProperty(
+            ServerProperties.sc_key_port,
+            ServerConfiguration.getDefaultPort(serverProtocol, isTls()));
     }
 
     /**
@@ -752,10 +765,6 @@ public class Server implements HsqlSocketRequestHandler {
         if (serverProtocol == ServerConstants.SC_PROTOCOL_HSQL) {
             r   = new ServerConnection(s, this);
             ctn = ((ServerConnection) r).getConnectionThreadName();
-
-            synchronized (serverConnSet) {
-                serverConnSet.add(r);
-            }
         } else {
             r   = new WebServerConnection(s, (WebServer) this);
             ctn = ((WebServerConnection) r).getConnectionThreadName();
@@ -778,7 +787,7 @@ public class Server implements HsqlSocketRequestHandler {
      */
     public boolean isNoSystemExit() {
         return serverProperties.isPropertyTrue(
-            ServerConstants.SC_KEY_NO_SYSTEM_EXIT);
+            ServerProperties.sc_key_no_system_exit);
     }
 
     /**
@@ -792,7 +801,7 @@ public class Server implements HsqlSocketRequestHandler {
      */
     public boolean isRestartOnShutdown() {
         return serverProperties.isPropertyTrue(
-            ServerConstants.SC_KEY_AUTORESTART_SERVER);
+            ServerProperties.sc_key_autorestart_server);
     }
 
     /**
@@ -821,7 +830,7 @@ public class Server implements HsqlSocketRequestHandler {
      *  description="Use TLS/SSL sockets?"
      */
     public boolean isTls() {
-        return serverProperties.isPropertyTrue(ServerConstants.SC_KEY_TLS);
+        return serverProperties.isPropertyTrue(ServerProperties.sc_key_tls);
     }
 
     /**
@@ -835,7 +844,7 @@ public class Server implements HsqlSocketRequestHandler {
      *  description="JDBC trace messages to System.out?"
      */
     public boolean isTrace() {
-        return serverProperties.isPropertyTrue(ServerConstants.SC_KEY_TRACE);
+        return serverProperties.isPropertyTrue(ServerProperties.sc_key_trace);
     }
 
     /**
@@ -846,7 +855,7 @@ public class Server implements HsqlSocketRequestHandler {
      *
      * @param path the path of the desired properties file, without the
      *      '.properties' file extension
-     * @throws RuntimeException if this server is running
+     * @throws HsqlException if this server is running
      * @return true if the indicated file was read sucessfully, else false
      *
      * @jmx.managed-operation
@@ -860,15 +869,28 @@ public class Server implements HsqlSocketRequestHandler {
      *   description="(optional) returns false if path is empty"
      */
     public boolean putPropertiesFromFile(String path) {
+        return putPropertiesFromFile(path, ".properties");
+    }
+
+    /**
+     * Attempts to put properties from the file with given extension.
+     *
+     * @param path the path of the desired properties file.
+     * @param extension extension to add to parth
+     * @throws HsqlException if this server is running
+     * @return true if the indicated file was read sucessfully, else false
+     *
+     */
+    public boolean putPropertiesFromFile(String path, String extension) {
 
         if (getState() != ServerConstants.SERVER_STATE_SHUTDOWN) {
-            throw new RuntimeException();
+            throw Error.error(ErrorCode.GENERAL_ERROR, "server properties");
         }
 
         path = FileUtil.getFileUtil().canonicalOrAbsolutePath(path);
 
-        HsqlProperties p = ServerConfiguration.getPropertiesFromFile(
-            ServerConstants.SC_PROTOCOL_HSQL, path);
+        ServerProperties p = ServerConfiguration.getPropertiesFromFile(
+            ServerConstants.SC_PROTOCOL_HSQL, path, extension);
 
         if (p == null || p.isEmpty()) {
             return false;
@@ -879,7 +901,9 @@ public class Server implements HsqlSocketRequestHandler {
         try {
             setProperties(p);
         } catch (Exception e) {
-            throw new RuntimeException("Failed to set properties: " + e);
+            throw Error.error(e, ErrorCode.GENERAL_ERROR,
+                              ErrorCode.M_Message_Pair,
+                              new String[]{ "Failed to set properties" });
         }
 
         return true;
@@ -892,7 +916,7 @@ public class Server implements HsqlSocketRequestHandler {
      *
      * @param s semicolon-delimited key=value pair string,
      *      e.g. silent=false;port=8080;...
-     * @throws RuntimeException if this server is running
+     * @throws HsqlException if this server is running
      *
      * @jmx.managed-operation
      *   impact="ACTION"
@@ -907,7 +931,7 @@ public class Server implements HsqlSocketRequestHandler {
     public void putPropertiesFromString(String s) {
 
         if (getState() != ServerConstants.SERVER_STATE_SHUTDOWN) {
-            throw new RuntimeException();
+            throw Error.error(ErrorCode.GENERAL_ERROR);
         }
 
         if (StringUtil.isEmpty(s)) {
@@ -917,12 +941,14 @@ public class Server implements HsqlSocketRequestHandler {
         printWithThread("putPropertiesFromString(): [" + s + "]");
 
         HsqlProperties p = HsqlProperties.delimitedArgPairsToProps(s, "=",
-            ";", ServerConstants.SC_KEY_PREFIX);
+            ";", ServerProperties.sc_key_prefix);
 
         try {
             setProperties(p);
         } catch (Exception e) {
-            throw new RuntimeException("Failed to set properties: " + e);
+            throw Error.error(e, ErrorCode.GENERAL_ERROR,
+                              ErrorCode.M_Message_Pair,
+                              new String[]{ "Failed to set properties" });
         }
     }
 
@@ -936,11 +962,11 @@ public class Server implements HsqlSocketRequestHandler {
      *    be retrieved by InetAddres.getByName(), or a null or empty string
      *    or "0.0.0.0" to signify that the server socket should be constructed
      *    using the signature that does not specify the InetAddress.
-     * @throws RuntimeException if this server is running
+     * @throws HsqlException if this server is running
      *
      * @jmx.managed-attribute
      */
-    public void setAddress(String address) throws RuntimeException {
+    public void setAddress(String address) {
 
         checkRunning(false);
 
@@ -949,7 +975,7 @@ public class Server implements HsqlSocketRequestHandler {
         }
 
         printWithThread("setAddress(" + address + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_ADDRESS, address);
+        serverProperties.setProperty(ServerProperties.sc_key_address, address);
     }
 
     /**
@@ -957,7 +983,7 @@ public class Server implements HsqlSocketRequestHandler {
      *
      * @param name external name (url alias) of the i'th HSQLDB database
      *      instance this server is to host.
-     * @throws RuntimeException if this server is running
+     * @throws HsqlException if this server is running
      *
      * @jmx.managed-operation
      *      impact="ACTION"
@@ -975,12 +1001,11 @@ public class Server implements HsqlSocketRequestHandler {
      *      position="1"
      *      description="url alias component for the hosted Database"
      */
-    public void setDatabaseName(int index,
-                                String name) throws RuntimeException {
+    public void setDatabaseName(int index, String name) {
 
         checkRunning(false);
         printWithThread("setDatabaseName(" + index + "," + name + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_DBNAME + "."
+        serverProperties.setProperty(ServerProperties.sc_key_dbname + "."
                                      + index, name);
     }
 
@@ -1008,12 +1033,11 @@ public class Server implements HsqlSocketRequestHandler {
      *      position="1"
      *      description="database uri path of the hosted Database"
      */
-    public void setDatabasePath(int index,
-                                String path) throws RuntimeException {
+    public void setDatabasePath(int index, String path) {
 
         checkRunning(false);
         printWithThread("setDatabasePath(" + index + "," + path + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_DATABASE + "."
+        serverProperties.setProperty(ServerProperties.sc_key_database + "."
                                      + index, path);
     }
 
@@ -1033,7 +1057,7 @@ public class Server implements HsqlSocketRequestHandler {
             return;
         }
 
-        serverProperties.setProperty(ServerConstants.SC_KEY_WEB_DEFAULT_PAGE,
+        serverProperties.setProperty(ServerProperties.sc_key_web_default_page,
                                      file);
     }
 
@@ -1044,11 +1068,11 @@ public class Server implements HsqlSocketRequestHandler {
      *
      * @jmx.managed-attribute
      */
-    public void setPort(int port) throws RuntimeException {
+    public void setPort(int port) {
 
         checkRunning(false);
         printWithThread("setPort(" + port + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_PORT, port);
+        serverProperties.setProperty(ServerProperties.sc_key_port, port);
     }
 
     /**
@@ -1083,7 +1107,7 @@ public class Server implements HsqlSocketRequestHandler {
     public void setNoSystemExit(boolean noExit) {
 
         printWithThread("setNoSystemExit(" + noExit + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_NO_SYSTEM_EXIT,
+        serverProperties.setProperty(ServerProperties.sc_key_no_system_exit,
                                      noExit);
     }
 
@@ -1097,8 +1121,8 @@ public class Server implements HsqlSocketRequestHandler {
     public void setRestartOnShutdown(boolean restart) {
 
         printWithThread("setRestartOnShutdown(" + restart + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_AUTORESTART_SERVER,
-                                     restart);
+        serverProperties.setProperty(
+            ServerProperties.sc_key_autorestart_server, restart);
     }
 
     /**
@@ -1112,7 +1136,7 @@ public class Server implements HsqlSocketRequestHandler {
     public void setSilent(boolean silent) {
 
         printWithThread("setSilent(" + silent + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_SILENT, silent);
+        serverProperties.setProperty(ServerProperties.sc_key_silent, silent);
 
         isSilent = silent;
     }
@@ -1121,7 +1145,7 @@ public class Server implements HsqlSocketRequestHandler {
      * Sets whether to use secure sockets
      *
      * @param tls true for secure sockets, else false
-     * @throws RuntimeException if this server is running
+     * @throws HsqlException if this server is running
      *
      * @jmx.managed-attribute
      */
@@ -1129,7 +1153,7 @@ public class Server implements HsqlSocketRequestHandler {
 
         checkRunning(false);
         printWithThread("setTls(" + tls + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_TLS, tls);
+        serverProperties.setProperty(ServerProperties.sc_key_tls, tls);
     }
 
     /**
@@ -1143,12 +1167,15 @@ public class Server implements HsqlSocketRequestHandler {
     public void setTrace(boolean trace) {
 
         printWithThread("setTrace(" + trace + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_TRACE, trace);
+        serverProperties.setProperty(ServerProperties.sc_key_trace, trace);
         JavaSystem.setLogToSystem(trace);
     }
 
     /**
-     * Sets whether server thread is a daemon. Used before starting
+     * Sets whether server thread is a daemon. Used before starting.
+     * The default is false.
+     *
+     * @param daemon if true, start the thread as a daemon thread
      *
      * @jmx.managed-attribute
      */
@@ -1156,7 +1183,7 @@ public class Server implements HsqlSocketRequestHandler {
 
         checkRunning(false);
         printWithThread("setDaemon(" + daemon + ")");
-        serverProperties.setProperty(ServerConstants.SC_KEY_DAEMON, daemon);
+        serverProperties.setProperty(ServerProperties.sc_key_daemon, daemon);
     }
 
     /**
@@ -1179,42 +1206,49 @@ public class Server implements HsqlSocketRequestHandler {
             return;
         }
 
-        serverProperties.setProperty(ServerConstants.SC_KEY_WEB_ROOT, root);
+        serverProperties.setProperty(ServerProperties.sc_key_web_root, root);
     }
 
     /**
      * Sets server properties using the specified properties object
      *
-     * @param p The object containing properties to set
+     * @param props The object containing properties to set
      * @throws ServerAcl.AclFormatException
      *          ACL list was requested but problem loading ACL.
      * @throws IOException
      *          ACL list was requested but I/O problem loading ACL.
      */
-    public void setProperties(HsqlProperties p)
+    public void setProperties(HsqlProperties props)
     throws IOException, ServerAcl.AclFormatException {
 
         checkRunning(false);
 
-        if (p != null) {
-            serverProperties.addProperties(p);
-            ServerConfiguration.translateAddressProperty(serverProperties);
+        if (props != null) {
+            props.validate();
+
+            String[] errors = props.getErrorKeys();
+
+            if (errors.length > 0) {
+                throw Error.error(ErrorCode.SERVER_NO_DATABASE, errors[0]);
+            }
+
+            serverProperties.addProperties(props);
         }
 
         maxConnections = serverProperties.getIntegerProperty(
-            ServerConstants.SC_KEY_MAX_CONNECTIONS, 16);
+            ServerProperties.sc_key_max_connections, 16);
 
         JavaSystem.setLogToSystem(isTrace());
 
         isSilent =
-            serverProperties.isPropertyTrue(ServerConstants.SC_KEY_SILENT);
+            serverProperties.isPropertyTrue(ServerProperties.sc_key_silent);
         isRemoteOpen = serverProperties.isPropertyTrue(
-            ServerConstants.SC_KEY_REMOTE_OPEN_DB);
+            ServerProperties.sc_key_remote_open_db);
         isDaemon =
-            serverProperties.isPropertyTrue(ServerConstants.SC_KEY_DAEMON);
+            serverProperties.isPropertyTrue(ServerProperties.sc_key_daemon);
 
         String aclFilepath =
-            serverProperties.getProperty(ServerConstants.SC_KEY_ACL_FILEPATH);
+            serverProperties.getProperty(ServerProperties.sc_key_acl);
 
         if (aclFilepath != null) {
             acl = new ServerAcl(new File(aclFilepath));;
@@ -1308,14 +1342,16 @@ public class Server implements HsqlSocketRequestHandler {
 
     /**
      * Retrieves whether the specified socket should be allowed
-     * to make a connection.  By default, this method always returns
-     * true, but it can be overidden to implement hosts allow-deny
-     * functionality.
+     * to make a connection.
      *
      * @param socket the socket to test.
      */
     protected boolean allowConnection(Socket socket) {
 
+        if (isShuttingDown) {
+            return false;
+        }
+
         return (acl == null) ? true
                              : acl.permitAccess(
                                  socket.getInetAddress().getAddress());
@@ -1385,8 +1421,9 @@ public class Server implements HsqlSocketRequestHandler {
      */
     final synchronized void releaseDatabase(int id) {
 
-        Iterator it;
-        boolean  found = false;
+        Iterator           it;
+        boolean            found = false;
+        ServerConnection[] array;
 
         printWithThread("releaseDatabase(" + id + ") entered");
 
@@ -1403,15 +1440,16 @@ public class Server implements HsqlSocketRequestHandler {
         }
 
         synchronized (serverConnSet) {
-            it = new WrapperIterator(serverConnSet.toArray(null));
+            array = new ServerConnection[serverConnSet.size()];
+
+            serverConnSet.toArray(array);
         }
 
-        while (it.hasNext()) {
-            ServerConnection sc = (ServerConnection) it.next();
+        for (int i = 0; i < array.length; i++) {
+            ServerConnection sc = array[i];
 
             if (sc.dbID == id) {
                 sc.signalClose();
-                serverConnSet.remove(sc);
             }
         }
 
@@ -1485,7 +1523,7 @@ public class Server implements HsqlSocketRequestHandler {
      * @param msg the message to print
      */
     final void printWithTimestamp(String msg) {
-        print(HsqlDateTime.getSytemTimeString() + " " + msg);
+        print(HsqlDateTime.getSystemTimeString() + " " + msg);
     }
 
     /**
@@ -1567,6 +1605,7 @@ public class Server implements HsqlSocketRequestHandler {
 
                 break;
             }
+            case ResultConstants.EXECUTE_INVALID :
             case ResultConstants.EXECUTE : {
                 sb.append("SQLCLI:SQLEXECUTE:");
                 sb.append(r.getStatementID());
@@ -1667,6 +1706,22 @@ public class Server implements HsqlSocketRequestHandler {
 
                 break;
             }
+            case ResultConstants.CLOSE_RESULT : {
+                sb.append("HQLCLI:CLOSE_RESULT:RESULT_ID ");
+                sb.append(r.getResultId());
+
+                break;
+            }
+            case ResultConstants.REQUESTDATA : {
+                sb.append("HQLCLI:REQUESTDATA:RESULT_ID ");
+                sb.append(r.getResultId());
+                sb.append(" ROWOFFSET ");
+                sb.append(r.getUpdateCount());
+                sb.append(" ROWCOUNT ");
+                sb.append(r.getFetchSize());
+
+                break;
+            }
             default : {
                 sb.append("SQLCLI:MODE:");
                 sb.append(r.getType());
@@ -1696,8 +1751,8 @@ public class Server implements HsqlSocketRequestHandler {
 
         if (dbIndex == -1) {
             if (filepath == null) {
-                RuntimeException e =
-                    new RuntimeException("database alias does not exist");
+                HsqlException e = Error.error(ErrorCode.GENERAL_ERROR,
+                                              "database alias does not exist");
 
                 printError("database alias=" + alias + " does not exist");
                 setServerError(e);
@@ -1717,8 +1772,8 @@ public class Server implements HsqlSocketRequestHandler {
     final int openDatabase(String alias, String datapath) {
 
         if (!isRemoteOpen) {
-            RuntimeException e =
-                new RuntimeException("remote open not allowed");
+            HsqlException e = Error.error(ErrorCode.GENERAL_ERROR,
+                                          "remote open not allowed");
 
             printError("Remote database open not allowed");
             setServerError(e);
@@ -1732,8 +1787,9 @@ public class Server implements HsqlSocketRequestHandler {
             i = closeOldestDatabase();
 
             if (i < -1) {
-                RuntimeException e =
-                    new RuntimeException("limit of open databases reached");
+                HsqlException e =
+                    Error.error(ErrorCode.GENERAL_ERROR,
+                                "limit of open databases reached");
 
                 printError("limit of open databases reached");
                 setServerError(e);
@@ -1745,7 +1801,8 @@ public class Server implements HsqlSocketRequestHandler {
         HsqlProperties newprops = DatabaseURL.parseURL(datapath, false, false);
 
         if (newprops == null) {
-            RuntimeException e = new RuntimeException("invalid database path");
+            HsqlException e = Error.error(ErrorCode.GENERAL_ERROR,
+                                          "invalid database path");
 
             printError("invalid database path");
             setServerError(e);
@@ -1866,9 +1923,9 @@ public class Server implements HsqlSocketRequestHandler {
         int           maxDatabases = dbNumberMap.size();
 
         if (serverProperties.isPropertyTrue(
-                ServerConstants.SC_KEY_REMOTE_OPEN_DB)) {
+                ServerProperties.sc_key_remote_open_db)) {
             int max = serverProperties.getIntegerProperty(
-                ServerConstants.SC_KEY_MAX_DATABASES,
+                ServerProperties.sc_key_max_databases,
                 ServerConstants.SC_DEFAULT_MAX_DATABASES);
 
             if (maxDatabases < max) {
@@ -1919,7 +1976,7 @@ public class Server implements HsqlSocketRequestHandler {
      */
     private IntKeyHashMap getDBNameArray() {
 
-        final String  prefix       = ServerConstants.SC_KEY_DBNAME + ".";
+        final String  prefix       = ServerProperties.sc_key_dbname + ".";
         final int     prefixLen    = prefix.length();
         IntKeyHashMap idToAliasMap = new IntKeyHashMap();
         Enumeration   en           = serverProperties.propertyNames();
@@ -2079,7 +2136,7 @@ public class Server implements HsqlSocketRequestHandler {
      * this method exists immediately, otherwise, the result is to fully
      * shut down the server.
      */
-    private void releaseServerSocket() {
+    private synchronized void releaseServerSocket() {
 
         printWithThread("releaseServerSocket() entered");
 
@@ -2163,13 +2220,16 @@ public class Server implements HsqlSocketRequestHandler {
         print(sw.elapsedTimeToMessage("Startup sequence completed"));
         printServerOnlineMessage();
 
+        // isShuttingDown is only read after socket connections are 'accept'ed.
+        isShuttingDown = false;    // In case shutdown was aborted previously.
+
         try {
             /*
              * This loop is necessary for UNIX w/ Sun Java 1.3 because
              * in that case the socket.close() elsewhere will not
              * interrupt this accept().
              */
-            while (true) {
+            while (socket != null) {
                 try {
                     handleConnection(socket.accept());
                 } catch (java.io.InterruptedIOException iioe) {}
@@ -2196,6 +2256,55 @@ public class Server implements HsqlSocketRequestHandler {
         serverError = t;
     }
 
+    /**
+     * Shuts down all the database served by this server. As a consequence,
+     * this server and any other server that is serving a subset of the
+     * databases will be shutdown, unless the server was started with
+     * server.remote_open property.
+     *
+     * The shutdownMode must be one of:
+     *
+     * <ul>
+     * <li>org.hsqldb.Database.CLOSEMODE_IMMEDIATELY
+     * <li>org.hsqldb.Database.CLOSEMODE_NORMAL
+     * <li>org.hsqldb.Database.CLOSEMODE_COMPACT
+     * <li>org.hsqldb.Database.CLOSEMODE_SCRIPT
+     * </ul>
+     * @param shutdownMode a value between 0-4, usually 0 or 1.
+     */
+    public void shutdownCatalogs(int shutdownMode) {
+        DatabaseManager.shutdownDatabases(this, shutdownMode);
+    }
+
+    /**
+     * Shuts down this server and all the database served by this server. As a
+     * consequence, any other server that is serving a subset of the databases
+     * will be shutdown, unless the server was started with server.remote_open
+     * property.
+     *
+     * The shutdownMode must be one of:
+     *
+     * <ul>
+     * <li>org.hsqldb.Database.CLOSEMODE_IMMEDIATELY
+     * <li>org.hsqldb.Database.CLOSEMODE_NORMAL
+     * <li>org.hsqldb.Database.CLOSEMODE_COMPACT
+     * <li>org.hsqldb.Database.CLOSEMODE_SCRIPT
+     * </ul>
+     * @param shutdownMode a value between 0-4, usually 0 or 1.
+     */
+    public void shutdownWithCatalogs(int shutdownMode) {
+
+        // If an unchecked exception is thrown, isShuttingDown will be left true,
+        // which is good from a security standpoint.
+        isShuttingDown = true;
+
+        // make handleConnection() reject new connection attempts
+        DatabaseManager.shutdownDatabases(this, shutdownMode);
+        shutdown(false);
+
+        isShuttingDown = false;
+    }
+
     /**
      * External method to shut down this server.
      */
@@ -2232,13 +2341,15 @@ public class Server implements HsqlSocketRequestHandler {
         }
 
         // Be nice and let applications exit if there are no
-        // running connection threads
+        // running connection threads - wait at most 100 ms per active thread
         if (serverConnectionThreadGroup != null) {
             if (!serverConnectionThreadGroup.isDestroyed()) {
-                for (int i = 0; serverConnectionThreadGroup.activeCount() > 0;
-                        i++) {
-                    int count;
+                int count = serverConnectionThreadGroup.activeCount();
 
+                for (int i = 0;
+                        serverConnectionThreadGroup.activeCount() > 0
+                        && i < count;
+                        i++) {
                     try {
                         Thread.sleep(100);
                     } catch (Exception e) {
@@ -2350,26 +2461,40 @@ public class Server implements HsqlSocketRequestHandler {
      */
     public static void main(String[] args) {
 
-        String propsPath =
-            FileUtil.getFileUtil().canonicalOrAbsolutePath("server");
-        ServerProperties fileProps = ServerConfiguration.getPropertiesFromFile(
-            ServerConstants.SC_PROTOCOL_HSQL, propsPath);
-        ServerProperties props =
-            fileProps == null
-            ? new ServerProperties(ServerConstants.SC_PROTOCOL_HSQL)
-            : fileProps;
-        HsqlProperties stringProps = null;
+        HsqlProperties argProps = null;
 
-        stringProps = HsqlProperties.argArrayToProps(args,
-                ServerConstants.SC_KEY_PREFIX);
+        argProps = HsqlProperties.argArrayToProps(args,
+                ServerProperties.sc_key_prefix);
 
-        if (stringProps.getErrorKeys().length != 0) {
+        String[] errors = argProps.getErrorKeys();
+
+        if (errors.length != 0) {
+            System.out.println("no value for argument:" + errors[0]);
             printHelp("server.help");
 
             return;
         }
 
-        props.addProperties(stringProps);
+        String propsPath = argProps.getProperty(ServerProperties.sc_key_props);
+        String propsExtension = "";
+
+        if (propsPath == null) {
+            propsPath      = "server";
+            propsExtension = ".properties";
+        } else {
+            argProps.removeProperty(ServerProperties.sc_key_props);
+        }
+
+        propsPath = FileUtil.getFileUtil().canonicalOrAbsolutePath(propsPath);
+
+        ServerProperties fileProps = ServerConfiguration.getPropertiesFromFile(
+            ServerConstants.SC_PROTOCOL_HSQL, propsPath, propsExtension);
+        ServerProperties props =
+            fileProps == null
+            ? new ServerProperties(ServerConstants.SC_PROTOCOL_HSQL)
+            : fileProps;
+
+        props.addProperties(argProps);
         ServerConfiguration.translateDefaultDatabaseProperty(props);
 
         // Standard behaviour when started from the command line
@@ -2377,16 +2502,13 @@ public class Server implements HsqlSocketRequestHandler {
         // course, be overridden by whatever, if any, security policy
         // is in place.
         ServerConfiguration.translateDefaultNoSystemExitProperty(props);
+        ServerConfiguration.translateAddressProperty(props);
 
         // finished setting up properties;
         Server server = new Server();
 
         try {
             server.setProperties(props);
-            props.validate();
-
-            // This must be called after setProperties, because stringProps
-            // isn't populated until then.
         } catch (Exception e) {
             server.printError("Failed to set properties");
             server.printStackTrace(e);
@@ -2399,7 +2521,7 @@ public class Server implements HsqlSocketRequestHandler {
 
         if (fileProps != null) {
             server.print("Loaded properties from [" + propsPath
-                         + ".properties]");
+                         + propsExtension + "]");
         } else {
             server.print("Could not load properties from file");
             server.print("Using cli/default properties only");
