@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,6 +41,9 @@ import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
 import java.net.HttpURLConnection;
 import java.net.Socket;
+import java.nio.ByteBuffer;
+import java.nio.charset.Charset;
+import java.nio.charset.CharsetDecoder;
 
 import org.hsqldb.DatabaseManager;
 import org.hsqldb.HsqlException;
@@ -56,6 +59,7 @@ import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultConstants;
 import org.hsqldb.rowio.RowInputBinary;
 import org.hsqldb.rowio.RowOutputBinary;
+import org.hsqldb.lib.HsqlByteArrayOutputStream;
 
 // fredt@users 20021002 - patch 1.7.1 - changed notification method
 // unsaved@users 20021113 - patch 1.7.2 - SSL support
@@ -79,12 +83,14 @@ import org.hsqldb.rowio.RowOutputBinary;
  *
  * @author Thomas Mueller (Hypersonic SQL Group)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since Hypersonic SQL
  */
 class WebServerConnection implements Runnable {
 
-    static final String         ENCODING = "ISO-8859-1";
+    static final String         ENCODING           = "ISO-8859-1";
+    private Charset             iso                = Charset.forName(ENCODING);
+    private CharsetDecoder      iso_8859_1_decoder = iso.newDecoder();
     private Socket              socket;
     private WebServer           server;
     private static final int    REQUEST_TYPE_BAD  = 0;
@@ -192,12 +198,10 @@ class WebServerConnection implements Runnable {
             inStream = new DataInputStream(socket.getInputStream());
 
             int    count;
-            String request;
             String name   = null;
             int    method = REQUEST_TYPE_BAD;
-            int    len    = -1;
 
-            // read line, ignoring any leading blank lines
+            // read line, ignoring any leading blank lines (there shouldn't be any, but just to be safe)
             do {
                 count = InOutUtil.readLine(inStream, rowOut);
 
@@ -206,7 +210,7 @@ class WebServerConnection implements Runnable {
                 }
             } while (count < 2);
 
-            byte[] byteArray = rowOut.getBuffer();
+            byte[] byteArray = rowOut.toByteArray();
             int    offset    = rowOut.size() - count;
 
             if (ArrayUtil.containsAt(byteArray, offset, BYTES_POST)) {
@@ -236,6 +240,10 @@ class WebServerConnection implements Runnable {
 
             switch (method) {
 
+                case REQUEST_TYPE_POST :
+                    processPost(inStream, name);
+                    break;
+
                 case REQUEST_TYPE_BAD :
                     processError(REQUEST_TYPE_BAD);
                     break;
@@ -247,10 +255,6 @@ class WebServerConnection implements Runnable {
                 case REQUEST_TYPE_HEAD :
                     processGet(name, false);
                     break;
-
-                case REQUEST_TYPE_POST :
-                    processPost(inStream, name);
-                    break;
             }
         } catch (Exception e) {
             server.printStackTrace(e);
@@ -269,43 +273,49 @@ class WebServerConnection implements Runnable {
 
     /**
      * POST is used only for database access. So we can assume the strings
-     * are those generated by HTTPClientConnection
+     * are those generated by ClientConnectionHTTP
      */
     private void processPost(InputStream inStream,
                              String name) throws IOException {
 
-        // fredt - parsing in this block is not actually necessary
         try {
 
-            // read the Content-Type line
-            InOutUtil.readLine(inStream, rowOut);
+            // In run() the first line of the requestHeader was already read into rowOut (for a POST
+            // this looks something like: 'POST /<some-path> HTTP/1.1')
+            //
+            // Now let's read the rest of the requestHeader until (including) the blank line that
+            // denotes the end of the requestHeader block (We can not assume a fixed number of
+            // lines since proxy's might add stuff, nor should we assume a certain order of the
+            // requestHeader lines)
+            int readLineLength;
 
-            // read and parse the Content-Length line
-            int count  = InOutUtil.readLine(inStream, rowOut);
-            int offset = rowOut.size() - count;
+            do {
+                readLineLength = InOutUtil.readLine(inStream, rowOut);
+            } while (readLineLength > 2);    // Blank line is usually 2 bytes ('\r\n')
 
-            // get buffer always after reading into rowOut, else old buffer may
-            // be returned
-            byte[] byteArray = rowOut.getBuffer();
+            // rowOut now contains the entire requestHeader as bytes. It is converted here
+            // into a java String, so that we can easily do some error checking on it and
+            //  inspect it during debugging
+            String requestHeader = iso_8859_1_decoder.decode(
+                ByteBuffer.wrap(rowOut.toByteArray())).toString();
 
-            if (!ArrayUtil.containsAt(byteArray, offset, BYTES_CONTENT)) {
+// System.out.println(requestHeader); //For debugging
+            // Throw an error if the Content-Type is something other than an what
+            // ClientConnectionHTTP is supposed to send
+            if (requestHeader.indexOf("Content-Type: application/octet-stream")
+                    < 0) {
                 throw new Exception();
             }
 
-            count  -= BYTES_CONTENT.length;
-            offset += BYTES_CONTENT.length;
-
-            // omit the last two characters
-            String lenStr = new String(byteArray, offset, count - 2);
-            int    length = Integer.parseInt(lenStr);
-
-            InOutUtil.readLine(inStream, rowOut);
+            //TODO: Determine presence of Keep-Alive in requestHeader and act upon it accordingly.
+            // if (requestHeader.indexOf("Connection: keep-alive") >= 0) { }
         } catch (Exception e) {
             processError(HttpURLConnection.HTTP_BAD_REQUEST);
 
             return;
         }
 
+        // inStream's read-pointer will now be positioned at the beginning of the request's pay-load
         processQuery(inStream);
     }
 
@@ -363,7 +373,7 @@ class WebServerConnection implements Runnable {
                         Error.error(ErrorCode.SERVER_DATABASE_DISCONNECTED));
                 } else {
                     resultIn.setSession(session);
-                    resultIn.readAdditionalResults(session, dataIn, rowIn);
+                    resultIn.readLobResults(session, dataIn, rowIn);
 
                     resultOut = session.execute(resultIn);
                 }
@@ -371,21 +381,47 @@ class WebServerConnection implements Runnable {
 
             int type = resultIn.getType();
 
+// patched 2.2.9 by Aart 2012-05-15: Make sure 'Content-length' is correctly set
             if (type == ResultConstants.DISCONNECT
                     || type == ResultConstants.RESETSESSION) {
+                DataOutputStream dataOut =
+                    new DataOutputStream(socket.getOutputStream());
+
+                // Upon DISCONNECT 6 bytes are read by the ClientConnectionHTTP": mode (1 byte), a length (int), and an 'additional results (1 byte)
+                String header = getHead(HEADER_OK, false,
+                                        "application/octet-stream", 6);
+
+                dataOut.write(header.getBytes(ENCODING));
+                dataOut.writeByte(ResultConstants.DISCONNECT);    // Mode
+                dataOut.writeInt(4);                              //Length Int of first result is always read! Minvalue is 4: It is the number of bytes of the current result (it includes the length of this Int itself)
+                dataOut.writeByte(ResultConstants.NONE);          // No Additional results
+                dataOut.close();
+
                 return;
             }
 
-//
+            // TODO: when doing Keep-Alive connections, try to retain buffer
+            HsqlByteArrayOutputStream memStream =
+                new HsqlByteArrayOutputStream();
+            DataOutputStream tempOutput = new DataOutputStream(memStream);
+
+            resultOut.write(session, tempOutput, rowOut);
+
             DataOutputStream dataOut =
                 new DataOutputStream(socket.getOutputStream());
+
+            // Write HTTP response header
             String header = getHead(HEADER_OK, false,
-                                    "application/octet-stream", rowOut.size());
+                                    "application/octet-stream",
+                                    memStream.size());
 
             dataOut.write(header.getBytes(ENCODING));
-            dataOut.flush();
-            resultOut.write(dataOut, rowOut);
+
+            // Write actual pay-load to response
+            memStream.writeTo(dataOut);
             dataOut.close();
+
+// patch-end 2.2.9 by Aart 2012-05-15
         } catch (Exception e) {
             server.printStackTrace(e);
         }
@@ -484,8 +520,10 @@ class WebServerConnection implements Runnable {
         }
 
         if (mimeType != null) {
+            sb.append("Cache-Control: no-cache\r\n");    // DB-traffic should not be cached by proxy's
             sb.append("Content-Type: ").append(mimeType).append("\r\n");
-            sb.append("Content-Length: ").append(length).append("\r\n");
+
+            //sb.append("Content-Length: ").append(length).append("\r\n");
         }
 
         sb.append("\r\n");
