@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,7 @@ import java.util.NoSuchElementException;
 
 import org.hsqldb.lib.ArrayCounter;
 import org.hsqldb.lib.Iterator;
+import org.hsqldb.lib.ObjectComparator;
 
 /**
  * Base class for hash tables or sets. The exact type of the structure is
@@ -45,7 +46,7 @@ import org.hsqldb.lib.Iterator;
  * Special getOrAddXXX() methods are used for object maps in some subclasses.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.7.2
  */
 public class BaseHashMap {
@@ -114,12 +115,13 @@ public class BaseHashMap {
     protected Object[]  objectValueTable2;
 
     //
-    final float       loadFactor;
-    final int         initialCapacity;
-    int               threshold;
-    protected int     maxCapacity;
-    protected int     purgePolicy = NO_PURGE;
-    protected boolean minimizeOnEmpty;
+    final float                loadFactor;
+    final int                  initialCapacity;
+    int                        threshold;
+    protected int              maxCapacity;
+    protected int              purgePolicy = NO_PURGE;
+    protected boolean          minimizeOnEmpty;
+    protected ObjectComparator comparator;
 
     //
     boolean hasZeroKey;
@@ -138,7 +140,7 @@ public class BaseHashMap {
     protected static final int PURGE_QUARTER = 3;
 
     //
-    public final static int ACCESS_MAX = Integer.MAX_VALUE - (1 << 20);
+    public static final int ACCESS_MAX = Integer.MAX_VALUE - (1 << 20);
 
     protected BaseHashMap(int initialCapacity, int keyType, int valueType,
                           boolean hasAccessCount)
@@ -204,7 +206,7 @@ public class BaseHashMap {
             tempKey = objectKeyTable[lookup];
 
             if (key.equals(tempKey)) {
-                return lookup;
+                break;
             }
         }
 
@@ -220,7 +222,7 @@ public class BaseHashMap {
             tempKey = intKeyTable[lookup];
 
             if (key == tempKey) {
-                return lookup;
+                break;
             }
         }
 
@@ -236,7 +238,23 @@ public class BaseHashMap {
             tempKey = longKeyTable[lookup];
 
             if (key == tempKey) {
-                return lookup;
+                break;
+            }
+        }
+
+        return lookup;
+    }
+
+    protected int getObjectLookup(long key) {
+
+        int  lookup = hashIndex.getLookup((int) key);
+        long tempKey;
+
+        for (; lookup >= 0; lookup = hashIndex.getNextLookup(lookup)) {
+            tempKey = comparator.longKey(objectKeyTable[lookup]);
+
+            if (tempKey == key) {
+                break;
             }
         }
 
@@ -256,6 +274,31 @@ public class BaseHashMap {
         return valuesIterator;
     }
 
+    protected int valueCount(Object key, int hash) {
+
+        int lookup = getLookup(key, hash);
+
+        if (lookup == -1) {
+            return 0;
+        }
+
+        int count = 1;
+
+        while (true) {
+            lookup = BaseHashMap.this.hashIndex.getNextLookup(lookup);
+
+            if (lookup == -1) {
+                break;
+            }
+
+            if (BaseHashMap.this.objectKeyTable[lookup].equals(key)) {
+                count++;
+            }
+        }
+
+        return count;
+    }
+
     /**
      * generic method for adding or removing keys
      */
@@ -270,7 +313,11 @@ public class BaseHashMap {
                 return null;
             }
 
-            hash = objectKey.hashCode();
+            if (comparator == null) {
+                hash = objectKey.hashCode();
+            } else {
+                hash = comparator.hashCode(objectKey);
+            }
         }
 
         int    index       = hashIndex.getHashIndex(hash);
@@ -282,8 +329,15 @@ public class BaseHashMap {
                 lastLookup = lookup,
                 lookup = hashIndex.getNextLookup(lookup)) {
             if (isObjectKey) {
-                if (objectKeyTable[lookup].equals(objectKey)) {
-                    break;
+                if (comparator == null) {
+                    if (objectKeyTable[lookup].equals(objectKey)) {
+                        break;
+                    }
+                } else {
+                    if (comparator.compare(objectKeyTable[lookup], objectKey)
+                            == 0) {
+                        break;
+                    }
                 }
             } else if (isIntKey) {
                 if (longKey == intKeyTable[lookup]) {
@@ -357,13 +411,11 @@ public class BaseHashMap {
         }
 
         if (hashIndex.elementCount >= threshold) {
-
-            // should throw maybe, if reset returns false?
             if (reset()) {
                 return addOrRemove(longKey, longValue, objectKey, objectValue,
                                    remove);
             } else {
-                return null;
+                throw new NoSuchElementException("BaseHashMap");
             }
         }
 
@@ -420,7 +472,11 @@ public class BaseHashMap {
                 return null;
             }
 
-            hash = objectKey.hashCode();
+            if (comparator == null) {
+                hash = objectKey.hashCode();
+            } else {
+                hash = comparator.hashCode(objectKey);
+            }
         }
 
         int     index       = hashIndex.getHashIndex(hash);
@@ -433,9 +489,38 @@ public class BaseHashMap {
                 lastLookup = lookup,
                 lookup = hashIndex.getNextLookup(lookup)) {
             if (isObjectKey) {
-                if (objectKeyTable[lookup].equals(objectKey)) {
-                    if (removeKey) {
-                        while (true) {
+                if (comparator == null) {
+                    if (objectKeyTable[lookup].equals(objectKey)) {}
+                    else {
+                        continue;
+                    }
+                } else {
+                    if (comparator.compare(objectKeyTable[lookup], objectKey)
+                            == 0) {}
+                    else {
+                        continue;
+                    }
+                }
+
+                if (removeKey) {
+                    while (true) {
+                        objectKeyTable[lookup]   = null;
+                        returnValue              = objectValueTable[lookup];
+                        objectValueTable[lookup] = null;
+
+                        hashIndex.unlinkNode(index, lastLookup, lookup);
+
+                        multiValueTable[lookup] = false;
+                        lookup                  = hashIndex.hashTable[index];
+
+                        if (lookup < 0
+                                || !objectKeyTable[lookup].equals(objectKey)) {
+                            return returnValue;
+                        }
+                    }
+                } else {
+                    if (objectValueTable[lookup].equals(objectValue)) {
+                        if (removeValue) {
                             objectKeyTable[lookup]   = null;
                             returnValue = objectValueTable[lookup];
                             objectValueTable[lookup] = null;
@@ -443,36 +528,16 @@ public class BaseHashMap {
                             hashIndex.unlinkNode(index, lastLookup, lookup);
 
                             multiValueTable[lookup] = false;
-                            lookup = hashIndex.hashTable[index];
+                            lookup                  = lastLookup;
 
-                            if (lookup < 0
-                                    || !objectKeyTable[lookup].equals(
-                                        objectKey)) {
-                                return returnValue;
-                            }
-                        }
-                    } else {
-                        if (objectValueTable[lookup].equals(objectValue)) {
-                            if (removeValue) {
-                                objectKeyTable[lookup]   = null;
-                                returnValue = objectValueTable[lookup];
-                                objectValueTable[lookup] = null;
-
-                                hashIndex.unlinkNode(index, lastLookup,
-                                                     lookup);
-
-                                multiValueTable[lookup] = false;
-                                lookup                  = lastLookup;
-
-                                return returnValue;
-                            } else {
-                                return objectValueTable[lookup];
-                            }
+                            return returnValue;
+                        } else {
+                            return objectValueTable[lookup];
                         }
                     }
-
-                    multiValue = true;
                 }
+
+                multiValue = true;
             } else if (isIntKey) {
                 if (longKey == intKeyTable[lookup]) {
                     if (removeKey) {
@@ -540,14 +605,12 @@ public class BaseHashMap {
         }
 
         if (hashIndex.elementCount >= threshold) {
-
-            // should throw maybe, if reset returns false?
             if (reset()) {
                 return addOrRemoveMultiVal(longKey, longValue, objectKey,
                                            objectValue, removeKey,
                                            removeValue);
             } else {
-                return null;
+                throw new NoSuchElementException("BaseHashMap");
             }
         }
 
@@ -722,6 +785,7 @@ public class BaseHashMap {
                 lastLookup = lookup,
                 lookup = hashIndex.getNextLookup(lookup)) {
             if (objectKeyTable[lookup].equals(objectKey)) {
+                returnValue            = objectKeyTable[lookup];
                 objectKeyTable[lookup] = null;
 
                 hashIndex.unlinkNode(index, lastLookup, lookup);
@@ -743,6 +807,73 @@ public class BaseHashMap {
         return returnValue;
     }
 
+    /**
+     * For object sets using long key attribute of object for equality and
+     * hash
+     */
+    protected Object addOrRemoveObject(Object object, long longKey,
+                                       boolean remove) {
+
+        int    hash        = (int) longKey;
+        int    index       = hashIndex.getHashIndex(hash);
+        int    lookup      = hashIndex.getLookup(hash);
+        int    lastLookup  = -1;
+        Object returnValue = null;
+
+        for (; lookup >= 0;
+                lastLookup = lookup,
+                lookup = hashIndex.getNextLookup(lookup)) {
+            returnValue = objectKeyTable[lookup];
+
+            if (comparator.longKey(returnValue) == longKey) {
+                break;
+            }
+        }
+
+        if (lookup >= 0) {
+            if (remove) {
+                objectKeyTable[lookup] = null;
+
+                hashIndex.unlinkNode(index, lastLookup, lookup);
+
+                if (accessTable != null) {
+                    accessTable[lookup] = 0;
+                }
+
+                if (minimizeOnEmpty && hashIndex.elementCount == 0) {
+                    rehash(initialCapacity);
+                }
+            } else {
+                objectKeyTable[lookup] = object;
+
+                if (accessTable != null) {
+                    accessTable[lookup] = ++accessCount;
+                }
+            }
+
+            return returnValue;
+        } else if (remove) {
+            return null;
+        }
+
+        if (hashIndex.elementCount >= threshold) {
+            if (reset()) {
+                return addOrRemoveObject(object, longKey, remove);
+            } else {
+                throw new NoSuchElementException("BaseHashMap");
+            }
+        }
+
+        lookup                 = hashIndex.linkNode(index, lastLookup);
+        objectKeyTable[lookup] = object;
+
+        if (accessTable != null) {
+            accessTable[lookup] = ++accessCount;
+        }
+
+        return returnValue;
+    }
+
     protected boolean reset() {
 
         if (maxCapacity == 0 || maxCapacity > threshold) {
@@ -978,60 +1109,61 @@ public class BaseHashMap {
      */
     void removeFromElementArrays(int lookup) {
 
-        int arrayLength = hashIndex.linkTable.length;
+        // this is newNodePointer post-removal
+        int lastPointer = hashIndex.newNodePointer;
 
         if (isIntKey) {
             Object array = intKeyTable;
 
             System.arraycopy(array, lookup + 1, array, lookup,
-                             arrayLength - lookup - 1);
+                             lastPointer - lookup);
 
-            intKeyTable[arrayLength - 1] = 0;
+            intKeyTable[lastPointer] = 0;
         }
 
         if (isLongKey) {
             Object array = longKeyTable;
 
             System.arraycopy(array, lookup + 1, array, lookup,
-                             arrayLength - lookup - 1);
+                             lastPointer - lookup);
 
-            longKeyTable[arrayLength - 1] = 0;
+            longKeyTable[lastPointer] = 0;
         }
 
         if (isObjectKey || objectKeyTable != null) {
             Object array = objectKeyTable;
 
             System.arraycopy(array, lookup + 1, array, lookup,
-                             arrayLength - lookup - 1);
+                             lastPointer - lookup);
 
-            objectKeyTable[arrayLength - 1] = null;
+            objectKeyTable[lastPointer] = null;
         }
 
         if (isIntValue) {
             Object array = intValueTable;
 
             System.arraycopy(array, lookup + 1, array, lookup,
-                             arrayLength - lookup - 1);
+                             lastPointer - lookup);
 
-            intValueTable[arrayLength - 1] = 0;
+            intValueTable[lastPointer] = 0;
         }
 
         if (isLongValue) {
             Object array = longValueTable;
 
             System.arraycopy(array, lookup + 1, array, lookup,
-                             arrayLength - lookup - 1);
+                             lastPointer - lookup);
 
-            longValueTable[arrayLength - 1] = 0;
+            longValueTable[lastPointer] = 0;
         }
 
         if (isObjectValue) {
             Object array = objectValueTable;
 
             System.arraycopy(array, lookup + 1, array, lookup,
-                             arrayLength - lookup - 1);
+                             lastPointer - lookup);
 
-            objectValueTable[arrayLength - 1] = null;
+            objectValueTable[lastPointer] = null;
         }
     }
 
@@ -1205,6 +1337,10 @@ public class BaseHashMap {
         return hashIndex.elementCount == 0;
     }
 
+    protected void setComparator(ObjectComparator comparator) {
+        this.comparator = comparator;
+    }
+
     protected boolean containsKey(Object key) {
 
         if (key == null) {
@@ -1413,6 +1549,13 @@ public class BaseHashMap {
             this.keys = keys;
         }
 
+        public void reset() {
+
+            this.lookup  = -1;
+            this.counter = 0;
+            this.removed = false;
+        }
+
         public boolean hasNext() {
             return counter < hashIndex.elementCount;
         }
@@ -1517,6 +1660,7 @@ public class BaseHashMap {
 
             if (isList) {
                 removeRow(lookup);
+
                 lookup--;
             }
         }
