@@ -86,8 +86,8 @@ class JDBCBench {
 
                     if (DriverName.equals("org.hsqldb.jdbc.JDBCDriver")
                             || DriverName.equals("org.hsqldb.jdbcDriver")) {
+                        tableExtension = "CREATE CACHED TABLE ";
 
-                        tableExtension  = "CREATE CACHED TABLE ";
 //                        ShutdownCommand = "SHUTDOWN";
                     }
                 }
@@ -173,7 +173,7 @@ class JDBCBench {
             JDBCBench Me = new JDBCBench(DBUrl, DBUser, DBPassword,
                                          initialize_dataset);
         } catch (Exception E) {
-            System.out.println(E.getMessage());
+            System.out.println(E.toString());
             E.printStackTrace();
         }
     }
@@ -222,7 +222,7 @@ class JDBCBench {
             oneRound(url, user, password, transactions, true);
 */
         } catch (Exception E) {
-            System.out.println(E.getMessage());
+            System.out.println(E.toString());
             E.printStackTrace();
         } finally {
             MemoryWatcher.end();
@@ -337,8 +337,9 @@ class JDBCBench {
         System.out.println("Time to execute " + transaction_count
                            + " transactions: " + completion_time
                            + " seconds.");
-        System.out.println("Max/Min memory usage: " + MemoryWatcher.max
-                           + " / " + MemoryWatcher.min + " kb");
+        System.out.println("Max/Min memory usage: "
+                           + (MemoryWatcher.max / 1024) + " / "
+                           + (MemoryWatcher.min / 1024) + " kb");
         System.out.println(failed_transactions + " / " + transaction_count
                            + " failed to complete.");
 
@@ -348,7 +349,8 @@ class JDBCBench {
         System.out.println("Transaction rate: " + rate + " txn/sec.");
 
         if (TabFile != null) {
-            TabFile.print(MemoryWatcher.max + ";" + MemoryWatcher.min + ";"
+            TabFile.print((MemoryWatcher.max / 1024) + ";"
+                          + (MemoryWatcher.min / 1024) + ";"
                           + failed_transactions + ";" + rate + "\n");
         }
 
@@ -708,7 +710,7 @@ class JDBCBench {
 
             Stmt.close();
         } catch (Exception E) {
-            System.out.println(E.getMessage());
+            System.out.println(E.toString());
             E.printStackTrace();
         }
 
@@ -757,7 +759,7 @@ class JDBCBench {
 
             return conn;
         } catch (Exception E) {
-            System.out.println(E.getMessage());
+            System.out.println(E.toString());
             E.printStackTrace();
         }
 
@@ -773,7 +775,7 @@ class JDBCBench {
         try {
             c.close();
         } catch (Exception E) {
-            System.out.println(E.getMessage());
+            System.out.println(E.toString());
             E.printStackTrace();
         }
     }
@@ -896,7 +898,7 @@ class JDBCBench {
                     pstmt5 = Conn.prepareStatement(Query);
                 }
             } catch (Exception E) {
-                System.out.println(E.getMessage());
+                System.out.println(E.toString());
                 E.printStackTrace();
             }
         }
@@ -935,7 +937,7 @@ class JDBCBench {
                         pstmt5.close();
                     }
                 } catch (Exception E) {
-                    System.out.println(E.getMessage());
+                    System.out.println(E.toString());
                     E.printStackTrace();
                 }
             }
@@ -1045,7 +1047,7 @@ class JDBCBench {
             } catch (Exception E) {
                 if (verbose) {
                     System.out.println("Transaction failed: "
-                                       + E.getMessage());
+                                       + E.toString());
                     E.printStackTrace();
                 }
 
