@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,23 +31,25 @@
 
 package org.hsqldb.test;
 
+import java.io.CharArrayReader;
+import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.Reader;
 import java.sql.Blob;
+import java.sql.CallableStatement;
 import java.sql.Clob;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
-
 import javax.sql.rowset.serial.SerialBlob;
 
 import org.hsqldb.jdbc.JDBCBlob;
 import org.hsqldb.jdbc.JDBCClob;
-import org.hsqldb.lib.StopWatch;
 import org.hsqldb.lib.HsqlByteArrayInputStream;
+import org.hsqldb.lib.StopWatch;
 
 public class TestLobs extends TestBase {
 
@@ -57,13 +59,14 @@ public class TestLobs extends TestBase {
     public TestLobs(String name) {
 
         super(name);
-//        super(name, "jdbc:hsqldb:file:test3", false, false);
 
+//        super(name, "jdbc:hsqldb:file:test3", false, false);
 //        super(name, "jdbc:hsqldb:mem:test3", false, false);
     }
 
     protected void setUp() {
 
+        System.out.println("SetUp (sub-)test: " + getName());
         super.setUp();
 
         try {
@@ -76,10 +79,12 @@ public class TestLobs extends TestBase {
 
     public void testBlobA() {
 
+        System.out.println("Starting (sub-)test: " + getName());
+
         try {
             String ddl0 = "DROP TABLE BLOBTEST IF EXISTS";
             String ddl1 =
-                "CREATE TABLE BLOBTEST(ID IDENTITY, BLOBFIELD BLOB(1000))";
+                "CREATE TABLE BLOBTEST(ID IDENTITY, BLOBFIELD BLOB(100000))";
 
             statement.execute(ddl0);
             statement.execute(ddl1);
@@ -130,6 +135,8 @@ public class TestLobs extends TestBase {
 
     public void testBlobB() {
 
+        System.out.println("Starting (sub-)test: " + getName());
+
         ResultSet rs;
         byte[]    ba;
         byte[]    baR1 = new byte[] {
@@ -148,6 +155,7 @@ public class TestLobs extends TestBase {
 
             Statement st = connection.createStatement();
 
+            st.executeUpdate("DROP TABLE blo IF EXISTS");
             st.executeUpdate("CREATE TABLE blo (id INTEGER, b blob( 100))");
 
             PreparedStatement ps = connection.prepareStatement(
@@ -247,10 +255,12 @@ public class TestLobs extends TestBase {
 
     public void testClobA() {
 
+        System.out.println("Starting (sub-)test: " + getName());
+
         try {
             String ddl0 = "DROP TABLE CLOBTEST IF EXISTS";
             String ddl1 =
-                "CREATE TABLE CLOBTEST(ID IDENTITY, CLOBFIELD CLOB(1000))";
+                "CREATE TABLE CLOBTEST(ID IDENTITY, CLOBFIELD CLOB(100000))";
 
             statement.execute(ddl0);
             statement.execute(ddl1);
@@ -283,7 +293,8 @@ public class TestLobs extends TestBase {
 
             rs.next();
 
-            Clob clob2 = rs.getClob(2);
+            Clob   clob2  = rs.getClob(2);
+            String string = rs.getString(2);
             int data1 = clob1.getSubString(1, data.length()).indexOf("insert");
             int data2 = clob2.getSubString(1, data.length()).indexOf("INSERT");
 
@@ -296,10 +307,12 @@ public class TestLobs extends TestBase {
 
     public void testClobB() {
 
+        System.out.println("Starting (sub-)test: " + getName());
+
         try {
             String ddl0 = "DROP TABLE CLOBTEST IF EXISTS";
             String ddl1 =
-                "CREATE TABLE CLOBTEST(ID IDENTITY, V VARCHAR(10), I INT, CLOBFIELD CLOB(1000))";
+                "CREATE TABLE CLOBTEST(ID IDENTITY, V VARCHAR(10), I INT, CLOBFIELD CLOB(100000))";
 
             statement.execute(ddl0);
             statement.execute(ddl1);
@@ -350,7 +363,6 @@ public class TestLobs extends TestBase {
 
             try {
                 ps.setCharacterStream(3, reader, (int) clob3.length());
-                assertTrue(false);
                 ps.executeUpdate();
             } catch (SQLException e) {}
 
@@ -363,6 +375,8 @@ public class TestLobs extends TestBase {
 
     public void testClobC() {
 
+        System.out.println("Starting (sub-)test: " + getName());
+
         try {
             String ddl0 = "DROP TABLE VARIABLE IF EXISTS";
             String ddl1 =
@@ -453,11 +467,13 @@ public class TestLobs extends TestBase {
 
     public void testClobD() {
 
+        System.out.println("Starting (sub-)test: " + getName());
+
         try {
             String ddl0 = "DROP TABLE VARIABLE IF EXISTS";
             String ddl1 =
                 "CREATE TABLE VARIABLE (stateid varchar(128), varid numeric(16,0), "
-                + "scalabilitypassivated char(1) DEFAULT 'N', value clob(2000), scopeguid varchar(128),"
+                + "scalabilitypassivated char(1) DEFAULT 'N', value clob(20000), scopeguid varchar(128),"
                 + "primary key (stateid, varid, scalabilitypassivated, scopeguid))";
 
             statement.execute(ddl0);
@@ -522,6 +538,8 @@ public class TestLobs extends TestBase {
 
     public void testClobE() {
 
+        System.out.println("Starting (sub-)test: " + getName());
+
         try {
             String ddl0 = "DROP TABLE VARIABLE IF EXISTS";
             String ddl1 =
@@ -531,7 +549,9 @@ public class TestLobs extends TestBase {
 
             statement.execute(ddl0);
             statement.execute(ddl1);
-        } catch (SQLException e) {}
+        } catch (SQLException e) {
+            System.out.println(e.getMessage());
+        }
 
         try {
             String dml0 = "INSERT INTO VARIABLE VALUES (?, ?, 'N', ?, ?)";
@@ -612,51 +632,343 @@ public class TestLobs extends TestBase {
 
     public void testClobF() {
 
+        System.out.println("Starting (sub-)test: " + getName());
+
         try {
             String ddl0 = "DROP TABLE CLOBTEST IF EXISTS";
             String ddl1 =
-                "CREATE TABLE CLOBTEST(ID IDENTITY, CLOBFIELD CLOB(1000))";
+                "CREATE TABLE CLOBTEST(ID IDENTITY, CLOBFIELD CLOB(100000))";
 
             statement.execute(ddl0);
             statement.execute(ddl1);
         } catch (SQLException e) {}
 
         try {
-            String dml0  = "insert into clobtest(clobfield) values('";
-            String value = "0123456789";
+            String dml0 = "insert into clobtest(clobfield) values ?";
+            String            value = "0123456789";
+            PreparedStatement ps    = connection.prepareStatement(dml0);
 
-            dml0 = dml0 + value + "')";
-
-            String dql0 = "select CHARACTER_LENGTH(clobfield) from clobtest;";
-            PreparedStatement ps = connection.prepareStatement(dml0);
-
-            //ps.setClob(1, clob);
+            ps.setString(1, value);
             ps.executeUpdate();
-            ps.close();
 
-            ps = connection.prepareStatement(dql0);
-
-            final ResultSet rs = ps.executeQuery();
+            String dq1   = "select CHARACTER_LENGTH(clobfield) from clobtest;";
+            ResultSet rs = statement.executeQuery(dq1);
 
             rs.next();
 
-            final int length = rs.getInt(1);
+            int length = rs.getInt(1);
 
             assertTrue(value.length() == length);
+            rs.close();
+
+            String dq3 = "delete from clobtest;";
+
+            statement.execute(dq3);
+
+            char[] testChars = new char[11111];
+
+            for (int i = 0, j = 32; i < testChars.length; i++, j++) {
+                if (j > 255) {
+                    j = 32;
+                }
+
+                testChars[i] = (char) j;
+            }
+
+            ps.setCharacterStream(1, new CharArrayReader(testChars),
+                                  testChars.length);
+            ps.executeUpdate();
+
+            String dq2 = "select clobfield from clobtest;";
+
+            rs = statement.executeQuery(dq2);
+
+            rs.next();
+
+            Reader reader   = rs.getCharacterStream(1);
+            char[] newChars = new char[testChars.length];
+
+            try {
+                reader.read(newChars);
+            } catch (IOException e) {
+                fail("test failure");
+            }
+
+            for (int i = 0; i < testChars.length; i++) {
+                if (testChars[i] != newChars[i]) {
+                    fail("test failure");
+                }
+            }
         } catch (SQLException e) {
             e.printStackTrace();
             fail("test failure");
         }
     }
 
+    public void testClobG() {
+
+        System.out.println("Starting (sub-)test: " + getName());
+
+        try {
+            String ddl0 = "DROP TABLE CLOBTEST IF EXISTS";
+            String ddl1 =
+                "CREATE TABLE CLOBTEST(ID IDENTITY, CLOBFIELD CLOB(100000))";
+
+            statement.execute(ddl0);
+            statement.execute(ddl1);
+        } catch (SQLException e) {}
+
+        try {
+            String dml0 = "insert into clobtest(clobfield) values ?";
+            String            value = "0123456789";
+            PreparedStatement ps    = connection.prepareStatement(dml0);
+            Clob              clob  = connection.createClob();
+
+            clob.setString(1, value);
+            ps.setClob(1, clob);
+            ps.executeUpdate();
+
+            String dq1   = "select CHARACTER_LENGTH(clobfield) from clobtest;";
+            ResultSet rs = statement.executeQuery(dq1);
+
+            rs.next();
+
+            int length = rs.getInt(1);
+
+            assertTrue(value.length() == length);
+            rs.close();
+
+            String dq3 = "delete from clobtest;";
+
+            statement.execute(dq3);
+
+            char[] testChars = new char[11111];
+
+            for (int i = 0, j = 32; i < testChars.length; i++, j++) {
+                if (j > 255) {
+                    j = 32;
+                }
+
+                testChars[i] = (char) j;
+            }
+
+            ps.setCharacterStream(1, new CharArrayReader(testChars),
+                                  testChars.length);
+            ps.executeUpdate();
+
+            String dq2 = "select clobfield from clobtest;";
+
+            rs = statement.executeQuery(dq2);
+
+            rs.next();
+
+            Reader reader   = rs.getCharacterStream(1);
+            char[] newChars = new char[testChars.length];
+
+            try {
+                reader.read(newChars);
+            } catch (IOException e) {
+                fail("test failure");
+            }
+
+            for (int i = 0; i < testChars.length; i++) {
+                if (testChars[i] != newChars[i]) {
+                    fail("test failure");
+                }
+            }
+        } catch (SQLException e) {
+            e.printStackTrace();
+            fail("test failure");
+        }
+    }
+
+    public void testClobH() {
+
+        System.out.println("Starting (sub-)test: " + getName());
+
+        try {
+            String ddl1 =
+                "create procedure PUBLIC.PROC_A(out p1 clob, out p2 int) READS SQL DATA BEGIN ATOMIC SET p1 = 'dafsdfasdfaefafeajfiwejifpjajsidojfakmvkamsdjfadpsjfoajsdifjaos'; SET p2 = 0; end";
+            String dml0 = "call PUBLIC.PROC_A(?, ?)";
+
+            statement.execute(ddl1);
+
+            CallableStatement ps = connection.prepareCall(dml0);
+
+            ps.registerOutParameter(1, java.sql.Types.CLOB);
+            ps.registerOutParameter(2, java.sql.Types.INTEGER);
+            ps.execute();
+
+            String string = ps.getClob(1).getSubString(1, 10);
+
+            System.out.println(string);
+        } catch (SQLException e) {
+            e.printStackTrace();
+        }
+    }
+
+    public void testBlobH() {
+
+        System.out.println("Starting (sub-)test: " + getName());
+
+        try {
+            String ddl1 = "DROP TABLE BLOBTEST IF EXISTS";
+            String ddl2 = "CREATE TABLE BLOBTEST(A INTEGER, B BLOB)";
+
+            statement.execute(ddl1);
+            statement.execute(ddl2);
+        } catch (SQLException e) {}
+
+        try {
+            String            dml0 = "insert into blobtest values(1, ?)";
+            String            dql0 = "select * from blobtest";
+            PreparedStatement ps   = connection.prepareStatement(dml0);
+            byte[]            data = new byte[] {
+                1, 2, 3, 4, 5, 6, 7, 8, 9, 10
+            };
+
+            connection.setAutoCommit(false);
+
+            Blob blob = connection.createBlob();
+
+            blob.setBytes(1, data);
+            ps.setBlob(1, blob);
+            ps.executeUpdate();
+
+            data[4] = 50;
+            blob    = new JDBCBlob(data);
+
+            ps.setBlob(1, blob);
+            ps.executeUpdate();
+            ps.close();
+            connection.commit();
+
+            ps = connection.prepareStatement(dql0);
+
+            ResultSet rs = ps.executeQuery();
+
+            rs.next();
+
+            Blob blob1 = rs.getBlob(2);
+
+            rs.next();
+
+            Blob   blob2 = rs.getBlob(2);
+            byte[] data1 = blob1.getBytes(1, 10);
+            byte[] data2 = blob2.getBytes(1, 10);
+
+            assertTrue(data1[4] == 5 && data2[4] == 50);
+        } catch (SQLException e) {
+            e.printStackTrace();
+            fail("test failure");
+        }
+    }
+
+    public void testBlobI() {
+
+        System.out.println("Starting (sub-)test: " + getName());
+
+        try {
+            Statement st = connection.createStatement();
+
+            st.executeUpdate("drop table BLOBTEST if exists");
+            st.executeUpdate("create table BLOBTEST (BT_BLOB BLOB)");
+            System.out.println("Running insert...");
+
+            PreparedStatement insert =
+                connection.prepareStatement("insert into BLOBTEST values(?)");
+
+            insert.setBytes(1, new byte[]{});
+            insert.executeUpdate();
+            System.out.println("Running update...");
+
+            PreparedStatement update = connection.prepareStatement(
+                "update BLOBTEST set BT_BLOB = CONCAT(BT_BLOB,?)");
+
+            update.setBytes(1, new byte[] {
+                1, 2, 3
+            });
+            update.executeUpdate();
+            System.out.println("Running select...");
+
+            PreparedStatement select =
+                connection.prepareStatement("select BT_BLOB from BLOBTEST");
+            ResultSet result = select.executeQuery();
+
+            System.out.println("Results: " + result.getFetchSize());
+
+            while (result.next()) {
+                byte[] data = result.getBytes(1);
+
+                System.out.println("Result: " + data == null ? "null"
+                                                             : data.length);
+            }
+        } catch (SQLException e) {
+            e.printStackTrace();
+            fail("test failure");
+        }
+    }
+
+/*
+    public static void main(String[] args) throws SQLException {
+         Connection con = null;
+         PreparedStatement st = null;
+         ResultSet rs = null;
+         try {
+
+             DriverManager.registerDriver(new org.hsqldb.jdbc.JDBCDriver());
+             con = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost:7001",
+ "sa", null);
+//            con = DriverManager.getConnection("jdbc:hsqldb:mem:mymemdb", "sa", null);
+
+//            con.createStatement().execute("drop table aaa");
+             con.createStatement().execute("create table aaa (a integer, b blob)");
+
+             String insert = "insert into aaa values (1, ?)";
+
+             st = con.prepareStatement(insert);
+
+             byte[] b = {0, 1};
+
+             Blob blob = con.createBlob();
+             blob.setBytes(1, b);
+             st.setBlob(1, blob);
+
+//            this does not work too !!!
+//            st.setBytes(1, b);
+
+             System.out.println("execute");
+             st.execute();
+             System.out.println("execute end");
+             con.commit();
+             System.out.println("commit");
+
+             con.createStatement().execute("shutdown script");
+         } catch (SQLException e) {
+             e.printStackTrace();
+         } finally {
+             if (st != null) {
+                 st.close();
+             }
+             if (con != null) {
+                 con.close();
+             }
+         }
+     }
+*/
     protected void tearDown() {
 
+        System.out.println("TearDown (sub-)test: " + getName());
+
         try {
             statement = connection.createStatement();
 
             statement.execute("SHUTDOWN");
             statement.close();
-            connection.close();
+
+            if (!isNetwork) {
+                connection.close();
+            }
         } catch (Exception e) {}
 
         super.tearDown();
