@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -1703,7 +1703,7 @@ public class TestOdbcTypes extends AbstractTestOdbc {
     }
     */
 
-    static public void main(String[] sa) {
+    public static void main(String[] sa) {
         staticRunner(TestOdbcTypes.class, sa);
     }
 
