@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -76,7 +76,7 @@ import org.hsqldb.lib.StringConverter;
  *
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.0.1
  * @since 1.9.0
  */
 public class BinaryType extends Type {
@@ -186,6 +186,14 @@ public class BinaryType extends Type {
 
     public Type getAggregateType(Type other) {
 
+        if (other == null) {
+            return this;
+        }
+
+        if (other == SQL_ALL_TYPES) {
+            return this;
+        }
+
         if (typeCode == other.typeCode) {
             return precision >= other.precision ? this
                                                 : other;
@@ -197,9 +205,6 @@ public class BinaryType extends Type {
 
         switch (other.typeCode) {
 
-            case Types.SQL_ALL_TYPES :
-                return this;
-
             case Types.SQL_BIT :
             case Types.SQL_BIT_VARYING : {
                 long bytePrecision = (other.precision + 7) / 8;
@@ -239,7 +244,7 @@ public class BinaryType extends Type {
     /**
      * Returns type for concat
      */
-    public Type getCombinedType(Type other, int operation) {
+    public Type getCombinedType(Session session, Type other, int operation) {
 
         if (operation != OpTypes.CONCAT) {
             return getAggregateType(other);
@@ -323,6 +328,26 @@ public class BinaryType extends Type {
                 return 0;
             }
 
+            if (typeCode == Types.SQL_BINARY) {
+                if (data1.length > data2.length) {
+                    for (int i = data2.length; i < data1.length; i++) {
+                        if (data1[i] != 0) {
+                            return 1;
+                        }
+                    }
+
+                    return 0;
+                } else {
+                    for (int i = data1.length; i < data2.length; i++) {
+                        if (data2[i] != 0) {
+                            return -1;
+                        }
+                    }
+
+                    return 0;
+                }
+            }
+
             return data1.length > data2.length ? 1
                                                : -1;
         }
@@ -378,6 +403,9 @@ public class BinaryType extends Type {
         switch (otherType.typeCode) {
 
             // non-SQL feature, for compatibility with previous versions
+            // also used by HEXTORAW function
+            case Types.SQL_CLOB :
+                a = Type.SQL_VARCHAR.convertToType(session, a, otherType);
             case Types.SQL_VARCHAR :
             case Types.SQL_CHAR : {
                 b = session.getScanner().convertToBinary((String) a);
@@ -500,6 +528,55 @@ public class BinaryType extends Type {
                || otherType.isBinaryType() || otherType.isCharacterType();
     }
 
+    public int canMoveFrom(Type otherType) {
+
+        if (otherType == this) {
+            return 0;
+        }
+
+        if (!otherType.isBinaryType()) {
+            return -1;
+        }
+
+        switch (typeCode) {
+
+            case Types.SQL_VARBINARY : {
+                if (otherType.typeCode == typeCode) {
+                    return precision >= otherType.precision ? 0
+                                                            : 1;
+                }
+
+                if (otherType.typeCode == Types.SQL_BINARY) {
+                    return precision >= otherType.precision ? 0
+                                                            : -1;
+                }
+
+                return -1;
+            }
+            case Types.SQL_BLOB : {
+                if (otherType.typeCode == typeCode) {
+                    return precision >= otherType.precision ? 0
+                                                            : 1;
+                }
+
+                return -1;
+            }
+            case Types.SQL_BIT :
+            case Types.SQL_BINARY : {
+                return otherType.typeCode == typeCode
+                       && precision == otherType.precision ? 0
+                                                           : -1;
+            }
+            case Types.SQL_BIT_VARYING : {
+                return otherType.typeCode == typeCode
+                       && precision >= otherType.precision ? 0
+                                                           : -1;
+            }
+            default :
+                return -1;
+        }
+    }
+
     public long position(SessionInterface session, BlobData data,
                          BlobData otherData, Type otherType, long offset) {
 
@@ -574,7 +651,13 @@ public class BinaryType extends Type {
             return null;
         }
 
-        byte[] bytes    = ((BlobData) data).getBytes();
+        long length = ((BlobData) data).length(session);
+
+        if (length > Integer.MAX_VALUE) {
+            throw Error.error(ErrorCode.X_22027);
+        }
+
+        byte[] bytes    = ((BlobData) data).getBytes(session, 0, (int) length);
         int    endindex = bytes.length;
 
         if (trailing) {
@@ -677,9 +760,10 @@ public class BinaryType extends Type {
         if (typeCode == Types.SQL_BLOB) {
             BlobData blob = session.createBlob(length);
 
-            blob.setBytes(session, 0, ((BlobData) b).getBytes());
+            blob.setBytes(session, 0, ((BlobData) a), 0,
+                          ((BlobData) a).length(session));
             blob.setBytes(session, ((BlobData) a).length(session),
-                          ((BlobData) b).getBytes());
+                          ((BlobData) b), 0, ((BlobData) b).length(session));
 
             return blob;
         } else {
