@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,7 +54,7 @@ import org.hsqldb.store.BitMap;
  * string<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.1
  * @since 1.9.0
  */
 public final class BitType extends BinaryType {
@@ -120,6 +120,14 @@ public final class BitType extends BinaryType {
 
     public Type getAggregateType(Type other) {
 
+        if (other == null) {
+            return this;
+        }
+
+        if (other == SQL_ALL_TYPES) {
+            return this;
+        }
+
         if (typeCode == other.typeCode) {
             return precision >= other.precision ? this
                                                 : other;
@@ -127,9 +135,6 @@ public final class BitType extends BinaryType {
 
         switch (other.typeCode) {
 
-            case Types.SQL_ALL_TYPES :
-                return this;
-
             case Types.SQL_BIT :
                 return precision >= other.precision ? this
                                                     : getBitType(typeCode,
@@ -153,7 +158,7 @@ public final class BitType extends BinaryType {
     /**
      * Returns type for concat
      */
-    public Type getCombinedType(Type other, int operation) {
+    public Type getCombinedType(Session session, Type other, int operation) {
 
         if (operation != OpTypes.CONCAT) {
             return getAggregateType(other);
@@ -178,7 +183,7 @@ public final class BitType extends BinaryType {
             case Types.SQL_BINARY :
             case Types.SQL_VARBINARY :
             case Types.SQL_BLOB :
-                return other.getCombinedType(this, operation);
+                return other.getCombinedType(session, this, operation);
 
             default :
                 throw Error.error(ErrorCode.X_42562);
@@ -378,6 +383,10 @@ public final class BitType extends BinaryType {
         throw Error.error(ErrorCode.X_22501);
     }
 
+    public Object convertJavaToSQL(SessionInterface session, Object a) {
+        return convertToDefaultType(session, a);
+    }
+
     public String convertToString(Object a) {
 
         if (a == null) {
