@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,12 +50,17 @@ import org.hsqldb.lib.StringConverter;
  * Type subclass for DATE, TIME and TIMESTAMP.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 public final class DateTimeType extends DTIType {
 
     public final boolean withTimeZone;
+    private String       nameString;
+    public static final long epochSeconds =
+        HsqlDateTime.getDateSeconds("1-01-01");
+    public static final TimestampData epochTimestamp =
+        new TimestampData(epochSeconds);
 
     public DateTimeType(int typeGroup, int type, int scale) {
 
@@ -63,6 +68,7 @@ public final class DateTimeType extends DTIType {
 
         withTimeZone = type == Types.SQL_TIME_WITH_TIME_ZONE
                        || type == Types.SQL_TIMESTAMP_WITH_TIME_ZONE;
+        nameString = getNameStringPrivate();
     }
 
     public int displaySize() {
@@ -148,6 +154,10 @@ public final class DateTimeType extends DTIType {
     }
 
     public String getNameString() {
+        return nameString;
+    }
+
+    private String getNameStringPrivate() {
 
         switch (typeCode) {
 
@@ -175,10 +185,6 @@ public final class DateTimeType extends DTIType {
 
     public String getDefinition() {
 
-        if (scale == DTIType.defaultTimeFractionPrecision) {
-            return getNameString();
-        }
-
         String token;
 
         switch (typeCode) {
@@ -237,6 +243,14 @@ public final class DateTimeType extends DTIType {
 
     public Type getAggregateType(Type other) {
 
+        if (other == null) {
+            return this;
+        }
+
+        if (other == SQL_ALL_TYPES) {
+            return this;
+        }
+
         // DATE with DATE returned here
         if (typeCode == other.typeCode) {
             return scale >= other.scale ? this
@@ -282,7 +296,7 @@ public final class DateTimeType extends DTIType {
         return getDateTimeType(newType, scale);
     }
 
-    public Type getCombinedType(Type other, int operation) {
+    public Type getCombinedType(Session session, Type other, int operation) {
 
         switch (operation) {
 
@@ -339,6 +353,19 @@ public final class DateTimeType extends DTIType {
                         return getDateTimeType(typeCode, other.scale);
                     }
 
+                    return this;
+                } else if (other.isDateTimeType()) {
+                    if (operation == OpTypes.SUBTRACT) {
+                        if (other.typeComparisonGroup == typeComparisonGroup) {
+                            if (typeCode == Types.SQL_DATE) {
+                                return Type.SQL_INTERVAL_DAY_MAX_PRECISION;
+                            } else {
+                                return Type
+                                    .SQL_INTERVAL_SECOND_MAX_FRACTION_MAX_PRECISION;
+                            }
+                        }
+                    }
+                } else if (other.isNumberType()) {
                     return this;
                 }
                 break;
@@ -679,7 +706,11 @@ public final class DateTimeType extends DTIType {
     }
 
     public Object convertToDefaultType(SessionInterface session, Object a) {
-        throw Error.error(ErrorCode.X_42561);
+
+        Type otherType = a instanceof TimeData ? Type.SQL_TIME
+                                               : Type.SQL_TIMESTAMP;
+
+        return convertToType(session, a, otherType);
     }
 
     /** @todo - check the time zone conversion */
@@ -756,8 +787,10 @@ public final class DateTimeType extends DTIType {
                             session.getCalendar(),
                             ((java.util.Date) a).getTime());
                     } else {
-                        millis      = ((java.util.Date) a).getTime();
-                        zoneSeconds = session.getZoneSeconds();
+                        millis = ((java.util.Date) a).getTime();
+                        zoneSeconds =
+                            HsqlDateTime.getZoneMillis(
+                                session.getCalendar(), millis) / 1000;
                     }
 
                     if (a instanceof java.sql.Timestamp) {
@@ -776,12 +809,6 @@ public final class DateTimeType extends DTIType {
         throw Error.error(ErrorCode.X_42561);
     }
 
-    /*
-                return (Time) Type.SQL_TIME.getJavaDateObject(session, t, 0);
-        java.sql.Timestamp value = new java.sql.Timestamp(
-            (((TimestampData) o).getSeconds() - ((TimestampData) o)
-                .getZone()) * 1000);
-    */
     public Object convertSQLToJavaGMT(SessionInterface session, Object a) {
 
         long millis;
@@ -791,6 +818,7 @@ public final class DateTimeType extends DTIType {
             case Types.SQL_TIME :
             case Types.SQL_TIME_WITH_TIME_ZONE :
                 millis = ((TimeData) a).getSeconds() * 1000;
+                millis += ((TimeData) a).getNanos() / 1000000;
 
                 return new java.sql.Time(millis);
 
@@ -1001,12 +1029,35 @@ public final class DateTimeType extends DTIType {
         return true;
     }
 
+    public int canMoveFrom(Type otherType) {
+
+        if (otherType == this) {
+            return 0;
+        }
+
+        if (typeCode == otherType.typeCode) {
+            return scale >= otherType.scale ? 0
+                                            : -1;
+        }
+
+        return -1;
+    }
+
     public Object add(Object a, Object b, Type otherType) {
 
         if (a == null || b == null) {
             return null;
         }
 
+        if (otherType.isNumberType()) {
+            if (typeCode == Types.SQL_DATE) {
+                b = ((NumberType) otherType).floor(b);
+            }
+
+            b = Type.SQL_INTERVAL_SECOND_MAX_PRECISION.multiply(
+                IntervalSecondData.oneDay, b);
+        }
+
         switch (typeCode) {
 
             /** @todo -  range checks for units added */
@@ -1017,7 +1068,7 @@ public final class DateTimeType extends DTIType {
                                              "DateTimeType");
                 } else if (b instanceof IntervalSecondData) {
                     return addSeconds((TimeData) a,
-                                      (int) ((IntervalSecondData) b).units,
+                                      ((IntervalSecondData) b).units,
                                       ((IntervalSecondData) b).nanos);
                 }
                 break;
@@ -1030,7 +1081,7 @@ public final class DateTimeType extends DTIType {
                                      (int) ((IntervalMonthData) b).units);
                 } else if (b instanceof IntervalSecondData) {
                     return addSeconds((TimestampData) a,
-                                      (int) ((IntervalSecondData) b).units,
+                                      ((IntervalSecondData) b).units,
                                       ((IntervalSecondData) b).nanos);
                 }
                 break;
@@ -1047,6 +1098,15 @@ public final class DateTimeType extends DTIType {
             return null;
         }
 
+        if (otherType.isNumberType()) {
+            if (typeCode == Types.SQL_DATE) {
+                b = ((NumberType) otherType).floor(b);
+            }
+
+            b = Type.SQL_INTERVAL_SECOND_MAX_PRECISION.multiply(
+                IntervalSecondData.oneDay, b);
+        }
+
         switch (typeCode) {
 
             case Types.SQL_TIME_WITH_TIME_ZONE :
@@ -1056,7 +1116,7 @@ public final class DateTimeType extends DTIType {
                                              "DateTimeType");
                 } else if (b instanceof IntervalSecondData) {
                     return addSeconds((TimeData) a,
-                                      -(int) ((IntervalSecondData) b).units,
+                                      -((IntervalSecondData) b).units,
                                       -((IntervalSecondData) b).nanos);
                 }
                 break;
@@ -1069,7 +1129,7 @@ public final class DateTimeType extends DTIType {
                                      -(int) ((IntervalMonthData) b).units);
                 } else if (b instanceof IntervalSecondData) {
                     return addSeconds((TimestampData) a,
-                                      -(int) ((IntervalSecondData) b).units,
+                                      -((IntervalSecondData) b).units,
                                       -((IntervalSecondData) b).nanos);
                 }
                 break;
@@ -1080,6 +1140,69 @@ public final class DateTimeType extends DTIType {
         throw Error.runtimeError(ErrorCode.U_S0500, "DateTimeType");
     }
 
+    public Object truncate(Object a, int part) {
+
+        if (a == null) {
+            return null;
+        }
+
+        long millis = getMillis(a);
+
+        millis = HsqlDateTime.getTruncatedPart(millis, part);
+        millis -= getZoneMillis(a);
+
+        switch (typeCode) {
+
+            case Types.SQL_TIME_WITH_TIME_ZONE :
+                millis = HsqlDateTime.getNormalisedTime(millis);
+            case Types.SQL_TIME : {
+                return new TimeData((int) (millis / 1000), 0,
+                                    ((TimeData) a).getZone());
+            }
+            case Types.SQL_DATE :
+            case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
+            case Types.SQL_TIMESTAMP : {
+                return new TimestampData(millis / 1000, 0,
+                                         ((TimestampData) a).getZone());
+            }
+            default :
+        }
+
+        throw Error.runtimeError(ErrorCode.U_S0500, "DateTimeType");
+    }
+
+    public Object round(Object a, int part) {
+
+        if (a == null) {
+            return null;
+        }
+
+        long millis = getMillis(a);
+
+        millis = HsqlDateTime.getRoundedPart(millis, part);
+        millis -= getZoneMillis(a);
+
+        switch (typeCode) {
+
+            case Types.SQL_TIME_WITH_TIME_ZONE :
+            case Types.SQL_TIME : {
+                millis = HsqlDateTime.getNormalisedTime(millis);
+
+                return new TimeData((int) (millis / 1000), 0,
+                                    ((TimeData) a).getZone());
+            }
+            case Types.SQL_DATE :
+            case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
+            case Types.SQL_TIMESTAMP : {
+                return new TimestampData(millis / 1000, 0,
+                                         ((TimestampData) a).getZone());
+            }
+            default :
+        }
+
+        throw Error.runtimeError(ErrorCode.U_S0500, "DateTimeType");
+    }
+
     public boolean equals(Object other) {
 
         if (other instanceof Type) {
@@ -1174,6 +1297,14 @@ public final class DateTimeType extends DTIType {
                                          "DateTimeType - " + part);
         }
 
+        long millis = getMillis(dateTime);
+
+        return HsqlDateTime.getDateTimePart(millis, calendarPart) / divisor
+               + increment;
+    }
+
+    long getMillis(Object dateTime) {
+
         long millis;
 
         if (typeCode == Types.SQL_TIME
@@ -1187,8 +1318,21 @@ public final class DateTimeType extends DTIType {
                     .getSeconds() + ((TimestampData) dateTime).getZone()) * 1000;
         }
 
-        return HsqlDateTime.getDateTimePart(millis, calendarPart) / divisor
-               + increment;
+        return millis;
+    }
+
+    long getZoneMillis(Object dateTime) {
+
+        long millis;
+
+        if (typeCode == Types.SQL_TIME
+                || typeCode == Types.SQL_TIME_WITH_TIME_ZONE) {
+            millis = ((TimeData) dateTime).getZone() * 1000;
+        } else {
+            millis = ((TimestampData) dateTime).getZone() * 1000;
+        }
+
+        return millis;
     }
 
     public BigDecimal getSecondPart(Object dateTime) {
@@ -1269,28 +1413,28 @@ public final class DateTimeType extends DTIType {
     public DateTimeType getDateTimeTypeWithoutZone() {
 
         if (this.withTimeZone) {
+            DateTimeType type;
+
             switch (typeCode) {
 
                 case Types.SQL_TIME_WITH_TIME_ZONE :
-                    if (scale != DTIType.defaultTimeFractionPrecision) {
-                        return new DateTimeType(Types.SQL_TIME,
-                                                Types.SQL_TIME, scale);
-                    }
-
-                    return SQL_TIME;
+                    type = new DateTimeType(Types.SQL_TIME, Types.SQL_TIME,
+                                            scale);
+                    break;
 
                 case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
-                    if (scale != DTIType.defaultTimestampFractionPrecision) {
-                        return new DateTimeType(Types.SQL_TIMESTAMP,
-                                                Types.SQL_TIMESTAMP, scale);
-                    }
-
-                    return SQL_TIMESTAMP;
+                    type = new DateTimeType(Types.SQL_TIMESTAMP,
+                                            Types.SQL_TIMESTAMP, scale);
+                    break;
 
                 default :
                     throw Error.runtimeError(ErrorCode.U_S0500,
                                              "DateTimeType");
             }
+
+            type.nameString = nameString;
+
+            return type;
         }
 
         return this;
@@ -1308,32 +1452,32 @@ public final class DateTimeType extends DTIType {
                 return SQL_DATE;
 
             case Types.SQL_TIME :
-                if (scale != DTIType.defaultTimeFractionPrecision) {
-                    return new DateTimeType(Types.SQL_TIME, type, scale);
+                if (scale == DTIType.defaultTimeFractionPrecision) {
+                    return SQL_TIME;
                 }
 
-                return SQL_TIME;
+                return new DateTimeType(Types.SQL_TIME, type, scale);
 
             case Types.SQL_TIME_WITH_TIME_ZONE :
-                if (scale != DTIType.defaultTimeFractionPrecision) {
-                    return new DateTimeType(Types.SQL_TIME, type, scale);
+                if (scale == DTIType.defaultTimeFractionPrecision) {
+                    return SQL_TIME_WITH_TIME_ZONE;
                 }
 
-                return SQL_TIME_WITH_TIME_ZONE;
+                return new DateTimeType(Types.SQL_TIME, type, scale);
 
             case Types.SQL_TIMESTAMP :
-                if (scale != DTIType.defaultTimestampFractionPrecision) {
-                    return new DateTimeType(Types.SQL_TIMESTAMP, type, scale);
+                if (scale == DTIType.defaultTimestampFractionPrecision) {
+                    return SQL_TIMESTAMP;
                 }
 
-                return SQL_TIMESTAMP;
+                return new DateTimeType(Types.SQL_TIMESTAMP, type, scale);
 
             case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
-                if (scale != DTIType.defaultTimestampFractionPrecision) {
-                    return new DateTimeType(Types.SQL_TIMESTAMP, type, scale);
+                if (scale == DTIType.defaultTimestampFractionPrecision) {
+                    return SQL_TIMESTAMP_WITH_TIME_ZONE;
                 }
 
-                return SQL_TIMESTAMP_WITH_TIME_ZONE;
+                return new DateTimeType(Types.SQL_TIMESTAMP, type, scale);
 
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500, "DateTimeType");
@@ -1426,7 +1570,7 @@ public final class DateTimeType extends DTIType {
             b[1] = b[0];
         }
 
-        Type commonType = ta[0].getCombinedType(tb[0], OpTypes.EQUAL);
+        Type commonType = ta[0].getCombinedType(session, tb[0], OpTypes.EQUAL);
 
         a[0] = commonType.castToType(session, a[0], ta[0]);
         b[0] = commonType.castToType(session, b[0], tb[0]);
@@ -1509,18 +1653,17 @@ public final class DateTimeType extends DTIType {
                 }
 
                 months += years * 12;
+            }
 
-                if (negate) {
-                    months = -months;
-                }
+            if (negate) {
+                months = -months;
             }
 
             return months;
         }
     }
 
-    /** @todo - overflow */
-    public static TimeData addSeconds(TimeData source, int seconds,
+    public static TimeData addSeconds(TimeData source, long seconds,
                                       int nanos) {
 
         nanos   += source.getNanos();
@@ -1536,7 +1679,7 @@ public final class DateTimeType extends DTIType {
         seconds += source.getSeconds();
         seconds %= (24 * 60 * 60);
 
-        TimeData ti = new TimeData(seconds, nanos, source.getZone());
+        TimeData ti = new TimeData((int) seconds, nanos, source.getZone());
 
         return ti;
     }
@@ -1559,8 +1702,7 @@ public final class DateTimeType extends DTIType {
         }
     }
 
-    /** @todo - overflow */
-    public static TimestampData addSeconds(TimestampData source, int seconds,
+    public static TimestampData addSeconds(TimestampData source, long seconds,
                                            int nanos) {
 
         nanos   += source.getNanos();
