@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,7 @@ import org.hsqldb.store.ValuePool;
  * Type subclass for all NUMBER types.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.6
  * @since 1.9.0
  */
 public final class NumberType extends Type {
@@ -57,8 +57,8 @@ public final class NumberType extends Type {
     static final int        integerPrecision             = 10;
     static final int        bigintPrecision              = 19;
     static final int        doublePrecision              = 0;
-    public static final int defaultNumericPrecision      = 100;
-    public static final int defaultNumericScale          = 10;
+    public static final int defaultNumericPrecision      = 128;
+    public static final int defaultNumericScale          = 32;
     public static final int maxNumericPrecision          = Integer.MAX_VALUE;
     static final int        bigintSquareNumericPrecision = 40;
 
@@ -157,6 +157,37 @@ public final class NumberType extends Type {
         }
     }
 
+    /**
+     * Returns decimal precision.
+     */
+    public int getDecimalPrecision() {
+
+        switch (typeCode) {
+
+            case Types.TINYINT :
+                return tinyintPrecision;
+
+            case Types.SQL_SMALLINT :
+                return smallintPrecision;
+
+            case Types.SQL_INTEGER :
+                return integerPrecision;
+
+            case Types.SQL_BIGINT :
+                return bigintPrecision;
+
+            case Types.SQL_REAL :
+            case Types.SQL_FLOAT :
+            case Types.SQL_DOUBLE :
+            case Types.SQL_NUMERIC :
+            case Types.SQL_DECIMAL :
+                return displaySize() - 1;
+
+            default :
+                throw Error.runtimeError(ErrorCode.U_S0500, "NumberType");
+        }
+    }
+
     public int displaySize() {
 
         switch (typeCode) {
@@ -256,6 +287,10 @@ public final class NumberType extends Type {
         }
     }
 
+    public int getJDBCPrecision() {
+        return getPrecision();
+    }
+
     public String getNameString() {
 
         switch (typeCode) {
@@ -428,6 +463,19 @@ public final class NumberType extends Type {
         }
     }
 
+    public boolean isDecimalType() {
+
+        switch (typeCode) {
+
+            case Types.SQL_NUMERIC :
+            case Types.SQL_DECIMAL :
+                return true;
+
+            default :
+                return false;
+        }
+    }
+
     public int getNominalWidth() {
         return typeWidth;
     }
@@ -445,6 +493,14 @@ public final class NumberType extends Type {
 
     public Type getAggregateType(Type other) {
 
+        if (other == null) {
+            return this;
+        }
+
+        if (other == SQL_ALL_TYPES) {
+            return this;
+        }
+
         if (this == other) {
             return this;
         }
@@ -455,9 +511,6 @@ public final class NumberType extends Type {
 
         switch (other.typeCode) {
 
-            case Types.SQL_ALL_TYPES :
-                return this;
-
             case Types.SQL_REAL :
             case Types.SQL_FLOAT :
             case Types.SQL_DOUBLE :
@@ -526,7 +579,7 @@ public final class NumberType extends Type {
      *  For ADD/SUBTRACT/DIVIDE, the scale is the larger of the two<br>
      *  For MULTIPLY, the scale is the sum of the two scales<br>
      */
-    public Type getCombinedType(Type other, int operation) {
+    public Type getCombinedType(Session session, Type other, int operation) {
 
         if (other.typeCode == Types.SQL_ALL_TYPES) {
             other = this;
@@ -535,18 +588,16 @@ public final class NumberType extends Type {
         switch (operation) {
 
             case OpTypes.ADD :
+            case OpTypes.DIVIDE :
                 break;
 
             case OpTypes.MULTIPLY :
                 if (other.isIntervalType()) {
-                    return other.getCombinedType(this, OpTypes.MULTIPLY);
+                    return other.getCombinedType(session, this,
+                                                 OpTypes.MULTIPLY);
                 }
                 break;
 
-            case OpTypes.DIVIDE :
-                if (typeWidth == DECIMAL_WIDTH) {
-                    break;
-                }
             case OpTypes.SUBTRACT :
             default :
 
@@ -564,14 +615,16 @@ public final class NumberType extends Type {
             return Type.SQL_DOUBLE;
         }
 
-        int sum = typeWidth + ((NumberType) other).typeWidth;
+        if (operation != OpTypes.DIVIDE || session.database.sqlAvgScale == 0) {
+            int sum = typeWidth + ((NumberType) other).typeWidth;
 
-        if (sum <= INTEGER_WIDTH) {
-            return Type.SQL_INTEGER;
-        }
+            if (sum <= INTEGER_WIDTH) {
+                return Type.SQL_INTEGER;
+            }
 
-        if (sum <= BIGINT_WIDTH) {
-            return Type.SQL_BIGINT;
+            if (sum <= BIGINT_WIDTH) {
+                return Type.SQL_BIGINT;
+            }
         }
 
         int  newScale;
@@ -593,6 +646,10 @@ public final class NumberType extends Type {
                 newDigits = precision - scale + other.scale;
                 newScale  = scale > other.scale ? scale
                                                 : other.scale;
+
+                if (session.database.sqlAvgScale > newScale) {
+                    newScale = session.database.sqlAvgScale;
+                }
                 break;
 
             case OpTypes.MULTIPLY :
@@ -689,6 +746,15 @@ public final class NumberType extends Type {
                 double ad = ((Number) a).doubleValue();
                 double bd = ((Number) b).doubleValue();
 
+                if (Double.isNaN(ad)) {
+                    return Double.isNaN(bd) ? 0
+                                            : -1;
+                }
+
+                if (Double.isNaN(bd)) {
+                    return 1;
+                }
+
                 return (ad > bd) ? 1
                                  : (bd > ad ? -1
                                             : 0);
@@ -759,13 +825,19 @@ public final class NumberType extends Type {
             switch (typeCode) {
 
                 case Types.SQL_NUMERIC :
-                case Types.SQL_DECIMAL :
-                    if (otherType.scale == scale
-                            && otherType.precision <= precision) {
-                        return a;
-                    }
-                    break;
+                case Types.SQL_DECIMAL : {
+                    BigDecimal dec = (BigDecimal) a;
 
+                    if (scale != dec.scale()) {
+                        dec = dec.setScale(scale, BigDecimal.ROUND_HALF_DOWN);
+                    }
+
+                    if (JavaSystem.precision(dec) > precision) {
+                        throw Error.error(ErrorCode.X_22003);
+                    }
+
+                    return dec;
+                }
                 default :
                     return a;
             }
@@ -843,10 +915,10 @@ public final class NumberType extends Type {
             case Types.TINYINT :
             case Types.SQL_SMALLINT :
             case Types.SQL_INTEGER :
-                return convertToInt(a, this.typeCode);
+                return convertToInt(session, a, this.typeCode);
 
             case Types.SQL_BIGINT :
-                return convertToLong(a);
+                return convertToLong(session, a);
 
             case Types.SQL_REAL :
             case Types.SQL_FLOAT :
@@ -855,7 +927,27 @@ public final class NumberType extends Type {
 
             case Types.SQL_NUMERIC :
             case Types.SQL_DECIMAL :
-                BigDecimal value = convertToDecimal(a);
+                BigDecimal value = null;
+
+                if (scale == 0 && a instanceof Double) {
+                    double d = ((Number) a).doubleValue();
+
+                    if (session instanceof Session) {
+                        if (!((Session) session).database.sqlConvertTruncate) {
+                            d = java.lang.Math.rint(d);
+                        }
+                    }
+
+                    if (Double.isInfinite(d) || Double.isNaN(d)) {
+                        throw Error.error(ErrorCode.X_22003);
+                    }
+
+                    value = BigDecimal.valueOf(d);
+                }
+
+                if (value == null) {
+                    value = convertToDecimal(a);
+                }
 
                 return convertToTypeLimits(session, value);
 
@@ -915,15 +1007,41 @@ public final class NumberType extends Type {
             } else if (a instanceof Double) {
                 otherType = Type.SQL_DOUBLE;
             } else if (a instanceof BigDecimal) {
-                if (typeCode == Types.SQL_DECIMAL
-                        || typeCode == Types.SQL_NUMERIC) {
-                    return a;
-                }
-
                 otherType = Type.SQL_DECIMAL_DEFAULT;
             } else {
                 throw Error.error(ErrorCode.X_42561);
             }
+
+            switch (typeCode) {
+
+                case Types.TINYINT :
+                case Types.SQL_SMALLINT :
+                case Types.SQL_INTEGER :
+                    return convertToInt(session, a, Types.INTEGER);
+
+                case Types.SQL_BIGINT :
+                    return convertToLong(session, a);
+
+                case Types.SQL_REAL :
+                case Types.SQL_FLOAT :
+                case Types.SQL_DOUBLE :
+                    return convertToDouble(a);
+
+                case Types.SQL_NUMERIC :
+                case Types.SQL_DECIMAL : {
+                    a = convertToDecimal(a);
+
+                    BigDecimal dec = (BigDecimal) a;
+
+                    if (scale != dec.scale()) {
+                        dec = dec.setScale(scale, BigDecimal.ROUND_HALF_DOWN);
+                    }
+
+                    return dec;
+                }
+                default :
+                    throw Error.error(ErrorCode.X_42561);
+            }
         } else if (a instanceof String) {
             otherType = Type.SQL_VARCHAR;
         } else {
@@ -933,6 +1051,10 @@ public final class NumberType extends Type {
         return convertToType(session, a, otherType);
     }
 
+    public Object convertJavaToSQL(SessionInterface session, Object a) {
+        return convertToDefaultType(session, a);
+    }
+
     /**
      * Type narrowing from DOUBLE/DECIMAL/NUMERIC to BIGINT / INT / SMALLINT / TINYINT
      * following SQL rules. When conversion is from a non-integral type,
@@ -943,7 +1065,7 @@ public final class NumberType extends Type {
      * Converter from a numeric object to Integer. Input is checked to be
      * within range represented by the given number type.
      */
-    static Integer convertToInt(Object a, int type) {
+    static Integer convertToInt(SessionInterface session, Object a, int type) {
 
         int value;
 
@@ -972,6 +1094,12 @@ public final class NumberType extends Type {
         } else if (a instanceof Double || a instanceof Float) {
             double d = ((Number) a).doubleValue();
 
+            if (session instanceof Session) {
+                if (!((Session) session).database.sqlConvertTruncate) {
+                    d = java.lang.Math.rint(d);
+                }
+            }
+
             if (Double.isInfinite(d) || Double.isNaN(d)
                     || d >= (double) Integer.MAX_VALUE + 1
                     || d <= (double) Integer.MIN_VALUE - 1) {
@@ -993,14 +1121,14 @@ public final class NumberType extends Type {
             }
         }
 
-        return ValuePool.getInt(value);
+        return Integer.valueOf(value);
     }
 
     /**
      * Converter from a numeric object to Long. Input is checked to be
      * within range represented by Long.
      */
-    static Long convertToLong(Object a) {
+    static Long convertToLong(SessionInterface session, Object a) {
 
         if (a instanceof Integer) {
             return ValuePool.getLong(((Integer) a).intValue());
@@ -1017,6 +1145,12 @@ public final class NumberType extends Type {
         } else if (a instanceof Double || a instanceof Float) {
             double d = ((Number) a).doubleValue();
 
+            if (session instanceof Session) {
+                if (!((Session) session).database.sqlConvertTruncate) {
+                    d = java.lang.Math.rint(d);
+                }
+            }
+
             if (Double.isInfinite(d) || Double.isNaN(d)
                     || d >= (double) Long.MAX_VALUE + 1
                     || d <= (double) Long.MIN_VALUE - 1) {
@@ -1093,10 +1227,10 @@ public final class NumberType extends Type {
             double value = ((Number) a).doubleValue();
 
             if (Double.isInfinite(value) || Double.isNaN(value)) {
-                return null;
+                throw Error.error(ErrorCode.X_22003);
             }
 
-            return new BigDecimal(value);
+            return BigDecimal.valueOf(value);
         } else {
             throw Error.runtimeError(ErrorCode.U_S0500, "NumberType");
         }
@@ -1185,6 +1319,69 @@ public final class NumberType extends Type {
         return false;
     }
 
+    public int canMoveFrom(Type otherType) {
+
+        if (otherType == this) {
+            return 0;
+        }
+
+        switch (typeCode) {
+
+            case Types.TINYINT :
+                if (otherType.typeCode == Types.SQL_SMALLINT
+                        || otherType.typeCode == Types.SQL_INTEGER) {
+                    return 1;
+                }
+                break;
+
+            case Types.SQL_SMALLINT :
+                if (otherType.typeCode == Types.TINYINT) {
+                    return 0;
+                }
+
+                if (otherType.typeCode == Types.SQL_INTEGER) {
+                    return 1;
+                }
+                break;
+
+            case Types.SQL_INTEGER :
+                if (otherType.typeCode == Types.SQL_SMALLINT
+                        || otherType.typeCode == Types.TINYINT) {
+                    return 0;
+                }
+                break;
+
+            case Types.SQL_BIGINT :
+                break;
+
+            case Types.SQL_DECIMAL :
+            case Types.SQL_NUMERIC :
+                if (otherType.typeCode == Types.SQL_DECIMAL
+                        || otherType.typeCode == Types.SQL_NUMERIC) {
+                    if (scale == otherType.scale) {
+                        if (precision >= otherType.precision) {
+                            return 0;
+                        } else {
+                            return 1;
+                        }
+                    }
+                }
+                break;
+
+            case Types.SQL_REAL :
+            case Types.SQL_FLOAT :
+            case Types.SQL_DOUBLE :
+                if (otherType.typeCode == Types.SQL_REAL
+                        || otherType.typeCode == Types.SQL_FLOAT
+                        || otherType.typeCode == Types.SQL_DOUBLE) {
+                    return 0;
+                }
+            default :
+        }
+
+        return -1;
+    }
+
     public int compareToTypeRange(Object o) {
 
         if (!(o instanceof Number)) {
@@ -1404,7 +1601,7 @@ public final class NumberType extends Type {
         }
     }
 
-    public Object divide(Object a, Object b) {
+    public Object divide(Session session, Object a, Object b) {
 
         if (a == null || b == null) {
             return null;
@@ -1418,7 +1615,9 @@ public final class NumberType extends Type {
                 double ad = ((Number) a).doubleValue();
                 double bd = ((Number) b).doubleValue();
 
-                if (bd == 0) {
+                if (bd == 0
+                        && (session == null
+                            || session.database.sqlDoubleNaN)) {
                     throw Error.error(ErrorCode.X_22012);
                 }
 
@@ -1478,7 +1677,7 @@ public final class NumberType extends Type {
             throw Error.error(ErrorCode.X_42561);
         }
 
-        Object temp = divide(a, b);
+        Object temp = divide(null, a, b);
 
         temp = multiply(temp, b);
         temp = convertToDefaultType(null, temp);
@@ -1798,15 +1997,45 @@ public final class NumberType extends Type {
             return null;
         }
 
-        if (s >= scale) {
-            return a;
+        BigDecimal dec = convertToDecimal(a);
+
+        dec = dec.setScale(s, BigDecimal.ROUND_DOWN);
+
+        if (typeCode == Types.SQL_DECIMAL || typeCode == Types.SQL_NUMERIC) {
+            dec = dec.setScale(scale, BigDecimal.ROUND_DOWN);
+        }
+
+        a = convertToDefaultType(null, dec);
+
+        return convertToTypeLimits(null, a);
+    }
+
+    public Object round(Object a, int s) {
+
+        if (a == null) {
+            return null;
         }
 
         BigDecimal dec = convertToDecimal(a);
 
-        dec = dec.setScale(s, BigDecimal.ROUND_DOWN);
-        dec = dec.setScale(scale, BigDecimal.ROUND_DOWN);
-        a   = convertToDefaultType(null, dec);
+        switch (typeCode) {
+
+            case Types.SQL_DOUBLE : {
+                dec = dec.setScale(s, BigDecimal.ROUND_HALF_EVEN);
+
+                break;
+            }
+            case Types.SQL_DECIMAL :
+            case Types.SQL_NUMERIC :
+            default : {
+                dec = dec.setScale(s, BigDecimal.ROUND_HALF_UP);
+                dec = dec.setScale(scale, BigDecimal.ROUND_DOWN);
+
+                break;
+            }
+        }
+
+        a = convertToDefaultType(null, dec);
 
         return convertToTypeLimits(null, a);
     }
