@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,23 +36,26 @@ import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.SchemaObject;
 import org.hsqldb.Session;
 import org.hsqldb.SessionInterface;
+import org.hsqldb.SortAndSlice;
+import org.hsqldb.Tokens;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
+import org.hsqldb.lib.IntKeyHashMap;
 import org.hsqldb.lib.IntValueHashMap;
+import org.hsqldb.lib.ObjectComparator;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.rights.Grantee;
-import org.hsqldb.store.ValuePool;
 
 /**
  * Base class for type objects.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 1.9.0
+ * @version 2.2.9
  * @since 1.9.0
  */
 public abstract class Type implements SchemaObject, Cloneable {
 
-    public final static Type[] emptyArray = new Type[]{};
+    public static final Type[] emptyArray = new Type[]{};
 
     //
     public final int        typeComparisonGroup;
@@ -61,9 +64,6 @@ public abstract class Type implements SchemaObject, Cloneable {
     public final int        scale;
     public UserTypeModifier userTypeModifier;
 
-    //
-    public final static int defaultArrayCardinality = 1024;
-
     //
     Type(int typeGroup, int type, long precision, int scale) {
 
@@ -191,9 +191,8 @@ public abstract class Type implements SchemaObject, Cloneable {
 
     public abstract Class getJDBCClass();
 
-    public Integer getJDBCScale() {
-        return acceptsScale() ? ValuePool.getInt(scale)
-                              : null;
+    public int getJDBCScale() {
+        return scale;
     }
 
     public int getJDBCPrecision() {
@@ -225,7 +224,23 @@ public abstract class Type implements SchemaObject, Cloneable {
     /**
      * Returns the full definition of the type, including parameters
      */
-    abstract String getDefinition();
+    public abstract String getDefinition();
+
+    public boolean hasCollation() {
+        return false;
+    }
+
+    public String getCollationDefinition() {
+        return "";
+    }
+
+    public Collation getCollation() {
+        return null;
+    }
+
+    public Charset getCharacterSet() {
+        return null;
+    }
 
     public final String getTypeDefinition() {
 
@@ -238,6 +253,29 @@ public abstract class Type implements SchemaObject, Cloneable {
 
     public abstract int compare(Session session, Object a, Object b);
 
+    public int compare(Session session, Object a, Object b,
+                       SortAndSlice sort) {
+
+        if (a == b) {
+            return 0;
+        }
+
+        if (a == null) {
+            return sort.sortNullsLast[0] ? 1
+                                         : -1;
+        }
+
+        if (b == null) {
+            return sort.sortNullsLast[0] ? -1
+                                         : 1;
+        }
+
+        int result = compare(session, a, b);
+
+        return sort.sortDescending[0] ? -result
+                                      : result;
+    }
+
     public abstract Object convertToTypeLimits(SessionInterface session,
             Object a);
 
@@ -293,11 +331,27 @@ public abstract class Type implements SchemaObject, Cloneable {
 
     public abstract boolean canConvertFrom(Type otherType);
 
+    /**
+     * Can convert without changing the object
+     * @return 0 always, 1 range check required, -1 never
+     */
+    public int canMoveFrom(Type otherType) {
+
+        if (otherType == this) {
+            return 0;
+        }
+
+        return -1;
+    }
+
     public boolean canBeAssignedFrom(Type otherType) {
 
-        return otherType == null ? true
-                                 : this.typeComparisonGroup
-                                   == otherType.typeComparisonGroup;
+        if (otherType == null) {
+            return true;
+        }
+
+        return otherType.typeCode == Types.SQL_ALL_TYPES
+               || typeComparisonGroup == otherType.typeComparisonGroup;
     }
 
     public int arrayLimitCardinality() {
@@ -340,6 +394,10 @@ public abstract class Type implements SchemaObject, Cloneable {
         return false;
     }
 
+    public boolean isDecimalType() {
+        return false;
+    }
+
     public boolean isDateTimeType() {
         return false;
     }
@@ -417,6 +475,11 @@ public abstract class Type implements SchemaObject, Cloneable {
     public int precedenceDegree(Type other) {
 
         if (other.typeCode == typeCode) {
+            if (typeCode == Types.SQL_ARRAY) {
+                return collectionBaseType().precedenceDegree(
+                    other.collectionBaseType());
+            }
+
             return 0;
         }
 
@@ -434,7 +497,8 @@ public abstract class Type implements SchemaObject, Cloneable {
      * other type is not allways comparable with this, but a operation should
      * be valid without any explicit CAST
      */
-    public abstract Type getCombinedType(Type other, int operation);
+    public abstract Type getCombinedType(Session session, Type other,
+                                         int operation);
 
     public int compareToTypeRange(Object o) {
         return 0;
@@ -463,7 +527,7 @@ public abstract class Type implements SchemaObject, Cloneable {
         throw Error.runtimeError(ErrorCode.U_S0500, "Type");
     }
 
-    public Object divide(Object a, Object b) {
+    public Object divide(Session session, Object a, Object b) {
         throw Error.runtimeError(ErrorCode.U_S0500, "Type");
     }
 
@@ -475,6 +539,15 @@ public abstract class Type implements SchemaObject, Cloneable {
         return 0;
     }
 
+    int hashCode(Object a) {
+
+        if (a == null) {
+            return 0;
+        }
+
+        return a.hashCode();
+    }
+
     public boolean equals(Object other) {
 
         if (other == this) {
@@ -482,6 +555,14 @@ public abstract class Type implements SchemaObject, Cloneable {
         }
 
         if (other instanceof Type) {
+            if (((Type) other).typeCode == Types.SQL_ARRAY) {
+                return false;
+            }
+
+            if (((Type) other).typeCode == Types.SQL_ROW) {
+                return false;
+            }
+
             return ((Type) other).typeCode == typeCode
                    && ((Type) other).precision == precision
                    && ((Type) other).scale == scale
@@ -495,6 +576,39 @@ public abstract class Type implements SchemaObject, Cloneable {
         return typeCode + (int) precision << 8 + scale << 16;
     }
 
+    public static TypedComparator newComparator(Session session) {
+        return new TypedComparator(session);
+    }
+
+    public static class TypedComparator implements ObjectComparator {
+
+        Session      session;
+        Type         type;
+        SortAndSlice sort;
+
+        TypedComparator(Session session) {
+            this.session = session;
+            this.sort    = sort;
+        }
+
+        public int compare(Object a, Object b) {
+            return type.compare(session, a, b, sort);
+        }
+
+        public int hashCode(Object a) {
+            return type.hashCode(a);
+        }
+
+        public long longKey(Object a) {
+            return 0;
+        }
+
+        public void setType(Type type, SortAndSlice sort) {
+            this.type = type;
+            this.sort = sort;
+        }
+    }
+
     /** @todo 1.9.0 - review all needs max implementation defined lengths, used for parameters */
 
     // null type
@@ -503,20 +617,22 @@ public abstract class Type implements SchemaObject, Cloneable {
     // character types
     public static final CharacterType SQL_CHAR =
         new CharacterType(Types.SQL_CHAR, 1);
+    public static final CharacterType SQL_CHAR_16 =
+        new CharacterType(Types.SQL_CHAR, 16);
     public static final CharacterType SQL_CHAR_DEFAULT =
         new CharacterType(Types.SQL_CHAR, CharacterType.defaultCharPrecision);
     public static final CharacterType SQL_VARCHAR =
         new CharacterType(Types.SQL_VARCHAR, 0);
     public static final CharacterType SQL_VARCHAR_DEFAULT =
         new CharacterType(Types.SQL_VARCHAR,
-                          CharacterType.defaultCharPrecision);
+                          CharacterType.defaultVarcharPrecision);
     public static final ClobType SQL_CLOB =
         new ClobType(ClobType.defaultClobSize);
     public static final CharacterType VARCHAR_IGNORECASE =
         new CharacterType(Types.VARCHAR_IGNORECASE, 0);
     public static final CharacterType VARCHAR_IGNORECASE_DEFAULT =
         new CharacterType(Types.VARCHAR_IGNORECASE,
-                          CharacterType.defaultCharPrecision);
+                          CharacterType.defaultVarcharPrecision);
 
     // binary types
     public static final BitType SQL_BIT = new BitType(Types.SQL_BIT, 1);
@@ -528,6 +644,8 @@ public abstract class Type implements SchemaObject, Cloneable {
     // binary types
     public static final BinaryType SQL_BINARY =
         new BinaryType(Types.SQL_BINARY, 1);
+    public static final BinaryType SQL_BINARY_16 =
+        new BinaryType(Types.SQL_BINARY, 16);
     public static final BinaryType SQL_BINARY_DEFAULT =
         new BinaryType(Types.SQL_BINARY, 32 * 1024);
     public static final BinaryType SQL_VARBINARY =
@@ -662,20 +780,29 @@ public abstract class Type implements SchemaObject, Cloneable {
                                      DTIType.maxIntervalPrecision,
                                      DTIType.maxFractionPrecision);
 
+    //
+    public static final IntervalType SQL_INTERVAL_YEAR_TO_MONTH_MAX_PRECISION =
+        IntervalType.newIntervalType(Types.SQL_INTERVAL_YEAR_TO_MONTH,
+                                     DTIType.maxIntervalPrecision, 0);
+    public static final IntervalType SQL_INTERVAL_DAY_TO_SECOND_MAX_PRECISION =
+        IntervalType.newIntervalType(Types.SQL_INTERVAL_DAY_TO_SECOND,
+                                     DTIType.maxIntervalPrecision,
+                                     DTIType.maxFractionPrecision);
+
     //
     public static final ArrayType SQL_ARRAY_ALL_TYPES =
         new ArrayType(SQL_ALL_TYPES, 0);
 
-
     public static ArrayType getDefaultArrayType(int type) {
-        return new ArrayType(getDefaultType(type), Type.defaultArrayCardinality);
+        return new ArrayType(getDefaultType(type),
+                             ArrayType.defaultArrayCardinality);
     }
 
-
     public static Type getDefaultType(int type) {
 
         try {
-            return getType(type, 0, 0, 0);
+            return getType(type, Type.SQL_VARCHAR.getCharacterSet(),
+                           Type.SQL_VARCHAR.getCollation(), 0, 0);
         } catch (Exception e) {
             return null;
         }
@@ -688,6 +815,9 @@ public abstract class Type implements SchemaObject, Cloneable {
             case Types.SQL_ALL_TYPES :
                 return SQL_ALL_TYPES;
 
+            case Types.SQL_ARRAY :
+                return SQL_ARRAY_ALL_TYPES;
+
             case Types.SQL_CHAR :
                 return SQL_CHAR_DEFAULT;
 
@@ -829,6 +959,9 @@ public abstract class Type implements SchemaObject, Cloneable {
             case Types.BLOB :
                 return Types.SQL_BLOB;
 
+            case Types.ARRAY :
+                return Types.SQL_ARRAY;
+
             default :
                 return jdbcTypeNumber;
         }
@@ -864,6 +997,9 @@ public abstract class Type implements SchemaObject, Cloneable {
             case Types.SQL_BIT_VARYING :
                 return Types.BIT;
 
+            case Types.SQL_ARRAY :
+                return Types.ARRAY;
+
             default :
                 return type;
         }
@@ -872,8 +1008,8 @@ public abstract class Type implements SchemaObject, Cloneable {
     /**
      * Enforces precision and scale limits on type
      */
-    public static Type getType(int type, int collation, long precision,
-                               int scale) {
+    public static Type getType(int type, Charset charset, Collation collation,
+                               long precision, int scale) {
 
         switch (type) {
 
@@ -885,7 +1021,8 @@ public abstract class Type implements SchemaObject, Cloneable {
             case Types.SQL_VARCHAR :
             case Types.VARCHAR_IGNORECASE :
             case Types.SQL_CLOB :
-                return CharacterType.getCharacterType(type, precision);
+                return CharacterType.getCharacterType(type, precision,
+                                                      collation);
 
             case Types.SQL_INTEGER :
                 return SQL_INTEGER;
@@ -974,44 +1111,74 @@ public abstract class Type implements SchemaObject, Cloneable {
 
     public static final IntValueHashMap typeAliases;
     public static final IntValueHashMap typeNames;
+    public static final IntKeyHashMap   jdbcConvertTypes;
 
     static {
         typeNames = new IntValueHashMap(37);
 
-        typeNames.put("CHARACTER", Types.SQL_CHAR);
-        typeNames.put("VARCHAR", Types.SQL_VARCHAR);
-        typeNames.put("VARCHAR_IGNORECASE", Types.VARCHAR_IGNORECASE);
-        typeNames.put("DATE", Types.SQL_DATE);
-        typeNames.put("TIME", Types.SQL_TIME);
-        typeNames.put("TIMESTAMP", Types.SQL_TIMESTAMP);
-        typeNames.put("INTERVAL", Types.SQL_INTERVAL);
-        typeNames.put("TINYINT", Types.TINYINT);
-        typeNames.put("SMALLINT", Types.SQL_SMALLINT);
-        typeNames.put("INTEGER", Types.SQL_INTEGER);
-        typeNames.put("BIGINT", Types.SQL_BIGINT);
-        typeNames.put("REAL", Types.SQL_REAL);
-        typeNames.put("FLOAT", Types.SQL_FLOAT);
-        typeNames.put("DOUBLE", Types.SQL_DOUBLE);
-        typeNames.put("NUMERIC", Types.SQL_NUMERIC);
-        typeNames.put("DECIMAL", Types.SQL_DECIMAL);
-        typeNames.put("BOOLEAN", Types.SQL_BOOLEAN);
-        typeNames.put("BINARY", Types.SQL_BINARY);
-        typeNames.put("VARBINARY", Types.SQL_VARBINARY);
-        typeNames.put("CLOB", Types.SQL_CLOB);
-        typeNames.put("BLOB", Types.SQL_BLOB);
-        typeNames.put("BIT", Types.SQL_BIT);
-        typeNames.put("OTHER", Types.OTHER);
+        typeNames.put(Tokens.T_CHARACTER, Types.SQL_CHAR);
+        typeNames.put(Tokens.T_VARCHAR, Types.SQL_VARCHAR);
+        typeNames.put(Tokens.T_VARCHAR_IGNORECASE, Types.VARCHAR_IGNORECASE);
+        typeNames.put(Tokens.T_NVARCHAR, Types.SQL_VARCHAR);
+        typeNames.put(Tokens.T_DATE, Types.SQL_DATE);
+        typeNames.put(Tokens.T_TIME, Types.SQL_TIME);
+        typeNames.put(Tokens.T_TIMESTAMP, Types.SQL_TIMESTAMP);
+        typeNames.put(Tokens.T_INTERVAL, Types.SQL_INTERVAL);
+        typeNames.put(Tokens.T_TINYINT, Types.TINYINT);
+        typeNames.put(Tokens.T_SMALLINT, Types.SQL_SMALLINT);
+        typeNames.put(Tokens.T_INTEGER, Types.SQL_INTEGER);
+        typeNames.put(Tokens.T_BIGINT, Types.SQL_BIGINT);
+        typeNames.put(Tokens.T_REAL, Types.SQL_REAL);
+        typeNames.put(Tokens.T_FLOAT, Types.SQL_FLOAT);
+        typeNames.put(Tokens.T_DOUBLE, Types.SQL_DOUBLE);
+        typeNames.put(Tokens.T_NUMERIC, Types.SQL_NUMERIC);
+        typeNames.put(Tokens.T_DECIMAL, Types.SQL_DECIMAL);
+        typeNames.put(Tokens.T_BOOLEAN, Types.SQL_BOOLEAN);
+        typeNames.put(Tokens.T_BINARY, Types.SQL_BINARY);
+        typeNames.put(Tokens.T_VARBINARY, Types.SQL_VARBINARY);
+        typeNames.put(Tokens.T_CLOB, Types.SQL_CLOB);
+        typeNames.put(Tokens.T_BLOB, Types.SQL_BLOB);
+        typeNames.put(Tokens.T_BIT, Types.SQL_BIT);
+        typeNames.put(Tokens.T_OTHER, Types.OTHER);
 
         //
         typeAliases = new IntValueHashMap(64);
 
-        typeAliases.put("CHAR", Types.SQL_CHAR);
-        typeAliases.put("INT", Types.SQL_INTEGER);
-        typeAliases.put("DEC", Types.SQL_DECIMAL);
-        typeAliases.put("LONGVARCHAR", Types.LONGVARCHAR);
-        typeAliases.put("DATETIME", Types.SQL_TIMESTAMP);
-        typeAliases.put("LONGVARBINARY", Types.LONGVARBINARY);
-        typeAliases.put("OBJECT", Types.OTHER);
+        typeAliases.put(Tokens.T_CHAR, Types.SQL_CHAR);
+        typeAliases.put(Tokens.T_INT, Types.SQL_INTEGER);
+        typeAliases.put(Tokens.T_DEC, Types.SQL_DECIMAL);
+        typeAliases.put(Tokens.T_LONGVARCHAR, Types.LONGVARCHAR);
+        typeAliases.put(Tokens.T_DATETIME, Types.SQL_TIMESTAMP);
+        typeAliases.put(Tokens.T_LONGVARBINARY, Types.LONGVARBINARY);
+        typeAliases.put(Tokens.T_OBJECT, Types.OTHER);
+
+        //
+        jdbcConvertTypes = new IntKeyHashMap(37);
+
+        jdbcConvertTypes.put(Tokens.SQL_CHAR, Type.SQL_CHAR_DEFAULT);
+        jdbcConvertTypes.put(Tokens.SQL_VARCHAR, Type.SQL_VARCHAR_DEFAULT);
+        jdbcConvertTypes.put(Tokens.SQL_LONGVARCHAR, Type.SQL_VARCHAR_DEFAULT);
+        jdbcConvertTypes.put(Tokens.SQL_NVARCHAR, Type.SQL_VARCHAR_DEFAULT);
+        jdbcConvertTypes.put(Tokens.SQL_DATE, Type.SQL_DATE);
+        jdbcConvertTypes.put(Tokens.SQL_TIME, Type.SQL_TIME);
+        jdbcConvertTypes.put(Tokens.SQL_TIMESTAMP, Type.SQL_TIMESTAMP);
+        jdbcConvertTypes.put(Tokens.SQL_TINYINT, Type.TINYINT);
+        jdbcConvertTypes.put(Tokens.SQL_SMALLINT, Type.SQL_SMALLINT);
+        jdbcConvertTypes.put(Tokens.SQL_INTEGER, Type.SQL_INTEGER);
+        jdbcConvertTypes.put(Tokens.SQL_BIGINT, Type.SQL_BIGINT);
+        jdbcConvertTypes.put(Tokens.SQL_REAL, Type.SQL_DOUBLE);
+        jdbcConvertTypes.put(Tokens.SQL_FLOAT, Type.SQL_DOUBLE);
+        jdbcConvertTypes.put(Tokens.SQL_DOUBLE, Type.SQL_DOUBLE);
+        jdbcConvertTypes.put(Tokens.SQL_NUMERIC, Type.SQL_NUMERIC);
+        jdbcConvertTypes.put(Tokens.SQL_DECIMAL, Type.SQL_DECIMAL);
+        jdbcConvertTypes.put(Tokens.SQL_BOOLEAN, Type.SQL_BOOLEAN);
+        jdbcConvertTypes.put(Tokens.SQL_BINARY, Type.SQL_BINARY_DEFAULT);
+        jdbcConvertTypes.put(Tokens.SQL_VARBINARY, Type.SQL_VARBINARY_DEFAULT);
+        jdbcConvertTypes.put(Tokens.SQL_LONGVARBINARY,
+                             Type.SQL_VARBINARY_DEFAULT);
+        jdbcConvertTypes.put(Tokens.SQL_CLOB, Type.SQL_CLOB);
+        jdbcConvertTypes.put(Tokens.SQL_BLOB, Type.SQL_BLOB);
+        jdbcConvertTypes.put(Tokens.SQL_BIT, Type.SQL_BIT);
     }
 
     public static int getTypeNr(String name) {
@@ -1025,6 +1192,10 @@ public abstract class Type implements SchemaObject, Cloneable {
         return i;
     }
 
+    public static Type getTypeForJDBCConvertToken(int name) {
+        return (Type) jdbcConvertTypes.get(name);
+    }
+
     public static boolean isSupportedSQLType(int typeNumber) {
 
         if (getDefaultType(typeNumber) == null) {
