@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2010, The HSQL Development Group
+/* Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@ import org.hsqldb.lib.IntValueHashMap;
 import org.hsqldb.persist.HsqlDatabaseProperties;
 
 // boucherb@users 20051207 - patch 1.8.x initial JDBC 4.0 support work
+// fredt@users - 2.0.0 code changes
 
 /**
  * Defines the constants that are used to identify SQL types for HSQLDB JDBC
@@ -47,7 +48,7 @@ import org.hsqldb.persist.HsqlDatabaseProperties;
  * has been added to differentiate HSQLDB-specific type specializations.
  *
  * @author  boucherb@users
- * @version 2.0.0
+ * @version 2.0.1
  * @since 1.7.2
  */
 public class Types {
@@ -408,7 +409,7 @@ public class Types {
      * @since JDK 1.6, HSQLDB 1.8.x
      *
      */
-    public final static int ROWID = 2008;
+    public static final int ROWID = 2008;
 
     /**
      * The constant in the Java programming language, sometimes referred to
@@ -542,13 +543,13 @@ public class Types {
     };
 
 // lookup for types
-    final static IntValueHashMap javaTypeNumbers;
+    static final IntValueHashMap javaTypeNumbers;
 
 //  boucherb@users - We can't handle method invocations in
 //                   Function.java whose number class is
 //                   narrower than the corresponding internal
 //                   wrapper
-    private final static HashSet illegalParameterClasses;
+    private static final HashSet illegalParameterClasses;
 
     static {
         javaTypeNumbers = new IntValueHashMap(32);
@@ -561,6 +562,8 @@ public class Types {
         javaTypeNumbers.put(DateClassName, Types.SQL_DATE);
         javaTypeNumbers.put(TimeClassName, Types.SQL_TIME);
         javaTypeNumbers.put(TimestampClassName, Types.SQL_TIMESTAMP);
+        javaTypeNumbers.put(BlobClassName, Types.SQL_BLOB);
+        javaTypeNumbers.put(ClobClassName, Types.SQL_CLOB);
         javaTypeNumbers.put("java.util.Date", Types.SQL_DATE);
         javaTypeNumbers.put(DecimalClassName, Types.SQL_DECIMAL);
         javaTypeNumbers.put("boolean", Types.SQL_BOOLEAN);
@@ -589,27 +592,9 @@ public class Types {
     }
 
     /**
-     * Retieves the type number corresponding to the class
-     * of an IN, IN OUT or OUT parameter.  <p>
+     * Retieves the type object corresponding to the class
+     * of an IN, IN OUT or OUT parameter or a return type.  <p>
      *
-     * This method extends getTypeNr to return OTHER for
-     * primitive arrays, classes that directly implement
-     * java.io.Serializable and non-primitive arrays whose
-     * base component implements java.io.Serializable,
-     * allowing, for instance, arguments and return types of
-     * primitive arrays, Serializable objects and arrays,
-     * of Serializable objects.  Direct primitive types
-     * other than those mapping directly to the internal
-     * wrapper form are not yet handled.  That is, HSQLDB
-     * cannot yet properly deal with CALLs involving methods
-     * with primitive byte, short, float or their
-     * corresponding wrappers, due to the way internal
-     * conversion works and lack of detection and narrowing
-     * code in Function to allow this.  In other words,
-     * passing in or retrieving any of the mentioned types
-     * always causes conversion to a wider internal wrapper
-     * which is genrally incompatible under reflective
-     * invocation, resulting in an IllegalArgumentException.
      *
      * @param  c a Class instance
      * @return java.sql.Types int value
@@ -653,6 +638,10 @@ public class Types {
             return Type.getDefaultTypeWithSize(typeCode);
         }
 
+        if (name.equals("java.sql.Array")) {
+            return Type.getDefaultArrayType(Types.SQL_ALL_TYPES);
+        }
+
         return null;
     }
 
@@ -677,7 +666,6 @@ public class Types {
             case Types.SQL_VARBINARY :
             case Types.SQL_VARCHAR :
             case Types.SQL_NVARCHAR :
-            case Types.VARCHAR_IGNORECASE :
                 return true;
 
             default :
