@@ -1,7 +1,7 @@
 /*
  * For work developed by the HSQL Development Group:
  *
- * Copyright (c) 2001-2010, The HSQL Development Group
+ * Copyright (c) 2001-2011, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -1268,7 +1268,7 @@ implements WindowListener, ActionListener, ItemListener, Traceable {
             String last = tMessage.getText();
 
             trace("Transfer stopped - " + last + " /  / Error: "
-                  + e.getMessage());
+                  + e.toString());
             e.printStackTrace();
         }
 
